"""
Filename: create_forbes_400_dataset.py
Python version: 2.7
Source: Capehart, Kevin W. Essays on the Wealthiest Americans. PhD dissertation, American University, Washington, DC, 2014. 
Description: This program creates a dataset based on Forbes Magazine's annual list of the 400 wealthiest Americans. The primary differences between the dataset created by this program and the information reported by the magazine are that unique individuals have the same names across years, some wealth estimates are imputed, and some of the ages reported by the magazine are corrected. See the dissertation cited above and the code below for more details on the manner in which unique individuals were identified, some wealths were imputed, and some ages were corrected. 
"""

## IMPORT SOME STANDARD MODULES ##

from __future__ import division # define division as in Python v3
import numpy # import for its many useful functions
from os.path import isfile # import for checking whether a file exists

## IMPORT RAW FORBES 400 DATA ##

# The data to be imported is organized so that each row corresponds to an individual on Forbes Magazine's list of the 400 wealthiest Americans in a given year. Some of the rows also correspond to a family on the magazine's accompanying list of some of the wealthiest families in America the given year, but we will ignore those rows. (The distinction between those two lists is discussed in the dissertation.) For the rows of interest, the columns correspond to an individual's name, age, wealth (in millions of current dollars), the specific source of his or her wealth, the general industry in which his or her wealth was made, and whether his or her wealth was self-made. That data is almost identical to the information reported by each year of the magazine's list, except that some of the names were modified so that unique individuals have the same names across years. See the dissertation for a discussion of how the data was collected and how the unique individuals were identified. Of note, data on whether someone's wealth was self-made, as well as data on the industry in which someone made his or her wealth, is only available for the years since 1996 when the magazine started making its list available online. Also of note, Forbes discontinued its list of some of the wealthiest families in America after 1999. 

# define the filename for the raw data
filename = "./data/raw_forbes_400_data.txt"

# assertion error if file does not exist
assert isfile(filename), "Forbes Magazine declined to directly share its data on the 400 wealthiest Americans with the author (personal communication on June 17, 2009, with Tanya Prive, circulation assistant at Forbes Magazine), so the author went through each year of the magazine's list and collected the data himself. The magazine also declined the author's request to republish that raw data as part of his dissertation (personal communication on May 8, 2014, with Elena Coster, sales coordinator at Forbes Magazine's content management firm, PARS International). As such, a file with the raw data collected from the magazine is not included as a supplementary file to the dissertation. The code for processing the rawer version of the data ('create_forbes_400_dataset.py') cannot be executed without that version of the data (named 'raw_forbes_400_data.txt' by the author). The processed version of the data ('forbes_400_dataset.txt') is included as a supplementary file to the dissertation, however, and the manner in which that version of the data was produced can be seen by looking at the code for processing the rawer version."

# get the header for the raw data
skiprows = 4 # number of rows to skip in file
delimiter = '\t' # file delimiter
f = open(filename, "r") # open file
header = f.readlines()[skiprows-1].replace('\n', '').split(delimiter)
f.close() # close file

# load the data with the headers
rawdata = numpy.loadtxt(filename, delimiter=delimiter, skiprows=skiprows, dtype={'names':(header), 'formats':(['<S99']*len(header))})

# get the most-recent year of the list
last_year = max([int(year) for year in set(rawdata['year'])])

## REMOVE ANY FAMILY MEMBERS FROM AN INDIVIDUAL'S NAME ##

# As discussed in the dissertation, an individual's wealth can include the wealth of family members in some cases. In such cases, an individual's name is usually given as his or name followed by the phrase ' and Family'. We'll remove that phrase. 

for i in range(0, len(rawdata)):
	if ' and Family' in rawdata['name'][i]:
		rawdata['name'][i] = rawdata['name'][i].replace(' and Family', '')

# There are also a few cases that require special attention. In 2011, in three cases, a husband and wife were listed together, rather than as one individual and their family. The husband-wife teams were: Michael and Marian Ilitch, Stewart and Lynda Resnick, and Do Won and Jin Sook Chang. Michael Illitch had been on the list in the past, as either himself or 'Michael Illitch and Family', so we will remove ' and Marian Ilitch' from the name 'Ilitch, Michael and Marian Ilitch'. The other individuals were new to the Forbes 400. Forbes appears to report the age of Do Won Chang, so we will remove ' and Jin Sook Chang' from the name 'Sook, Jin and Do Won Chang'. Forbes does not report an age for the Resnicks, so we will arbitrarily remove ' and Lynda Resnick' from the name 'Resnick, Stewart and Lynda Resnick'. We'll do the same for all other years, as well. In 2012 and 2013, the brothers Hank and Doug Meijer were listed together, rather than as one individual and their family. Hank is the older brother, so we'll remove ' and Doug' from the name 'Meijer, Hank and Doug'. For the same years, we'll also arbitrarily remove ' and Judy' from the husband-wife team 'Love, Tom and Judy' and remove ' and Peggy' from the husband-wife team 'Cherng, Andrew and Peggy'. Note that, if someone in any one of those pairs eventually dies while the other remains on the list, then, for any sort of longitudinal analysis, we should reconsider which family members (if any) should be ignored.  

for i in range(0, len(rawdata)):
	if rawdata['name'][i] == 'Ilitch, Michael and Marian Ilitch':
		rawdata['name'][i] = rawdata['name'][i].replace(' and Marian Ilitch', '')
	if rawdata['name'][i] == 'Chang, Do Won and Jin Sook Chang':
		rawdata['name'][i] = rawdata['name'][i].replace(' and Jin Sook Chang', '')
	if rawdata['name'][i] == 'Resnick, Stewart and Lynda Resnick':
		rawdata['name'][i] = rawdata['name'][i].replace(' and Lynda Resnick', '')
	if rawdata['name'][i] == 'Meijer, Hank and Doug':
		rawdata['name'][i] = rawdata['name'][i].replace(' and Doug', '')
	if rawdata['name'][i] == 'Love, Tom and Judy':
		rawdata['name'][i] = rawdata['name'][i].replace(' and Judy', '')
	if rawdata['name'][i] == 'Cherng, Andrew and Peggy':
		rawdata['name'][i] = rawdata['name'][i].replace(' and Peggy', '')

## GET INDIVIDUALS' NAMES ##

# construct a list of non-unique names
names = dict(zip(['including families', 'excluding families'], \
	[list(), list()]))
for i in range(0, len(rawdata)):
	name = rawdata['name'][i]
	names['including families'].append(name)
	# the age associated with a family is 'n.a.' in the raw data in most cases
	if rawdata['age'][i] != 'n.a.':
		names['excluding families'].append(name)

# get unique names
for _ in ['including families', 'excluding families']:
	names[_] = list(set(names[_]))

# If excluding families, exclude family names that have associated ages that were not excluded by the code excuted above. It is unclear why these names have ages associated with them in the magazine, but they are families and not individuals. 
family_names_to_remove = ['Alfond Family', 'Breed Family', 'Magness Family', 'Martin Family', 'Miner Family', 'Ward Family']
for name in family_names_to_remove:
	names['excluding families'].remove(name)

# alphabetize names
for _ in ['including families', 'excluding families']:
	names[_].sort()

## EXPORT DATA ORGANIZED BY UNIQUE INDIVIDUALS ##

# if the file already exists, set the following to True to recreate it
re_create_file = False
filename = "./data/reorganized_forbes_400_data_1982_to_" + str(last_year) + ".txt"
if not isfile(filename) or re_create_file:
	# open file
	table = open(filename, 'w')
	# write first header for the table
	infos = ['wealth', 'age', 'source', 'industry', 'selfmade or inherited']
	years = [str(y) for y in range(1982, last_year+1, 1)]
	header = 'name\t'
	for info in infos:
		header += info + '\t'
		for year in years[1:]:
			if (info == infos[-1]) and (year == years[-1]):
				header += '\n'
			else:
				header += '\t' # if last info and last year, then break line
	table.write(header)
	# write second header for the table
	header = '/year\t'
	for info in infos:
		for year in years:
			header += year
			if (info == infos[-1]) and (year == years[-1]):
				header += '\n'
			else:
				header += '\t'
	table.write(header)
	# write data
	for name in names['excluding families']: # note: excluding families
		table.write(name + '\t') # write name
		for info in infos:
			for year in years:
				for i in range(0, len(rawdata)):
					if (name == rawdata['name'][i]) and (year == rawdata['year'][i]):
						table.write(rawdata[info][i])
						break
				if (info == infos[-1]) and (year == years[-1]):
					table.write('\n')
				else:
					table.write('\t')
	# close file
	table.close()
else:
	print "Note that 're_create_file' must be set to True in order to recreate the file that reorganizes the raw data."
	
## IMPORT DATA THAT WAS JUST EXPORTED ##

# infos and years
infos = ['wealth', 'age', 'source', 'industry', 'selfmade or inherited']
years = [str(y) for y in range(1982, last_year+1, 1)]
# define header
header = ['name']
for info in infos:
	header.extend(info+year for year in years)
# import data
rawdata = numpy.loadtxt(filename, delimiter='\t', skiprows=2, \
	dtype={'names':(header), 'formats':(['<S99']*len(header))})
# create a dictionary of the data
data = dict(zip(rawdata['name'], [dict(zip(['wealth', 'age', 'source', 'industry', 'selfmade or inherited'], [dict(zip(years, [rawdata['wealth'+y][i] for y in years])), dict(zip(years, [rawdata['age'+y][i] for y in years])), dict(zip(years, [rawdata['source'+y][i] for y in years])), dict(zip(years, [rawdata['industry'+y][i] for y in years])), dict(zip(years, [rawdata['selfmade or inherited'+y][i] for y in years]))])) for i in range(0, len(rawdata['name']))]))
# get list of names sorted alphabetically
names = sorted(data.keys())

## USE OTHER INFO TO SOME FIX AGES ##

# The following age is missing, but it can be fixed with other information from forbes. According to http://www.forbes.com/global/2001/0709/030tab1.html, David Oros was 41 in 2001, so he would've been about 40 in 2000. 
data['Oros, David S']['age']['2000'] = '40'

## EXTRAPOLATE AND CORRECT SOME AGES ##

# some ages are "bad" in the sense that they're not reported or not reported as an integer
bad_ages = ['n.a.', 'blank', 'page23', 'died after press closing', \
	'unknown', '40s', '50s', 'early50s', '60s', 'late60s', '70s', '70+', 'early70s', \
	'mid70s', '75+', 'late70s', '80s', 'late80s']

# keep track of names with ages that are extrapolated or corrected
names_with_ages_extrapolated, names_with_corrected_ages = [], []

# fix the following ages by extrapolating forward
data['Lauder, Estee']['age']['1995'] = '87' # roll her age forward
data['Busch, August Anheuser Jr']['age']['1989'] = '90' # roll his age forward
names_with_ages_extrapolated.append('Lauder, Estee')
names_with_ages_extrapolated.append('Busch, August Anheuser Jr')

# define years in descending order
descending_years = [str(y) for y in range(last_year, 1982-1, -1)]

# fix the following ages by extrapolating backwards or correcting inconsistencies
# for each name
for name in names:
	# define a variable for the last age that was observed, extrapolated backwards, or corrected
	last_age = ''
	# look at each year in descending order
	for year in descending_years:
		# get the name's age in a given year
		age = data[name]['age'][year]
		# if an age is not observed in the given year and an age wasn't observed in later years, then pass
		if (age == '') and (last_age == ''):
			pass
		# if an age is not observed in the given year, but an age was observed in later years
		elif (age == '') and (last_age != ''):
			# continue to extrapolate the observed age backwards by subtracting unity
			last_age = str(int(last_age) - 1)
		# if a bad age is observed in the given year, but an age was observed in later years
		elif (age != '') and (age in bad_ages) and (last_age != ''):
			# extrapolate the observed age backwards by subtracting unity
			data[name]['age'][year] = str(int(last_age) - 1)
			# redefine the last age as this age
			last_age = str(int(last_age) - 1)
			# keep track of the names with extrapolated ages
			names_with_ages_extrapolated.append(name)
		# if a good age is observed in the given year and an age wasn't observed in later years
		elif (age != '') and (age not in bad_ages) and (last_age == ''):
			# redefine the last age as this age
			last_age = data[name]['age'][year]
		# if a good age is observed in the given year and an age was also observed in later years
		elif (age != '') and (age not in bad_ages) and (last_age != ''):
			# check to see if the age is consistent by calculating the difference between the age observed in later years extrapolated backwards, on the one hand, and the age observed in the given year, on the other hand. 
			age_difference = int(last_age) - int(age)
			# depending on when their birthday occurs, the difference should be between zero and two years
			# if age is consistent, then redefine last age
			if 2 >= age_difference >= 0:
				last_age = age
			# if age is inconsistent
			else:
				# correct it
				data[name]['age'][year] = str(int(last_age) - 1)
				# redefine the last age as this age
				last_age = str(int(last_age) - 1)
				# keep track of the names with corrected ages
				names_with_corrected_ages.append(name)

# EXTRAPOLATIONS: If a person's age was missing in a given year, but she appeared on the list in a latter year with a non-missing age, then it was assumed that the most-recent non-missing age was correct, and the earlier missing age was extrapolated based on it. The ages of 22 people (61 observations) were extrapolated, in total. 
#print "The ages of " + str(len(list(set(names_with_ages_extrapolated)))) + " people (" + str(len(names_with_ages_extrapolated)) + " observations) were extrapolated, in total. "

# CORRECTIONS: Note that, if the ages are correct, then the age difference for a given person between two lists should be no less than zero years (or else they would've grown younger) and at most two years (since the list is not constructed on the same date every year, it is possible that a person could have two birthdays between two lists). If those conditions did not hold, then we assumed that the most-recent age was correct while the latter age was incorrect, which is reasonable since new information presumably came to light. The ages of 53 people (111 observations) were corrected, in total. 
#print "The ages of " + str(len(list(set(names_with_corrected_ages)))) + " people (" + str(len(names_with_corrected_ages)) + " observations) were corrected, in total. "

## IMPUTE SOME AGES ##

# For each year, we'll impute unknown ages as the average age of a relevant group. M.S. Forbes' ages will be inclued in all of the imputations, as they should be, given that he was a member of the Forbes 400.  

# create list of age categories
age_categories = ['all age categories']
rough_ages = ['40s', '60s', '70s', '80s', 'late70s', '70+', '75+']
age_categories.extend(rough_ages)
# create dict of ages by year by age category
ages_in = dict(zip(age_categories, [dict(zip(years, [[] for year in years])) for age_category in age_categories]))

# define a function that takes an age and returns an age group
def get_rough_age(age):
	if 40 <= age < 50:
		rough_age = rough_ages[0]
	elif 60 <= age < 70:
		rough_age = rough_ages[1]
	elif 70 <= age < 80:
		rough_age = rough_ages[2]
	elif 80 <= age < 90:
		rough_age = rough_ages[3]
	else:
		rough_age = 'ignore'
	return rough_age

# fill the dictionary
for name in names:
	for year in years:
		age = data[name]['age'][year]
		if (age != '') and (age not in bad_ages):
			ages_in['all age categories'][year].append(int(age))
			rough_age = get_rough_age(int(age))
			# age categories
			if rough_age != 'ignore':
				ages_in[rough_age][year].append(int(age))
			# late 70s
			if 75 <= int(age) < 80:
				ages_in['late70s'][year].append(int(age))
			# 70+
			if int(age) >= 70:
				ages_in['70+'][year].append(int(age))
			# 75 +
			if int(age) >= 75:
				ages_in['75+'][year].append(int(age))

# calculate average age by age category by year
avg_age_in = dict(zip(age_categories, [dict(zip(years, [str(int(round(numpy.mean(ages_in[age_category][year])))) for year in years])) for age_category in age_categories]))

# 'unknown' ages are imputed as the average age for the given year (3 people, 5 observations)
data['Pan, Jing Jong']['age']['2000'] = avg_age_in['all age categories']['2000']
data['Pan, Theresa']['age']['2000'] = avg_age_in['all age categories']['2000']
data['Wiskemann, Elizabeth S']['age']['2004'] = avg_age_in['all age categories']['2004']
data['Wiskemann, Elizabeth S']['age']['2005'] = avg_age_in['all age categories']['2005']
data['Wiskemann, Elizabeth S']['age']['2006'] = avg_age_in['all age categories']['2006']

# conditional ages are imputed as average conditional age (12 people, 29 observations)
data['Bettingen, Burton Green']['age']['1982'] = avg_age_in['70+']['1982']
data['Bettingen, Burton Green']['age']['1983'] = avg_age_in['70+']['1983']
data['Bettingen, Burton Green']['age']['1984'] = avg_age_in['70s']['1984']
data['Bettingen, Burton Green']['age']['1985'] = avg_age_in['70s']['1985']
data['Bettingen, Burton Green']['age']['1986'] = avg_age_in['70s']['1986']
data['Copeland, Gerret van Sweringen']['age']['1985'] = avg_age_in['40s']['1985']
data['Copeland, Pamela Cunningham']['age']['1985'] = avg_age_in['70s']['1985']
data['Franchetti, Anne']['age']['1991'] = avg_age_in['70s']['1991']
data['Franchetti, Anne']['age']['1992'] = avg_age_in['70s']['1992']
data['Franchetti, Anne']['age']['1993'] = avg_age_in['70s']['1993']
data['Franchetti, Anne']['age']['1994'] = avg_age_in['70s']['1994']
data['Green, Dorothy (Dolly)']['age']['1982'] = avg_age_in['70+']['1982']
data['Green, Dorothy (Dolly)']['age']['1983'] = avg_age_in['70s']['1983']
data['Green, Dorothy (Dolly)']['age']['1984'] = avg_age_in['70s']['1984']
data['Green, Dorothy (Dolly)']['age']['1985'] = avg_age_in['70s']['1985']
data['Green, Dorothy (Dolly)']['age']['1986'] = avg_age_in['70s']['1986']
data['Green, Dorothy (Dolly)']['age']['1987'] = avg_age_in['late70s']['1987']
data['Green, Dorothy (Dolly)']['age']['1988'] = avg_age_in['80s']['1988']
data['Jones, Arthur']['age']['1983'] = avg_age_in['60s']['1983']
data['Kauffman, Muriel']['age']['1994'] = avg_age_in['70s']['1994']
data['Lauder, Joseph']['age']['1982'] = avg_age_in['70s']['1982']
data['Rains, Liliore Green']['age']['1982'] = avg_age_in['70+']['1982']
data['Rains, Liliore Green']['age']['1983'] = avg_age_in['70s']['1983']
data['Rains, Liliore Green']['age']['1984'] = avg_age_in['70s']['1984']
data['Rains, Liliore Green']['age']['1985'] = avg_age_in['70s']['1985']
data['Rains, Liliore Green']['age']['1986'] = avg_age_in['70s']['1986']
data['Whittier, Leland K']['age']['1982'] = avg_age_in['75+']['1982']
data['Whittier, N Paul']['age']['1982'] = avg_age_in['75+']['1982']
data['Woodward, Helen Whittier']['age']['1982'] = avg_age_in['75+']['1982']

# INTERPOLATIONS: The ages of 15 people (34 observations) were imputed, of which 3 people (5 observations) had unknown ages and 12 people (29 observations) had conditional ages (e.g., "70s"). Unknown ages were imputed as the year-specific average age. Conditional ages were imputed as the age-specific average conditional age; "70s" was imputed as the average age of people in their 70s, for example. 

# SUMMARY: The ages of 22 people (61 observations) were extrapolated. The ages of 53 people (111 observations) were corrected. And the ages of 15 people (34 observations) were imputed. So, in total, the ages of 90 people (206 observations) were extrapolated, corrected, or imputed. That represents a relatively small number of people: about six percent, specifically, 100. * (90. / 1474.) percent, of the (unique) people in the sample. It also represents a relatively small number of the age observations: about two percent, specifically, 100. * (206. / (400. * 32.)) percent, of the age observations

## IMPUTE THE WEALTH OF M.S. FORBES ##

# M.S. Forbes was one of Forbes 400 in the 1982 to 1989 lists (dying by 1990 list), but the magazine did not report an estimate of his wealth. We'll impute his wealth in a given year as the median wealth of the other 399 wealthiest Americans on the list in the given year 

# remove M.S. Forbes from the list of names
names.remove('Forbes, Malcolm Stevenson')
# calculate median wealth with M.S. Forbes
median_wealth_in = dict(zip(years, [[] for year in years]))
for year in years:
	for name in names:
		wealth = data[name]['wealth'][year]
		if wealth != '':
			median_wealth_in[year].append(float(wealth))
for year in years:
	median_wealth_in[year] = numpy.median(median_wealth_in[year])
# impute his wealth as the median wealth (1 person, 8 observations)
for year in [str(y) for y in range(1982, 1989+1)]:
	data['Forbes, Malcolm Stevenson']['wealth'][year] = str(int(median_wealth_in[year]))
# add M.S. Forbes back to list of names and sort the names alphabetically
names.append('Forbes, Malcolm Stevenson')
names.sort()

# SUMMARY: The wealths of one person (eight observations) were imputed as the year-specific median wealth. 

## INDUSTRY DATA ##

# impute missing industries
data['Anderson, John Edward']['industry']['2000'] = 'Beverages'
data['Diller, Barry']['industry']['2000'] = 'Media/Entertainment'
data['Disney, Roy Edward']['industry']['2000'] = 'Media/Entertainment'
data['Eisner, Michael D']['industry']['2000'] = 'Media/Entertainment'
data['Gallo, Ernest']['industry']['2000'] = 'Beverages'
data['Gonda, Louis L']['industry']['2000'] = 'Investments' # source of wealth in 1999 was 'AIG stock', so industry should be investments. Industry was also investments in 1999
data['Kimmel, Sidney']['industry']['2000'] = 'Apparel'
data['Stowers, James Evans Jr']['industry']['2000'] = 'Finance'

# change 'Technology/Medicine' to 'Technology' for the following
data['Bezos, Jeffrey P']['industry']['2009'] = 'Technology'
data['Dangermond, Jack']['industry']['2009'] = 'Technology'
data['Dolby, Ray Milton']['industry']['2009'] = 'Technology'
data['Gates, William Henry III']['industry']['2009'] = 'Technology'
data['Kim, James']['industry']['2009'] = 'Technology'
data['Sun, David']['industry']['2009'] = 'Technology'
data['Zuckerberg, Mark']['industry']['2009'] = 'Technology'

# change 'Technology/Medicine' to 'Healthcare' for the following
data['Brown, John W']['industry']['2009'] = 'Healthcare'
data['Rahr, Stewart']['industry']['2009'] = 'Healthcare'

# should be 'Investments' for the following
data['Picower, Jeffry']['industry']['2009'] = 'Investments'

# change to 'Information Technology', 'Internet', and 'Software' to 'Technology'  for simplicity
# also change 'Investments' and 'Finance/Investment' to 'Finance' for simplicity
for year in years:
	for name in names:
		wealth = data[name]['wealth'][year]
		if wealth != '':
			industry = data[name]['industry'][year]
			# change to 'Information Technology', 'Internet', and 'Software' to 'Technology'
			if industry in ['Information Technology', 'Internet', 'Software']:
				data[name]['industry'][year] = 'Technology'
			# change 'Investments' and 'Finance/Investment' to 'Finance'
			if industry in ['Investments', 'Finance/Investment']:
				data[name]['industry'][year] = 'Finance'

## SELF-MADE OR INHERITED DATA ##

# impute missing
data['McGlothlin, James']['selfmade or inherited']['1997'] = 'Self made' # this imputation can be justified in different ways, but based on earlier years of the Forbes 400 list and other information, he does not appear to have inherited his wealth
data['Corn, Elizabeth Turner']['selfmade or inherited']['1996'] = 'Inherited' # if someone's priminary source of wealth is 'Inheritance' and wealth is inherited in other years, then wealth shouldn't be 'Self made'
data['Boudjakdji, Millicent V']['selfmade or inherited']['1997'] = 'Inherited' # same justification
# it should be noted that 'Copley, Helen Kinney' has a priminary source of 'Inheritance' and her wealth is 'Self made' in 1996, but it's not clear that this is an error when looking at other years, so we'll leave it be

for year in years:
	for name in names:
		wealth = data[name]['wealth'][year]
		if wealth != '':
			# change mixed to 'Both'
			type = data[name]['selfmade or inherited'][year]
			if type in ['Both', 'Built up inheritance', 'Inherited and growing']:
				data[name]['selfmade or inherited'][year] = 'Both'

## EXPORT PROCESSED DATA ##

# open file
filename = "./data/forbes_400_dataset_1982_to_" + str(last_year) + ".txt"
table = open(filename, 'w')
# write license
table.write("This work is licensed under a Creative Commons Attribution-ShareAlike 4.0 International License. For more information on that license see http://creativecommons.org/licenses/by-sa/4.0/. Under the terms of the license, you are free to share and adapt the work for any purpose, but you must give appropriate credit and, if any changes are made to the original version, you must distribute your adapted version under the same license as the original. Preferred citation is: Capehart, Kevin W. Essays on the Wealthiest Americans. PhD dissertation, American University, Washington, DC, 2014.\n\n")
# wite notes
table.write("Note:\n")
table.write("The following table is based on each year of Forbes Magazine's list of the 400 wealthiest Americans between its inaugural year in 1982 and its most-recent year as of writing, which was 2013. Each row of the table corresponds to a unique individual who appeared on the magazine's list in at least one year between those years. Unique individuals were identified by the author in the manner discussed in the dissertation. For individuals who appeared on the magazine's list in a given year, the columns of the table show an individual's wealth (in millions of current dollars), his or her age, the specific source of his or her wealth, the general industry in which his or her wealth was made, and whether his or her wealth was self-made. If an individual did not appear on the list in a given year, then the corresponding entries in the table are blank. All of the entries in the table are based on, but not always the same as, the information reported by the magazine. As discussed in the dissertation, the magazine did not report a wealth estimate for Malcolm Stevenson Forbes during the years in which he appeared on its list, so we imputed his wealth in a given year as the median wealth of the 399 other people on the list in the given year. Some of the ages are also different than those reported by the magazine for the reasons discussed in the dissertation. Of note, information on whether someone's wealth was self-made, as well as information on the industry in which someone made his or her wealth, is only available for the years since 1996 when the magazine started making its list available online. \n\n")
# write first header for the table
header = '\t'
for info in infos:
	header += info + ' in ...' + '\t'
	for year in years[1:]:
		if (info == infos[-1]) and (year == years[-1]):
			header += '\n'
		else:
			header += '\t' # if last info and last year, then break line
table.write(header)
# write second header for the table
header = 'name/year\t'
for info in infos:
	for year in years:
		header += year
		if (info == infos[-1]) and (year == years[-1]):
			header += '\n'
		else:
			header += '\t'
table.write(header)
# write data
for name in names:
	table.write(name + '\t') # write name
	for info in infos:
		for year in years:
			table.write(data[name][info][year])
			if (info == infos[-1]) and (year == years[-1]):
				table.write('\n')
			else:
				table.write('\t')
# close file
table.close()