"""
Filename: data_appendix_code.py
Python version: 2.7
Source: Capehart, Kevin W. Essays on the Wealthiest Americans. PhD dissertation, American University, Washington, DC, 2014. 
Description: This file generates the summary statistics reported by the data appendix. 
"""

from __future__ import division
from load_forbes_400_dataset import *

## LOAD FLOW OF FUNDS DATA ##

# load wealth estimates from the June 5, 2014 release of the Flow of Funds
filename = "./data/data_specific_to_data_appendix/flow_of_funds_data.txt"
header = ['year and quarter', 'net worth of households and non-profits in millions', 'net worth of non-profits in millions']
raw_flow_of_funds_data = numpy.loadtxt(filename, delimiter='\t', skiprows=7, dtype={'names':header, 'formats':['<S50']*len(header)})

# get total household wealth at end of given quarter in given year
quarters = ['1', '2', '3', '4']
total_wealth_of_households_at_end_of_in = dict(zip(quarters, [dict(zip(years, ['TBD' for year in years])) for quarter in quarters]))
for i in range(0, len(raw_flow_of_funds_data['year and quarter'])):
	year, quarter = raw_flow_of_funds_data['year and quarter'][i].split('Q')
	total_wealth_of_households_at_end_of_in[quarter][year] = float(raw_flow_of_funds_data['net worth of households and non-profits in millions'][i]) - float(raw_flow_of_funds_data['net worth of non-profits in millions'][i])

# interpolate total household wealth on the date that Forbes valued wealth
total_wealth_of_households_in = dict()
# for each year
for year in years:
	# use a weighted average of end-of-second and end-of-third quarter wealth
	wealth_at_end_of_2nd_quarter = total_wealth_of_households_at_end_of_in['2'][year]
	wealth_at_end_of_3rd_quarter = total_wealth_of_households_at_end_of_in['3'][year]
	# use a weight of one half if the Forbes valuation date is unknown
	if year in years_w_imputed_valuation_dates:
		weight = 0.5
	# use a weight based the Forbes valuation date if it is known
	else:
		days_btw_end_of_2nd_quarter_and_valuation_date = (datetime.date(int(year), 6, 30) - valued_on_in[year]).days
		days_btw_end_of_2nd_and_3rd_quarter = (datetime.date(int(year), 6, 30) - datetime.date(int(year), 9, 30)).days
		weight = (days_btw_end_of_2nd_quarter_and_valuation_date / days_btw_end_of_2nd_and_3rd_quarter)
	total_wealth_of_households_in[year] = ((1. - weight) * wealth_at_end_of_2nd_quarter) + (weight * wealth_at_end_of_3rd_quarter)	
	# print total wealth of households by year
	if year == years[0]: print "\nyear \tweight \ttotal wealth of households in year"
	print year, "\t", weight, "\t", total_wealth_of_households_in[year]

## LOAD POPULATION ESTIMATES ##

# load the Census Bureau's mid-year estimates of the US resident population
filename = "./data/data_specific_to_data_appendix/population_data.txt"
raw_population_data = numpy.loadtxt(filename, delimiter='\t', skiprows=3, dtype={'names':['year', 'population'], 'formats':['<S50','f']})
total_population_in = dict(zip(raw_population_data['year'], raw_population_data['population']))
# print 400 people as a share of the population
for year in years:
	if year == years[0]: print "\nyear \t400 wealthiest as a percentage of the population in year"
	print year, "\t", 100. * (400./total_population_in[year])

## DEFINE FUNCTIONS FOR CONVERTING DOLLAR AMOUNTS ###

# load CPI-U-RS data
filename = "./data/cpi_u_rs_data.txt"
raw_cpi_data = numpy.loadtxt(filename, delimiter='\t', skiprows=4, dtype={'names':['year', 'index'], 'formats':['<S50','f']})
cpi_u_rs_w_1977_as_base_year = dict(zip(raw_cpi_data['year'], raw_cpi_data['index']))

# define a function for deflating current dollars into constant dollars
def deflate_by_CPI(current_dollar_amount, year, base_year='2013'):
	""" Takes a current dollar amount and returns a constant dollar amount in CPI-U-RS, base year dollars """
	# re-base the CPI-U-RS
	index_in_new_base_year = cpi_u_rs_w_1977_as_base_year[base_year]
	# get re-based CPI-U-RS in the given year
	index_in_year = 100. * (cpi_u_rs_w_1977_as_base_year[year] / index_in_new_base_year)
	# get constant dollar amount in CPI-U-RS, base year dollars
	constant_dollar_amount = current_dollar_amount / (index_in_year / 100.0)
	return constant_dollar_amount

# define a function for re-inflating Kennickell's constant dollar (CPI-U-RS, 2009) estimates
def not_in_2009_dollars(constant_dollar_amount, year):
	""" Takes a constant dollar amount in 2009 dollars and returns a current dollar amount """
	# re-base the CPI-U-RS to 2009 dollars
	index_in_new_base_year = cpi_u_rs_w_1977_as_base_year['2009']
	# get re-based CPI-U-RS in the given year
	index_in_year = 100. * (cpi_u_rs_w_1977_as_base_year[year] / index_in_new_base_year)
	# get current dollar amount
	current_dollar_amount = constant_dollar_amount * (index_in_year / 100.0)
	return current_dollar_amount

## ABSOLUTE WEALTH OF 400 WEALTHIEST AMERICANS ##

# get wealth of 400 wealthiest Americans in current and constant dollars
nominal_wealth_in = dict()
cpi_deflated_wealth_in = dict()
for year in years:
	nominal_wealth_in[year] = float(summary_stats['sum'][year])
	cpi_deflated_wealth_in[year] = deflate_by_CPI( current_dollar_amount=nominal_wealth_in[year], year=year, base_year='2013')
	# print total current- and constant-dollar wealth of 400 wealthiest by year
	if year == years[0]: print "\ntotal wealth of the 400 wealthiest by year \nyear \tnominal wealth in year \tcpi deflated wealth in year"
	print year, "\t", nominal_wealth_in[year], "\t", cpi_deflated_wealth_in[year]

## RELATIVE WEALTH BASED ON FOFA ##

# get wealth share based on Flow of Funds data
for year in years: 
	if year == years[0]: print "\nwealth share based on Flow of Funds data using weighted average of end-of-2nd-quarter and end-of-3rd-quarter data\n year \t wealth share in year"
	print year, "\t", 100. * (summary_stats['sum'][year] / total_wealth_of_households_in[year])

# using any quarter of the FOFA from a given year
print "\nwealth share based on Flow of Funds data using end of any quarter in given year"
print "1982Q \t2013Q \tinitial wealth share \tfinal wealth share \tdifference \tmultiple \ttripled?"
for one_quarter in quarters:
	for another_quarter in quarters:
		initial = (summary_stats['sum']['1982'] / total_wealth_of_households_at_end_of_in[one_quarter]['1982'])
		final = (summary_stats['sum']['2013'] / total_wealth_of_households_at_end_of_in[another_quarter]['2013'])
		diff = final - initial
		print one_quarter, "\t", another_quarter, "\t", 100. * initial, "\t", 100. * final, "\t", 100. * diff, "\t", final/initial, "\t", (final/initial > 3.0)

## RELATIVE WEALTH BASED ON SCF ##

# load SCF data
years_of_scf = ['1989', '1992', '1995', '1998', '2001', '2004', '2007', '2010']
scf_mean_wealth_in_thousands_of_2009_dollars_in = dict(zip(years_of_scf, [315.3, 279.5, 296.1, 373.2, 483.0, 509.6, 575.9, 493.1])) # Source: Table 2 of Kennickell's (2012) "The Other, Other Half: Changes in the Finances of the Least Wealthy 50%, 2007-2009."
number_of_households_in_millions = dict(zip(years_of_scf, [93.1, 95.9, 99.0, 102.6, 106.5, 112.1, 116.1, 117.6])) # Sources: Kennickell and Starr (1994, p. 880, n. 22); Kennickell et al. (2000, p. 27, n. 35); Bricker et al. (2012, p. 78, table A.3)
perct_of_wealth_held_by_least_wealthy_half_in = dict(zip(years_of_scf, [3.0, 3.3, 3.6, 3.0, 2.8, 2.5, 2.5, 1.1])) # Source: Table 5 of Kennickell's (2011) "Tossed and Turned: Wealth Dynamics of US Households, 2007-2009"
perct_of_wealth_held_by_wealthiest_one_perct_in = dict(zip(years_of_scf, [30.1, 30.1, 34.6, 33.9, 32.6, 33.4, 33.8, 34.5])) # Source: Ibid.

# define dicts for storing calculations
recalculated_perct_of_wealth_held_by_least_wealthy_half_in = dict()
recalculated_perct_of_wealth_held_by_400_wealthiest_in = dict()
recalculated_perct_of_wealth_held_by_wealthiest_one_perct_in = dict()
recalculated_perct_of_wealth_held_by_wealthiest_one_perct_to_400_wealthiest_in = dict()

# for each year
for year in years_of_scf:
	# get the wealth of each group in millions of current dollars
	mean_wealth = not_in_2009_dollars((scf_mean_wealth_in_thousands_of_2009_dollars_in[year] * 1000.), year) # in current dollars per households
	number_of_households = number_of_households_in_millions[year] * 1000000. # in households
	total_wealth = number_of_households * mean_wealth # in current dollars
	wealth_of_least_wealthy_half = ((perct_of_wealth_held_by_least_wealthy_half_in[year] / 100.) * total_wealth) / 1000000. # in millions of current dollars
	wealth_of_400_wealthiest = summary_stats['sum'][year] # in millions of current dollars
	wealth_of_wealthiest_one_perct = ((perct_of_wealth_held_by_wealthiest_one_perct_in[year] / 100.) * total_wealth) / 1000000. # in millions of current dollars
	# compare wealths of the different groups
	print "\nIn " + year + ",", 
	print "the mean wealth of American households was " + str(mean_wealth) + " current dollars.", 
	print "Their total wealth would be " + str(total_wealth) + " current dollars, given that the total number of American households was " + str(number_of_households) + " households.", 
	print "If the least-wealthy half of American households was worth " + str(perct_of_wealth_held_by_least_wealthy_half_in[year]) + " percent of that, then they would be worth " + str(wealth_of_least_wealthy_half) + " million current dollars.", 
	print "The 400 wealthiest Americans were worth " + str(wealth_of_400_wealthiest) + " million current dollars.", 
	print "The difference between the latter and the former is " + str(wealth_of_400_wealthiest - wealth_of_least_wealthy_half) + " million current dollars or " + str(round(100. * (wealth_of_400_wealthiest - wealth_of_least_wealthy_half) / wealth_of_400_wealthiest, 2)) + " percent.",
	print "The wealth (wealth share) of the wealthiest one percent of American households was " + str(wealth_of_wealthiest_one_perct) + " million current dollars (" + str(perct_of_wealth_held_by_wealthiest_one_perct_in[year]) + " percent).",
	print "The difference between the wealth of the 400 wealthiest Americans and the wealth of the wealthiest one percent of households is " + str(wealth_of_wealthiest_one_perct - wealth_of_400_wealthiest) + " million current dollars.",
	print "The wealth of the 400 wealthiest Americans is " + str(round(100. * wealth_of_400_wealthiest / wealth_of_wealthiest_one_perct, 2)) + " percent of the wealth of the wealthiest one percent of American households.", 
	print "Or, including the wealth of the 400 wealthiest Americans,",
	print "the wealth (wealth share) of the wealthiest one percent of American households is",
	print str(wealth_of_400_wealthiest + wealth_of_wealthiest_one_perct),
	print "(" + str(round(100. * (wealth_of_400_wealthiest + wealth_of_wealthiest_one_perct) / (wealth_of_400_wealthiest + (total_wealth / 1000000.)), 4)) + ")", 
	print "and the wealth of the 400 wealthiest Americans is " + str(round(100. * wealth_of_400_wealthiest / (wealth_of_400_wealthiest + wealth_of_wealthiest_one_perct), 2)) + " percent of the wealth of the wealthiest one percent of American households."
	# calculate percentages to draw in figure
	recalculated_perct_of_wealth_held_by_least_wealthy_half_in[year] = 100. * (wealth_of_least_wealthy_half) / (wealth_of_400_wealthiest + (total_wealth / 1000000.))
	recalculated_perct_of_wealth_held_by_400_wealthiest_in[year] = 100. * (wealth_of_400_wealthiest) / (wealth_of_400_wealthiest + (total_wealth / 1000000.))
	recalculated_perct_of_wealth_held_by_wealthiest_one_perct_in[year] = 100. * (wealth_of_400_wealthiest + wealth_of_wealthiest_one_perct) / (wealth_of_400_wealthiest + (total_wealth / 1000000.))
	recalculated_perct_of_wealth_held_by_wealthiest_one_perct_to_400_wealthiest_in[year] = 100. * (wealth_of_wealthiest_one_perct - wealth_of_400_wealthiest) / (wealth_of_400_wealthiest + (total_wealth / 1000000.))
	# de-comment to print table of percentages
	#if year == years_of_scf[0]: print "\nyear \t(total wealth / 1000000.) \trecalculated perct of wealth held by least wealthy half in year \trecalculated perct of wealth held by 400 wealthiest in year \trecalculated perct of wealth held by wealthiest one perct in year"
	#print year, "\t", (total_wealth / 1000000.), "\t", recalculated_perct_of_wealth_held_by_least_wealthy_half_in[year], "\t", recalculated_perct_of_wealth_held_by_400_wealthiest_in[year], "\t", recalculated_perct_of_wealth_held_by_wealthiest_one_perct_in[year]

## DEFINE SOME FUNCTIONS FOR ARCSINH TRANSFORMATIONS ###

def arcsinh(x, scale=0.001):
	""" Returns the inverse hyperbolic sine of x """
	return numpy.log((scale * x) + (((scale * x) ** 2.) + 1.) ** 0.5) / scale

def sinh(x, scale=0.001):
	""" Returns the hyperbolic sine of x """
	return (1. / (2. * scale)) * numpy.exp(- scale * x) * (numpy.exp(2. * scale * x) - 1.)

## LOAD DATA ON IMMIGRANTS

#load data Forbes 400 members who were immigrants (note that we only have that data for the 2012 and 2013 Forbes 400s)
filename = "./data/data_specific_to_data_appendix/immigrants_in_data.txt"
rawdata = numpy.loadtxt(filename, delimiter='\t', skiprows=7, dtype={'names':['name', 'year'], 'formats':['<S50', '<S50']})
# get dict of names of those who renounced their citizenship by year
year_immigrant_in_for = dict(zip(rawdata['name'], rawdata['year']))
immigrants_in = dict(zip(years, [[] for year in years]))
for name in year_immigrant_in_for.keys():
	year = year_immigrant_in_for[name]
	if year in years:
		immigrants_in[year].append(name)

## PRO-WEALTHINESS OF ACCUMULATION ##

# get wealth by rank
ranks = range(1, 400+1)
base_year_for_price_index = '2013'
wealth_by_rank = dict(zip(ranks, [dict(zip(years, ['' for year in years])) for rank in ranks]))
for year in years:
	for name in names:
		wealth = data[name]['wealth'][year]
		if (wealth != ''):
			wealth = float(wealth)
			rank = summary_stats['wealths'][year].index(wealth) + 1 # add one
			wealth_by_rank[rank][year] = deflate_by_CPI( current_dollar_amount=wealth, year=year, base_year=base_year_for_price_index) # cpi-deflated

# create dicts for storing changes in wealth between years by initial rank
dyears = ['1982-2013']
anonymous_accumulation_by_rank_btw = dict(zip(ranks, [dict(zip(dyears, ['' for dyear in dyears])) for rank in ranks]))
non_anonymous_accumulation_by_rank_btw = dict(zip(ranks, [dict(zip(dyears, [list() for dyear in dyears])) for rank in ranks]))
new_entrant_accumulation = dict(zip(dyears, [list() for dyear in dyears]))
on_list_in = dict(zip(dyears, [dict(zip(['initial year', 'final year', 'initial year but dropped off due to death', 'initial year but renounced citizenship', 'initial year but dropped off due to decline'], [list() for _ in ['initial year', 'final year', 'initial year but dropped off due to death', 'initial year but renounced citizenship', 'initial year but dropped off due to decline']])) for dyear in dyears]))

# for each set of years
for dyear in dyears:
	initial_year, final_year = dyear.split('-')
	# anonymous changes in wealth by rank
	for rank in ranks:
		# initial wealth associated with given rank
		initial_wealth = wealth_by_rank[rank][initial_year]
		i = ranks.index(rank)
		while initial_wealth == '':
			i -= 1
			initial_wealth = wealth_by_rank[ranks[i]][initial_year] # rank or next-highest rank
		# final wealth associated with given rank
		final_wealth = wealth_by_rank[rank][final_year]
		i = ranks.index(rank)
		while final_wealth == '':
			i -= 1
			final_wealth = wealth_by_rank[ranks[i]][final_year] # rank or next-highest rank
		# change in wealth of given rank
		anonymous_accumulation_by_rank_btw[rank][dyear] = (final_wealth - initial_wealth)
		#if rank == 1: print "rank\tchange in wealth in billions of constant dollars"
		#print rank, "\t", round(anonymous_accumulation_by_rank_btw[rank][dyear] / 1000., 2)
	# non-anonymous changes in wealth
	for name in names:
		# on list in the initial year?
		if (data[name]['wealth'][initial_year] != ''):
			on_list_in[dyear]['initial year'].append(name)
			initial_wealth = deflate_by_CPI( current_dollar_amount=float(data[name]['wealth'][initial_year]), year=initial_year, base_year=base_year_for_price_index)
			rank = summary_stats['wealths'][initial_year].index(float(data[name]['wealth'][initial_year])) + 1 # add one
			# also on list in final year?
			if (data[name]['wealth'][final_year] != ''):
				on_list_in[dyear]['final year'].append(name)
				final_wealth = deflate_by_CPI( current_dollar_amount=float(data[name]['wealth'][final_year]), year=final_year, base_year=base_year_for_price_index)
				non_anonymous_accumulation_by_rank_btw[rank][dyear].append(((final_wealth - initial_wealth), 'uncensored', name))
			else:
				# get year of most-recent drop off
				dropped_off_in = 'TBD'
				for y in range(int(initial_year)+1, int(final_year)+1):
					if (data[name]['wealth'][str(y)] == '') and (data[name]['wealth'][str(y-1)] != ''):
						dropped_off_in = str(y)
				# get min wealth after drop off
				censored_final_wealth = deflate_by_CPI( current_dollar_amount=summary_stats['min'][dropped_off_in], year=dropped_off_in, base_year=base_year_for_price_index)
				for y in range(int(dropped_off_in)+1, int(final_year)+1):
					if deflate_by_CPI( current_dollar_amount=summary_stats['min'][str(y)], year=str(y), base_year=base_year_for_price_index) < censored_final_wealth:
						censored_final_wealth = deflate_by_CPI( current_dollar_amount=summary_stats['min'][str(y)], year=str(y), base_year=base_year_for_price_index)
				# dropped off by death?
				if True in [name in dead_in[str(y)] for y in range(int(initial_year)+1, min(int(final_year)+1, '2012'))]:
					on_list_in[dyear]['initial year but dropped off due to death'].append(name)
					non_anonymous_accumulation_by_rank_btw[rank][dyear].append(((censored_final_wealth - initial_wealth), 'died', name))
				# renounced citizenship?
				elif True in [name in renunciants_in[str(y)] for y in range(int(initial_year)+1, min(int(final_year)+1, '2013'))]:
					on_list_in[dyear]['initial year but renounced citizenship'].append(name)
					non_anonymous_accumulation_by_rank_btw[rank][dyear].append(((censored_final_wealth - initial_wealth), 'renounced citizenship', name))
				# censored change in wealth
				else:
					on_list_in[dyear]['initial year but dropped off due to decline'].append(name)
					non_anonymous_accumulation_by_rank_btw[rank][dyear].append(((censored_final_wealth - initial_wealth), 'censored', name))
		# not on list in initial year ...
		else:
			# but on list in final year
			if (data[name]['wealth'][final_year] != ''):
				on_list_in[dyear]['final year'].append(name)
				# wealth was at most the min wealth in any year before up to and including the year of his or her first appearance
				censored_initial_wealth = deflate_by_CPI( current_dollar_amount=summary_stats['min'][final_year], year=final_year, base_year=base_year_for_price_index)
				for y in range(int(initial_year)+1, int(final_year)+1):
					if (data[name]['wealth'][str(y)] != '') and (data[name]['wealth'][str(y-1)] == ''):
						if min(deflate_by_CPI( current_dollar_amount=summary_stats['min'][str(y)], year=str(y), base_year=base_year_for_price_index), deflate_by_CPI( current_dollar_amount=summary_stats['min'][str(y-1)], year=str(y-1), base_year=base_year_for_price_index)) < censored_initial_wealth:
							censored_initial_wealth = min(deflate_by_CPI( current_dollar_amount=summary_stats['min'][str(y)], year=str(y), base_year=base_year_for_price_index), deflate_by_CPI( current_dollar_amount=summary_stats['min'][str(y-1)], year=str(y-1), base_year=base_year_for_price_index))
				final_wealth = deflate_by_CPI( current_dollar_amount=float(data[name]['wealth'][final_year]), year=final_year, base_year=base_year_for_price_index)
				new_entrant_accumulation[dyear].append(((final_wealth - censored_initial_wealth), 'censored', name))

## SOME CALCULATIONS ##

for dyear in dyears:
	print "\ndyear=\t", dyear
	people_on_list_in_initial_year = set(on_list_in[dyear]['initial year'])
	people_on_list_in_final_year = set(on_list_in[dyear]['final year'])
	assert (len(list(people_on_list_in_initial_year)) == int(400)) and (len(list(people_on_list_in_final_year)) == int(400)), "There should be 400 people on the list in both years"
	people_on_list_in_initial_year_and_final_year = people_on_list_in_initial_year.intersection(people_on_list_in_final_year)
	print "number of people on list in both initial year and final year:\t", len(list(people_on_list_in_initial_year_and_final_year))
	assert len(list(people_on_list_in_initial_year)) == len(list(people_on_list_in_initial_year_and_final_year)) + sum([len(list(set(on_list_in[dyear][_]))) for _ in ['initial year but dropped off due to death', 'initial year but renounced citizenship', 'initial year but dropped off due to decline']]), "everyone either stayed or dropped out somehow"
	assert not set(on_list_in[dyear]['initial year but dropped off due to death']).intersection(set(on_list_in[dyear]['initial year but renounced citizenship'])), "A person should drop off for only one reason"
	assert not set(on_list_in[dyear]['initial year but dropped off due to death']).intersection(set(on_list_in[dyear]['initial year but dropped off due to decline'])), "A person should drop off for only one reason"
	assert not set(on_list_in[dyear]['initial year but renounced citizenship']).intersection(set(on_list_in[dyear]['initial year but dropped off due to decline'])), "A person should drop off for only one reason"
	print "number of people on list in initial year who dropped off due to death:\t", len(on_list_in[dyear]['initial year but dropped off due to death'])
	print "number of people on list in initial year who dropped off due to decline:\t", len(on_list_in[dyear]['initial year but dropped off due to decline'])
	print "number of people on list in initial year who renounced citizenship:\t", len(on_list_in[dyear]['initial year but renounced citizenship'])
	print "number of people who came onto list by final year:\t", len(list(people_on_list_in_final_year.difference(people_on_list_in_initial_year)))
	immigrants = 0
	for name in list(people_on_list_in_final_year.difference(people_on_list_in_initial_year)):
		if name in immigrants_in[final_year]:
			immigrants += 1
	print "number of people who came onto list by final year who were immigrants:\t", immigrants
	for name in people_on_list_in_final_year.intersection(set(immigrants_in[final_year])):
		if data[name]['wealth'][initial_year] != '': 
			print name + " was an immigrant on the list in both the initial and final year"	
	self_made_in_final_year = 0
	self_made_in_final_year_and_on_list_in_initial_year = 0
	for name in people_on_list_in_final_year:
		if data[name]['selfmade or inherited'][final_year] == 'Self made':
			self_made_in_final_year += 1
			if data[name]['wealth'][initial_year] != '':
				self_made_in_final_year_and_on_list_in_initial_year += 1
	print "number of self-made in final year:\t", self_made_in_final_year
	print "self-made in final year and on list in initial year:\t", self_made_in_final_year_and_on_list_in_initial_year
	
# print info for incumbents
for dyear in dyears:
	people_on_list_in_initial_year_and_final_year = list(set(on_list_in[dyear]['initial year']).intersection(set(on_list_in[dyear]['final year'])))
	people_on_list_in_initial_year_and_final_year.sort() # alphabetize names
	for name in people_on_list_in_initial_year_and_final_year:
		if name == people_on_list_in_initial_year_and_final_year[0]:
			print "\n",	
		print name[name.index(',')+2:] + ' ' + name[:name.index(',')] + " was ranked", 
		rank_in_initial_year = summary_stats['wealths'][initial_year].index(float(data[name]['wealth'][initial_year])) + 1
		print str(rank_in_initial_year) + " in " + initial_year + " with a wealth of",
		print data[name]['wealth'][initial_year] + "M current dollars or about ", 
		print str(round(deflate_by_CPI( current_dollar_amount=float(data[name]['wealth'][initial_year]), year=initial_year, base_year=final_year), 0)) + "M constant (i.e., CPI-deflated, final_year) dollars.", 
		print "A person with the same rank in " + final_year + " would be worth",
		print str(summary_stats['wealths'][final_year][int(rank_in_initial_year-1)]) + "M constant dollars.", 
		print "By" + final_year + ", he/she was ranked", 
		print str(summary_stats['wealths'][final_year].index(float(data[name]['wealth'][final_year])) + 1), 
		print "with a wealth of ",
		print data[name]['wealth'][final_year] + "M constant dollars."

# print info for new entrants
for dyear in dyears:
	entrants = list(people_on_list_in_final_year.difference(people_on_list_in_initial_year))
	entrants.sort()
	print "\nThe minimum wealth of the Forbes 400 in " + initial_year + " was about " + str(round(deflate_by_CPI( current_dollar_amount=float(summary_stats['min'][initial_year]), year=initial_year, base_year=base_year_for_price_index), 0)) + " M constant dollars. The following people were not on the list back in that year, so they must have been worth less than that amount, assuming they could have been on the list if only they were wealthy enough."
	for name in entrants:
		print "By " + final_year + ", " + name[name.index(',')+2:] + ' ' + name[:name.index(',')] + " was ranked", 
		current_dollar_wealth_in_final_year = data[name]['wealth'][final_year]
		assert (data[name]['wealth'][initial_year] == '') and (current_dollar_wealth_in_final_year != ''), "Name is not a new entrant"
		rank_in_final_year = summary_stats['wealths'][final_year].index(float(current_dollar_wealth_in_final_year)) + 1
		print str(rank_in_final_year) + " with a wealth of about " + str(round(deflate_by_CPI( current_dollar_amount=float(current_dollar_wealth_in_final_year), year=final_year, base_year=base_year_for_price_index), 0)) + " M constant dollars"

# print info for dropouts
for dyear in dyears:
	dropoffs = list(on_list_in[dyear]['initial year but dropped off due to death'])
	dropoffs.extend(list(on_list_in[dyear]['initial year but dropped off due to decline']))
	dropoffs.extend(list(on_list_in[dyear]['initial year but renounced citizenship']))
	dropoffs.sort()
	print "\nThe following were on the list in " + initial_year + " but they dropped off by " + final_year + ". In " + initial_year + ", they had the following ranks and wealths."
	for name in dropoffs:
		print name[name.index(',')+2:] + ' ' + name[:name.index(',')] + " was ranked", 
		current_dollar_wealth_in_initial_year = data[name]['wealth'][initial_year]
		assert (current_dollar_wealth_in_initial_year != '') and (data[name]['wealth'][final_year] == ''), "Name is not a dropout"
		rank_in_initial_year = summary_stats['wealths'][initial_year].index(float(current_dollar_wealth_in_initial_year)) + 1
		print str(rank_in_initial_year) + " with a wealth of about " + str(round(deflate_by_CPI( current_dollar_amount=float(current_dollar_wealth_in_initial_year), year=initial_year, base_year=base_year_for_price_index), 0)) + " M constant dollars"

## DECOMPOSE CHANGE IN WEALTH ##

# create dicts for decomposing change in wealth
components = ['incumbents', 'entrants', 'decedents', 'renunciants', 'other dropouts', 'all dropouts', 'self-made entrants', 'other entrants']
change_in_wealth_btw = dict(zip(dyears, ['TBD' for dyear in dyears]))
change_in_wealth_due_to = dict(zip(components, [dict(zip(dyears, [list() for dyear in dyears])) for component in components]))

for dyear in dyears:
	initial_year, final_year = dyear.split('-')
	initial_and_final_year = [initial_year, final_year]
	# wealths in years
	wealths = dict(zip([initial_year, final_year], [list(), list()]))
	for year in [initial_year, final_year]:
		for name in names:
			wealth = data[name]['wealth'][year]
			if (wealth != ''):
				wealth = deflate_by_CPI( current_dollar_amount=float(wealth), year=year, base_year=base_year_for_price_index) / 1000. # in billions, deflated
				wealths[year].append(wealth)
	# change in total wealth between years
	change_in_wealth_btw[dyear] = sum(wealths[final_year]) - sum(wealths[initial_year])
	#print "\nchange in wealth btw " + dyear + ":\t" + str(change_in_wealth_btw[dyear]) + " in millions of constant dollars"
	# min wealths in each year
	min_wealth_in_final_year = min(wealths[final_year]) # already in billions and deflated
	min_wealth_in_initial_year = min(wealths[initial_year]) # already in billions and deflated
	# reset components, given that we might look at some dyears twice
	for component in components:
		change_in_wealth_due_to[component][dyear] = list()
	# decompose change in index
	for name in names:
		# last year's wealth
		wealth_in_initial_year = data[name]['wealth'][initial_year]
		if (wealth_in_initial_year != ''):
			wealth_in_initial_year = deflate_by_CPI( current_dollar_amount=float(wealth_in_initial_year), year=initial_year, base_year=base_year_for_price_index)  / 1000. # in billions
		# this year's wealth
		wealth_in_final_year = data[name]['wealth'][final_year]
		if (wealth_in_final_year != ''):
			wealth_in_final_year = deflate_by_CPI( current_dollar_amount=float(wealth_in_final_year), year=final_year, base_year=base_year_for_price_index) / 1000. # in billions
		# incumbent
		if (wealth_in_initial_year != '') and (wealth_in_final_year != ''):
			contribution = wealth_in_final_year - wealth_in_initial_year
			change_in_wealth_due_to['incumbents'][dyear].append([contribution, name])
		# entrant
		elif (wealth_in_initial_year == '') and (wealth_in_final_year != ''):
			contribution = wealth_in_final_year - min_wealth_in_initial_year
			change_in_wealth_due_to['entrants'][dyear].append([contribution, name])
			if data[name]['selfmade or inherited'][final_year] == 'Self made':
				change_in_wealth_due_to['self-made entrants'][dyear].append([contribution, name])
			else:
				assert data[name]['selfmade or inherited'][final_year] == 'Not self made'
				change_in_wealth_due_to['other entrants'][dyear].append([contribution, name])
		# dropouts
		elif (wealth_in_initial_year != '') and (wealth_in_final_year == ''):
			contribution = [- (wealth_in_initial_year - min_wealth_in_initial_year), name]
			change_in_wealth_due_to['all dropouts'][dyear].append(contribution)
			if True in [name in dead_in[str(_ + 1)] for _ in range(1982, int(final_year))]:
				change_in_wealth_due_to['decedents'][dyear].append(contribution)
			elif True in [name in renunciants_in[str(_ + 1)] for _ in range(1982, int(final_year))]:
				change_in_wealth_due_to['renunciants'][dyear].append(contribution)
			else:
				change_in_wealth_due_to['other dropouts'][dyear].append(contribution)
		# not wealthy
		else:
			assert ((wealth_in_initial_year == '') and (wealth_in_final_year == ''))

# print contribution by component
for component in components:
	if component == components[0]: print "\ncomponent \tchange in wealth due to component"
	print component, "\t", sum([_[0] for _ in change_in_wealth_due_to[component]['1982-2013']])

# print contribution by name
#for component in components:
#	if component == components[0]: print "\ncomponent \tname \tchange in wealth due to name"
#	for _ in change_in_wealth_due_to[component]['1982-2013']:
#		print component, "\t", _[1], "\t", _[0]

## SOME MORE CALCULATIONS ##

min_wealth_in_1982 = deflate_by_CPI( current_dollar_amount=float(summary_stats['min']['1982']), year='1982', base_year=base_year_for_price_index)

# select incumbents
name = 'Buffett, Warren Edward'
wealth_in_1982 = deflate_by_CPI( current_dollar_amount=float(data[name]['wealth']['1982']), year='1982', base_year=base_year_for_price_index)
wealth_in_2013 = deflate_by_CPI( current_dollar_amount=float(data[name]['wealth']['2013']), year='2013', base_year=base_year_for_price_index)
contribution = wealth_in_2013 - wealth_in_1982
contributions = sum([_[0] for _ in change_in_wealth_due_to['incumbents']['1982-2013']]) * 1000. # in millions
print "\nchange in incumbent wealth due to " + name + ":\t", 100. * contribution / contributions, "\tpercent or\t", contribution / 1000., "\tbillion"

# select self-made entrants
name = 'Gates, William Henry III'
wealth_in_2013 = deflate_by_CPI( current_dollar_amount=float(data[name]['wealth']['2013']), year='2013', base_year=base_year_for_price_index)
contribution = wealth_in_2013 - min_wealth_in_1982
contributions = sum([_[0] for _ in change_in_wealth_due_to['entrants']['1982-2013']]) * 1000. # in millions
print "\nchange in self-made entrant wealth due to " + name + ":\t", 100. * contribution / contributions, "\tpercent or\t", contribution / 1000., "\tbillion"

# select decedents
name = 'Ludwig, Daniel Keith'
wealth_in_1982 = deflate_by_CPI( current_dollar_amount=float(data[name]['wealth']['1982']), year='1982', base_year=base_year_for_price_index)
contribution = - (wealth_in_1982 - min_wealth_in_1982)
contributions = sum([_[0] for _ in change_in_wealth_due_to['decedents']['1982-2013']]) * 1000. # in millions
print "\nchange in decedent wealth due to " + name + ":\t", 100. * contribution / contributions, "\tpercent or\t", contribution / 1000., "\tbillion"

# other dropouts
name = 'Hunt, Nelson Bunker'
wealth_in_1982 = deflate_by_CPI( current_dollar_amount=float(data[name]['wealth']['1982']), year='1982', base_year=base_year_for_price_index)
contribution = - (wealth_in_1982 - min_wealth_in_1982)
contributions = sum([_[0] for _ in change_in_wealth_due_to['decedents']['1982-2013']]) * 1000. # in millions
print "\nchange in other dropouts wealth due to " + name + ":\t", 100. * contribution / contributions, "\tpercent or\t", contribution / 1000., "\tbillion"

# group of self-made entrants
contribution = 0. 
for name in ['Gates, William Henry III', 'Ellison, Lawrence Joseph', 'Bezos, Jeffrey P', 'Page, Larry E', 'Brin, Sergey', 'Bloomberg, Michael Rubens', 'Adelson, Sheldon']:
	wealth_in_2013 = deflate_by_CPI( current_dollar_amount=float(data[name]['wealth']['2013']), year='2013', base_year=base_year_for_price_index)
	contribution += wealth_in_2013 - min_wealth_in_1982
contributions = sum([_[0] for _ in change_in_wealth_due_to['self-made entrants']['1982-2013']]) * 1000. # in millions
print "\nchange in self-made entrant wealth due to select self-made entrants" + ":\t",  100. * contribution / contributions, "\tpercent or\t", contribution / 1000., "\tbillion"

# group of Walmart heirs 
contribution = 0. 
for name in ['Walton, Christy', 'Walton, Jim C', 'Walton, Alice L', 'Walton, S Robson']:
	wealth_in_2013 = deflate_by_CPI( current_dollar_amount=float(data[name]['wealth']['2013']), year='2013', base_year=base_year_for_price_index)
	contribution += wealth_in_2013 - min_wealth_in_1982
contributions = sum([_[0] for _ in change_in_wealth_due_to['other entrants']['1982-2013']]) * 1000. # in millions
print "\nchange in not self-made entrant wealth due to Walmart heirs" + ":\t",  100. * contribution / contributions, "\tpercent or\t", contribution / 1000., "\tbillion"

## lastline=478
if __name__ == '__main__':
	
	#### FIGURES ####
	
	import pylab
	from matplotlib import rc
	from matplotlib import lines
	from matplotlib.ticker import MultipleLocator

	# size of figures
	figwidth = 430.00627 / 72.27 # \textwidth in pts times pts per inch, following cookbook example 
	figheight = figwidth * (numpy.sqrt(5)-1.0) / 2.0 # golden ratio, following cookbook example
	figsize = (figwidth, figheight)

	rc('text', usetex=True)
	#rc('font',**{'family':'sans-serif','sans-serif':['Computer Modern Sans Serif']}) # for beamer
	#rc('font', family='sans-serif')
	rc('font',**{'family':'serif','serif':['Computer Modern Roman']}) # for document
	rc('font', family='serif')
	fontsize = 11
	pylab.rcParams['font.size'] = fontsize
	pylab.rcParams['axes.labelsize'] = fontsize
	pylab.rcParams['legend.fontsize'] = fontsize
	pylab.rcParams['xtick.labelsize'] = fontsize
	pylab.rcParams['ytick.labelsize'] = fontsize

	## FIG D.1: ABSOLUTE WEALTH ##

	# draw twice, once with recessions, once without
	for show_recessions in [True, False]:
		try:
			fignumber += 1
		except NameError:
			fignumber = 1
		fig = pylab.figure(fignumber, figsize=figsize)
		fig_ax = fig.gca()
		# plot wealth in billions of current and constant dollars
		if not show_recessions:
			fig_ax.plot([int(year) for year in years], list(cpi_deflated_wealth_in[year] / 1000. for year in years), color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='Constant dollars') # no markers if more than 10 data points
			fig_ax.plot([int(year) for year in years], list(nominal_wealth_in[year] / 1000. for year in years), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label='Current dollars') # no markers if more than 10 data points
		else:
			fig_ax.plot_date([valued_on_in[year] for year in years], list(cpi_deflated_wealth_in[year] / 1000. for year in years), color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='Constant dollars') # no markers if more than 10 data points
			fig_ax.plot_date([valued_on_in[year] for year in years], list(nominal_wealth_in[year] / 1000. for year in years), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label='Current dollars') # no markers if more than 10 data points
		# make a legend
		leg = fig_ax.legend(loc='upper left', frameon=False, handlelength=3.3, handletextpad=0.5)
		# label the x axis
		if not show_recessions:
			fig_ax.set_xlabel('Year')
		else:
			fig_ax.set_xlabel('Date')
		# label the y axis
		fig_ax.set_ylabel('Wealth ($\mathdollar$Ts)')
		# log the y axis
		#fig_ax.set_yscale('log')
		# set x limits
		if not show_recessions:
			pylab.xlim(1982, 2013)
		else:
			pylab.xlim(datetime.date(1982, 1, 1), datetime.date(2014, 1, 1))
		# set y limits
		pylab.ylim(0, 2100)
		# major xticks
		if not show_recessions:
			pylab.xticks([1982, 1992, 2002, 2012], ['1982', '1992', '2002', '2012'])
			fig_ax.xaxis.set_minor_locator(MultipleLocator(1)) # minor tick every year
		else:
			pylab.xticks([datetime.date(1982, 1, 1), datetime.date(1990, 1, 1), datetime.date(1998, 1, 1), datetime.date(2006, 1, 1), datetime.date(2014, 1, 1)], ['1/1/1982', '1/1/1990', '1/1/1998', '1/1/2006', '1/1/2014'])
		# NBER recessions
		if show_recessions:
			y1, y2 = fig_ax.get_ylim()
			fig_ax.fill_between((datetime.date(1981, 7, 1), datetime.date(1982, 11, 1)), y1=y1, y2=y2, alpha=.10, color='k')
			fig_ax.fill_between((datetime.date(1990, 7, 1), datetime.date(1991, 3, 1)), y1=y1, y2=y2, alpha=.10, color='k')
			fig_ax.fill_between((datetime.date(2001, 3, 1), datetime.date(2001, 11, 1)), y1=y1, y2=y2, alpha=.10, color='k')
			fig_ax.fill_between((datetime.date(2007, 12, 1), datetime.date(2009, 6, 1)), y1=y1, y2=y2, alpha=.10, color='k')
			from matplotlib.dates import YearLocator, MonthLocator, DateFormatter
			year_locator = YearLocator()
			fig_ax.xaxis.set_minor_locator(year_locator)
		# add y ticks
		pylab.yticks([0, 1000, 2000], ['0', '1', '2'])
		# minor yticks
		#fig_ax.yaxis.set_minor_locator(MultipleLocator(100)) # tick every 100
		# clean ticks
		for tickline in fig_ax.get_xticklines():
			tickline.set_markersize(8)
			tickline.set_markeredgewidth(0.5)
			tickline.set_marker(lines.TICKDOWN)
		for tickline in fig_ax.xaxis.get_minorticklines():
			tickline.set_markersize(4)
			tickline.set_markeredgewidth(0.5)
			tickline.set_marker(lines.TICKDOWN)
		for tickline in fig_ax.get_yticklines():
			tickline.set_markersize(8)
			tickline.set_markeredgewidth(0.5)
			tickline.set_marker(lines.TICKLEFT)
		for tickline in fig_ax.yaxis.get_minorticklines():
			tickline.set_markersize(4)
			tickline.set_markeredgewidth(0.5)
			tickline.set_marker(lines.TICKLEFT)
		# shift labels
		for ticklabel in fig_ax.get_xticklabels():
			ticklabel.set_y(0-0.04)
		for ticklabel in fig_ax.get_yticklabels():
			ticklabel.set_x(0-0.02)
		# set frame width
		for i in fig_ax.spines.keys():
			fig_ax.spines[i].set_linewidth(0.5)
		# get rid of ticks on top and right
		fig_ax.get_yaxis().tick_left()
		fig_ax.get_xaxis().tick_bottom()
		# adjust dimensions
		fig.subplots_adjust(left=0.12, bottom=0.14, right=0.95, top=0.96, wspace=0.0, hspace=0.0)
		# save figure
		if not show_recessions:
			pylab.savefig("./figures/figures_for_data_appendix/absolute_wealth.pdf", bbox_inches='tight')
		else:
			pylab.savefig("./figures/figures_for_data_appendix/absolute_wealth_w_recessions.pdf", bbox_inches='tight')

	## FIG D.2: CHANGES IN ABSOLUTE WEALTH ##

	sets_of_select_years = [['2007', '2008', '2009', '2010'], ['1987', '1988', '1989', '1990'], ['1989', '1990', '1991', '1992'], ['2000', '2001', '2002', '2003']]
	linestyle_for_sets_of_select_years = ['solid', 'dashed', 'dotted', 'dashdot']
	clip_on_for_sets_of_select_year = [False, True, False, False]
	labels_for_sets_of_select_years = ['2007--2010', '1987--1990', '1989--1992', '2000--2003']

	# draw twice, once with recessions, once without
	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=figsize)
	fig_ax = fig.gca()
	# plot changes in CPI-deflated wealth
	for i in range(0, len(sets_of_select_years)):
		select_years = sets_of_select_years[i]
		base_year = select_years[0]
		linestyle = linestyle_for_sets_of_select_years[i]
		clip_on = clip_on_for_sets_of_select_year[i]
		label = labels_for_sets_of_select_years[i]
		fig_ax.plot(range(0, 3+1), list(100. * ((cpi_deflated_wealth_in[year] / cpi_deflated_wealth_in[base_year]) - 1.) for year in select_years), color='k', linestyle=linestyle, lw=0.5, marker='o', markersize=2, clip_on=clip_on, label=label) # no markers if more than 10 data points
		#print list(100. * ((cpi_deflated_wealth_in[year] / cpi_deflated_wealth_in[base_year]) - 1.) for year in select_years)
	# make a legend
	leg = fig_ax.legend(loc='lower left', frameon=False, handlelength=3.3, handletextpad=0.5)
	# label the x axis
	fig_ax.set_xlabel('Years since peak year') # sentence-style capitalization
	# label the y axis
	fig_ax.set_ylabel('Change in wealth (\%)')
	# set x limits
	pylab.xlim(0, 3)
	# set y limits
	pylab.ylim(-30.65176132036026, 0.65176132036026)
	# major xticks
	pylab.xticks([0, 1, 2, 3], ['0', '1', '2', '3'])
	# add y ticks
	pylab.yticks([-30, -15, 0], ['$-$30', '$-$15', '0'])
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# adjust dimensions
	fig.subplots_adjust(left=0.12, bottom=0.14, right=0.95, top=0.96, wspace=0.0, hspace=0.0)
	# save figure
	pylab.savefig("./figures/figures_for_data_appendix/declines_in_absolute_wealth.pdf", bbox_inches='tight')

	## FIGS D.3 TO D.5: RELATIVE WEALTH ##

	for show_recessions in [True, False]:
		try:
			fignumber += 1
		except NameError:
			fignumber = 1
		fig = pylab.figure(fignumber, figsize=figsize)
		fig_ax = fig.gca()
		if not show_recessions:
			# plot wealth as a share of total wealth of households according to FOFA
			fig_ax.plot([int(year) for year in years], list(100. * (summary_stats['sum'][year] / total_wealth_of_households_in[year]) for year in years), color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='FOFA') # no markers if more than 10 data points
			# plot wealth as a share of total wealth of households according to SCF
			fig_ax.plot([int(year) for year in years_of_scf], list(recalculated_perct_of_wealth_held_by_400_wealthiest_in[year] for year in years_of_scf), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=2, clip_on=True, label='SCF') # no markers if more than 10 data points
			# minor tick for every year
			fig_ax.xaxis.set_minor_locator(MultipleLocator(1))
		else:
			# plot shares with dates
			fig_ax.plot([valued_on_in[year] for year in years], list(100. * (summary_stats['sum'][year] / total_wealth_of_households_in[year]) for year in years), color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='FOFA') # no markers if more than 10 data points ##
			fig_ax.plot([valued_on_in[year] for year in years_of_scf], list(recalculated_perct_of_wealth_held_by_400_wealthiest_in[year] for year in years_of_scf), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=2, clip_on=True, label='SCF') # no markers if more than 10 data points ##
		# make a legend
		leg = fig_ax.legend(loc='upper left', frameon=False, handlelength=3.3, handletextpad=0.5)
		# label the axes
		if not show_recessions:
			fig_ax.set_xlabel('Year')
		else:
			fig_ax.set_xlabel('Date')
		fig_ax.set_ylabel('Wealth share (\%)') # sentence-style capitalization
		# set limits
		if not show_recessions:
			pylab.xlim(1982, 2013)
		else:
			pylab.xlim(datetime.date(1982, 1, 1), datetime.date(2014, 1, 1)) ##
		pylab.ylim(0.852602332947, 3. + (1. - 0.852602332947))
		# major xticks
		if not show_recessions:
			pylab.xticks([1982, 1992, 2002, 2012], ['1982', '1992', '2002', '2012'])
		else:
			pylab.xticks([datetime.date(1982, 1, 1), datetime.date(1990, 1, 1), datetime.date(1998, 1, 1), datetime.date(2006, 1, 1), datetime.date(2014, 1, 1)], ['1/1/1982', '1/1/1990', '1/1/1998', '1/1/2006', '1/1/2014']) ##
			# minor xticks
			fig_ax.xaxis.set_minor_locator(MultipleLocator(1)) # tick every year
		# add y ticks
		pylab.yticks([1, 2, 3], ['1', '2', '3'])
		# minor yticks
		#fig_ax.yaxis.set_minor_locator(MultipleLocator(0.1)) # tick every 0.1
		## nber dates
		if show_recessions:
			y1, y2 = fig_ax.get_ylim()
			fig_ax.fill_between((datetime.date(1981, 7, 1), datetime.date(1982, 11, 1)), y1=y1, y2=y2, alpha=.10, color='k')
			fig_ax.fill_between((datetime.date(1990, 7, 1), datetime.date(1991, 3, 1)), y1=y1, y2=y2, alpha=.10, color='k')
			fig_ax.fill_between((datetime.date(2001, 3, 1), datetime.date(2001, 11, 1)), y1=y1, y2=y2, alpha=.10, color='k')
			fig_ax.fill_between((datetime.date(2007, 12, 1), datetime.date(2009, 6, 1)), y1=y1, y2=y2, alpha=.10, color='k')
			from matplotlib.dates import YearLocator, MonthLocator, DateFormatter ##
			year_locator = YearLocator()
			fig_ax.xaxis.set_minor_locator(year_locator)
		# clean ticks
		for tickline in fig_ax.get_xticklines():
			tickline.set_markersize(8)
			tickline.set_markeredgewidth(0.5)
			tickline.set_marker(lines.TICKDOWN)
		for tickline in fig_ax.xaxis.get_minorticklines():
			tickline.set_markersize(4)
			tickline.set_markeredgewidth(0.5)
			tickline.set_marker(lines.TICKDOWN)
		for tickline in fig_ax.get_yticklines():
			tickline.set_markersize(8)
			tickline.set_markeredgewidth(0.5)
			tickline.set_marker(lines.TICKLEFT)
		for tickline in fig_ax.yaxis.get_minorticklines():
			tickline.set_markersize(4)
			tickline.set_markeredgewidth(0.5)
			tickline.set_marker(lines.TICKLEFT)
		# shift labels
		for ticklabel in fig_ax.get_xticklabels():
			ticklabel.set_y(0-0.04)
		for ticklabel in fig_ax.get_yticklabels():
			ticklabel.set_x(0-0.02)
		# set frame width
		for i in fig_ax.spines.keys():
			fig_ax.spines[i].set_linewidth(0.5)
		# get rid of ticks on top and right
		fig_ax.get_yaxis().tick_left()
		fig_ax.get_xaxis().tick_bottom()
		# adjust dimensions
		fig.subplots_adjust(left=0.12, bottom=0.14, right=0.95, top=0.96, wspace=0.0, hspace=0.0)
		# save figure
		if not show_recessions:
			pylab.savefig("./figures/figures_for_data_appendix/share_of_wealth.pdf", bbox_inches='tight')
		else:
			pylab.savefig("./figures/figures_for_data_appendix/share_of_wealth_w_recessions.pdf", bbox_inches='tight')

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=figsize)
	fig_ax = fig.gca()
	# plot wealth shares
	fig_ax.plot([int(year) for year in years_of_scf], list(recalculated_perct_of_wealth_held_by_least_wealthy_half_in[year] for year in years_of_scf), color='k', linestyle='solid', lw=0.5, marker='o', markersize=2, clip_on=False, label='Least-wealthy half of American households') # no markers if more than 10 data points
	fig_ax.plot([int(year) for year in years_of_scf], list(recalculated_perct_of_wealth_held_by_400_wealthiest_in[year] for year in years_of_scf), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=2, clip_on=False, label='Four hundred wealthiest Americans') # no markers if more than 10 data points
	# make a legend
	leg = fig_ax.legend(loc='upper left', frameon=False, handlelength=3.3, handletextpad=0.5)
	# label the axes
	fig_ax.set_xlabel('Year')
	fig_ax.set_ylabel('Wealth share (\%)') # sentence-style capitalization
	# set limits
	pylab.xlim(1989, 2010)
	pylab.ylim(0, 5) #pylab.ylim(0, 40)
	# major xticks
	pylab.xticks([1989, 1992, 1995, 1998, 2001, 2004, 2007, 2010], years_of_scf)
	# minor xticks
	fig_ax.xaxis.set_minor_locator(MultipleLocator(1)) # tick every year
	# add y ticks
	pylab.yticks([0, 2.5, 5], ['0', '2.5', '5.0']) #pylab.yticks([0, 20, 40], ['0', '20', '40'])
	# minor yticks
	#fig_ax.yaxis.set_minor_locator(MultipleLocator(1)) # tick every unit
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# adjust dimensions
	fig.subplots_adjust(left=0.12, bottom=0.14, right=0.95, top=0.96, wspace=0.0, hspace=0.0)
	# save figure
	pylab.savefig("./figures/figures_for_data_appendix/shares_of_wealth.pdf", bbox_inches='tight')

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=figsize)
	fig_ax = fig.gca()
	# plot wealth shares
	fig_ax.plot([int(year) for year in years_of_scf], list(recalculated_perct_of_wealth_held_by_wealthiest_one_perct_in[year] for year in years_of_scf), color='k', linestyle='solid', lw=0.5, marker='o', markersize=2, clip_on=True, label='Wealthiest one percent of American households') # no markers if more than 10 data points
	fig_ax.plot([int(year) for year in years_of_scf], list(recalculated_perct_of_wealth_held_by_400_wealthiest_in[year] for year in years_of_scf), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=2, clip_on=False, label='Four hundred wealthiest Americans') # no markers if more than 10 data points
	# make a legend
	leg = fig_ax.legend(loc='upper left', frameon=False, handlelength=3.3, handletextpad=0.5)
	# label the axes
	fig_ax.set_xlabel('Year')
	fig_ax.set_ylabel('Wealth share (\%)') # sentence-style capitalization
	# log the y axis
	#fig_ax.set_yscale('log')
	# set limits
	pylab.xlim(1989, 2010)
	pylab.ylim(0.0, 50.0)
	# major xticks
	pylab.xticks([1989, 1992, 1995, 1998, 2001, 2004, 2007, 2010], years_of_scf)
	# minor xticks
	fig_ax.xaxis.set_minor_locator(MultipleLocator(1)) # tick every year
	# add y ticks
	pylab.yticks([0.0, 25.0, 50.0], ['0', '25', '50'])
	# minor yticks
	#fig_ax.yaxis.set_minor_locator(MultipleLocator(1)) # tick every unit
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# adjust dimensions
	fig.subplots_adjust(left=0.12, bottom=0.14, right=0.95, top=0.96, wspace=0.0, hspace=0.0)
	# save figure
	pylab.savefig("./figures/figures_for_data_appendix/more_shares_of_wealth.pdf", bbox_inches='tight')

	## GROWTH INCIDENCE CURVE ##

	for dyear in dyears:
		try:
			fignumber += 1
		except NameError:
			fignumber = 1
		fig = pylab.figure(fignumber, figsize=figsize)
		fig_ax = fig.gca()
		# growth incidence curve
		minimum_value, maximum_value = 0.0, max(list(arcsinh(anonymous_accumulation_by_rank_btw[1][dyear]) for rank in ranks))
		#print "max value in dollar terms:\t", sinh(maximum_value)
		fig_ax.plot(ranks, list(arcsinh(anonymous_accumulation_by_rank_btw[rank][dyear]) for rank in ranks), color='k', drawstyle='step-post', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='')
		# avg change in wealth
		avg_change_in_wealth = (1. / 400.) * (deflate_by_CPI(current_dollar_amount=summary_stats['sum'][dyear.split('-')[1]], year=dyear.split('-')[1], base_year='2013') - deflate_by_CPI(current_dollar_amount=summary_stats['sum'][dyear.split('-')[0]], year=dyear.split('-')[0], base_year='2013'))
		fig_ax.plot([400+1, 1-1], [arcsinh(avg_change_in_wealth), arcsinh(avg_change_in_wealth)], color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label='')
		#print "avg change in wealth in dollar terms:\t", avg_change_in_wealth
		# label the axes
		fig_ax.set_xlabel('Rank in ' + dyear.split('-')[0]) # sentence-style capitalization
		fig_ax.set_ylabel('Change in wealth ($\mathdollar$Bs)') # between ' + dyear.split('-')[0] + ' and ' + dyear.split('-')[1] + ' ($\mathdollar$Bs)')
		# set x limit
		pylab.xlim(1, 400)
		# major xticks
		pylab.xticks([1, 100, 200, 300, 400], ['1rst', '100th', '200th', '300th', '400th'])
		# minor xticks
		fig_ax.xaxis.set_minor_locator(MultipleLocator(50)) # tick every 50 ranks
		# major yticks
		ticks = [0, 1000, 4516.89428869, 10000, 67364.4988511]
		pylab.yticks([arcsinh(tick) for tick in ticks], ['0', '1', '5', '10', '67'])
		# set limits again
		pylab.ylim(arcsinh(0), arcsinh(67364.4988511))
		# invert x axis
		fig_ax.invert_xaxis()
		# clean ticks
		for tickline in fig_ax.get_xticklines():
			tickline.set_markersize(8)
			tickline.set_markeredgewidth(0.5)
			tickline.set_marker(lines.TICKDOWN)
		for tickline in fig_ax.xaxis.get_minorticklines():
			tickline.set_markersize(4)
			tickline.set_markeredgewidth(0.5)
			tickline.set_marker(lines.TICKDOWN)
		for tickline in fig_ax.get_yticklines():
			tickline.set_markersize(8)
			tickline.set_markeredgewidth(0.5)
			tickline.set_marker(lines.TICKLEFT)
		for tickline in fig_ax.yaxis.get_minorticklines():
			tickline.set_markersize(4)
			tickline.set_markeredgewidth(0.5)
			tickline.set_marker(lines.TICKLEFT)
		# shift labels
		for ticklabel in fig_ax.get_xticklabels():
			ticklabel.set_y(0-0.04)
		for ticklabel in fig_ax.get_yticklabels():
			ticklabel.set_x(0-0.02)
		# set frame width
		for i in fig_ax.spines.keys():
			fig_ax.spines[i].set_linewidth(0.5)
		# get rid of ticks on top and right
		fig_ax.get_yaxis().tick_left()
		fig_ax.get_xaxis().tick_bottom()
		# adjust dimensions
		fig.subplots_adjust(left=0.12, bottom=0.14, right=0.95, top=0.96, wspace=0.0, hspace=0.0)
		# save figure
		pylab.savefig("figures/figures_for_data_appendix/gic_" + dyear.replace('-', '_to_') + ".pdf")

	## GROWTH INCIDENCE CURVE WITH WEALTH CHANGES ##

	for dyear in dyears:
		try:
			fignumber += 1
		except NameError:
			fignumber = 1
		fig = pylab.figure(fignumber, figsize=figsize)
		fig_ax = fig.gca()
		# growth incidence curve
		minimum_value, maximum_value = 0.0, max(list(arcsinh(anonymous_accumulation_by_rank_btw[1][dyear]) for rank in ranks))
		fig_ax.plot(ranks, list(arcsinh(anonymous_accumulation_by_rank_btw[rank][dyear]) for rank in ranks), color='k', drawstyle='step-post', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='')
		# incumbents and drop outs
		for rank in ranks:
			if len(non_anonymous_accumulation_by_rank_btw[rank][dyear]) > 0:
				for value, type, name in non_anonymous_accumulation_by_rank_btw[rank][dyear]:
					value = arcsinh(value) # inverse hyperbolic sine transformation
					if name == 'Buffett, Warren Edward':
						# plot point
						fig_ax.scatter([rank], [value], marker='o', c='k', s=10, clip_on=False, zorder=99)
						# plot arrow
						fig_ax.annotate('Warren Buffett', 
										xy=(rank, value), xycoords='data', 
										xytext=(rank + 15, value - arcsinh(1000)), textcoords='data',
										arrowprops=dict(facecolor='0.50', edgecolor='0.50', arrowstyle="->"),
										horizontalalignment='right', verticalalignment='center')
					elif name == 'Rockefeller, David Sr':
						# plot point
						fig_ax.scatter([rank], [value], marker='o', c='k', s=10, clip_on=False, zorder=99)
						# plot arrow
						fig_ax.annotate('David Rockefeller', 
										xy=(rank, value), xycoords='data', 
										xytext=(0.75, 0.15), textcoords='axes fraction',
										arrowprops=dict(facecolor='0.50', edgecolor='0.50', arrowstyle="->"),
										horizontalalignment='center', verticalalignment='center')
					elif name == 'Trump, Donald John':
						# plot point
						fig_ax.scatter([rank], [value], marker='o', c='k', s=10, clip_on=False, zorder=99)
						# plot arrow
						fig_ax.annotate('Donald Trump', 
										xy=(rank, value), xycoords='data', 
										xytext=(rank + 10, value + arcsinh(1000)), textcoords='data',
										arrowprops=dict(facecolor='0.50', edgecolor='0.50', arrowstyle="->"),
										horizontalalignment='right', verticalalignment='center')
					elif name == 'Hunt, Nelson Bunker': # other Hunt brother, 'Hunt, William Herbert', returned to list in 2013
						# plot point
						fig_ax.scatter([rank], [value], marker='o', c='k', s=10, clip_on=False, zorder=99)
						# plot arrow
						fig_ax.annotate('Nelson Hunt', 
										xy=(rank, value), xycoords='data', 
										xytext=(0.75, 0.10), textcoords='axes fraction',
										arrowprops=dict(facecolor='0.50', edgecolor='0.50', arrowstyle="->"),
										horizontalalignment='center', verticalalignment='center')
					#elif name == 'Hunt, William Herbert': # returned to list in 2013
					#	# plot point
					#	fig_ax.scatter([rank], [value], marker='o', c='k', s=10, clip_on=False, zorder=99)
					elif name == 'Ludwig, Daniel Keith':
						# plot point
						fig_ax.scatter([rank], [value], marker='o', c='k', s=10, clip_on=False, zorder=99)
						# plot arrow
						fig_ax.annotate('Daniel Ludwig', 
										xy=(rank, value), xycoords='data', 
										xytext=(0.75, 0.05), textcoords='axes fraction',
										arrowprops=dict(facecolor='0.50', edgecolor='0.50', arrowstyle="->"),
										horizontalalignment='center', verticalalignment='center')
					else: 
						fig_ax.scatter([rank], [value], marker='o', facecolor='0.75', edgecolor='0.75', s=10, clip_on=False, zorder=1)
					# keep track of min and max values for setting y limits
					if value < minimum_value: minimum_value = value	
					if value > maximum_value: maximum_value = value
		# new entrants
		if len(new_entrant_accumulation[dyear]) > 0:
			for value, type, name in new_entrant_accumulation[dyear]:
				value = arcsinh(value) # inverse hyperbolic sine transformation
				if name == 'Gates, William Henry III':
					# plot point
					fig_ax.scatter([rank], [value], marker='o', c='k', s=10, clip_on=False, zorder=99)
					# plot arrow
					fig_ax.annotate('Bill Gates', 
									xy=(rank, value), xycoords='data', 
									xytext=(rank - 15, value - arcsinh(1000)), textcoords='data',
									arrowprops=dict(facecolor='0.50', edgecolor='0.50', arrowstyle="->"),
									horizontalalignment='left', verticalalignment='center')
				else:
					fig_ax.scatter([rank], [value], marker='o', facecolor='0.75', edgecolor='0.75', s=10, clip_on=False, zorder=1)			
				# keep track of min and max values for setting y limits
				if value < minimum_value: minimum_value = value
				if value > maximum_value: maximum_value = value
		#print "min and max value in dollar terms:\t", sinh(minimum_value), "\t", sinh(maximum_value)
		# avg change in wealth
		#avg_change_in_wealth = (1. / 400.) * (deflate_by_CPI(current_dollar_amount=summary_stats['sum'][dyear.split('-')[1]], year=dyear.split('-')[1], base_year='2013') - deflate_by_CPI(current_dollar_amount=summary_stats['sum'][dyear.split('-')[0]], year=dyear.split('-')[0], base_year='2013'))
		#fig_ax.plot([400+1, 1-1], [arcsinh(avg_change_in_wealth), arcsinh(avg_change_in_wealth)], color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label='')
		#print "avg change in wealth in dollar terms:\t", avg_change_in_wealth
		# label the axes
		fig_ax.set_xlabel('Rank in ' + dyear.split('-')[0]) # sentence-style capitalization
		fig_ax.set_ylabel('Change in wealth ($\mathdollar$Bs)') # between ' + dyear.split('-')[0] + ' and ' + dyear.split('-')[1] + ' ($\mathdollar$Bs)')
		# set x limit
		pylab.xlim(1, 400)
		# major xticks
		pylab.xticks([1, 100, 200, 300, 400], ['1rst', '100th', '200th', '300th', r'$\leq$400th'])
		# minor xticks
		fig_ax.xaxis.set_minor_locator(MultipleLocator(50)) # tick every 50 ranks
		# major yticks
		ticks = [-4204.41933963, -1000, 0, 1000, 10000, 71690.3138157]
		pylab.yticks([arcsinh(tick) for tick in ticks], ['$-$4', '$-$1', '0', '1', '10', '72'])
		# set limits again
		pylab.ylim(arcsinh(-4204.41933963), arcsinh(71690.3138157))
		# invert x axis
		fig_ax.invert_xaxis()
		# clean ticks
		for tickline in fig_ax.get_xticklines():
			tickline.set_markersize(8)
			tickline.set_markeredgewidth(0.5)
			tickline.set_marker(lines.TICKDOWN)
		for tickline in fig_ax.xaxis.get_minorticklines():
			tickline.set_markersize(4)
			tickline.set_markeredgewidth(0.5)
			tickline.set_marker(lines.TICKDOWN)
		for tickline in fig_ax.get_yticklines():
			tickline.set_markersize(8)
			tickline.set_markeredgewidth(0.5)
			tickline.set_marker(lines.TICKLEFT)
		for tickline in fig_ax.yaxis.get_minorticklines():
			tickline.set_markersize(4)
			tickline.set_markeredgewidth(0.5)
			tickline.set_marker(lines.TICKLEFT)
		# shift labels
		for ticklabel in fig_ax.get_xticklabels():
			ticklabel.set_y(0-0.04)
		for ticklabel in fig_ax.get_yticklabels():
			ticklabel.set_x(0-0.02)
		# set frame width
		for i in fig_ax.spines.keys():
			fig_ax.spines[i].set_linewidth(0.5)
		# get rid of ticks on top and right
		fig_ax.get_yaxis().tick_left()
		fig_ax.get_xaxis().tick_bottom()
		# adjust dimensions
		fig.subplots_adjust(left=0.12, bottom=0.14, right=0.95, top=0.96, wspace=0.0, hspace=0.0)
		# save figure
		pylab.savefig("figures/figures_for_data_appendix/gic_w_names_" + dyear.replace('-', '_to_') + ".pdf")

	#### TABLES ####

	## DECOMPOSED CHANGE IN WEALTH ##

	# uses packages ctable and dcolumn
	f = open("./tables/tables_specific_to_data_appendix/decomposition_of_change_in_wealth_of_the_400_wealthiest_Americans.txt", 'w') # write to this file
	f.write("{\ssp\n") # single-spacing
	f.write("\ctable[\n")
	f.write("\tcaption\t= {Decomposition of the Change in the Wealth of the 400 Wealthiest Americans, 1982--2013},\n") # sentence-style without terminal period
	f.write("\tlabel\t= decomposition_of_change_in_wealth_of_the_400_wealthiest_Americans,\n")
	#f.write("\tdoinside\t=\\footnotesize,\n")
	#f.write("\tsideways\t,\n") # flip table sideways
	f.write("\tpos\t= tp,\n") # top or page by itself
	f.write("\tnotespar,\n") # footnotes must be flush left (Turabian 2007, sec. 26.2.7)
	f.write("\tnosuper,\n") # typeset sources and notes on the line, manually superscript everything else
	f.write("\tmincapwidth\t= 1.0\\textwidth,\n") # make the table caption and footnotes as wide as the text
	f.write("\t]{lD{.}{.}{-1}D{.}{.}{-1}}{\n") # first column is flush left, other columns are centered
	f.write("\\tnote[Source:~\\textnormal{Data adapted from \\textit{Forbes Magazine} (1982--2013).}]{\\\} \n") # terminal period
	f.write("\\tnote[]{\\\} % for spacing\n")
	f.write("\\tnote[Note:~\\textnormal{This table shows how the change in the total wealth of the 400 wealthiest Americans between 1982 and 2013 can be decomposed into the contributions made by different groups of unique individuals. The decomposition is discussed in the text. The dollar values are billons of CPI-deflated, 2013 dollars.}]{\\\} \n")
	#f.write("\\tnote[]{\\\} % for spacing\n")
	#f.write("\\tnote[$^{*}p<.10$ { } $^{**}p<.05$ { } $^{***}p<0.01$]{}") # notes for p-values (see Turabian 26.2.7)
	f.write("}{\FL[\lightrulewidth]\n")
	f.write("Group\t&\t\multicolumn{1}{c}{Contribution ($\mathdollar$Bs)}\t\ML\n")
	# ...
	dyear = '1982-2013'
	select_components = ['incumbents', 'self-made entrants', 'other entrants', 'decedents', 'renunciants', 'other dropouts']
	for component in select_components:
		if component == 'incumbents':
			s = component.capitalize()
		else:
			# group or sub-group name
			if component == 'self-made entrants':
				s = 'Entrants\t\NN\n'
				s += '\hspace{1em}' + component.capitalize()
			elif component == 'decedents':
				s = 'Dropouts\t\NN\n'
				s += '\hspace{1em}' + component.capitalize()
			else:
				s = '\hspace{1em}' + component.capitalize()
		s += '\t&\t'
		s += str(format(round(sum([_[0] for _ in change_in_wealth_due_to[component][dyear]]), 0), ',.0f'))
		s += '\t&\t'
		if component != select_components[-1]:
			s += '\t\NN\n'
		else:
			s += "\t\ML[\lightrulewidth]\n" # rule 
			s += 'All groups'
			s += '\t&\t'
			s += str(format(round(change_in_wealth_btw[dyear], 0), ',.0f'))
		f.write(s)
	#...
	f.write('\t\LL[\lightrulewidth]\n')
	f.write("\NN % for spacing\n") # for spacing
	f.write("}\n") # end ctable
	f.write("}\n") # end single-spacing
	f.close()