"""
Filename: essay_on_distribution_of_wealth_code.py
Python version: 2.7
Source: Capehart, Kevin W. Essays on the Wealthiest Americans. PhD dissertation, American University, Washington, DC, 2014. 
Description: This file generates results reported by the essay on the distribution of the wealth of the wealthiest Americans. 
"""

from __future__ import division
import numpy
import scipy.stats # import for confidence intervals
import scipy.optimize # import for maximum likelihood estimation
import scipy.special # import for gamma distribution
from scipy.special import erf # import for error function erf(z) = 2 / sqrt(pi) * integral(exp(-x ** 2), x=0...z)
from os.path import isfile # import for checking whether a file exists
import pickle # import for dumping and loading computationally intensive results

## SEC. 3.2: REPLICATION OF KLASS ET AL. (2006)

# The following code estimates the shape parameters of Pareto distributions using Klass et al.'s (2006) data and methods

def get_pareto_index_using_klass_et_al_method(wealths, shift_term_for_ranks=0., ignore_ties=True, drop_wealthiest_and_poorest=True, regress_wealth_on_rank_instead_of_rank_on_wealth=False):
	"""
	Returns OLS estimate of shape parameter of pareto distribution from a regression of log-ranks on log-wealths 
	
	Parameters
	==========
	shift_term_for_ranks : float for shift term to subtract from ranks as in the rank-1/2 OLS method
	ignore_ties : bool for whether to ignore or account for ties in wealth when ranking wealths
	drop_wealthiest_and_poorest : bool for whether to drop wealths that are strictly poorer than the 400th and strictly wealthier than the 10th
	regress_wealth_on_rank_instead_of_rank_on_wealth : bool for whether to run a log-log weak-rank rather than a log-log rank-wealth regression	
	"""
	# sort wealths in descending order
	wealths.sort(reverse=True)
	# create ranks
	# ignore ties?
	if ignore_ties:
		ranks = list(rank - shift_term_for_ranks for rank in range(1, len(wealths)+1))
	else:
		ranks = [1]
		for i in range(1, len(wealths)):
			if wealths[i-1] == wealths[i]:
				ranks.append(ranks[-1])
			else:
				ranks.append(i+1)
	# drop wealthiest and poorest?
	if drop_wealthiest_and_poorest:
		# drop wealths that are strictly poorer than the 400th
		while ranks[-1] > 400:
			ranks.pop()
			wealths.pop()
		# drop wealths that are strictly wealthier than the 10th
		i = 0
		rank = ranks[i]
		while rank < 10:
			i += 1
			rank = ranks[i]
		index_of_10th_wealthiest = i
		wealths, ranks = wealths[index_of_10th_wealthiest:], ranks[index_of_10th_wealthiest:] # note: re-defining wealths and ranks
	# log wealths and ranks
	log_wealths = numpy.log(wealths)
	log_ranks = numpy.log(ranks)
	# get means of log wealth and ranks
	mean_of_log_wealths = numpy.mean(log_wealths)
	mean_of_log_ranks = numpy.mean(log_ranks)	
	# estimate shape parameter
	# regress log wealths on log ranks or vice versa?
	if regress_wealth_on_rank_instead_of_rank_on_wealth:
		cov_btw_log_ranks_and_log_wealths = (1. / len(ranks)) * sum([(log_ranks[i] - mean_of_log_ranks) * (log_wealths[i] - mean_of_log_wealths) for i in range(0, len(ranks))])
		var_of_log_ranks = (1. / len(ranks)) * sum([(log_ranks[i] - mean_of_log_ranks) ** 2. for i in range(0, len(ranks))])
		shape_parameter = - 1. / (cov_btw_log_ranks_and_log_wealths / var_of_log_ranks)
	else:
		cov_btw_log_wealths_and_log_ranks = (1. / len(wealths)) * sum([(log_wealths[i] - mean_of_log_wealths) * (log_ranks[i] - mean_of_log_ranks) for i in range(0, len(wealths))])
		var_of_log_wealths = (1. / len(wealths)) * sum([(log_wealths[i] - mean_of_log_wealths) ** 2. for i in range(0, len(wealths))])
		shape_parameter = - 1. * (cov_btw_log_wealths_and_log_ranks / var_of_log_wealths)
	# return estimated shape parameter
	return shape_parameter

# load Klass et al.'s (2006) data
filepath = "./data/data_specific_to_second_essay/"
filename = filepath + "data_for_klass_et_al_2006.txt"
select_years = list(str(y) for y in range(1988, 2003+1))
header = ['Name']
header.extend(select_years)
rawdata = numpy.loadtxt(filename, delimiter=',', skiprows=5, dtype={'names':(header), 'formats':(['<S50' for _ in range(0, len(header))])})

# define a dict for storing their data with its errors while preparing to correct errors
wealths_for_klass_et_al_data_in = dict(zip(select_years, [dict(zip(['with errors', 'without errors'], ['TBD' for _ in ['with errors', 'without errors']])) for year in select_years]))
# store raw data, removing zeros
for year in select_years:
	wealths_for_klass_et_al_data_in[year]['with errors'] = [float(rawdata[year][i]) for i in range(0, len(rawdata['Name']))]
	while 0. in wealths_for_klass_et_al_data_in[year]['with errors']:
		wealths_for_klass_et_al_data_in[year]['with errors'].remove(0.)

# estimate Pareto indexes using klass et al.'s (2006) data while also preparing to estimate them after correcting errors in their data
estimated_pareto_index_using_klass_et_al_methods_in = dict(zip(select_years, list(dict(zip(['with errors', 'without errors'], list(dict(zip(['point', 'se'], ['TBD', 'TBD'])) for _ in ['with errors', 'without errors']))) for year in select_years)))
w_or_wo = 'with errors'
for year in select_years:
	wealths = wealths_for_klass_et_al_data_in[year][w_or_wo]
	point = get_pareto_index_using_klass_et_al_method(wealths, shift_term_for_ranks=0., ignore_ties=True, drop_wealthiest_and_poorest=True, regress_wealth_on_rank_instead_of_rank_on_wealth=False)
	se = point * (2. / 391.) ** 0.5 # use the standard error suggested by Gabaix and Ibragimov (2011)
	estimated_pareto_index_using_klass_et_al_methods_in[year][w_or_wo]['point'] = point
	estimated_pareto_index_using_klass_et_al_methods_in[year][w_or_wo]['se'] = se
	# print results
	if year == select_years[0]: print "\npareto indexes estimated using Klass et al.'s (2006) data and methods \nyear \tpoint estimate"
	print year, "\t", round(estimated_pareto_index_using_klass_et_al_methods_in[year][w_or_wo]['point'], 2)

## 3.3.1: EFFECT OF ERRORS IN KLASS ET AL.'s (2006) DATA ##

# The following code shows the effect of errors in Klass et al.'s (2006) data on estimates of the Pareto indexes. Their estimation method is still used. 

# correct errors related to the wealths of individuals/families who appear in Klass et al.'s (2006) data
# Richard Alexander Manoogian's wealth was 625 million dollars in 1988 (24 October 1988, p. 188, 340), not 885 million dollars, as in Klass et al.'s (2006) data.
rawdata['1988'][list(rawdata['Name']).index('ManoogianRichardAlexander')] = '625'
# Henry Ross Perot Sr.'s wealth was 2500 million dollars in 1989 (23 October 1989, p. 156, 352), not 500 million dollars, as in Klass et al.'s (2006) data.
rawdata['1989'][list(rawdata['Name']).index('PerotHenryRoss')] = '2500'
# Klass et al.'s (2006) data includes Roy Michael Huffington with a worth of 400 million dollars in the year 1994, but he dropped off the 1994 Forbes Four Hundred (17 October 1994, p. 331, 310).
rawdata['1994'][list(rawdata['Name']).index('HuffingtonRoyMichael')] = '0'
# Leonard Samuel Skaggs Jr.'s wealth was 950 million million dollars in 1996 (14 October 1996, p. 204, 354)
rawdata['1996'][list(rawdata['Name']).index('SkaggsLeonardSamuelJr')] = '950'

# store corrected data, removing zeros
for year in select_years:
	wealths_for_klass_et_al_data_in[year]['without errors'] = [float(rawdata[year][i]) for i in range(0, len(rawdata['Name']))]
	while 0. in wealths_for_klass_et_al_data_in[year]['without errors']:
		wealths_for_klass_et_al_data_in[year]['without errors'].remove(0.)

# correct errors related to individuals/families who do not appear in Klass et al.'s (2006) data
# Klass et al.'s (2006) data includes the family of Charles E. Smith with a worth of 290 million dollars in the year 1988, but the 1988 Forbes Four Hundred included Charles E. Smith and Robert H. Smith each with worths of 290 million dollars (24 October 1988, p. 258, 344--345).
wealths_for_klass_et_al_data_in['1988']['without errors'].append(290.)
# Klass et al.'s (2006) data does not include Shelby Cullom Davis for the year 1993, but he was on the 1993 Forbes Four Hundred with a worth of 800 million dollars (18 October 1993, p. 180). 
wealths_for_klass_et_al_data_in['1993']['without errors'].append(800.)
# Klass et al.'s (2006) data includes a Frank Batten with a worth 1.6 billion dollars in the year 1999, but there was a Frank Batten Sr. who was worth twice that much (i.e., 2.1 billion dollars) and a Frank Batten Jr. who was worth half that much (i.e., 1.1 billion dollars) on the 1999 Forbes Four Hundred (11 October 1999, p. 242). 
wealths_for_klass_et_al_data_in['1999']['without errors'].remove(1600.)
wealths_for_klass_et_al_data_in['1999']['without errors'].append(2100.)
wealths_for_klass_et_al_data_in['1999']['without errors'].append(1100.)

# estimate Pareto indexes without errors in the data
w_or_wo = 'without errors'
for year in select_years:
	wealths = wealths_for_klass_et_al_data_in[year][w_or_wo]
	point = get_pareto_index_using_klass_et_al_method(wealths, shift_term_for_ranks=0., ignore_ties=True, drop_wealthiest_and_poorest=True, regress_wealth_on_rank_instead_of_rank_on_wealth=False)
	se = point * (2. / 391.) ** 0.5 # use the standard error suggested by Gabaix and Ibragimov (2011)
	estimated_pareto_index_using_klass_et_al_methods_in[year][w_or_wo]['point'] = point
	estimated_pareto_index_using_klass_et_al_methods_in[year][w_or_wo]['se'] = se
	# print results
	if year == select_years[0]: print "\npareto indexes with and without errors \nyear \twith \twithout \tdiff"
	with_errors, without_errors = estimated_pareto_index_using_klass_et_al_methods_in[year]['with errors']['point'], estimated_pareto_index_using_klass_et_al_methods_in[year]['without errors']['point']
	print year, "\t", format(round(with_errors, 2), '.2f'), "\t", 
	print format(round(without_errors, 2), '.2f'), "\t", 
	print format(round(with_errors - without_errors, 2), '.2f')

## LOAD OUR FORBES 400 DATASET ##

from load_forbes_400_dataset import *

## SEC. 3.4.1: VARIATION OVER TIME IN PARETO INDEX ##

# The following code looks at variation over time in the maximum-likelihood estimate of the shape parameter of a Pareto distribution fit the wealths of the Forbes 400

class fit_pareto_to(object):
	""" Class for getting the (unbiased) maximum-likelihood estimates of the parameters of a two-parameter Pareto distribution """

	def __init__(self, wealths):
		self.wealths = wealths # observations
		self.n = len(wealths) # number of observations
		self.min_wealth = min(self.wealths) # minimum observation
		
	def estimate_biased_parameters(self):
		""" Calculates biased MLEs of lower bound and shape parameters. Returns None """
		self.biased_lower_bound = self.min_wealth
		self.biased_shape = self.n / sum(list(numpy.log(wealth / self.min_wealth) for wealth in self.wealths))
	
	def get_estimated_parameters(self):
		""" Returns unbiased MLEs of lower bound and shape parameters """
		self.estimate_biased_parameters()
		self.lower_bound = (1. - (1. / ((self.n - 1) * self.biased_shape))) * self.biased_lower_bound
		self.shape = ((self.n - 2.) / self.n) * self.biased_shape
		return self.lower_bound, self.shape
	
	def get_standard_errors_of_estimated_parameters(self):
		""" Returns standard errors of estimated lower bound and shape parameters """
		se_of_lower_bound = numpy.sqrt((self.lower_bound ** 2.) * (self.shape ** -1.) * ((self.n - 1.) ** -1.) * ((self.shape * self.n - 2.) ** -1.))
		se_of_shape = numpy.sqrt((self.shape ** 2.) * ((self.n - 3.) ** -1.))		
		return se_of_lower_bound, se_of_shape

# define dict for storing estimates
estimated_pareto_index_in = dict(zip(years, [dict(zip(['point', 'se'], ['TBD' for _ in ['point', 'se']])) for year in years]))
# get estimates
for year in years:
	wealths = summary_stats['wealths'][year]
	fit = fit_pareto_to(wealths=wealths)
	_, estimated_pareto_index_in[year]['point'] = fit.get_estimated_parameters()
	_, estimated_pareto_index_in[year]['se'] = fit.get_standard_errors_of_estimated_parameters()
	# print results
	if year == years[0]: print "\nmles of pareto indexes \nyear \tpoint \tse"
	print year, "\t", estimated_pareto_index_in[year]['point'], "\t", estimated_pareto_index_in[year]['se']

## SEC. 3.4.2: PARTS RELATED TO FIG. 3.1 ##

# The following code generates the empirical CCDF, fitted CCDF, and largest absolute difference between those two distributions shown in Fig. 3.1 of the essay

class k_s_like_test_against_unknown_pareto_distribution(object):
	""" Class for a Kolmogorov-Smirnov-like test against a two-parameter Pareto distribution. The test is exactly like a K-S-like test, except it is on the CCDF rather than the CDF, but that has no effect on the conclusions as long as the empirical CCDF and CDF are the complement of one another. 
	
	Parameters
	==========
	wealths : observed wealths
	a_priori_parameters_of_null_distribution : None for unknown parameters of Pareto distribution or tuple for specifying parameters a priori
	"""

	def __init__(self, wealths, a_priori_parameters_of_null_distribution=None):
		self.wealths = wealths
		self.wealths.sort() # sort wealths in ascending order
		self.number_of_observations = len(self.wealths)
		self.distinct_wealths = list(set(self.wealths))
		self.distinct_wealths.sort() # sort distinct wealths in ascending order
		self.number_of_distinct_observations = len(self.distinct_wealths)
		if a_priori_parameters_of_null_distribution == None:
			self.lower_bound_parameter, self.shape_parameter = self.get_estimated_parameters()
		else:
			self.lower_bound_parameter, self.shape_parameter = a_priori_parameters_of_null_distribution
		self.max_discrepancy = None # maximum discrepancy
				
	def get_estimated_parameters(self):
		fit = fit_pareto_to(wealths=self.wealths)
		return fit.get_estimated_parameters()
	
	def get_predicted_ccdf_at(self, given_wealth):
		if given_wealth < self.lower_bound_parameter:
			return 1.
		else:
			return (self.lower_bound_parameter / given_wealth) ** self.shape_parameter
	
	def get_predicted_wealth_for(self, prob_wealth_greater_than_or_equal_to):
		return self.lower_bound_parameter * (prob_wealth_greater_than_or_equal_to ** (- self.shape_parameter))
	
	def get_empirical_ccdf_at(self, given_wealth):
		""" Returns proportion of wealths that are strictly greater than the given wealth """
		return sum(list(int(wealth > given_wealth) for wealth in self.wealths)) / float(self.number_of_observations)

	def get_discrepancy_statistic(self):
		""" Returns the largest absolute difference between an empirical and fitted CCDF """
		# get discrepancies
		discrepancies = list()
		# observations, except left most
		for i in range(1, self.number_of_distinct_observations):
			discrepancy = max(abs(self.get_empirical_ccdf_at(self.distinct_wealths[i-1]) - self.get_predicted_ccdf_at(self.distinct_wealths[i])), abs(self.get_empirical_ccdf_at(self.distinct_wealths[i]) - self.get_predicted_ccdf_at(self.distinct_wealths[i])))
			discrepancies.append(discrepancy)
		# left-most observation
		discrepancy = max(abs(1. - self.get_predicted_ccdf_at(self.distinct_wealths[0])), abs(self.get_empirical_ccdf_at(self.distinct_wealths[0]) - self.get_predicted_ccdf_at(self.distinct_wealths[0])))
		discrepancies.append(discrepancy)
		# get maximum discrepancy
		max_discrepancy = max(discrepancies)
		self.max_discrepancy = max_discrepancy
		return max_discrepancy
	
	def get_wealths_and_probs_at_max_discrepancy(self):
		""" Returns wealth(s) and prob(s) at the maximum discrepancy between an empirical and fitted CCDF """
		if self.max_discrepancy == None:
			self.get_discrepancy_statistic()
		else:
			pass
		# get wealths and probs at max discrepancy
		wealths_at_max_discrepancy = list()
		probs_at_max_discrepancy = list()
		# left-most observation
		discrepancy = max(abs(1. - self.get_predicted_ccdf_at(self.distinct_wealths[0])), abs(self.get_empirical_ccdf_at(self.distinct_wealths[0]) - self.get_predicted_ccdf_at(self.distinct_wealths[0])))
		if discrepancy < self.max_discrepancy:
			pass
		else:
			wealths_at_max_discrepancy.append(self.distinct_wealths[0])
			# just before step?
			if abs(1. - self.get_predicted_ccdf_at(self.distinct_wealths[0])) > abs(self.get_empirical_ccdf_at(self.distinct_wealths[0]) - self.get_predicted_ccdf_at(self.distinct_wealths[0])):
				probs_at_max_discrepancy.append([1., self.get_predicted_ccdf_at(self.distinct_wealths[0])])
			# just after step?
			else:
				probs_at_max_discrepancy.append([self.get_empirical_ccdf_at(self.distinct_wealths[0]), self.get_predicted_ccdf_at(self.distinct_wealths[0])])
		# others observations
		for i in range(1, self.number_of_distinct_observations):
			discrepancy = max(abs(self.get_empirical_ccdf_at(self.distinct_wealths[i-1]) - self.get_predicted_ccdf_at(self.distinct_wealths[i])), abs(self.get_empirical_ccdf_at(self.distinct_wealths[i]) - self.get_predicted_ccdf_at(self.distinct_wealths[i])))
			if discrepancy < self.max_discrepancy:
				pass
			else:
				wealths_at_max_discrepancy.append(self.distinct_wealths[i])
				# just before step?
				if abs(self.get_empirical_ccdf_at(self.distinct_wealths[i-1]) - self.get_predicted_ccdf_at(self.distinct_wealths[i])) > abs(self.get_empirical_ccdf_at(self.distinct_wealths[i]) - self.get_predicted_ccdf_at(self.distinct_wealths[i])):
					probs_at_max_discrepancy.append([self.get_empirical_ccdf_at(self.distinct_wealths[i-1]), self.get_predicted_ccdf_at(self.distinct_wealths[i])])
				# just after step?
				else:
					probs_at_max_discrepancy.append([self.get_empirical_ccdf_at(self.distinct_wealths[i]), self.get_predicted_ccdf_at(self.distinct_wealths[i])])
		return wealths_at_max_discrepancy, probs_at_max_discrepancy

# get empirical CCDF for wealth of the 2003 Forbes 400
wealths_in_2003 = summary_stats['wealths']['2003']
k_s_like_test = k_s_like_test_against_unknown_pareto_distribution(wealths_in_2003) # use this classes empirical CCDF function
empirical_ccdf_at_for_2003 = [k_s_like_test.get_empirical_ccdf_at(wealth) for wealth in wealths_in_2003]
# get fitted CCDF
fitted_ccdf_for_2003 = [k_s_like_test.get_predicted_ccdf_at(wealth) for wealth in wealths_in_2003]
# get largest discrepancy and the wealth(s) and prob(s) associated with that discrepancy
discrepancy_statistic_in_2003 = k_s_like_test.get_discrepancy_statistic()
wealths_at_max_discrepancy_for_2003, probs_at_max_discrepancy_for_2003 = k_s_like_test.get_wealths_and_probs_at_max_discrepancy()
# largest discrepancy only occurs once, so only need first element of lists
assert len(wealths_at_max_discrepancy_for_2003) == 1, "Largest discrepancy occurs more than once"
wealth_at_max_discrepancy_for_2003 = wealths_at_max_discrepancy_for_2003[0]
probs_at_max_discrepancy_for_2003 = probs_at_max_discrepancy_for_2003[0]

## FUNCTION AND CLASS FOR CRITICAL VALUES OF K-S-LIKE TEST AGAINST UNBOUNDED PARETO ##

def random_pareto(lower_bound, shape_parameter, size):
	""" Returns random draws from a two-parameter Pareto distribution """
	uniforms = 1. - numpy.random.random(size=size) # uniform draws from (0.0, 1.0]
	paretos = lower_bound / (uniforms ** (1. / shape_parameter))
	return list(paretos)

class resample_d_stats_for_unknown_pareto_distribution(object):
	""" Class for resampling the discrepancy statistic for a K-S-like test against an unknown Pareto distribution 
	
	Parameters
	==========
	wealths : observed wealths
	round_to_nearest_for : dict of instructions for way in which the wealths of billionaires and sub-billionaires were rounded
	number_of_resamples : int for number of resamples to draw
	fitting : True for estimating parameters or tuple for specifying parameters a priori
	rounding : bool for whether to round resamples using instructions from round_to_nearest_for
	diffs_to_draw_from : False for no perturbation of resamples or set of kernel density estimates to draw from	
	"""

	def __init__(self, wealths=list(), round_to_nearest_for={}, number_of_resamples=1, fitting=True, rounding=True, diffs_to_draw_from=None):
		self.wealths = wealths
		self.number_of_observations = len(self.wealths)
		self.round_to_nearest_for = round_to_nearest_for
		k_s_like_test = k_s_like_test_against_unknown_pareto_distribution(self.wealths)
		self.lower_bound_parameter, self.shape_parameter = k_s_like_test.get_estimated_parameters()
		self.number_of_resamples = number_of_resamples
		if fitting:
			self.a_priori_parameters_of_null_distribution = None
		else:
			self.a_priori_parameters_of_null_distribution = (self.lower_bound_parameter, self.shape_parameter)
		self.rounding = rounding
		self.diffs_to_draw_from = diffs_to_draw_from
		if self.diffs_to_draw_from != None:
			self.kde = kernel_density_estimator(x=diffs_to_draw_from)
		self.resampled_discrepancy_statistics = list()

	def get_resample_discrepancy_statistics(self):
		""" Returns discrepancy statistics from applying the same K-S-like test to sets of random draws from a bounded Pareto distribution """
		for resample in range(1, self.number_of_resamples+1):
			resampled_wealths = self.get_resampled_wealths()
			k_s_like_test_on_resampled_wealths = k_s_like_test_against_unknown_pareto_distribution(resampled_wealths, a_priori_parameters_of_null_distribution=self.a_priori_parameters_of_null_distribution)
			discrepancy_statistic = k_s_like_test_on_resampled_wealths.get_discrepancy_statistic()
			self.resampled_discrepancy_statistics.append(discrepancy_statistic)
		self.resampled_discrepancy_statistics.sort(reverse=True) # sort in descending order
		return self.resampled_discrepancy_statistics
	
	def get_p_value(self, discrepancy_statistic):
		""" Returns estimated p-value associated with given a discrepancy statistic """		
		# resample discrepancy statistic, if haven't done so already
		if len(self.resampled_discrepancy_statistics) == 0:
			self.get_resample_discrepancy_statistics()
		else:
			pass
		# find the proportion of resampled discrepancy statistics that are larger than the given discrepancy_statistic
		i = 0
		while (discrepancy_statistic < self.resampled_discrepancy_statistics[i]) and (i < self.number_of_resamples-1):
			i += 1
		return i / self.number_of_resamples
	
	def get_perct_critical_value(self, perct=5):
		""" Returns the critical value associated with a given p-value """
		# resample discrepancy statistic, if haven't done so already
		if len(self.resampled_discrepancy_statistics) == 0:
			self.get_resample_discrepancy_statistics()
		else:
			pass
		# find the proportion of resampled discrepancy statistics that are larger than the given discrepancy_statistic
		return self.resampled_discrepancy_statistics[int(int(round((perct / 100.) * self.number_of_resamples, 0)) - 1)]
	
	def get_resampled_wealths(self):
		""" Returns a set of random draws from a Pareto distribution """
		if self.diffs_to_draw_from == None:
			wealths = random_pareto(lower_bound=self.lower_bound_parameter, shape_parameter=self.shape_parameter, size=self.number_of_observations)
		else:
			uniform_randoms = 1. - numpy.random.uniform(low=0.0, high=1.0, size=self.number_of_observations * 10.) # order of magnitude more observations
			wealths = list((self.lower_bound_parameter / 10.) / (uniform_randoms ** (1. / self.shape_parameter))) # order of magnitude lower lower-bound parameter
			wealths = list(wealth * self.kde.get_random_sample_from_estimated_kernel_density() for wealth in wealths) # perturb
			wealths.sort(reverse=True)
			wealths = wealths[0:self.number_of_observations] # only use the number_of_observations largest samples
		# round wealths
		if self.rounding:
			wealths = list(self.get_rounded(wealth) for wealth in wealths)
		# return resampled and rounded wealth
		return wealths
		
	def get_rounded(self, wealth):
		""" Returns a wealth rounded according to the rounding instructions """
		if wealth < 1000.:
			round_to_nearest = self.round_to_nearest_for['sub-billionaires']
		else:
			round_to_nearest = self.round_to_nearest_for['billionaires']
		wealth = round(wealth / round_to_nearest, 0) * round_to_nearest
		return wealth

## CLASSES FOR FITTING TO A BOUNDED PARETO AND TESTING THE GOODNESS OF THE FIT ##

class fit_bounded_pareto_to(object):
	""" Class for getting the maximum-likelihood estimates of the three parameters of a truncated or tappered or bounded Pareto distribution """

	def __init__(self, wealths):
		self.wealths = wealths # the data
		self.n = len(wealths) # number of observations
		self.min_wealth = min(self.wealths) # minimum observation
		self.max_wealth = max(self.wealths) # maximum observation
		self.min_over_max = (self.min_wealth / self.max_wealth)
		self.found_root = False
		
	def get_estimated_parameters(self):
		""" Returns unbiased MLEs of parameters """
		# get MLE of shape parameter
		tries = 0
		initial_guess_for_shape = self.get_initial_guess_for_shape_parameter()
		while (self.found_root == False) and (tries < 1000+1):
			guess_for_shape = numpy.random.uniform(initial_guess_for_shape / 2., initial_guess_for_shape * 2.)
			biased_shape = scipy.optimize.newton(func=self.func, x0=guess_for_shape, fprime=None)
			self.found_root = (not (self.func(biased_shape) > 1.48e-08))
			tries += 1
		# adjust for bias by following the suggestion of Maschberger and Kroupa 2009
		bias_adjusted_shape = ((self.n - 3.) / self.n) * biased_shape
		# get bias-adjusted MLEs of upper- and lower-bound parameters by following the suggestions of Zhang (2012)
		lower_bound = self.min_wealth * (1. + (((self.min_over_max ** biased_shape) - 1.) / (self.n * biased_shape)))
		upper_bound = self.max_wealth * (1. + (numpy.log(self.max_wealth / self.min_wealth) / self.n))
		return lower_bound, upper_bound, bias_adjusted_shape
	
	def get_initial_guess_for_shape_parameter(self):
		""" Returns an initial guess for the shape parameter based on the MLE of the shape parameter for an unbounded Pareto """
		return self.n / sum(list(numpy.log(wealth / self.min_wealth) for wealth in self.wealths))

	def func(self, shape):
		""" Returns the shape parameter that solves func = 0 """
		return (self.n / shape) + ((self.n * ((self.min_over_max ** shape) * numpy.log(self.min_over_max))) / (1. - (self.min_over_max ** shape))) - sum(list(numpy.log(wealth / self.min_wealth) for wealth in self.wealths))
	
	def get_found_root(self):
		""" Returns a bool for whether a root was found or not """
		return self.found_root
	
class k_s_like_test_against_unknown_bounded_pareto_distribution(object):
	""" Class for a K-S-like test against an unknown bounded pareto distribution """

	def __init__(self, wealths):
		self.wealths = wealths
		self.wealths.sort() # sort wealths in ascending order
		self.number_of_observations = len(self.wealths)
		self.distinct_wealths = list(set(self.wealths))
		self.distinct_wealths.sort() # sort distinct wealths in ascending order
		self.number_of_distinct_observations = len(self.distinct_wealths)
		self.lower_bound_parameter, self.upper_bound_parameter, self.shape_parameter = self.get_estimated_parameters()
		self.max_discrepancy = None # maximum discrepancy
				
	def get_estimated_parameters(self):
		fit = fit_bounded_pareto_to(wealths=self.wealths)
		estimated_parameters = fit.get_estimated_parameters()
		self.found_root = fit.get_found_root()
		return estimated_parameters
	
	def get_found_root(self):
		return self.found_root
	
	def get_predicted_ccdf_at(self, given_wealth):
		if given_wealth < self.lower_bound_parameter:
			return 1.
		elif given_wealth > self.upper_bound_parameter:
			return 0.
		else:
			return ((self.lower_bound_parameter ** self.shape_parameter) * ((given_wealth ** (- self.shape_parameter)) - (self.upper_bound_parameter ** (- self.shape_parameter)))) / (1. - ((self.lower_bound_parameter / self.upper_bound_parameter) ** self.shape_parameter))
	
	def get_empirical_ccdf_at(self, given_wealth):
		return sum(list(int(wealth > given_wealth) for wealth in self.wealths)) / float(self.number_of_observations)

	def get_discrepancy_statistic(self):
		# record discrepancies
		discrepancies = list()
		# observations, except left most
		for i in range(1, self.number_of_distinct_observations):
			discrepancy = max(abs(self.get_empirical_ccdf_at(self.distinct_wealths[i-1]) - self.get_predicted_ccdf_at(self.distinct_wealths[i])), abs(self.get_empirical_ccdf_at(self.distinct_wealths[i]) - self.get_predicted_ccdf_at(self.distinct_wealths[i])))
			discrepancies.append(discrepancy)
		# left-most observation
		discrepancy = max(abs(1. - self.get_predicted_ccdf_at(self.distinct_wealths[0])), abs(self.get_empirical_ccdf_at(self.distinct_wealths[0]) - self.get_predicted_ccdf_at(self.distinct_wealths[0])))
		discrepancies.append(discrepancy)
		# maximum discrepancy
		max_discrepancy = max(discrepancies)
		self.max_discrepancy = max_discrepancy
		return max_discrepancy
	
	def get_wealths_and_probs_at_max_discrepancy(self):
		if self.max_discrepancy == None:
			self.get_discrepancy_statistic()
		else:
			pass
		wealths_at_max_discrepancy = list()
		probs_at_max_discrepancy = list()
		# left-most observation
		discrepancy = max(abs(1. - self.get_predicted_ccdf_at(self.distinct_wealths[0])), abs(self.get_empirical_ccdf_at(self.distinct_wealths[0]) - self.get_predicted_ccdf_at(self.distinct_wealths[0])))
		if discrepancy < self.max_discrepancy:
			pass
		else:
			wealths_at_max_discrepancy.append(self.distinct_wealths[0])
			# just before step?
			if abs(1. - self.get_predicted_ccdf_at(self.distinct_wealths[0])) > abs(self.get_empirical_ccdf_at(self.distinct_wealths[0]) - self.get_predicted_ccdf_at(self.distinct_wealths[0])):
				probs_at_max_discrepancy.append([1., self.get_predicted_ccdf_at(self.distinct_wealths[0])])
			# just after step?
			else:
				probs_at_max_discrepancy.append([self.get_empirical_ccdf_at(self.distinct_wealths[0]), self.get_predicted_ccdf_at(self.distinct_wealths[0])])
		# others observations
		for i in range(1, self.number_of_distinct_observations):
			discrepancy = max(abs(self.get_empirical_ccdf_at(self.distinct_wealths[i-1]) - self.get_predicted_ccdf_at(self.distinct_wealths[i])), abs(self.get_empirical_ccdf_at(self.distinct_wealths[i]) - self.get_predicted_ccdf_at(self.distinct_wealths[i])))
			if discrepancy < self.max_discrepancy:
				pass
			else:
				wealths_at_max_discrepancy.append(self.distinct_wealths[i])
				# just before step?
				if abs(self.get_empirical_ccdf_at(self.distinct_wealths[i-1]) - self.get_predicted_ccdf_at(self.distinct_wealths[i])) > abs(self.get_empirical_ccdf_at(self.distinct_wealths[i]) - self.get_predicted_ccdf_at(self.distinct_wealths[i])):
					probs_at_max_discrepancy.append([self.get_empirical_ccdf_at(self.distinct_wealths[i-1]), self.get_predicted_ccdf_at(self.distinct_wealths[i])])
				# just after step?
				else:
					probs_at_max_discrepancy.append([self.get_empirical_ccdf_at(self.distinct_wealths[i]), self.get_predicted_ccdf_at(self.distinct_wealths[i])])
		return wealths_at_max_discrepancy, probs_at_max_discrepancy

def random_bounded_pareto(lower_bound, shape_parameter, upper_bound, size):
	""" Returns random draws from a bounded Pareto distribution """
	uniform_randoms = 1. - numpy.random.uniform(low=0.0, high=1.0, size=size)
	bounded_paretos = list((- (uniform_randoms * (upper_bound ** shape_parameter) - uniform_randoms * (lower_bound ** shape_parameter) - (upper_bound ** shape_parameter)) / ((lower_bound ** shape_parameter) * (upper_bound ** shape_parameter))) ** (- 1. / shape_parameter))
	return list(bounded_paretos)

class resample_d_stats_for_unknown_bounded_pareto_distribution(object):
	""" Class for resampling the discrepancy statistic for a K-S-like test against an unknown bounded Pareto distribution """

	def __init__(self, wealths=list(), round_to_nearest_for={}, number_of_resamples=10000, diffs_to_draw_from=None):
		self.wealths = wealths
		self.number_of_observations = len(self.wealths)
		self.round_to_nearest_for = round_to_nearest_for
		k_s_like_test = k_s_like_test_against_unknown_bounded_pareto_distribution(self.wealths)
		self.lower_bound_parameter, self.upper_bound_parameter, self.shape_parameter = k_s_like_test.get_estimated_parameters()
		self.number_of_resamples = number_of_resamples
		self.diffs_to_draw_from = diffs_to_draw_from
		if self.diffs_to_draw_from != None:
			self.kde = kernel_density_estimator(x=diffs_to_draw_from)		
		self.resampled_discrepancy_statistics = list()

	def get_resample_discrepancy_statistics(self):
		for resample in range(1, self.number_of_resamples+1):
			#tries = 0
			found_root = False
			while found_root == False:		
				resampled_wealths = self.get_resampled_wealths()
				k_s_like_test_on_resampled_wealths = k_s_like_test_against_unknown_bounded_pareto_distribution(resampled_wealths)
				found_root = k_s_like_test_on_resampled_wealths.get_found_root()
			discrepancy_statistic = k_s_like_test_on_resampled_wealths.get_discrepancy_statistic()
			self.resampled_discrepancy_statistics.append(discrepancy_statistic)
		self.resampled_discrepancy_statistics.sort(reverse=True) # sort in descending order
		return self.resampled_discrepancy_statistics
	
	def get_p_value(self, discrepancy_statistic):
		# resample discrepancy statistic, if haven't done so already
		if len(self.resampled_discrepancy_statistics) == 0:
			self.get_resample_discrepancy_statistics()
		else:
			pass
		# find the proportion of resampled discrepancy statistics that are larger than the given discrepancy_statistic
		i = 0
		while (discrepancy_statistic < self.resampled_discrepancy_statistics[i]) and (i < self.number_of_resamples-1):
			i += 1
		return i / self.number_of_resamples
		
	def get_resampled_wealths(self):
		if self.diffs_to_draw_from == None:
			wealths = random_bounded_pareto(lower_bound=self.lower_bound_parameter, shape_parameter=self.shape_parameter, upper_bound=self.upper_bound_parameter, size=self.number_of_observations)
		else:
			uniform_randoms = 1. - numpy.random.uniform(low=0.0, high=1.0, size=self.number_of_observations * 10.) # order of magnitude more observations
			wealths = list((- (uniform_randoms * (self.upper_bound_parameter ** self.shape_parameter) - uniform_randoms * ((self.lower_bound_parameter / 10.) ** self.shape_parameter) - (self.upper_bound_parameter ** self.shape_parameter)) / (((self.lower_bound_parameter / 10.) ** self.shape_parameter) * (self.upper_bound_parameter ** self.shape_parameter))) ** (- 1. / self.shape_parameter)) # order of magnitude lower lower-bound parameter
			wealths = list(wealth * self.kde.get_random_sample_from_estimated_kernel_density() for wealth in wealths) # mismeasure
			wealths.sort(reverse=True)
			wealths = wealths[0:self.number_of_observations] # only use the number_of_observations largest samples
		# round wealths, following the methodology of forbes
		wealths = list(self.get_rounded(wealth) for wealth in wealths)
		# return resampled and rounded wealth
		return wealths
		
	def get_rounded(self, wealth):
		""" Returns a wealth rounded according to the rounding instructions """
		if wealth < 1000.:
			round_to_nearest = self.round_to_nearest_for['sub-billionaires']
		else:
			round_to_nearest = self.round_to_nearest_for['billionaires']
		wealth = round(wealth / round_to_nearest, 0) * round_to_nearest
		return wealth

## K-S-LIKE TEST WITH ROUNDING ERRORS ##

# The following code runs the K-S-like tests

# define dict for storing results for K-S-like test with rounding errors against unbounded and bounded Pareto distributions
unbounded_and_bounded_pareto = ['pareto', 'bounded pareto']
k_s_like_test_against_w_rounding_errors = dict(zip(unbounded_and_bounded_pareto, [dict(zip(years, [dict(zip(['D stat', 'p-value', 'D stats'], ['TBD' for _ in ['D stat', 'p-value', 'D stats']])) for year in years])) for distribution in unbounded_and_bounded_pareto]))
# set the following to True to re-estimate if already estimated
re_estimate = False
# check if already been estimated
filename = filepath + "k_s_like_test_against_w_rounding_errors.pkl"
already_estimated = isfile(filename)
# estimate or re-estimate
if not already_estimated or re_estimate:
	# set number of resamples
	number_of_resamples = 10000
	# for each year
	for year in years:
		# get wealths
		wealths = summary_stats['wealths'][year]
		# get rounding instructions
		round_to_nearest_for = dict()
		unique_wealths = list(set(wealths))
		unique_wealths.sort(reverse=True)
		# if wealthiest person isn't a billionaire
		if summary_stats['max'][year] < 1000.0:
			min_difference = min(list(unique_wealths[i] - unique_wealths[i+1] for i in range(0, len(unique_wealths)-1) if unique_wealths[i] < 1000.0))
			round_to_nearest_for['billionaires'] = min_difference # if a billion dollar wealth is drawn, use the same rounding
			round_to_nearest_for['sub-billionaires'] = min_difference
		# if poorest person is a billionaire
		elif summary_stats['min'][year] >= 1000.0:
			min_difference = min(list(unique_wealths[i] - unique_wealths[i+1] for i in range(0, len(unique_wealths)-1) if unique_wealths[i+1] >= 1000.0))
			round_to_nearest_for['billionaires'] = min_difference # if a sub-billion dollar wealth is drawn, use the same rounding
			round_to_nearest_for['sub-billionaires'] = min_difference
		else:
			round_to_nearest_for['billionaires'] = min(list(unique_wealths[i] - unique_wealths[i+1] for i in range(0, len(unique_wealths)-1) if unique_wealths[i+1] >= 1000.0))
			round_to_nearest_for['sub-billionaires'] = min(list(unique_wealths[i] - unique_wealths[i+1] for i in range(0, len(unique_wealths)-1) if unique_wealths[i] < 1000.0))		
		## k-s test with errors against unbounded pareto ##
		# get discrepancy statistic
		numpy.random.seed(250624)
		k_s_like_test = k_s_like_test_against_unknown_pareto_distribution(wealths)
		discrepancy_statistic = k_s_like_test.get_discrepancy_statistic()
		k_s_like_test_against_w_rounding_errors['pareto'][year]['D stat'] = discrepancy_statistic
		# resample discrepancy statistic
		numpy.random.seed(250624)
		resample_d_stats = resample_d_stats_for_unknown_pareto_distribution(wealths=wealths, round_to_nearest_for=round_to_nearest_for, number_of_resamples=number_of_resamples, fitting=True, rounding=True)
		k_s_like_test_against_w_rounding_errors['pareto'][year]['p-value'] = resample_d_stats.get_p_value(discrepancy_statistic)
		#k_s_like_test_against_w_rounding_errors['pareto'][year]['D stats'] = resample_d_stats.resampled_discrepancy_statistics # de-comment to store D-stats
		## k-s test with errors against bounded pareto ##
		# get discrepancy statistic against bounded pareto distribution
		numpy.random.seed(250624)
		k_s_like_test = k_s_like_test_against_unknown_bounded_pareto_distribution(wealths=wealths)
		discrepancy_statistic = k_s_like_test.get_discrepancy_statistic()
		k_s_like_test_against_w_rounding_errors['bounded pareto'][year]['D stat'] = discrepancy_statistic	
		# resample discrepancy statistic
		numpy.random.seed(250624)
		resample_d_stats = resample_d_stats_for_unknown_bounded_pareto_distribution(wealths=wealths, round_to_nearest_for=round_to_nearest_for, number_of_resamples=number_of_resamples) # always fitting and rounding
		k_s_like_test_against_w_rounding_errors['bounded pareto'][year]['p-value'] = resample_d_stats.get_p_value(discrepancy_statistic)
		#k_s_like_test_against_w_rounding_errors['bounded pareto'][year]['D stats'] = resample_d_stats.resampled_discrepancy_statistics # de-comment to store D-stats
		print year
	# dump results
	pickle.dump(k_s_like_test_against_w_rounding_errors, open(filename, 'wb'))
else:
	# load result
	k_s_like_test_against_w_rounding_errors = pickle.load(open(filename, 'rb'))

## BLOOMBERG VS. FORBES DIFFERENCES ##

class kernel_density_estimator(object):
	
	def __init__(self, x):
		self.x = x
		self.N = len(self.x)
		self.global_bandwidth = self.get_silverman_plug_in_estimate()
		
	def get_sample_inter_quartile_range(self):
		""" Computes the sample interquartile range using one of the functions recommended by Hyndman and Fan (1996), specifically, the eighth fuction they consider. According to the authors, it gives approximately median-unbiased estimates of a quartile regardless of the underlying distribution """
		upper_quartile = scipy.stats.mstats.mquantiles(self.x, prob=[75/100.], alphap=1/3., betap=1/3.)[0]
		lower_quartile = scipy.stats.mstats.mquantiles(self.x, prob=[25/100.], alphap=1/3., betap=1/3.)[0]
		return upper_quartile - lower_quartile
	
	def get_silverman_plug_in_estimate(self):
		""" Return's Silverman's plug-in estimate of the optimal band width """
		constant = (3 / (8 * numpy.sqrt(numpy.pi))) ** -0.2 # = about 1.3643
		delta = (1 / (2 * numpy.sqrt(numpy.pi))) ** 0.2 # = about 0.7764
		sample_standard_deviation = numpy.std(self.x, ddof=1) # standard deviation with (N-1) adjustment
		iqr = self.get_sample_inter_quartile_range()
		normalized_iqr = iqr / (scipy.stats.norm.ppf(0.75) - scipy.stats.norm.ppf(0.25)) # if the distribution is normal, then the standard deviation should be the inter-quartile range deflated by about 1.349
		return constant * delta * ( self.N ** (-0.2) ) * min(sample_standard_deviation, normalized_iqr)
	
	def guassian_kernel(self, z):
		""" Returns Guassian kernel """
		return ((2. * numpy.pi) ** (-0.5)) * numpy.exp(-0.5 * (z ** 2.))

	def get_kernel_density_estimates(self, xrange=None):
		""" Returns a dictionary of the Guassian kernel density estimates at each data point. Bandwidth is equal to Silverman's plug-in estimate """
		density_estimate_at = dict()
		if xrange == None:
			self.xrange = self.x
		else:
			self.xrange = xrange
		bandwidth_at = dict(zip(self.x, [self.global_bandwidth] * self.N))
		for x0 in self.xrange:
			density_estimate_at[x0] = (1. / self.N) * sum([(1. / bandwidth_at[xi]) * self.guassian_kernel((x0 - xi) / bandwidth_at[xi]) for xi in self.x])
		return density_estimate_at
	
	def get_random_sample_from_estimated_kernel_density(self):
		""" Returns random draws from the kernel density estimate """
		return numpy.random.normal(self.x[numpy.random.randint(low=int(0), high=int(self.N-1))], self.global_bandwidth)

# load forbes vs. bloomberg data
filename = filepath + "bloomberg_vs_forbes_data.txt"
header = ['name', 'forbes wealth', 'bloomberg wealth', 'year']
raw_data = numpy.loadtxt(filename, delimiter='\t', skiprows=8, dtype={'names':header, 'formats':['<S50']*len(header)})

# define dicts for storing results
sources = ['forbes 2012', 'bloomberg 2012', 'forbes 2013', 'bloomberg 2013']
wealths_from = dict(zip(sources, [list() for source in sources]))
n_on = dict(zip(sources, [0 for source in sources]))
comparisions = ['forbes and bloomberg 2012', 'forbes and bloomberg 2013']
n_on_both = dict(zip(comparisions, [0 for comparison in comparisions])) # ignoring Rick Cohen
diffs_btw = dict(zip(comparisions, [list() for comparison in comparisions]))
for i in range(len(raw_data)):
	name = raw_data['name'][i]
	forbes_wealth = raw_data['forbes wealth'][i]
	bloomberg_wealth = raw_data['bloomberg wealth'][i]
	year = raw_data['year'][i]
	if (forbes_wealth != ''):
		n_on['forbes' + ' ' + year] += 1
		wealths_from['forbes' + ' ' + year].append(float(forbes_wealth))
		if (bloomberg_wealth != ''):
			n_on_both['forbes and bloomberg' + ' ' + year] += 1
			diff = (float(bloomberg_wealth) / float(forbes_wealth)) # bloomberg as perct of forbes
			diffs_btw['forbes and bloomberg' + ' ' + year].append(diff)
	if (bloomberg_wealth != ''):
		n_on['bloomberg' + ' ' + year] += 1
		wealths_from['bloomberg' + ' ' + year].append(float(bloomberg_wealth))
		# note: diff already recorded
for comparison in comparisions:
	diffs_btw[comparison].sort() # sort in ascending order

print "\nnumber on forbes and bloomberg lists, respectively"
for year in ['2012', '2013']:
	print year, "\t", str(n_on['forbes' + ' ' + year]), "\t", str(n_on['bloomberg' + ' ' + year])

print "\nnumber on both lists",
for year in ['2012', '2013']:
	print year, "\t", str(n_on_both['forbes and bloomberg' + ' ' + year])

print "\nrange of differences between the forbes and bloomberg estimates of someone's wealth (as a percentage of the Forbes estimate)"
for year in ['2012', '2013']:
	print "min/max in " + str(year) + ":\t", str(100. * diffs_btw['forbes and bloomberg' + ' ' + year][0]), "\t", str(round(100. * diffs_btw['forbes and bloomberg' + ' ' + year][-1], 0))

# kde for 2013 diffs
xrange_for_bloomberg_forbes_diffs = list(set(diffs_btw['forbes and bloomberg 2012']).union(set(diffs_btw['forbes and bloomberg 2013']))) # use diffs from both years
xrange_for_bloomberg_forbes_diffs.append(0) # include zero to pick up any left tail
xrange_for_bloomberg_forbes_diffs.sort()
kde = kernel_density_estimator(x=diffs_btw['forbes and bloomberg 2013'])
kde_for_bloomberg_and_forbes_2013_diffs_at = kde.get_kernel_density_estimates(xrange=xrange_for_bloomberg_forbes_diffs)
# kde for 2012 diffs
kde = kernel_density_estimator(x=diffs_btw['forbes and bloomberg 2012'])
kde_for_bloomberg_and_forbes_2012_diffs_at = kde.get_kernel_density_estimates(xrange=xrange_for_bloomberg_forbes_diffs)

## K-S-LIKE TEST WITH ROUNDING AND OTHER MEASUREMENT ERRORS ##

# define dict for storing results for K-S-like test with rounding and other measurement errors against unbounded and bounded Pareto distributions
k_s_like_test_against_w_rounding_and_other_measurement_errors = dict(zip(unbounded_and_bounded_pareto, [dict(zip(years, [dict(zip(['D stat', 'p-value', 'D stats'], ['TBD' for _ in ['D stat', 'p-value', 'D stats']])) for year in years])) for distribution in unbounded_and_bounded_pareto]))
# set the following to True to re-estimate if already estimate
re_estimate = False
# check if already been estimated
filename = filepath + "k_s_like_test_against_w_rounding_and_other_measurement_errors.pkl"
already_estimated = isfile(filename)
# estimate or re-estimate
if not already_estimated or re_estimate:
	# set number of resamples
	number_of_resamples = 10000
	# for each year
	for year in years:
		# get wealths
		wealths = summary_stats['wealths'][year]
		# get rounding instructions
		round_to_nearest_for = dict()
		unique_wealths = list(set(wealths))
		unique_wealths.sort(reverse=True)
		# if wealthiest person isn't a billionaire
		if summary_stats['max'][year] < 1000.0:
			min_difference = min(list(unique_wealths[i] - unique_wealths[i+1] for i in range(0, len(unique_wealths)-1) if unique_wealths[i] < 1000.0))
			round_to_nearest_for['billionaires'] = min_difference # if a billion dollar wealth is drawn, use the same rounding
			round_to_nearest_for['sub-billionaires'] = min_difference
		# if poorest person is a billionaire
		elif summary_stats['min'][year] >= 1000.0:
			min_difference = min(list(unique_wealths[i] - unique_wealths[i+1] for i in range(0, len(unique_wealths)-1) if unique_wealths[i+1] >= 1000.0))
			round_to_nearest_for['billionaires'] = min_difference # if a sub-billion dollar wealth is drawn, use the same rounding
			round_to_nearest_for['sub-billionaires'] = min_difference
		else:
			round_to_nearest_for['billionaires'] = min(list(unique_wealths[i] - unique_wealths[i+1] for i in range(0, len(unique_wealths)-1) if unique_wealths[i+1] >= 1000.0))
			round_to_nearest_for['sub-billionaires'] = min(list(unique_wealths[i] - unique_wealths[i+1] for i in range(0, len(unique_wealths)-1) if unique_wealths[i] < 1000.0))
		## k-s like test with rounding and other measurement errors against unbounded pareto ##
		# get discrepancy statistic
		numpy.random.seed(250624)
		k_s_like_test = k_s_like_test_against_unknown_pareto_distribution(wealths)
		discrepancy_statistic = k_s_like_test.get_discrepancy_statistic()
		k_s_like_test_against_w_rounding_and_other_measurement_errors['pareto'][year]['D stat'] = discrepancy_statistic
		# resample discrepancy statistic
		numpy.random.seed(250624)
		resample_d_stats = resample_d_stats_for_unknown_pareto_distribution(wealths=wealths, round_to_nearest_for=round_to_nearest_for, number_of_resamples=number_of_resamples, fitting=True, rounding=True, diffs_to_draw_from=diffs_btw['forbes and bloomberg 2013'])
		k_s_like_test_against_w_rounding_and_other_measurement_errors['pareto'][year]['p-value'] = resample_d_stats.get_p_value(discrepancy_statistic)
		#k_s_like_test_against_w_rounding_and_other_measurement_errors['pareto'][year]['D stats'] = resample_d_stats.resampled_discrepancy_statistics # de-comment to store D-stats		
		## k-s like test with rounding and other measurement errors against bounded pareto ##
		# get discrepancy statistic against bounded pareto distribution
		numpy.random.seed(250624)
		k_s_like_test = k_s_like_test_against_unknown_bounded_pareto_distribution(wealths=wealths)
		discrepancy_statistic = k_s_like_test.get_discrepancy_statistic()
		k_s_like_test_against_w_rounding_and_other_measurement_errors['bounded pareto'][year]['D stat'] = discrepancy_statistic	
		# resample discrepancy statistic
		numpy.random.seed(250624)
		resample_d_stats = resample_d_stats_for_unknown_bounded_pareto_distribution(wealths=wealths, round_to_nearest_for=round_to_nearest_for, number_of_resamples=number_of_resamples, diffs_to_draw_from=diffs_btw['forbes and bloomberg 2013']) # always rounding and fitting
		k_s_like_test_against_w_rounding_and_other_measurement_errors['bounded pareto'][year]['p-value'] = resample_d_stats.get_p_value(discrepancy_statistic)
		#k_s_like_test_against_w_rounding_and_other_measurement_errors['bounded pareto'][year]['D stats'] = resample_d_stats.resampled_discrepancy_statistics # de-comment to store D-stats
		print year
	# dump results
	pickle.dump(k_s_like_test_against_w_rounding_and_other_measurement_errors, open(filename, 'wb'))
else:
	# load result
	k_s_like_test_against_w_rounding_and_other_measurement_errors = pickle.load(open(filename, 'rb'))

# print results
distribution = 'pareto'
print "\nresults of k-s test for " + distribution + "\nt-stat \tp-value accounting for rounding \tp-value accounting for rounding and other measurement errors"
for year in years:
	_ = k_s_like_test_against_w_rounding_errors['pareto'][year]
	print year, "\t", _['D stat'], "\t", _['p-value'], "\t", 
	print k_s_like_test_against_w_rounding_and_other_measurement_errors['pareto'][year]['p-value']
	
## CLASSES FOR FITTING A TRUNCATED LOG-NORMAL AND COMPARING ITS FIT TO THAT OF A PARETO ##

class fit_truncated_log_normal_distribution_to(object):
	""" Class for getting the maximum likelihood estimates of the parameters of a two-parameter log-normal distribution truncated from below at a known truncation point """

	def __init__(self, x, truncated_from_below_at):
		self.x = x # the data
		self.truncated_from_below_at = truncated_from_below_at # the truncation point
		self.converged = False # optimization method converged
		
	def pdf_at(self, xi, parameters):
		""" Returns the PDF at a given xi of a log-normal with given mean and stdev parameters """
		# unpack the parameters
		mean, stdev = parameters
		# return the pdf evaluated at xi
		return (1. / (xi * numpy.sqrt(2. * numpy.pi * (stdev ** 2.)))) * numpy.exp((- (numpy.log(xi) - mean) ** 2.) / (2. * (stdev ** 2.)))

	def cdf_at(self, xi, parameters):
		""" Returns the CDF at a given xi of a log-normal with given mean and stdev parameters """
		# unpack the parameters
		mean, stdev = parameters
		# return the cdf evaluated at xi
		return 0.5 * (1. + erf((numpy.log(xi) - mean) / numpy.sqrt(2. * (stdev ** 2.))))

	def negative_log_likelihood_function(self, parameters):
		""" Returns the negative log-likelihood function for given data and parameters """
		# create a variable for the sum of contributions to the negative log likelihood function
		sum_of_contributions_to_log_likelihood_function = 0.
		# create scaling factor
		ln_of_1_minus_cdf_at_truncation_point = numpy.log((1. - self.cdf_at(self.truncated_from_below_at, parameters)))
		# sum the contributions
		for xi in self.x:
			sum_of_contributions_to_log_likelihood_function += numpy.log(self.pdf_at(xi, parameters)) - ln_of_1_minus_cdf_at_truncation_point
		# return the (negative of the) sum of contributions
		return - sum_of_contributions_to_log_likelihood_function
	
	def partial_derivative_of_ln_of_pdf_wrt_mean_at(self, xi, parameters):
		mean, stdev = parameters
		return (numpy.log(xi) - mean) / (stdev ** 2.)
	
	def partial_derivative_of_ln_of_pdf_wrt_stdev_at(self, xi, parameters):
		mean, stdev = parameters
		return (- 1. / stdev) + (((numpy.log(xi) - mean) ** 2.) / (stdev ** 2.))
	
	def partial_derivative_of_cdf_wrt_mean_at(self, xi, parameters):
		mean, stdev = parameters
		return (- 1. / numpy.sqrt(2 * numpy.pi * (stdev ** 2.))) * numpy.exp((- (numpy.log(xi) - mean) ** 2.) / (2. * (stdev ** 2.)))
	
	def partial_derivative_of_cdf_wrt_stdev_at(self, xi, parameters):
		mean, stdev = parameters
		return (- (numpy.log(xi) - mean) / ((stdev ** 2.) * numpy.sqrt(2. * numpy.pi))) * numpy.exp((- (numpy.log(xi) - mean) ** 2.) / (2. * (stdev ** 2.)))
		
	def gradient_of_negative_log_likelihood_function(self, parameters):	
		""" Returns the gradient of the negative log-likelihood function """
		# create variables for the components of the gradient
		partial_derivative_wrt_mean = 0.
		partial_derivative_wrt_stdev = 0.
		# create scaling factors
		cdf_at_truncation_point = self.cdf_at(self.truncated_from_below_at, parameters)
		partial_derivative_of_cdf_wrt_mean_at_truncation_point = self.partial_derivative_of_cdf_wrt_mean_at(self.truncated_from_below_at, parameters)
		partial_derivative_of_cdf_wrt_stdev_at_truncation_point = self.partial_derivative_of_cdf_wrt_stdev_at(self.truncated_from_below_at, parameters)
		# sum the contributions
		for xi in self.x:
			partial_derivative_wrt_mean += self.partial_derivative_of_ln_of_pdf_wrt_mean_at(xi, parameters) + (1. / cdf_at_truncation_point) * partial_derivative_of_cdf_wrt_mean_at_truncation_point
			partial_derivative_wrt_stdev += self.partial_derivative_of_ln_of_pdf_wrt_stdev_at(xi, parameters) + (1. / cdf_at_truncation_point) * partial_derivative_of_cdf_wrt_stdev_at_truncation_point
		# return the (negative of the) components of the gradient
		return numpy.array([- partial_derivative_wrt_mean, - partial_derivative_wrt_stdev])

	def get_initial_guess(self):
		""" Returns an initial guess for the parameters based on the MLEs of an untruncated log-normal """
		mean0 = (1. / len(self.x)) * sum([numpy.log(xi) for xi in self.x])
		stdev0 = numpy.sqrt((1. / len(self.x)) * sum([(numpy.log(xi) - mean0) ** 2. for xi in self.x]))
		return mean0, stdev0
	
	def get_estimated_parameters(self):
		""" Returns the MLEs of the parameters """
		# if optimization fails to converge, try a new guess
		tries = 0
		fmins = list()
		initial_guess = self.get_initial_guess()		
		while (self.converged == False) and (tries < 100+1):
			guess = numpy.random.uniform(initial_guess[0] / 2., initial_guess[0] * 2.), numpy.random.uniform(initial_guess[1] / 2., initial_guess[1] * 2.) # take anywhere between half and twice the initial guess
			fmin = scipy.optimize.fmin_tnc(func=self.negative_log_likelihood_function, fprime=self.gradient_of_negative_log_likelihood_function, bounds=[(None, None), (0.+10**-100, None)], x0=guess, maxfun=100, messages=0) # 8 is "Exit reasons"
			parameters = (fmin[0][0], fmin[0][1])
			negative_log_likelihood = self.negative_log_likelihood_function(parameters)
			converged = (fmin[2] == 1) # function converged
			if numpy.isnan(negative_log_likelihood) or numpy.isinf(negative_log_likelihood):
				pass
			else:
				fmins.append((negative_log_likelihood, parameters, converged))
				fmins.sort()
				self.estimated_parameters = fmins[0][1]
				self.converged = fmins[0][2]
			tries += 1
			if len(fmins) == 0:
				tries -= 1 # try again, if only found log-likelihood of NaN or +/Infinity
			if (self.converged == True) and (tries < 10+1):
				self.converged = False # try at least ten starting points
		return self.estimated_parameters

	def get_converged(self):
		return self.converged
	
	def get_std_errors_of_estimated_parameters(self):
		""" Returns std errors of the estimated mean and stdev parameters """
		# following gretl, standard errors are based on the outer product of the gradient
		G = list()
		# create scaling factors
		cdf_at_truncation_point = self.cdf_at(self.truncated_from_below_at, self.estimated_parameters)
		partial_derivative_of_cdf_wrt_mean_at_truncation_point = self.partial_derivative_of_cdf_wrt_mean_at(self.truncated_from_below_at, self.estimated_parameters)
		partial_derivative_of_cdf_wrt_stdev_at_truncation_point = self.partial_derivative_of_cdf_wrt_stdev_at(self.truncated_from_below_at, self.estimated_parameters)
		# sum the contributions
		for xi in self.x:
			partial_derivative_of_ln_of_pdf_wrt_mean_at_xi = self.partial_derivative_of_ln_of_pdf_wrt_mean_at(xi, self.estimated_parameters) + (1. / cdf_at_truncation_point) * partial_derivative_of_cdf_wrt_mean_at_truncation_point
			partial_derivative_of_ln_of_pdf_wrt_stdev_at_xi = self.partial_derivative_of_ln_of_pdf_wrt_stdev_at(xi, self.estimated_parameters) + (1. / cdf_at_truncation_point) * partial_derivative_of_cdf_wrt_stdev_at_truncation_point
			G.append([- partial_derivative_of_ln_of_pdf_wrt_mean_at_xi, - partial_derivative_of_ln_of_pdf_wrt_stdev_at_xi])
		G = numpy.matrix(G)
		VCV = numpy.array((G.T * G).I)
		return numpy.sqrt(VCV[0][0]), numpy.sqrt(VCV[1][1])

class vuong_test_for_pareto_against_truncated_log_normal_distribution(object):
	""" Class for a Vuong (1989) test of a two-parameter Pareto distribution against a truncated log-normal distribution. Follows Cameron and Trivedi (2006, sec. 8.5.3--5) """

	def __init__(self, wealths, truncated_from_below_at):
		self.wealths = wealths
		self.truncated_from_below_at = truncated_from_below_at # for truncated log-normal
		self.fit_distributions()
		self.n = len(wealths)
	
	def fit_distributions(self):
		# fit parteo
		fit_to_pareto = fit_pareto_to(wealths=self.wealths)
		self.estimated_parameters_for_pareto = fit_to_pareto.get_estimated_parameters()
		# fit log normal
		self.fit_to_truncated_log_normal = fit_truncated_log_normal_distribution_to(x=self.wealths, truncated_from_below_at=self.truncated_from_below_at)
		self.estimated_parameters_for_truncated_log_normal = self.fit_to_truncated_log_normal.get_estimated_parameters()
	
	def get_log_likelihood_at_for_fitted_pareto(self, wealth):
		lower_bound, shape = self.estimated_parameters_for_pareto # unpack parameters
		return numpy.log(shape * (lower_bound ** shape) * (wealth ** (- 1. - shape)))
	
	def get_log_likelihood_at_for_fitted_truncated_log_normal(self, wealth):
		ln_of_1_minus_cdf_at_truncation_point = numpy.log((1. - self.fit_to_truncated_log_normal.cdf_at(self.truncated_from_below_at, self.estimated_parameters_for_truncated_log_normal)))
		return numpy.log(self.fit_to_truncated_log_normal.pdf_at(xi=wealth, parameters=self.estimated_parameters_for_truncated_log_normal)) - ln_of_1_minus_cdf_at_truncation_point
	
	def get_test_statistic(self):
		""" Returns the test statistic """
		likelihood_ratios = list(self.get_log_likelihood_at_for_fitted_pareto(wealth=wealth) - self.get_log_likelihood_at_for_fitted_truncated_log_normal(wealth=wealth) for wealth in self.wealths)
		LR = sum(likelihood_ratios)
		omega_squared = (1. / self.n) * sum(list(likelihood_ratio ** 2. for likelihood_ratio in likelihood_ratios)) - ((1. / self.n) * LR) ** 2.
		return LR / numpy.sqrt(self.n * omega_squared)

## CLASSES FOR FITTING A TRUNCATED GAMMA AND COMPARING ITS FIT TO THAT OF A PARETO ##

class fit_truncated_gamma_distribution_to(object):
	""" Class for getting the maximum likelihood estimates of the parameters of a two-parameter gamma distribution truncated from below at a known truncation point """
	
	def __init__(self, x, truncated_from_below_at):
		self.x = x # the data
		self.truncated_from_below_at = truncated_from_below_at # the truncation point
		self.converged = False # optimization method converged

	def pdf_at(self, xi, parameters):
		# unpack the parameters
		shape, scale = parameters
		# return the pdf evaluated at xi
		return (1. / scipy.special.gamma(shape)) * (scale ** (- shape)) * (xi ** (shape - 1)) * numpy.exp(- xi / scale)

	def cdf_at(self, xi, parameters):
		# unpack the parameters
		shape, scale = parameters
		# return the cdf evaluated at xi
		return scipy.special.gammainc(shape, xi / scale)
		# note that scipy.special.gammainc is the regularized version of the lower incomplete gamma function
		
	def negative_log_likelihood_function(self, parameters):
		# create a variable for the sum of contributions to the negative log likelihood function
		sum_of_contributions_to_log_likelihood_function = 0.
		# create scaling factor, note that the scipy.special.gammaincc could be used in calculating the complement
		ln_of_1_minus_cdf_at_truncation_point = numpy.log((1. - self.cdf_at(self.truncated_from_below_at, parameters)))
		# sum the contributions
		for xi in self.x:
			sum_of_contributions_to_log_likelihood_function += numpy.log(self.pdf_at(xi, parameters)) - ln_of_1_minus_cdf_at_truncation_point		
		# return the (negative of the) sum of contributions
		return - sum_of_contributions_to_log_likelihood_function
	
	def partial_derivative_of_ln_of_pdf_wrt_shape_at(self, xi, parameters):
		shape, scale = parameters
		return numpy.log(xi) - numpy.log(scale) - scipy.special.psi(shape)
	
	def partial_derivative_of_ln_of_pdf_wrt_scale_at(self, xi, parameters):
		shape, scale = parameters
		return - (shape / scale) - (xi / scale)
	
	def partial_derivative_of_cdf_wrt_shape_at(self, xi, parameters):
		shape, scale = parameters
		return "TBD"
	
	def partial_derivative_of_cdf_wrt_scale_at(self, xi, parameters):
		shape, scale = parameters
		return - (numpy.exp(- x / scale) * ((x / scale) ** shape)) / (scale * scipy.special.gamma(shape))
	
	def gradient_of_negative_log_likelihood_function(self, parameters):	
		# create variables for the components of the gradient
		partial_derivative_wrt_shape = 0.
		partial_derivative_wrt_scale = 0.
		# create scaling factors
		cdf_at_truncation_point = self.cdf_at(self.truncated_from_below_at, parameters)
		partial_derivative_of_cdf_wrt_shape_at_truncation_point = self.partial_derivative_of_cdf_wrt_shape_at(self.truncated_from_below_at, parameters)
		partial_derivative_of_cdf_wrt_scale_at_truncation_point = self.partial_derivative_of_cdf_wrt_scale_at(self.truncated_from_below_at, parameters)
		# sum the contributions
		for xi in self.x:
			partial_derivative_wrt_shape += self.partial_derivative_of_ln_of_pdf_wrt_shape_at(xi, parameters) + (1. / cdf_at_truncation_point) * partial_derivative_of_cdf_wrt_shape_at_truncation_point
			partial_derivative_wrt_scale += self.partial_derivative_of_ln_of_pdf_wrt_scale_at(xi, parameters) + (1. / cdf_at_truncation_point) * partial_derivative_of_cdf_wrt_scale_at_truncation_point
		# return the (negative of the) components of the gradient
		return numpy.array([- partial_derivative_wrt_shape, - partial_derivative_wrt_scale])
	
	def get_initial_guess(self):
		""" Returns an initial guess for the parameters based on Thom's (1958) approximation to the MLEs for an untruncated gamma """
		arithmetic_mean = (1. / len(self.x)) * sum(self.x)
		d = numpy.log(arithmetic_mean) - (1. / len(self.x)) * sum(list(numpy.log(xi) for xi in self.x))
		shape0 = (1. + numpy.sqrt(1. + ((4. * d) / 3.))) / (4. * d)
		scale0 = arithmetic_mean / shape0
		return shape0, scale0
				
	def get_estimated_parameters(self):
		""" Returns the MLEs """
		tries = 0
		fmins = list()
		initial_guess = self.get_initial_guess()
		while (self.converged == False) and (tries < 100+1):
			guess = numpy.random.uniform(initial_guess[0] / 2., initial_guess[0] * 2.), numpy.random.uniform(initial_guess[1] / 2., initial_guess[1] * 2.)
			fmin = scipy.optimize.fmin_l_bfgs_b(func=self.negative_log_likelihood_function, x0=guess, fprime=None, approx_grad=True, bounds=[(0.+10**-100, None), (0.+10**-100, None)], maxfun=100)
			parameters = (fmin[0][0], fmin[0][1])
			negative_log_likelihood = float(fmin[1])
			converged = (fmin[2]['warnflag'] == 0)
			if numpy.isnan(negative_log_likelihood) or numpy.isinf(negative_log_likelihood):
				pass
			else:
				fmins.append((negative_log_likelihood, parameters, converged))
				fmins.sort()
				self.estimated_parameters = fmins[0][1]
				self.converged = fmins[0][2]
			tries += 1
			if len(fmins) == 0:
				tries -= 1 # try again, if only found log-likelihood of NaN or +/Inf
			if (self.converged == True) and (tries < 10+1):
				self.converged = False # try at least ten starting points
		return self.estimated_parameters	
	
	def get_converged(self):
		return self.converged

class vuong_test_for_pareto_against_truncated_gamma_distribution(object):
	""" Class for a Vuong (1989) test of of a two-parameter Pareto distribution against a left-truncated gamma distribution """
	
	def __init__(self, wealths, truncated_from_below_at):
		self.wealths = wealths
		self.truncated_from_below_at = truncated_from_below_at # for truncated gamma
		self.fit_distributions()
		self.n = len(wealths)
	
	def fit_distributions(self):
		# fit parteo
		fit_to_pareto = fit_pareto_to(wealths=self.wealths)
		self.estimated_parameters_for_pareto = fit_to_pareto.get_estimated_parameters()
		# fit truncated gamma
		self.fit_to_truncated_gamma = fit_truncated_gamma_distribution_to(x=self.wealths, truncated_from_below_at=self.truncated_from_below_at)
		self.estimated_parameters_for_truncated_gamma = self.fit_to_truncated_gamma.get_estimated_parameters()
	
	def get_log_likelihood_at_for_fitted_pareto(self, wealth):
		lower_bound, shape = self.estimated_parameters_for_pareto # unpack parameters
		return numpy.log(shape * (lower_bound ** shape) * (wealth ** (- 1. - shape)))
	
	def get_log_likelihood_at_for_fitted_truncated_gamma(self, wealth):
		ln_of_1_minus_cdf_at_truncation_point = numpy.log((1. - self.fit_to_truncated_gamma.cdf_at(xi=self.truncated_from_below_at, parameters=self.estimated_parameters_for_truncated_gamma)))
		return numpy.log(self.fit_to_truncated_gamma.pdf_at(xi=wealth, parameters=self.estimated_parameters_for_truncated_gamma)) - ln_of_1_minus_cdf_at_truncation_point
	
	def get_test_statistic(self):
		likelihood_ratios = list(self.get_log_likelihood_at_for_fitted_pareto(wealth=wealth) - self.get_log_likelihood_at_for_fitted_truncated_gamma(wealth=wealth) for wealth in self.wealths)
		LR = sum(likelihood_ratios)
		omega_squared = (1. / self.n) * sum(list(likelihood_ratio ** 2. for likelihood_ratio in likelihood_ratios)) - ((1. / self.n) * LR) ** 2.
		return LR / numpy.sqrt(self.n * omega_squared)

## CLASS FOR COMPARING FIT OF BOUNDED AND UNBOUNDED PARETOS ##

class lr_test_for_pareto_against_bounded_pareto_distribution(object):
	""" Class for a likelihood ratio test of an unbounded Pareto distribution against a bounded one. A standard likelihood ratio test can be applied because the former is nested in the latter. The two distributions are the same as the bounding parameter goes to infinity """
	
	def __init__(self, wealths):
		self.wealths = wealths
		self.fit_distributions()
	
	def fit_distributions(self):
		# fit parteo
		fit_to_pareto = fit_pareto_to(wealths=self.wealths)
		self.estimated_parameters_for_pareto = fit_to_pareto.get_estimated_parameters()
		# fit bounded pareto
		fit_to_bounded_pareto = fit_bounded_pareto_to(wealths=self.wealths)
		self.estimated_parameters_for_bounded_pareto = fit_to_bounded_pareto.get_estimated_parameters()
	
	def get_log_likelihood_at_for_fitted_pareto(self, wealth):
		lower_bound, shape = self.estimated_parameters_for_pareto # unpack parameters
		return numpy.log(shape * (lower_bound ** shape) * (wealth ** (- 1. - shape)))

	def get_log_likelihood_at_for_fitted_bounded_pareto(self, wealth):
		lower_bound, upper_bound, shape = self.estimated_parameters_for_bounded_pareto # unpack parameters
		return numpy.log((shape * (lower_bound ** shape) * (wealth ** (- 1. - shape))) / (1. - (lower_bound / upper_bound) ** shape))
	
	def get_test_statistic(self):
		""" Returns the test statistic """
		likelihood_for_pareto = sum(list(self.get_log_likelihood_at_for_fitted_pareto(wealth=wealth) for wealth in self.wealths))
		likelihood_for_bounded_pareto = sum(list(self.get_log_likelihood_at_for_fitted_bounded_pareto(wealth=wealth) for wealth in self.wealths))
		return -2. * (likelihood_for_pareto - likelihood_for_bounded_pareto)

## LIKELIHOOD-RATIO TESTS OF PARETO AGAINST OTHER DISTRIBUTIONS ##

def get_p_value(z_stat):
	""" Returns p-value for z-test """
	return scipy.stats.norm.cdf(-abs(z_stat), loc=0, scale=1) + scipy.stats.norm.sf(abs(z_stat), loc=0, scale=1)

# define dicts for storing results of tests
other_distributions = ['gamma', 'log-normal', 'bounded pareto']
lr_test_of_pareto_against = dict(zip(other_distributions, [dict(zip(years, [dict(zip(['t-stat', 'p-value'], ['TBD' for _ in ['t-stat', 'p-value']])) for year in years])) for distribution in other_distributions]))
# set the following to True to re-estimate if already estimate
re_estimate = False
# check if already been estimated
filename = filepath + "lr_test_of_pareto_against.pkl"
already_estimated = isfile(filename)
# estimate or re-estimate
if not already_estimated or re_estimate:
	# for each year
	for year in years:
		# get wealths
		wealths = summary_stats['wealths'][year]
		# get rounding
		round_to_nearest_for = dict()
		unique_wealths = list(set(wealths))
		unique_wealths.sort(reverse=True)
		# if wealthiest person isn't a billionaire
		if summary_stats['max'][year] < 1000.0:
			min_difference = min(list(unique_wealths[i] - unique_wealths[i+1] for i in range(0, len(unique_wealths)-1) if unique_wealths[i] < 1000.0))
			round_to_nearest_for['billionaires'] = min_difference # if a billion dollar wealth is drawn, use the same rounding
			round_to_nearest_for['sub-billionaires'] = min_difference
		# if poorest person is a billionaire
		elif summary_stats['min'][year] >= 1000.0:
			min_difference = min(list(unique_wealths[i] - unique_wealths[i+1] for i in range(0, len(unique_wealths)-1) if unique_wealths[i+1] >= 1000.0))
			round_to_nearest_for['billionaires'] = min_difference # if a sub-billion dollar wealth is drawn, use the same rounding
			round_to_nearest_for['sub-billionaires'] = min_difference
		else:
			round_to_nearest_for['billionaires'] = min(list(unique_wealths[i] - unique_wealths[i+1] for i in range(0, len(unique_wealths)-1) if unique_wealths[i+1] >= 1000.0))
			round_to_nearest_for['sub-billionaires'] = min(list(unique_wealths[i] - unique_wealths[i+1] for i in range(0, len(unique_wealths)-1) if unique_wealths[i] < 1000.0))
		# get truncation point
		truncated_from_below_at = summary_stats['min'][year] - (round_to_nearest_for['sub-billionaires'] / 2.)
		## test pareto against gamma ##
		numpy.random.seed(250624)
		vuong_test = vuong_test_for_pareto_against_truncated_gamma_distribution(wealths=wealths, truncated_from_below_at=truncated_from_below_at)
		test_statistic = vuong_test.get_test_statistic()
		lr_test_of_pareto_against['gamma'][year]['t-stat'] = test_statistic
		lr_test_of_pareto_against['gamma'][year]['p-value'] = get_p_value(z_stat=test_statistic)
		## test pareto against truncated log-normal ##
		numpy.random.seed(250624)
		vuong_test = vuong_test_for_pareto_against_truncated_log_normal_distribution(wealths=wealths, truncated_from_below_at=truncated_from_below_at)
		test_statistic = vuong_test.get_test_statistic()
		lr_test_of_pareto_against['log-normal'][year]['t-stat'] = test_statistic
		lr_test_of_pareto_against['log-normal'][year]['p-value'] = get_p_value(z_stat=test_statistic)
		## test pareto against bounded pareto ##
		# note that the unbounded pareto is nested in the bounded pareto
		numpy.random.seed(250624)
		lr_test = lr_test_for_pareto_against_bounded_pareto_distribution(wealths=wealths)
		test_statistic = lr_test.get_test_statistic()
		lr_test_of_pareto_against['bounded pareto'][year]['t-stat'] = test_statistic
		lr_test_of_pareto_against['bounded pareto'][year]['p-value'] = 1. - scipy.stats.chi2.cdf(test_statistic, 1) # chi-squared with one degree of freedom
		print year
	# dump results
	pickle.dump(lr_test_of_pareto_against, open(filename, 'wb'))
else:
	# load result
	lr_test_of_pareto_against = pickle.load(open(filename, 'rb'))

# print results
print "\nresults of LR tests"
for distribution in other_distributions:
	# intepretation of Vuong (1989) tests
	if distribution not in ['bounded pareto']:
		n_years_favors_dist = int(sum([lr_test_of_pareto_against[distribution][year]['t-stat'] > 0. and lr_test_of_pareto_against[distribution][year]['p-value'] < 0.10 for year in years]))
		n_years_favors_pareto = int(sum([lr_test_of_pareto_against[distribution][year]['t-stat'] < 0. and lr_test_of_pareto_against[distribution][year]['p-value'] < 0.10 for year in years]))
		neither_favored = len(years) - (n_years_favors_dist + n_years_favors_pareto)
	# intepretation of standard LR tests
	else:
		n_years_favors_dist = int(sum([lr_test_of_pareto_against[distribution][year]['p-value'] < 0.10 for year in years]))
		n_years_favors_pareto = len(years)
	print "A LR test favors a " + distribution + " over an unbounded pareto in " + str(n_years_favors_dist) + " years.", 
	if distribution not in ['bounded pareto']:
		print "The same test favors the pareto over the " + distribution + " in " + str(n_years_favors_pareto) + " years.", 
		print "Neither is favored over the other in " + str(neither_favored) + " years." 

#lastline=1225
#### FIGURES ####
if __name__ == '__main__':

	import pylab
	from matplotlib import rc
	from matplotlib import lines
	from matplotlib.ticker import MultipleLocator

	# size of figures
	figwidth = 430.00627 / 72.27 # \textwidth in pts times pts per inch, following cookbook example 
	figheight = figwidth * (numpy.sqrt(5)-1.0) / 2.0 # golden ratio, following cookbook example
	figsize = (figwidth, figheight)

	rc('text', usetex=True)
	#rc('font',**{'family':'sans-serif','sans-serif':['Computer Modern Sans Serif']}) # for beamer
	#rc('font', family='sans-serif')
	rc('font',**{'family':'serif','serif':['Computer Modern Roman']}) # for document
	rc('font', family='serif')
	fontsize = 11
	pylab.rcParams['font.size'] = fontsize
	pylab.rcParams['axes.labelsize'] = fontsize
	pylab.rcParams['legend.fontsize'] = fontsize
	pylab.rcParams['xtick.labelsize'] = fontsize
	pylab.rcParams['ytick.labelsize'] = fontsize

	## FIG. 3.1: WEALTH DISTRIBUTION IN 2003 ##

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=figsize)
	fig_ax = fig.gca()
	# draw empirical CCDF for wealths in the year 2003
	fig_ax.plot(list(wealth / 1000.0 for wealth in wealths_in_2003), empirical_ccdf_at_for_2003, color='k', linestyle='steps-post', lw=0.5, zorder=1, clip_on=True, label='Empirical')
	# draw predicted CCDF for Pareto distribution estimated from wealths in the year 2003
	fig_ax.plot(list(wealth / 1000.0 for wealth in wealths_in_2003), fitted_ccdf_for_2003, color='k', linestyle='dashed', lw=0.5, zorder=2, clip_on=True, label='Fitted')
	# draw the largest difference between the empirical and fitted CCDFs
	fig_ax.plot([wealth_at_max_discrepancy_for_2003 / 1000.0, wealth_at_max_discrepancy_for_2003 / 1000.0], [probs_at_max_discrepancy_for_2003[0], probs_at_max_discrepancy_for_2003[-1]], color='k', linestyle='solid', lw=0.5, marker='o', markersize=3, zorder=3, label='Largest difference')
	# make a legend
	leg = fig_ax.legend(loc='upper right', frameon=False, handlelength=3.3, handletextpad=0.5)
	# log the axes
	fig_ax.set_xscale('log')
	fig_ax.set_yscale('log')
	# label the axes
	fig_ax.set_xlabel('Wealth (billions of 2003 dollars)')
	fig_ax.set_ylabel('Cumulative probability')
	# set limits
	pylab.ylim(0.0025, 1)
	pylab.xlim(0.6, 46)
	# add ticks
	pylab.yticks([0.0025, 10**-2, 10**-1, 1], ['0.0025', '0.01', '0.1', '1'])
	pylab.xticks([0.6, 1, 10, 46], ['0.6', '1', '10', '46'])
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# adjust dimensions
	fig.subplots_adjust(left=0.19, bottom=0.14, right=0.96, top=0.96, wspace=0.0, hspace=0.0)
	# save figure
	pylab.savefig("./figures/figures_for_second_essay/wealth_distribution_in_2003.pdf", bbox_inches='tight')

	## FIG 3.2: PARETO INDEX BY YEAR ##

	def get_ci(point, se, confidence_level=0.95):
		""" Returns lower and upper confidence intervals based on a Normal distribution """
		z = scipy.stats.norm.ppf(1.0 - ((1.0 - confidence_level) / 2.0))
		return point - z * se, point + z * se

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=figsize)
	fig_ax = fig.gca()
	# plot point estimates of pareto indexes
	fig_ax.plot([int(year) for year in years], list(estimated_pareto_index_in[year]['point'] for year in years), color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True) # no markers if more than 10 data points
	# plot confidence intervals
	lowers, uppers = list(), list()
	for year in years:
		point, se = estimated_pareto_index_in[year]['point'], estimated_pareto_index_in[year]['se']
		lower, upper = get_ci(point, se, confidence_level=0.95)
		lowers.append(lower)
		uppers.append(upper)
	fig_ax.plot([int(year) for year in years], lowers, color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label='95\% CI')
	fig_ax.plot([int(year) for year in years], uppers, color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True)
	# make a legend
	leg = fig_ax.legend(loc='upper right', frameon=False, handlelength=3.3, handletextpad=0.5)
	# label the axes
	fig_ax.set_xlabel('Year')
	fig_ax.set_ylabel('Pareto index') # sentence-style capitalization
	# set limits
	pylab.xlim(1982, 2013)
	pylab.ylim(0.9, 2.1)
	# major xticks
	pylab.xticks([1982, 1992, 2002, 2012], ['1982', '1992', '2002', '2012'])
	# minor xticks
	fig_ax.xaxis.set_minor_locator(MultipleLocator(1)) # tick every year
	# add y ticks
	pylab.yticks([1.0, 1.5, 2.0], ['1.0', '1.5', '2.0'])
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# adjust dimensions
	fig.subplots_adjust(left=0.12, bottom=0.14, right=0.95, top=0.96, wspace=0.0, hspace=0.0)
	# save figure
	pylab.savefig("./figures/figures_for_second_essay/pareto_index.pdf", bbox_inches='tight')

	## FIG. 3.3: BLOOMBERG VS. FORBES ##

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=figsize)
	fig_ax = fig.gca()
	# 2013 diffs
	fig_ax.plot(xrange_for_bloomberg_forbes_diffs, [kde_for_bloomberg_and_forbes_2013_diffs_at[xi] for xi in xrange_for_bloomberg_forbes_diffs], color='k', linestyle='solid', lw=0.5, zorder=1, clip_on=True, label='2013')
	# 2012 diffs
	fig_ax.plot(xrange_for_bloomberg_forbes_diffs, [kde_for_bloomberg_and_forbes_2012_diffs_at[xi] for xi in xrange_for_bloomberg_forbes_diffs], color='k', linestyle='dashed', lw=0.5, zorder=2, clip_on=True, label='2012')
	# make a legend
	leg = fig_ax.legend(loc='upper right', frameon=False, handlelength=3.3, handletextpad=0.5)
	# label the axes
	fig_ax.set_xlabel("Ratio between estimates of a person's wealth")
	fig_ax.set_ylabel('Probability density')
	# set limits
	pylab.xlim(0.5, 1.5)
	pylab.xticks([0.5, 1., 1.5], ['0.5', '1.0', '1.5'])
	pylab.ylim(0., 3.25)
	pylab.yticks([0., 1.5, 3.], ['0', '1.5', '3.0'])
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# save figure
	pylab.savefig("./figures/figures_for_second_essay/bloomberg_vs_forbes_diffs.pdf", bbox_inches='tight')

	#### TABLES ####

	def get_note(p_value):
		""" Returns LaTeX note for p-value """
		note = "\\tmark[$^{"
		if p_value < 10. / 100.0:
			note += '*'
		else:
			pass
		if p_value < 5. / 100.0:
			note += '*'
		else:
			pass
		if p_value < 1. / 100.0:
			note += '*'
		else:
			pass
		note += "}$]"
		return note

	## TABLE 3.1: PARETO INDEXES USING KLASS ET AL.'S (2006) DATA AND METHODS ##

	# uses packages ctable and dcolumn
	f = open("./tables/tables_specific_to_second_essay/replication_of_klass_et_al_2006.txt", 'w') # write to this file
	f.write("{\ssp\n") # single-spacing
	f.write("\ctable[\n")
	f.write("\tcaption\t= {Pareto Indexes for Some of the Wealthiest Americans, 1988--2003},\n") # sentence-style without terminal period (Turabian 2007, sec. 26.2.2)
	f.write("\tlabel\t= replication_of_klass_et_al_2006,\n")
	f.write("\tpos\t= tp,\n") # top or page by itself
	f.write("\tnotespar,\n") # footnotes must be flush left (Turabian 2007, sec. 26.2.7)
	f.write("\tnosuper,\n") # typeset sources and notes on the line, manually superscript everything else
	f.write("\tmincapwidth\t= 1.0\\textwidth,\n") # make the table caption and footnotes as wide as the text
	f.write("\t]{lc}{\n") # first column is flush left, other columns are centered
	f.write("\\tnote[Source:~\\textnormal{Data adapted from \\citet{klass.et.al.-2006}.}]{\\\} \n") # terminal period
	f.write("\\tnote[]{\\\} % for spacing\n")
	f.write("\\tnote[Note:~\\textnormal{This table shows, for the years 1988 to 2003, a point estimate for the shape parameter of a Pareto distribution fitted to the distribution of wealth among some of the wealthiest Americans in \citeauthor{klass.et.al.-2006}'s (\citeyear{klass.et.al.-2006}) dataset in a given year. Following \citet{klass.et.al.-2006}, the estimates are based on log-log rank-wealth regressions for Americans who ranked between the 10th and 400th wealthiest in a given year.}]{} \n")
	f.write("}{\FL[\lightrulewidth]\n")
	f.write("Year\t&\t\multicolumn{1}{c}{Pareto index}\t\ML\n") # sentence style
	for year in select_years:
		s = year + '\t&\t'
		s += str(format(round(estimated_pareto_index_using_klass_et_al_methods_in[year]['with errors']['point'], 2), '.2f'))
		if year != '2003':
			s += '\t\NN\n'
		else:
			s += '\t\LL[\lightrulewidth]\n'
		f.write(s)
	f.write("\NN % for spacing\n") # for spacing
	f.write("}\n") # end ctable
	f.write("}\n") # end single-spacing
	f.close()

	## TABLE 3.2: EFFECT OF ERRORS IN KLASS ET AL.'S (2006) DATA ##

	# uses packages ctable and dcolumn
	f = open("./tables/tables_specific_to_second_essay/errors_of_klass_et_al_2006.txt", 'w') # write to this file
	f.write("{\ssp\n") # single-spacing
	f.write("\ctable[\n") # ctable
	f.write("\tcaption\t= {Effect of Errors in \citeauthor{klass.et.al.-2006}'s (\citeyear{klass.et.al.-2006}) Dataset on the Pareto Index for Some of the Wealthiest Americans, 1988--2003},\n") # sentence-style without terminal period (Turabian 2007, sec. 26.2.2)
	f.write("\tlabel\t= errors_of_klass_et_al_2006,\n")
	f.write("\tpos\t= H,\n") # top or page by itself
	f.write("\tnotespar,\n") # footnotes must be flush left (Turabian 2007, sec. 26.2.7)
	f.write("\tnosuper,\n") # typeset sources and notes on the line, manually superscript everything else
	f.write("\tmincapwidth\t= 1.0\\textwidth,\n") # make the table caption and footnotes as wide as the text
	f.write("\t]{lccc}{\n") # first column is flush left, other columns are centered
	f.write("\\tnote[Sources:~\\textnormal{Data adapted from \\textit{Forbes Magazine} (1988--2003); \citet{klass.et.al.-2006}.}]{\\\} \n")
	f.write("\\tnote[]{\\\} % for spacing\n")
	f.write("\\tnote[Note:~\\textnormal{This table shows, for years in which errors were identified in \citeauthor{klass.et.al.-2006}'s (\citeyear {klass.et.al.-2006}) dataset, a Pareto index estimated from their dataset with the errors, a Pareto index estimated from a dataset without the errors, and the difference between the two estimated indexes. Following \citet{klass.et.al.-2006}, the point estimates for the Pareto indexes are based on log-log rank-wealth regressions for Americans who ranked between the 10th and 400th wealthiest in their dataset in a given year. Standard errors are reported in parentheses. The standard error of an estimated index was calculated in the manner suggested by \citet{gabaix.ibragimov-2011}, while the standard error of the difference between two indexes was calculated as the squared sum of the variances of each index. None of the differences are statistically significant at conventional levels.}]{} \n")
	f.write("}{\FL[\lightrulewidth]\n")
	f.write("\t&\t\multicolumn{2}{c}{Pareto index}\t\NN\n")
	f.write("\cmidrule[\lightrulewidth](rl){2-3}\n") # rule 
	f.write("Year\t&\t\multicolumn{1}{c}{With errors}\t&\t\multicolumn{1}{c}{Without errors}\t&\t\multicolumn{1}{c}{Difference}\t\ML\n")
	for year in select_years:
		s = year + '\t&\t'
		if year in ['1988', '1989', '1993', '1994', '1996', '1999']: # years with errors
			for w_or_wo in ['with errors', 'without errors']:
				s += str(format(round(estimated_pareto_index_using_klass_et_al_methods_in[year][w_or_wo]['point'], 2), '.2f'))
				s += ' '
				s += '(' + str(format(round(estimated_pareto_index_using_klass_et_al_methods_in[year][w_or_wo]['se'], 2), '.2f')) + ')'
				s += '\t&\t'
			diff = estimated_pareto_index_using_klass_et_al_methods_in[year]['with errors']['point'] - estimated_pareto_index_using_klass_et_al_methods_in[year]['without errors']['point']
			if not diff > 0.:
				s += '$-$' + str(format(round(abs(diff), 2), '.2f'))
			else:
				s += str(format(round(diff, 2), '.2f'))
			se = numpy.sqrt(estimated_pareto_index_using_klass_et_al_methods_in[year]['with errors']['se'] ** 2. + estimated_pareto_index_using_klass_et_al_methods_in[year]['without errors']['se'] ** 2.) # the se of the difference can be estimated by the square of the sum of the variances
			z_stat = diff / se
			p_value = get_p_value(z_stat)
			s += get_note(p_value)
			s += ' '
			s += '(' + str(format(round(se, 2), '.2f')) + ')'
			if not diff > 0.:
				s += '{\color{white}$-$}'
		if year != '2003':
			s += '\t\NN\n'
		else:
			s += '\t\LL[\lightrulewidth]\n'
		f.write(s)
	f.write("\NN % for spacing\n") # for spacing
	f.write("}\n") # end ctable
	f.write("}\n") # end single-spacing
	f.close()

	## TABLE 3.4: GOODNESS-OF-FIT TEST AGAINST UNBOUNDED PARETO ##

	# export goodness-of-fit tests, just for pareto distribution, with rounding errors, with or without other measurement errors
	# uses packages ctable and dcolumn
	f = open("./tables/tables_specific_to_second_essay/goodness_of_fit_w_measurement_errors.txt", 'w') # write to this file
	f.write("{\ssp\n") # single-spacing
	f.write("\ctable[\n")
	f.write("\tcaption\t= {Goodness-of-fit Tests Against a Pareto Distribution, 1982--2013},\n") # sentence-style without terminal period
	f.write("\tlabel\t= goodness_of_fit_w_measurement_errors,\n")
	f.write("\tpos\t= H,\n") # top or page by itself
	f.write("\tnotespar,\n") # footnotes must be flush left (Turabian 2007, sec. 26.2.7)
	f.write("\tnosuper,\n") # typeset sources and notes on the line, manually superscript everything else
	f.write("\tmincapwidth\t= 1.0\\textwidth,\n") # make the table caption and footnotes as wide as the text
	f.write("\tdoinside\t=\\small,\n") # table is too big for page, so shrink text size
	f.write("\t]{lD{.}{.}{-2}D{.}{.}{-2}D{.}{.}{-2}}{\n") # first column is flush left, other columns are centered
	f.write("\\tnote[Note:~\\textnormal{This table shows, for the years 1982 to 2013, the test statistic for a test of the goodness of the fit between the empirical CCDF for the wealth of the 400 wealthiest Americans in a given year and the CCDF for a Pareto distribution estimated from their wealths.}]{\\\} \n")
	#f.write("\\tnote[]{\\\} % for spacing\n")
	#f.write("\\tnote[$^{*}p<.10$ { } $^{**}p<.05$ { } $^{***}p<0.01$]{}") # notes for p-values (see Turabian 26.2.7)
	f.write("}{\FL[\lightrulewidth]\n")
	f.write("	\t&\t\multicolumn{1}{c}{}		 \t&\t\multicolumn{2}{c}{$p$-value accounting for}	 \t\NN\n")
	f.write("\cmidrule[\lightrulewidth](r){3-4}\n")
	f.write("	\t&\t\multicolumn{1}{c}{Test}	 \t&\t\multicolumn{1}{c}{Rounding}  \t&\t\multicolumn{1}{c}{Rounding and other}	 \t\NN\n")
	f.write("Year\t&\t\multicolumn{1}{c}{statistic}\t&\t\multicolumn{1}{c}{errors}	 \t&\t\multicolumn{1}{c}{measurement errors}	 \t\ML\n")
	for year in years:
		s = year + '\t&\t'
		s += str(format(round(k_s_like_test_against_w_rounding_errors['pareto'][year]['D stat'], 2), '.2f')) # same test stat either way
		s += '\t&\t'
		s += str(format(round(k_s_like_test_against_w_rounding_errors['pareto'][year]['p-value'], 2), '.2f')) 
		s += '\t&\t'
		s += str(format(round(k_s_like_test_against_w_rounding_and_other_measurement_errors['pareto'][year]['p-value'], 2), '.2f')) 	
		if year != years[-1]:
			s += '\t\NN\n'
		else:
			s += '\t\LL[\lightrulewidth]\n'
		f.write(s)
	f.write("\NN % for spacing\n") # for spacing
	f.write("}\n") # end ctable
	f.write("}\n") # end single-spacing
	f.close()

	## TABLE 3.6: GOODNESS-OF-FIT TEST AGAINST BOUNDED PARETO ##

	# export goodness of fit tests, just for pareto distribution, with rounding errors, with or without other measurement errors
	# uses packages ctable and dcolumn
	f = open("./tables/tables_specific_to_second_essay/goodness_of_fit_against_bounded_pareto_w_measurement_errors.txt", 'w') # write to this file
	f.write("{\ssp\n") # single-spacing
	f.write("\ctable[\n")
	f.write("\tcaption\t= {Goodness-of-fit Tests Against the Truncated Version, 1982--2013},\n") # sentence-style without terminal period
	f.write("\tlabel\t= goodness_of_fit_against_bounded_pareto_w_measurement_errors,\n")
	f.write("\tpos\t= H,\n") # top or page by itself
	f.write("\tnotespar,\n") # footnotes must be flush left (Turabian 2007, sec. 26.2.7)
	f.write("\tnosuper,\n") # typeset sources and notes on the line, manually superscript everything else
	f.write("\tmincapwidth\t= 1.0\\textwidth,\n") # make the table caption and footnotes as wide as the text
	f.write("\tdoinside\t=\\small,\n") # table is too big for page, so shrink text size
	f.write("\t]{lD{.}{.}{-2}D{.}{.}{-2}D{.}{.}{-2}}{\n") # first column is flush left, other columns are centered
	f.write("\\tnote[Note:~\\textnormal{This table shows, for the years 1982 to 2013, the test statistic for a test of the goodness of the fit between the empirical CCDF for the wealth of the 400 wealthiest Americans in a given year and the CCDF for a truncated Pareto distribution estimated from their wealths.}]{\\\} \n")
	f.write("}{\FL[\lightrulewidth]\n")
	f.write("	\t&\t\multicolumn{1}{c}{}		 \t&\t\multicolumn{2}{c}{$p$-value accounting for}	 \t\NN\n")
	f.write("\cmidrule[\lightrulewidth](r){3-4}\n")
	f.write("	\t&\t\multicolumn{1}{c}{Test}	 \t&\t\multicolumn{1}{c}{Rounding}  \t&\t\multicolumn{1}{c}{Rounding and other}	 \t\NN\n")
	f.write("Year\t&\t\multicolumn{1}{c}{statistic}\t&\t\multicolumn{1}{c}{errors}	 \t&\t\multicolumn{1}{c}{measurement errors}	 \t\ML\n")
	for year in years:
		s = year + '\t&\t'
		s += str(format(round(k_s_like_test_against_w_rounding_errors['bounded pareto'][year]['D stat'], 2), '.2f')) # same test stat either way
		s += '\t&\t'
		s += str(format(round(k_s_like_test_against_w_rounding_errors['bounded pareto'][year]['p-value'], 2), '.2f')) 
		s += '\t&\t'
		s += str(format(round(k_s_like_test_against_w_rounding_and_other_measurement_errors['bounded pareto'][year]['p-value'], 2), '.2f')) 	
		if year != years[-1]:
			s += '\t\NN\n'
		else:
			s += '\t\LL[\lightrulewidth]\n'
		f.write(s)
	f.write("\NN % for spacing\n") # for spacing
	f.write("}\n") # end ctable
	f.write("}\n") # end single-spacing
	f.close()

	## TABLE 3.5: LIKELIHOOD RATIO TESTS ##

	# uses packages ctable and dcolumn
	f = open("./tables/tables_specific_to_second_essay/lr_tests.txt", 'w') # write to this file
	f.write("{\ssp\n") # single-spacing
	f.write("\ctable[\n")
	f.write("\tcaption\t= {Likelihood-ratio Tests, 1982--2013},\n")
	f.write("\tlabel\t= likelihood_ratio_tests,\n")
	f.write("\tmincapwidth\t= 1.0\\textwidth,\n")
	f.write("\tpos\t= H,\n") # top or page by itself
	f.write("\tnotespar,\n") # footnotes must be flush left (Turabian 2007, sec. 26.2.7)
	f.write("\tnosuper,\n") # typeset sources and notes on the line, manually superscript everything else
	f.write("\tmincapwidth\t= 1.0\\textwidth,\n") # make the table caption and footnotes as wide as the text
	f.write("\tdoinside\t=\\small,\n") # table is too big for page, so shrink text size
	f.write("\t]{lD{.}{.}{-2}D{.}{.}{-2}D{.}{.}{-2}}{\n") # first column is flush left, other columns are centered
	f.write("\\tnote[Note:~\\textnormal{This table shows, for the years 1982 to 2013 and for different distributions, the test statistic for a likelihood-ratio test of a Pareto distribution against another distribution.}]{\\\} \n") # 
	f.write("\\tnote[]{\\\} % for spacing\n")
	f.write("\\tnote[$^{*}p<.10$ { } $^{**}p<.05$ { } $^{***}p<0.01$]{}") # notes for p-values (see Turabian 26.2.7)
	f.write("}{\FL[\lightrulewidth]\n")
	f.write("	\t&\t\multicolumn{3}{c}{Distribution}\t\NN\n")
	f.write("\cmidrule[\lightrulewidth](r){2-4}\n")
	f.write("\t&\t\multicolumn{1}{c}{Lower-truncated}\t&\t\multicolumn{1}{c}{Lower-truncated}\t&\t\multicolumn{1}{c}{Upper-truncated}\t\NN\n")
	f.write("Year\t&\t\multicolumn{1}{c}{gamma}\t&\t\multicolumn{1}{c}{log-normal}\t&\t\multicolumn{1}{c}{Pareto}\t\ML\n")
	for year in years:
		s = year + '\t&\t'
		for distribution in other_distributions:
			s += str(format(round(lr_test_of_pareto_against[distribution][year]['t-stat'], 2), '.2f'))
			s += get_note(lr_test_of_pareto_against[distribution][year]['p-value'])
			if distribution != other_distributions[-1]:
				s += '\t&\t'
			else:
				if year != years[-1]:
					s += '\t\NN\n'
				else:
					s += '\t\LL[\lightrulewidth]\n'
		f.write(s)
	f.write("\NN % for spacing\n") # for spacing
	f.write("}\n") # end ctable
	f.write("}\n") # end single-spacing
	f.close()