"""
Filename: essay_on_duration_of_wealth_code.py
Python version: 2.7
Source: Capehart, Kevin W. Essays on the Wealthiest Americans. PhD dissertation, American University, Washington, DC, 2014. 
Description: This file generates results reported by the essay on the duration of the wealth of the wealthiest Americans. 
"""

from __future__ import division
from load_forbes_400_dataset import * # load our Forbes 400 dataset
import scipy.stats # import for confidence intervals
from scipy.special import btdtri # import for quantiles of beta distribution
import statsmodels.api as sm # import for multionomial logit model
from os.path import isfile # import for checking whether a file exists
import pickle # import for dumping and loading computationally intensive results

## SEC. 4.3 AND C.1: NUMBER APPEARING AGAIN OR DROPPING OFF ##

# The following code looks at the year-over-year movement of unique individuals onto, off of, and throughout the Forbes 400. 

dyears = [year + '--' + str(int(year) + 1) for year in years_less_last]
dyear_types = ['remain wealthy', 'remain wealthy and move up', 'remain wealthy but move down', 'remain wealthy and stay the same', 'exit', 'exit by decline', 'exit by death', 'exit by renunciation of citizenship', 'entrants', 'first-timers', 'returnees']
n_by_dyear_types_in = dict(zip(dyears, [dict(zip(dyear_types, [0 for type in dyear_types])) for dyear in dyears]))

for dyear in dyears:
	year, next_year = dyear.split('--')
	for name in names:
		if data[name]['wealth'][year] != '':
			# remained wealthy?
			if (data[name]['wealth'][next_year] != ''):
				n_by_dyear_types_in[dyear]['remain wealthy'] += 1
				rank_in_year = summary_stats['wealths'][year].index(float(data[name]['wealth'][year]))
				rank_in_next_year = summary_stats['wealths'][next_year].index(float(data[name]['wealth'][next_year]))
				# up, down, or same?
				if rank_in_next_year > rank_in_year:
					n_by_dyear_types_in[dyear]['remain wealthy and move up'] += 1
				elif rank_in_next_year < rank_in_year:
					n_by_dyear_types_in[dyear]['remain wealthy but move down'] += 1
				else:
					assert (rank_in_next_year == rank_in_year), "If a person didn't move up or down in the rankings, their rank should be the same"
					n_by_dyear_types_in[dyear]['remain wealthy and stay the same'] += 1
			# exited?
			else:
				n_by_dyear_types_in[dyear]['exit'] += 1
				# exited by death, decline, or renunciation of citizenship?
				if (name in dead_in[next_year]):
					n_by_dyear_types_in[dyear]['exit by death'] += 1
				elif (name in renunciants_in[next_year]):
					n_by_dyear_types_in[dyear]['exit by renunciation of citizenship'] += 1
				else:
					n_by_dyear_types_in[dyear]['exit by decline'] += 1
		else:
			# not on list in first year, but comes on by next?
			if data[name]['wealth'][next_year] != '':
				n_by_dyear_types_in[dyear]['entrants'] += 1
				# first-timer or returnee?
				if True in [data[name]['wealth'][str(y)] != '' for y in range(1982, int(year))]:
					n_by_dyear_types_in[dyear]['returnees'] += 1
					assert (True in [data[name]['wealth'][str(y)] != '' for y in range(1982, int(year))]) == (True in [data[name]['wealth'][str(y)] != '' for y in range(1982, int(next_year))])
				else:
					n_by_dyear_types_in[dyear]['first-timers'] += 1

print "\nyear \tremain wealthy \texit by decline \texit by death \texit by renunciation of citizenship"
for dyear in dyears: print year, "\t", n_by_dyear_types_in[dyear]['remain wealthy'], "\t", n_by_dyear_types_in[dyear]['exit by decline'], "\t", n_by_dyear_types_in[dyear]['exit by death'], "\t", n_by_dyear_types_in[dyear]['exit by renunciation of citizenship']

print "\ntype \tavg number across years"
for type in dyear_types: print type, "\t", round(numpy.average([n_by_dyear_types_in[dyear][type] for dyear in dyears]), 0)

# quick calculation to compare to highest income 400
print "\nnumber of unique individuals on Forbes 400 btw 1992 and 2009:\t", len(list(set([name for name in names for year in [str(y) for y in range(1992, 2009+1)] if (data[name]['wealth'][year] != '')])))

## SEC. 4.3: PATTERNS OF APPEARANCES ###

# The following code uses Ashworth et al.'s (1994) classification scheme in order to look at the patterns with which unique individuals appeared on the Forbes 400 over the years. 

patterns = ['transient', 'occasional', 'recurrent', 'persistent', 'chronic', 'permanent']
persons_by_pattern = dict(zip(patterns, [list() for pattern in patterns]))
longest_spell_for_multiple_spells = 0
# for each name
for name in names:
	# get spells
	spells = []
	spell = []
	for year in years:
		wealth = data[name]['wealth'][year]
		if wealth != '':
			spell.append(year)
		else:
			spells.append(spell)
			spell = []
	spells.append(spell)
	# remove empty spells
	while [] in spells:
		spells.remove([])
	# record single-spell patterns
	if len(spells) == 1:
		# 'transient': one 'short' (i.e., one year) spell
		if len(spells[0]) == 1:
			persons_by_pattern['transient'].append(name)
		# 'permanent': one continuous spell
		elif len(spells[0]) == len(years):
			persons_by_pattern['permanent'].append(name)
		# 'persistent': one 'non-short' spell (i.e., longer than one year), but not continuous
		else:
			persons_by_pattern['persistent'].append(name)
	# record multiple-spell patterns
	else:
		# get shortest spell 
		shortest_spell = min([len(spell) for spell in spells])
		# get longest spell, too
		longest_spell = max([len(spell) for spell in spells])
		if longest_spell > longest_spell_for_multiple_spells:
			longest_spell_for_multiple_spells = longest_spell
		# get longest time between observed spells
		longest_time_between_spells = []
		for i in range(0, len(spells)-1):
			longest_time_between_spells.append(int(spells[i+1][0]) - int(spells[i][-1]) - 1)
		longest_time_between_spells = max(longest_time_between_spells)
		# 'occasional': multiple 'short' spells (all lasting one year)
		if (shortest_spell == 1):
			persons_by_pattern['occasional'].append((name, [len(spell) for spell in spells]))
		# 'recurrent': multiple 'non-short' spells (some lasting over one year) and 'non-short' out-of-poverty spells (some lasting over one year)
		elif (shortest_spell > 1) and (longest_time_between_spells > 1):
			persons_by_pattern['recurrent'].append((longest_time_between_spells, name, [len(spell) for spell in spells]))
		# 'chronic': multiple 'non-short' poverty spells (some lasting over one year) and 'short' out-of-poverty spells (lasting one year)
		else:
			persons_by_pattern['chronic'].append((name, [len(spell) for spell in spells]))
		#if len(spells) == 4:
		#	print name

## SOME CALCULATIONS ##

# persons for all patterns
persons_by_pattern['all'] = list()
for pattern in patterns:
	persons_by_pattern['all'].extend(persons_by_pattern[pattern])
total_number_of_persons = len(persons_by_pattern['all'])

# single spell patterns
print "\nBetween 1982 and 2013,", 
print "out of the " + str(format(total_number_of_persons, ',.0f')) + " people who appeared at least once on the magazine's list of the 400 wealthiest Americans,", 
print "over three quarters of them (", 
number_with_single_spell = len(persons_by_pattern['transient']) + len(persons_by_pattern['persistent']) + len(persons_by_pattern['permanent'])
print str(format(number_with_single_spell, ',.0f')) + " or about " + str(int(round(100. * number_with_single_spell / total_number_of_persons))),
print "percent of them) came onto the list, appeared in consecutive years, and then dropped off without reappearing again.",
# transient, persistent, permanent
print "Out of these people who had only one spell on the magazine's list,",
print "a minority (", 
print str(len(persons_by_pattern['transient'])) + " or about " + str(int(round(100. * len(persons_by_pattern['transient']) / number_with_single_spell))), 
print "percent) only appeared in one year, a majority (", 
print str(len(persons_by_pattern['persistent']) + len(persons_by_pattern['permanent'])) + " or about " + str(int(round(100. * (len(persons_by_pattern['persistent']) + len(persons_by_pattern['permanent'])) / number_with_single_spell))), 
print "percent) appeared in more than one year, and a small fraction", 
print "(" + str(len(persons_by_pattern['permanent'])), 
print "people or about", 
print str(int(round(100. * len(persons_by_pattern['permanent']) / number_with_single_spell))), 
print "percent) appeared in every year.", 
number_with_multiple_spells = len(persons_by_pattern['occasional']) + len(persons_by_pattern['recurrent']) + len(persons_by_pattern['chronic'])
# occassional
print "\nOut of the " + str(number_with_multiple_spells) + " people who had more than one spell, about half of them (", 
print str(len(persons_by_pattern['occasional'])) + " or about " + str(int(round(100. * len(persons_by_pattern['occasional']) / number_with_multiple_spells))), 
print "percent of them) had a pattern wherein all of their spells (of which the median number was",
print str(int(round(numpy.median([len(spells) for name, spells in persons_by_pattern['occasional']])))),
print "and the maximum number was",
print str(max([len(spells) for name, spells in persons_by_pattern['occasional']])),
print "spells) only lasted one year.",
# recurrent
print str(len(persons_by_pattern['recurrent'])) + " or about " + str(int(round(100. * len(persons_by_pattern['recurrent']) / number_with_multiple_spells))), 
print "percent of them had a pattern wherein at least one of their spells (of which the median number was",
print str(int(round(numpy.median([len(spells) for name, spells, longest_time_between_spells in persons_by_pattern['recurrent']])))),
print "and the maximum number was",
print str(max([len(spells) for name, spells, longest_time_between_spells in persons_by_pattern['recurrent']])),
print "spells) lasted longer than one year, but at least one of the times in between their spells was longer than one year.",
# chronic
print str(len(persons_by_pattern['chronic'])) + " or about " + str(int(round(100. * len(persons_by_pattern['chronic']) / number_with_multiple_spells))), 
print "percent of them had a pattern wherein at least one of their spells (of which the median number was",
print str(int(round(numpy.median([len(spells) for name, spells in persons_by_pattern['chronic']])))),
print "and the maximum number was",
print str(max([len(spells) for name, spells in persons_by_pattern['chronic']])),
print "spells) lasted longer than one year, and none of the times in between their spells were longer than one year.",
# longest and shortest
print "The longest spell for anyone with more than one spell was", 
print str(longest_spell_for_multiple_spells),
print "years (the heiress Phoebe Hearst Cooke and the media mogul Sumner Murray Redstone both had spells of that length),", 
print "while the longest time between any two spells was",
persons_by_pattern['recurrent'].sort(reverse=True)
assert persons_by_pattern['recurrent'][0][0] != persons_by_pattern['recurrent'][1][0] # no ties for the longest time between spells
print str(persons_by_pattern['recurrent'][0][0]), 
print "years (Kenneth Stanley ``Bud'' Adams Jr. was off the list for that long between his two spells)."

## DEFINE FUNCTIONS FOR CI'S AND P-VALUES ##

def get_normal_ci(point, se, confidence_level=0.95):
	""" Returns lower and upper confidence intervals based on a Normal distribution """
	z_stat = scipy.stats.norm.ppf(1. - ((1. - confidence_level) / 2.))
	lower = point - z_stat * se
	upper = point + z_stat * se
	return lower, upper

def get_p_value(z_stat):
	""" Returns p-value for a z-test """
	return scipy.stats.norm.cdf(-abs(z_stat), loc=0, scale=1) + scipy.stats.norm.sf(abs(z_stat), loc=0, scale=1)

def get_cp_ci(trials, failures, confidence_level=0.95):
	""" Returns lower and upper Clopper-Pearson confidence intervals, which are calculated based on the quantiles of a beta distribution """
	lower = btdtri(failures, trials - failures + 1., (1. - confidence_level) / 2.) 
	upper = btdtri(failures + 1., trials - failures, 1. - ((1. - confidence_level) / 2.))
	return lower, upper

## SEC. 4.3, ESP. FIG. 4.3: SURVIVAL FUNCTION ##

# The following code looks at consecutive appearances of unique invidiauls on the Forbes 400 and calculates a survival function for appearing again and again for at least a given number of consecutive years. 

durations = range(1, len(years), 1)
n_at_risk_at = dict(zip(durations, [0 for duration in durations]))
n_exits_by_decline_in_wealth_at = dict(zip(durations, [0 for duration in durations]))
n_exits_by_death_at = dict(zip(durations, [0 for duration in durations]))
n_exits_by_renunciation_of_citizenship_at = dict(zip(durations, [0 for duration in durations]))

# omit left-censored spells?
omit_left_censored_spells = False
if omit_left_censored_spells:
	durations = range(1, len(years)-1, 1) # re-defines durations
# for each name
for name in names:
	# get spells
	spells = []
	spell = []
	for year in years:
		wealth = data[name]['wealth'][year]
		if wealth != '':
			spell.append(year)
		else:
			spells.append(spell)
			spell = []
	spells.append(spell)
	# change left-censored spells to empty spells, so they'll be removed when empty spells are removed
	if omit_left_censored_spells:
		for i in range(0, len(spells)):
			if '1982' in spells[i]:
				spells[i] = []
	# remove empty spells
	while [] in spells:
		spells.remove([])
	# for each spell, record data
	while len(spells) > 0:
		spell = spells.pop()
		# as long as the spell didn't start in the last year
		if spell == [years[-1]]:
			pass
		else:
			# for each year of the spell
			for i in range(0, len(spell)):
				# year
				year = spell[i]
				next_year = str(int(year)+1)
				# if year is not the last year
				if year != years[-1]:
					# record at risk
					duration = (i + 1)
					n_at_risk_at[duration] += 1
					# remains wealthy?
					remains_wealthy = int(i != len(spell)-1)
					# exits by decline in wealth?
					exits_by_decline_in_wealth = int(data[name]['wealth'][next_year] == '')
					# exits by death?
					exits_by_death = int(name in dead_in[next_year])
					# exits by renunciation of citizenship?
					exits_by_renunciation_of_citizenship = int(name in renunciants_in[next_year])
					# record any incidence
					if exits_by_renunciation_of_citizenship:
						n_exits_by_renunciation_of_citizenship_at[duration] += 1
					elif exits_by_death:
						n_exits_by_death_at[duration] += 1
					elif exits_by_decline_in_wealth: 
						n_exits_by_decline_in_wealth_at[duration] += 1
					else: # remains wealthy
						pass

n_exits_at = dict(zip(durations, [n_exits_by_decline_in_wealth_at[duration] + n_exits_by_death_at[duration] + n_exits_by_renunciation_of_citizenship_at[duration] for duration in durations]))
point_estimate_for_survivor_function = dict()
lower_ci_for_survivor_function, upper_ci_for_survivor_function = dict(), dict()
duration = durations[0]
lower_ci_for_survivor_function[duration], point_estimate_for_survivor_function[duration], upper_ci_for_survivor_function[duration] = 1., 1., 1.

print "\nduration \tpoint estimate for survivor function \t std error"
print duration, "\t", point_estimate_for_survivor_function[duration], "\t", 0.0
for duration in durations[1:]:
	shorter_durations = range(1, duration, 1)
	point_estimate_for_survivor_function[duration] = numpy.prod([1. - (n_exits_at[shorter_duration] / n_at_risk_at[shorter_duration]) for shorter_duration in shorter_durations])
	var = (point_estimate_for_survivor_function[duration] ** 2.) * sum([n_exits_at[shorter_duration] / (n_at_risk_at[shorter_duration] * (n_at_risk_at[shorter_duration] - n_exits_at[shorter_duration])) for shorter_duration in shorter_durations])
	lower_ci_for_survivor_function[duration], upper_ci_for_survivor_function[duration] = get_normal_ci(point=point_estimate_for_survivor_function[duration], se=numpy.sqrt(var))
	print duration, "\t", point_estimate_for_survivor_function[duration], "\t", 100. * numpy.sqrt(var)
print "\nOf note, if survival curves with and without left-censored observations are computed, then the largest difference between them is 0.027692595 and that occurs once at three years, but that difference is not quite statistically significant at the 10 percent level (with a p-value of about " +  str(get_p_value(z_stat=(0.657313437 - 0.629620841) / numpy.sqrt(0.000124073 + 0.000164852295641))) + ")."

## SEC. 4.3, ESP. FIG. 4.2: HAZARD FUNCTION ##

# The following code calculates a hazard function for appearing again after a given number of consecutive appearances. Normal CI's contain impossible values less than zero, so we use Clopper-Pearson CI's instead. 

durations = range(1, len(years), 1)
point_estimate_for_hazard_function = dict()
lower_ci_for_hazard_function, upper_ci_for_hazard_function = dict(), dict()
for duration in durations:
	# Normal CI's
	point_estimate_for_hazard_function[duration] = n_exits_at[duration] / n_at_risk_at[duration]
	se = numpy.sqrt((point_estimate_for_hazard_function[duration] * (1. - point_estimate_for_hazard_function[duration])) / n_at_risk_at[duration])
	lower, upper = get_normal_ci(point=point_estimate_for_hazard_function[duration], se=se)
	# print results
	if duration == durations[0]: print "\nduration \tpoint estimate for hazard function duration \tlower Normal CI \tupper Normal CI \tlower CP CI \tupper CP CI"
	print duration, "\t", point_estimate_for_hazard_function[duration], "\t", lower, "\t", upper, "\t", 	
	# Clopper-Pearson CI's
	lower, upper = get_cp_ci(trials=n_at_risk_at[duration], failures=n_exits_at[duration])
	lower_ci_for_hazard_function[duration] = lower
	upper_ci_for_hazard_function[duration] = upper
	# print results
	print lower, "\t", upper

## SEC. 4.4: GET DATA FOR BASELINE MODEL ##

# keep track of names associated with each spell
names_associated_w_spells = list()
# keep track of ages and ranks
ages_as_dependent_vars, ranks_as_dependent_vars = list(), list()
# list of data for spells
data_for_spells = list()
# for each name
for name in names:
	# get spells
	spells = []
	spell = []
	for year in years:
		wealth = data[name]['wealth'][year]
		if wealth != '':
			spell.append(year)
		else:
			spells.append(spell)
			spell = []
	spells.append(spell)
	# remove empty spells
	while [] in spells:
		spells.remove([])
	# for each spell, record data
	while len(spells) > 0:
		spell = spells.pop()
		# as long as the spell didn't start in the last year
		if spell == [years[-1]]:
			pass
		else:
			# list of data for spell
			data_for_spell = list()
			# for each year of the spell
			for i in range(0, len(spell)):
				# year
				year = spell[i]
				next_year = str(int(year)+1)
				# if year is not the last year
				if year != years[-1]:
					# data for given year of the spell
					data_for_year_of_spell = list()
					# remains wealthy?
					remains_wealthy = int(i != len(spell)-1)
					# exits by decline in wealth?
					exits_by_decline_in_wealth = int(data[name]['wealth'][next_year] == '')
					# exits by death? superseding exit by decline
					exits_by_death = int(name in dead_in[next_year])
					# exits by renunciation of citizenship? superseding exit by decline
					exits_by_renunciation_of_citizenship = int(name in renunciants_in[next_year])
					# record dependent variable
					if exits_by_renunciation_of_citizenship:
						dependent_var = 3 # where 3 = exits by renunciation of citizenship:
					elif exits_by_death:
						dependent_var = 2 # where 2 = exits by death
					elif exits_by_decline_in_wealth: 
						dependent_var = 1 # where 1 = exits by decline in wealth
					else: # remains wealthy
						dependent_var = 0 # where 0 = remains wealthy
					data_for_year_of_spell.append(dependent_var)
					# independent variables
					# age
					age = int(data[name]['age'][year])
					age_squared = age * age
					# rank
					rank = summary_stats['wealths'][year].index(float(data[name]['wealth'][year])) + 1
					# left-censored dummy
					left_censored = int('1982' in spell)
					data_for_year_of_spell.extend([1., age, age_squared, rank, left_censored]) # include a constant
					# dummies for years, but drop the dummy for 1982
					data_for_year_of_spell.extend([int(year == _) for _ in years_less_initial_and_last])
					# test:
					if 1 in data_for_year_of_spell[6:6+len(years_less_initial_and_last)]:
						assert year == years[data_for_year_of_spell[6:6+len(years_less_initial_and_last)].index(1)+1]
					else:
						assert year == '1982'
					# dummies for durations of given number of years or 28 or more years (see below), but drop the dummy for a duration of one year
					duration = (i + 1)
					for _ in range(2, 27+1):
						data_for_year_of_spell.append(int(duration == _))
					data_for_year_of_spell.append(int(duration > 27))
					# add to list
					if (data_for_year_of_spell[0] != 3): # ignore spells that end with exit by renunciation of citizenship for reasons discussed in dissertation
						data_for_spell.append(data_for_year_of_spell)
						ages_as_dependent_vars.append(age)
						ranks_as_dependent_vars.append(rank)
			# add to list
			if (data_for_year_of_spell[0] != 3): # again, ignorning spells that end with exit by renunciation of citizenship
				data_for_spells.append(data_for_spell)
				names_associated_w_spells.append(name)

# Each of the 25 people who were on the list for 30 years by 2011 remained on the list into 2012 and 2013, so there will be perfect prediction problem if we include a dummy variable for a duration of 30 or more years. Moreover, none of the people who who were on the list for 29 years died, they all either remained wealthy or exited by a decline in wealth, so there would again be a perfect prediction problem if we included a dummy variable for a duration of 29 years. In order to avoid these problems while still allowing for a non-parameteric approach to duration dependence, we use a dummy for a duration of 28 or more years, instead of dummies for durations of 28, 29, and 30 years. 

print "\nThere are " + str(sum(len(_) for _ in data_for_spells)) + " person-year observations associated with " + str(len(data_for_spells)) + " spells of " + str(len(list(set(names_associated_w_spells)))) + " unique individuals." 

# Observations of 400 people in each year for 31 years should yield 400 * 31 = 12,400 person-year observations. But if we ignore the observations associated with spells for the four people who renounced their citizenship, then we get 12,379 person-year observations associated with the 1,854 spells of 1,451 unique individuals. (One person who would eventually exit by renunciation of citizenship, John Thompson Dorrance III, had an earlier spell.)

## ORGANIZE DATA AND EXPORT IT ##

# organize data for estimation
dependent_vars = list()
independent_vars = list()
for data_for_spell in data_for_spells:
	for data_for_year_of_spell in data_for_spell:
		dependent_vars.append(data_for_year_of_spell[0])
		independent_vars.append(numpy.array(data_for_year_of_spell[1:]))
dependent_vars = numpy.array(dependent_vars)
independent_vars = numpy.array(independent_vars)
# export to file
filepath = "./data/data_specific_to_third_essay/"
filename = filepath + "exported_survival_data.csv"
f = open(filename, 'w')
# write header
s = "dependent_var,constant,age,age_squared,rank,left_censored,"
for year in years_less_initial_and_last:
	s += "d" + year + ","
for duration in range(2, 27+1):
	s += "d" + str(duration) + ","
s += "d28_or_more" + "\n"
f.write(s)
header = s.replace('\n', '')
header = header.split(',')
n_of_independent_vars = len(header[1:])
# write data
for i in range(0, len(dependent_vars)):
	s = str(dependent_vars[i])
	for var in independent_vars[i]:
		s += ','
		s += str(var)
	s += '\n'
	f.write(s)
# close file
f.close()

## SOME SUMMARY STATISTICS ##

# import the data that was just exported
raw_data = numpy.loadtxt(filename, delimiter=',', skiprows=1, dtype={'names':header, 'formats':['f']*len(header)})
# calculate some summary statistics
for var in ['age','rank']:
	print "\nThe minimum, median, and maximum " + var + " are " + str(min(raw_data[var])) + ', ' + str(numpy.median(raw_data[var])) + ", and " + str(max(raw_data[var])) + ", respectively."
for var in header[5:]:
	if var == header[5:][0]: print "\nThe total number of observations where each dummy variable is true are as follows."
	print var, "\t", sum(raw_data[var])

## ESTIMATE BASELINE MODEL ##

model = sm.MNLogit(dependent_vars, independent_vars)
estimate_using_gretl = False
if estimate_using_gretl:
	# use the optimizers used by the MNLogit function in StatsModel to estimate the model
	converged = False
	while converged == False:
		fitted_for_baseline = model.fit(disp=False)
		converged = fitted_for_baseline.mle_retvals['converged']
else:
	# use estimated parameters from gretl and just use the MNLogit function for var-covar estimates
	start_params = []
	filename = filepath + "start_params_for_baseline_model.txt"
	f = open(filename, "r")
	lines = f.readlines()
	for line in lines[len(lines) - n_of_independent_vars * 2:]:
		start_params.append(float(line.replace('\n', '')))
	f.close()
	# fit model
	fitted_for_baseline = model.fit(method='bfgs', maxiter=0, disp=False, start_params=numpy.array(start_params))
#print fitted_for_baseline.summary(yname=header[0], xname=header[1:]) # de-comment to print summary of estimated model
# As a check, it can be shown that the econometric program gretl 1.9.14 generates the same estimates (ignoring floating point errors) as the MNLogit function in the StatsModel, so we will simply use the estimates generated by the latter. 

## SEC. 4.4.3., ESP. FIGS. 4.4 TO 4.7: PREDICTED PROBABILITIES ##

exits = ['remain wealthy', 'exit by decline', 'exit by death'] # ignoring exit by renunciation of citizenship for reasons discussed in the essay
ages = range(20, 100+1) # select ages
ranks = [1]
ranks.extend(range(10, 400+1, 10)) # select ranks
durations = range(1, 28+1) # select durations

# set the following to True to re-estimate confidence intervals if they've already been estimated
re_estimate = False
# check if confidence intervals have already been estimated
filenames = [filepath + _ + ".pkl" for _ in ["prob_of_exit_by_age", "prob_of_exit_by_rank", "prob_of_exit_by_duration", "prob_of_exit_by_year"]]
already_estimated = False not in [isfile(filename) for filename in filenames]
# estimate or re-estimate
if not already_estimated or re_estimate:

	# construct Krinsky-Robb confidence intervals by taking a large number of draws from a multivariate normal with means equal to the estimated parameters and var-covar matrix equal to the estimated var-covar matrix
	numpy.random.seed(250624) 
	redrawn_parameters = numpy.random.multivariate_normal(mean=fitted_for_baseline.params.flatten('F'), cov=fitted_for_baseline.cov_params(), size=10000)
	
	# probs of exits by ages	
	prob_of_exit_by_age = dict(zip(exits, [dict(zip(['lower', 'point', 'upper'], [dict(zip(ages, [[] for age in ages])) for _ in ['lower', 'point', 'upper']])) for exit in exits]))
	# for each age
	for age in ages:
		X = [1, age, age * age, 194, 0] # rank of 194, no left censoring
		X.extend([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]) # 1996
		X.extend([0 for _ in range(2, 28+1)]) # duration of one year
		n = len(X)
		# point estimates for probs, given estimated parameters
		eXb1 = numpy.exp(numpy.dot(X, fitted_for_baseline.params.flatten('F')[0:n]))
		eXb2 = numpy.exp(numpy.dot(X, fitted_for_baseline.params.flatten('F')[n:]))
		denominator = 1. + eXb1 + eXb2
		prob_of_exit_by_age['remain wealthy']['point'][age] = 1. / denominator
		prob_of_exit_by_age['exit by decline']['point'][age] = eXb1 / denominator
		prob_of_exit_by_age['exit by death']['point'][age] = eXb2 / denominator
		# point estimates for probs, given redrawn parameters
		estimates_for = dict(zip(exits, [[] for exit in exits]))
		for b in redrawn_parameters:
			eXb1 = numpy.exp(numpy.dot(X, b[0:n]))
			eXb2 = numpy.exp(numpy.dot(X, b[n:]))
			denominator = 1. + eXb1 + eXb2
			estimates_for['remain wealthy'].append(1. / denominator)
			estimates_for['exit by decline'].append(eXb1 / denominator)
			estimates_for['exit by death'].append(eXb2 / denominator)
		# get 95% CI
		for exit in exits:
			prob_of_exit_by_age[exit]['lower'][age] = scipy.stats.mstats.mquantiles(estimates_for[exit], prob=[2.5/100.], alphap=1/3., betap=1/3.)[0]
			prob_of_exit_by_age[exit]['upper'][age] = scipy.stats.mstats.mquantiles(estimates_for[exit], prob=[97.5/100.], alphap=1/3., betap=1/3.)[0]
	# dump results
	filename = filepath + "prob_of_exit_by_age.pkl"
	pickle.dump(prob_of_exit_by_age, open(filename, 'wb'))

	# probs of exits by rank
	prob_of_exit_by_rank = dict(zip(exits, [dict(zip(['lower', 'point', 'upper'], [dict(zip(ranks, [None for rank in ranks])) for _ in ['lower', 'point', 'upper']])) for exit in exits]))
	# for each rank
	for rank in ranks:
		X = [1, 64, 4096, rank, 0] # 64 years of age, no left censoring
		X.extend([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]) # 1996
		X.extend([0 for _ in range(2, 28+1)]) # duration of one year
		n = len(X)
		# point estimates for probs, given estimated parameters
		eXb1 = numpy.exp(numpy.dot(X, fitted_for_baseline.params.flatten('F')[0:n]))
		eXb2 = numpy.exp(numpy.dot(X, fitted_for_baseline.params.flatten('F')[n:]))
		denominator = 1. + eXb1 + eXb2
		prob_of_exit_by_rank['remain wealthy']['point'][rank] = 1. / denominator
		prob_of_exit_by_rank['exit by decline']['point'][rank] = eXb1 / denominator
		prob_of_exit_by_rank['exit by death']['point'][rank] = eXb2 / denominator
		# point estimates for probs, given redrawn parameters
		estimates_for = dict(zip(exits, [[] for exit in exits]))
		for b in redrawn_parameters:
			eXb1 = numpy.exp(numpy.dot(X, b[0:n]))
			eXb2 = numpy.exp(numpy.dot(X, b[n:]))
			denominator = 1. + eXb1 + eXb2
			estimates_for['remain wealthy'].append(1. / denominator)
			estimates_for['exit by decline'].append(eXb1 / denominator)
			estimates_for['exit by death'].append(eXb2 / denominator)
		# get 95% CI
		for exit in exits:
			prob_of_exit_by_rank[exit]['lower'][rank] = scipy.stats.mstats.mquantiles(estimates_for[exit], prob=[2.5/100.], alphap=1/3., betap=1/3.)[0]
			prob_of_exit_by_rank[exit]['upper'][rank] = scipy.stats.mstats.mquantiles(estimates_for[exit], prob=[97.5/100.], alphap=1/3., betap=1/3.)[0]
	# dump results
	filename = filepath + "prob_of_exit_by_rank.pkl"
	pickle.dump(prob_of_exit_by_rank, open(filename, 'wb'))

	# probs of exits by duration
	prob_of_exit_by_duration = dict(zip(exits, [dict(zip(['lower', 'point', 'upper'], [dict(zip(years_less_last, [None for duration in durations])) for _ in ['lower', 'point', 'upper']])) for exit in exits]))
	# for each duration
	for duration in durations:
		X = [1, 64, 4096, 194, 0] # 64 years of age, rank of 194, no left censoring
		X.extend([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]) # 1996
		duration_dummies = [0 for _ in range(2, 28+1)]
		if duration == 1:
			pass
		else:
			duration_dummies[range(2, 28+1).index(duration)] = 1
		X.extend(duration_dummies)
		n = len(X)
		# point estimates for probs, given estimated parameters
		eXb1 = numpy.exp(numpy.dot(X, fitted_for_baseline.params.flatten('F')[0:n]))
		eXb2 = numpy.exp(numpy.dot(X, fitted_for_baseline.params.flatten('F')[n:]))
		denominator = 1. + eXb1 + eXb2
		prob_of_exit_by_duration['remain wealthy']['point'][duration] = 1. / denominator
		prob_of_exit_by_duration['exit by decline']['point'][duration] = eXb1 / denominator
		prob_of_exit_by_duration['exit by death']['point'][duration] = eXb2 / denominator
		# point estimates for probs, given redrawn parameters
		estimates_for = dict(zip(exits, [[] for exit in exits]))
		for b in redrawn_parameters:
			eXb1 = numpy.exp(numpy.dot(X, b[0:n]))
			eXb2 = numpy.exp(numpy.dot(X, b[n:]))
			denominator = 1. + eXb1 + eXb2
			estimates_for['remain wealthy'].append(1. / denominator)
			estimates_for['exit by decline'].append(eXb1 / denominator)
			estimates_for['exit by death'].append(eXb2 / denominator)
		# get 95% CI
		for exit in exits:
			prob_of_exit_by_duration[exit]['lower'][duration] = scipy.stats.mstats.mquantiles(estimates_for[exit], prob=[2.5/100.], alphap=1/3., betap=1/3.)[0]
			prob_of_exit_by_duration[exit]['upper'][duration] = scipy.stats.mstats.mquantiles(estimates_for[exit], prob=[97.5/100.], alphap=1/3., betap=1/3.)[0]
	# dump results
	filename = filepath + "prob_of_exit_by_duration.pkl"
	pickle.dump(prob_of_exit_by_duration, open(filename, 'wb'))

	# probs of exits by year
	prob_of_exit_by_year = dict(zip(exits, [dict(zip(['lower', 'point', 'upper'], [dict(zip(years_less_last, [None for year in years_less_last])) for _ in ['lower', 'point', 'upper']])) for exit in exits]))
	# for each year
	for year in years_less_last:
		X = [1, 64, 4096, 194, 0] # 64 years of age, rank of 194, no left censoring
		year_dummies = [0 for _ in years_less_initial_and_last]
		if year == '1982':
			pass
		else:
			year_dummies[years_less_initial_and_last.index(year)] = 1
		X.extend(year_dummies)
		X.extend([0 for _ in range(2, 28+1)]) # duration of one year
		n = len(X)
		# point estimates for probs, given estimated parameters
		eXb1 = numpy.exp(numpy.dot(X, fitted_for_baseline.params.flatten('F')[0:n]))
		eXb2 = numpy.exp(numpy.dot(X, fitted_for_baseline.params.flatten('F')[n:]))
		denominator = 1. + eXb1 + eXb2
		prob_of_exit_by_year['remain wealthy']['point'][year] = 1. / denominator
		prob_of_exit_by_year['exit by decline']['point'][year] = eXb1 / denominator
		prob_of_exit_by_year['exit by death']['point'][year] = eXb2 / denominator
		# point estimates for probs, given redrawn parameters
		estimates_for = dict(zip(exits, [[] for exit in exits]))
		for b in redrawn_parameters:
			eXb1 = numpy.exp(numpy.dot(X, b[0:n]))
			eXb2 = numpy.exp(numpy.dot(X, b[n:]))
			denominator = 1. + eXb1 + eXb2
			estimates_for['remain wealthy'].append(1. / denominator)
			estimates_for['exit by decline'].append(eXb1 / denominator)
			estimates_for['exit by death'].append(eXb2 / denominator)
		# get 95% CI
		for exit in exits:
			prob_of_exit_by_year[exit]['lower'][year] = scipy.stats.mstats.mquantiles(estimates_for[exit], prob=[2.5/100.], alphap=1/3., betap=1/3.)[0]
			prob_of_exit_by_year[exit]['upper'][year] = scipy.stats.mstats.mquantiles(estimates_for[exit], prob=[97.5/100.], alphap=1/3., betap=1/3.)[0]

	# dump results
	filename = filepath + "prob_of_exit_by_year.pkl"
	pickle.dump(prob_of_exit_by_year, open(filename, 'wb'))

else:

	# load results that were dumped using pickle
	prob_of_exit_by_age = pickle.load(open(filepath + "prob_of_exit_by_age.pkl", 'rb'))
	prob_of_exit_by_rank = pickle.load(open(filepath + "prob_of_exit_by_rank.pkl", 'rb'))
	prob_of_exit_by_duration = pickle.load(open(filepath + "prob_of_exit_by_duration.pkl", 'rb'))
	prob_of_exit_by_year = pickle.load(open(filepath + "prob_of_exit_by_year.pkl", 'rb'))

## SEC. 4.4.3: PREDICTION ERRORS ##

prediction_error_for = dict()
translation_for_exit_number = dict(zip([0, 1, 2], ['Appeared again', 'Exited by decline', 'Exited by death']))
for i in range(0, len(data_for_spells)):
	data_for_spell = data_for_spells[i]
	name = names_associated_w_spells[i]
	for data_for_year_of_spell in data_for_spell:
		# point estimates for probs, given estimated parameters
		prob_of = dict()
		X = data_for_year_of_spell[1:]
		n = len(X)
		eXb1 = numpy.exp(numpy.dot(X, fitted_for_baseline.params.flatten('F')[0:n]))
		eXb2 = numpy.exp(numpy.dot(X, fitted_for_baseline.params.flatten('F')[n:]))
		denominator = 1. + eXb1 + eXb2
		prob_of['Appeared again'] = 1. / denominator
		prob_of['Exited by decline'] = eXb1 / denominator
		prob_of['Exited by death'] = eXb2 / denominator
		# actual outcome
		actual_outcome = translation_for_exit_number[data_for_year_of_spell[0]]
		# actual outcome was most likely outcome(s)?
		max_prob = max([prob_of[outcome] for outcome in ['Appeared again', 'Exited by decline', 'Exited by death']])
		most_prob_outcome = list()
		for outcome in ['Appeared again', 'Exited by decline', 'Exited by death']:
			if prob_of[outcome] == max_prob:
				most_prob_outcome.append(outcome)
		# if actual outcome was not the most probable outcome, get the prediction error
		if actual_outcome in most_prob_outcome:
			pass
		else:
			if 1 in data_for_year_of_spell[6:6+len(years_less_initial_and_last)]:
				year = years[data_for_year_of_spell[6:6+len(years_less_initial_and_last)].index(1)+1]
			else:
				year = '1982'
			next_year = str(int(year) + 1)
			dyear = year + '--' + next_year
			# prediction error
			for outcome in ['Appeared again', 'Exited by decline', 'Exited by death']:
				if outcome == actual_outcome:
					assert len(most_prob_outcome) == 1, "Ties in most-probable outcome"
					if i==0: print "\nname \tdyear \tactual outcome \tmost probable outcome \tprediction error"
					# only print especially egregious prediction errors
					if (1. - prob_of[outcome]) > 0.995:
						print name, "\t", dyear, "\t", actual_outcome, "\t", most_prob_outcome[0], "\t", (1. - prob_of[outcome])

## SEC. C.3: TEST OF THE IIA ASSUMPTION FOR BASELINE DURATION MODEL ##

# define dict for storing results
results_of_iaa_test = dict()
# set the following to True to re-estimate if already estimated
re_estimate = False
# check if already been estimated
filename = filepath + _ + "results_of_iaa_test.pkl"
already_estimated = isfile(filename)
# estimate or re-estimate
if not already_estimated or re_estimate:
	
	# get point estimates, var-covar matrix, and relative risks for full model
	b_full = fitted_for_baseline.params.flatten('F')
	cov_full = fitted_for_baseline.cov_params()
	X = [1, 64, 4096, 194, 0] # 64 years of age, rank of 194, no left censoring
	X.extend([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]) # 1996
	X.extend([0 for _ in range(2, 28+1)]) # duration of one year
	n = len(X)
	eXb1 = numpy.exp(numpy.dot(X, b_full[0:n]))
	eXb2 = numpy.exp(numpy.dot(X, b_full[n:]))
	denominator = 1. + eXb1 + eXb2
	prob_of_remain_wealthy = 1. / denominator
	prob_of_exit_by_decline = eXb1 / denominator
	prob_of_exit_by_death = eXb2 / denominator
	relative_risk_of_exit_by_decline_to_remain_wealthy = prob_of_exit_by_decline / prob_of_remain_wealthy
	relative_risk_of_exit_by_death_to_remain_wealthy = prob_of_exit_by_death / prob_of_remain_wealthy
	relative_risk_of_exit_by_decline_to_exit_by_death = prob_of_exit_by_decline / prob_of_exit_by_death
	
	# re-estimate model ignoring exit by decline
	dependent_vars = list()
	independent_vars = list()
	for data_for_spell in data_for_spells:
		for data_for_year_of_spell in data_for_spell:
			dependent_var = data_for_year_of_spell[0]
			if (data_for_year_of_spell[0] == 0) or (data_for_year_of_spell[0] == 2): # ignore spells that end with exit by decline
				dependent_vars.append(data_for_year_of_spell[0])
				independent_vars.append(numpy.array(data_for_year_of_spell[1:]))
	dependent_vars = numpy.array(dependent_vars)
	independent_vars = numpy.array(independent_vars)
	model_subset = sm.MNLogit(dependent_vars, independent_vars)
	numpy.random.seed(250624)
	fitted_subset = model_subset.fit(disp=False)
	# get t-stat and p-value
	b_subset = fitted_subset.params.flatten('F')
	cov_subset = fitted_subset.cov_params()
	n = len(fitted_subset.params)
	stat = numpy.matrix(b_subset - b_full[n:]) * numpy.matrix(cov_subset - cov_full[n:,n:]).I * numpy.matrix(b_subset - b_full[n:]).T
	dof = numpy.linalg.matrix_rank(cov_subset - cov_full[n:,n:])
	# store results
	_ = "t-stat and p-value for ignoring exit by decline" 
	results_of_iaa_test[_] = (stat[(0,0)], 1.0 - scipy.stats.chi2.cdf(stat[(0,0)], dof))
	# get relative risks
	eXb2 = numpy.exp(numpy.dot(X, b_subset))
	denominator = 1. + eXb2
	prob_of_remain_wealthy = 1. / denominator
	prob_of_exit_by_death = eXb2 / denominator
	relative_risk_of_exit_by_death_to_remain_wealthy_subset = prob_of_exit_by_death / prob_of_remain_wealthy
	# store results
	_ = "relative risk of exit by death to remain wealthy, full model" 
	results_of_iaa_test[_] = relative_risk_of_exit_by_death_to_remain_wealthy
	_ = "relative risk of exit by death to remain wealthy, ignoring exit by decline" 
	results_of_iaa_test[_] = relative_risk_of_exit_by_death_to_remain_wealthy_subset
	
	# re-estimate model ignoring exit by death
	dependent_vars = list()
	independent_vars = list()
	for data_for_spell in data_for_spells:
		for data_for_year_of_spell in data_for_spell:
			dependent_var = data_for_year_of_spell[0]
			if (data_for_year_of_spell[0] == 0) or (data_for_year_of_spell[0] == 1): # ignore spells that end with exit by death
				dependent_vars.append(data_for_year_of_spell[0])
				independent_vars.append(numpy.array(data_for_year_of_spell[1:]))
	dependent_vars = numpy.array(dependent_vars)
	independent_vars = numpy.array(independent_vars)
	model_subset = sm.MNLogit(dependent_vars, independent_vars)
	# estimate model on subset
	numpy.random.seed(250624) 
	fitted_subset = model_subset.fit(disp=False)
	# get t-stat and p-value
	b_subset = fitted_subset.params.flatten('F')
	cov_subset = fitted_subset.cov_params()
	n = len(fitted_subset.params)
	stat = numpy.matrix(b_subset - b_full[0:n]) * numpy.matrix(cov_subset - cov_full[0:n,0:n]).I * numpy.matrix(b_subset - b_full[0:n]).T
	dof = numpy.linalg.matrix_rank(cov_subset - cov_full[0:n,0:n])
	# store results
	_ = "t-stat and p-value for ignoring exit by death" 
	results_of_iaa_test[_] = (stat[(0,0)], 1.0 - scipy.stats.chi2.cdf(stat[(0,0)], dof))
	# get relative risk for model estimated on subset
	eXb1 = numpy.exp(numpy.dot(X, b_subset))
	denominator = 1. + eXb1
	prob_of_remain_wealthy = 1. / denominator
	prob_of_exit_by_decline = eXb1 / denominator
	relative_risk_of_exit_by_decline_to_remain_wealthy_subset = prob_of_exit_by_decline / prob_of_remain_wealthy
	# store results
	_ = "relative risk of exit by decline to remain wealthy, full model" 
	results_of_iaa_test[_] = relative_risk_of_exit_by_decline_to_remain_wealthy
	_ = "relative risk of exit by decline to remain wealthy, ignoring exit by death" 
	results_of_iaa_test[_] = relative_risk_of_exit_by_decline_to_remain_wealthy_subset

	# re-estimate model ignoring remain wealthy
	dependent_vars = list()
	independent_vars = list()
	for data_for_spell in data_for_spells:
		for data_for_year_of_spell in data_for_spell:
			dependent_var = data_for_year_of_spell[0]
			if (data_for_year_of_spell[0] == 1) or (data_for_year_of_spell[0] == 2): # ignore remain wealthy
				dependent_vars.append(data_for_year_of_spell[0])
				independent_vars.append(numpy.array(data_for_year_of_spell[1:]))
	dependent_vars = numpy.array(dependent_vars)
	independent_vars = numpy.array(independent_vars)
	model_subset = sm.MNLogit(dependent_vars, independent_vars)
	# estimate model on subset
	numpy.random.seed(250624) 
	fitted_subset = model_subset.fit(disp=False)
	b_subset = fitted_subset.params.flatten('F')
	cov_subset = fitted_subset.cov_params()
	n = len(fitted_subset.params)
	# re-estimate full model with exit by decline as base so that the estimates are comparable
	dependent_vars = list()
	independent_vars = list()
	for data_for_spell in data_for_spells:
		for data_for_year_of_spell in data_for_spell:
			# re-define dependent variable so that exit by decline is the base
			if data_for_year_of_spell[0] == 0:
				dependent_vars.append(1) # 0=remain wealthy becomes 1
			if data_for_year_of_spell[0] == 1:
				dependent_vars.append(0) # 1=exit by decline becomes 0
			if data_for_year_of_spell[0] == 2:
				dependent_vars.append(2) # 2=exit by death stays as 2
			independent_vars.append(numpy.array(data_for_year_of_spell[1:]))
	dependent_vars = numpy.array(dependent_vars)
	independent_vars = numpy.array(independent_vars)
	model = sm.MNLogit(dependent_vars, independent_vars)
	numpy.random.seed(250624) 
	fitted_w_different_base = model.fit(disp=False)
	# get t-stat and p-value
	b_full = fitted_w_different_base.params.flatten('F')
	cov_full = fitted_w_different_base.cov_params()
	stat = numpy.matrix(b_subset - b_full[n:]) * numpy.matrix(cov_subset - cov_full[n:,n:]).I * numpy.matrix(b_subset - b_full[n:]).T
	dof = numpy.linalg.matrix_rank(cov_subset - cov_full[n:,n:])
	# store results
	_ = "t-stat and p-value for ignoring remaining wealthy" 
	results_of_iaa_test[_] = (stat[(0,0)], 1.0 - scipy.stats.chi2.cdf(stat[(0,0)], dof))
	# get relative risk for full model with different base
	eXb2 = numpy.exp(numpy.dot(X, b_subset))
	denominator = 1. + eXb2
	prob_of_exit_by_decline = 1. / denominator
	prob_of_exit_by_death = eXb2 / denominator
	relative_risk_of_exit_by_decline_to_exit_by_death_subset = prob_of_exit_by_decline / prob_of_exit_by_death
	# store results
	_ = "relative risk of exit by decline to exit by death, full model" 
	results_of_iaa_test[_] = relative_risk_of_exit_by_decline_to_exit_by_death
	_ = "relative risk of exit by decline to exit by death, ignoring remaining wealthy" 
	results_of_iaa_test[_] = relative_risk_of_exit_by_decline_to_exit_by_death_subset

	# dump all results
	filename = filepath + "results_of_iaa_test.pkl"
	pickle.dump(results_of_iaa_test, open(filename, 'wb'))
	
else:

	# load results that were dumped using pickle
	results_of_iaa_test = pickle.load(open(filepath + "results_of_iaa_test.pkl", 'rb'))

organized_keys = ['t-stat and p-value for ignoring exit by decline', 
	't-stat and p-value for ignoring exit by death', 
	't-stat and p-value for ignoring remaining wealthy', 
	'relative risk of exit by death to remain wealthy, full model', 
	'relative risk of exit by death to remain wealthy, ignoring exit by decline', 
	'relative risk of exit by decline to remain wealthy, full model', 
	'relative risk of exit by decline to remain wealthy, ignoring exit by death', 
	'relative risk of exit by decline to exit by death, full model', 
	'relative risk of exit by decline to exit by death, ignoring remaining wealthy']

# print results
print "\nresults related to test of IIA assumption in baseline duration model"
for key in organized_keys:
	print key, "\t", results_of_iaa_test[key]

## GIVING IT AWAY? ##

# The following code relates to the extension to the baseline duration model that includes a philanthropic dummy

## GET SURVIVAL DATA FOR PHILANTHROPIC MODEL ##

# load data on Forbes 400 members on the Philanthropy 50 or Slate 60 (note that we only have that data since 1996)
filename = filepath + "philanthropy_50_and_slate_60_data.txt"
header = ['year', 'philanthropy 50 or slate 60 donor name(s)', 'forbes 400 name(s)', 'total amount committed', 'total amount committed in millions']
rawdata = numpy.loadtxt(filename, delimiter='\t', skiprows=19, dtype={'names':(header), 'formats':(['<S100' for _ in range(0, len(header))])})
# get dict of names on Philanthropy 50 and Slate 60 data
years_since_1996 = [str(y) for y in range(1996, 2013+1)]
philanthropists_in = dict(zip(years_since_1996, [[] for year in years_since_1996]))
for i in range(0, len(rawdata['year'])):
	year = rawdata['year'][i]
	if year in years:
		donor_or_donors = rawdata['forbes 400 name(s)'][i]
		# in the data, different names are separated by ' and '
		if ' and ' in donor_or_donors:
			donors = donor_or_donors.split(' and ')
		else:
			donors = [donor_or_donors]
		for name in donors:
			if name in names:
				if data[name]['wealth'][year] != '':
					philanthropists_in[year].append(name)

# philantropy dummy for just the same year or any earlier year?
just_the_same_year = False
# select years
years_since_1996 = [str(y) for y in range(1996, 2013+1)]
years_since_1996_less_initial_and_last = years_since_1996[1:-1]
# keep track of names associated with each spell
names_associated_w_spells = list()
# keep track of ages and ranks
ages_as_dependent_vars, ranks_as_dependent_vars = list(), list()
# list of data for spells
data_for_spells = list()
# for each name
for name in names:
	# get spells
	spells = []
	spell = []
	for year in years:
		wealth = data[name]['wealth'][year]
		if wealth != '':
			spell.append(year)
		else:
			spells.append(spell)
			spell = []
	spells.append(spell)
	# remove empty spells
	while [] in spells:
		spells.remove([])
	# for each spell, record data
	while len(spells) > 0:
		spell = spells.pop()
		# as long as the spell didn't start in the last year
		if spell == [years[-1]]:
			pass
		else:
			# list of data for spell
			data_for_spell = list()
			# for each year of the spell
			for i in range(0, len(spell)):
				# year
				year = spell[i]
				next_year = str(int(year)+1)
				# if year is not the last year
				if year != years[-1]:
					# data for given year of the spell
					data_for_year_of_spell = list()
					# remains wealthy?
					remains_wealthy = int(i != len(spell)-1)
					# exits by decline in wealth?
					exits_by_decline_in_wealth = int(data[name]['wealth'][next_year] == '')
					# exits by death?
					exits_by_death = int(name in dead_in[next_year])
					# exits by renunciation of citizenship?
					exits_by_renunciation_of_citizenship = int(name in renunciants_in[next_year])
					# record dependent variable
					if exits_by_renunciation_of_citizenship:
						dependent_var = 3 # where 3 = exits by renunciation of citizenship:
					elif exits_by_death:
						dependent_var = 2 # where 2 = exits by death
					elif exits_by_decline_in_wealth: 
						dependent_var = 1 # where 1 = exits by decline in wealth
					else: # remains wealthy
						dependent_var = 0 # where 0 = remains wealthy
					data_for_year_of_spell.append(dependent_var)
					# independent variables
					# age
					age = int(data[name]['age'][year])
					age_squared = age * age
					# rank
					rank = summary_stats['wealths'][year].index(float(data[name]['wealth'][year])) + 1
					# left-censored dummy
					left_censored = int('1982' in spell)
					data_for_year_of_spell.extend([1., age, age_squared, rank, left_censored]) # include a constant
					# philanthropists?
					if int(year) > 1995:
						# same year?
						if just_the_same_year:
							philanthropic = int(name in philanthropists_in[year])
						# any earlier year or same year?
						else:
							philanthropic = int(True in [name in philanthropists_in[str(earlier_year)] for earlier_year in range(1996, int(year)+1)])
						# append to data set
						data_for_year_of_spell.append(philanthropic)
					else:
						data_for_year_of_spell.append('-999')
					# dummies for years, but drop 1996
					data_for_year_of_spell.extend([int(year == _) for _ in years_since_1996_less_initial_and_last])
					# dummies for durations of given number of years, except when there is a perfect prediction problem
					duration = (i + 1)
					for _ in range(2, 27+1):
						if 13 <= _ <= 15:
							pass
						else:
							data_for_year_of_spell.append(int(duration == _))
					data_for_year_of_spell.append(int(13 <= duration <= 15)) # 13, 14, or 15 years
					data_for_year_of_spell.append(int(duration > 27)) # 28 or more years
					# add to list
					# ignore spells that end with exit by renunciation of citizenship
					# also, ignore years of spells that are before 1996
					if (data_for_year_of_spell[0] != 3) and (int(year) > 1995):
						data_for_spell.append(data_for_year_of_spell)
						ages_as_dependent_vars.append(age)
						ranks_as_dependent_vars.append(rank)
			# add to list
			if (data_for_year_of_spell[0] != 3) and (int(year) > 1995):
				data_for_spells.append(data_for_spell)
				names_associated_w_spells.append(name)

# No one died after their 14th consecutive year on the list, so we'll make a dummy for 13, 14, or 15 years. 

## ORGANIZE PHILANTHROPIC DATA AND EXPORT IT ##

dependent_vars = list()
independent_vars = list()
for data_for_spell in data_for_spells:
	for data_for_year_of_spell in data_for_spell:
		dependent_vars.append(data_for_year_of_spell[0])
		independent_vars.append(numpy.array(data_for_year_of_spell[1:]))
dependent_vars = numpy.array(dependent_vars)
independent_vars = numpy.array(independent_vars)
# export to file
f = open(filepath + "exported_survival_data_w_philanthropic_just_the_same_year_" + str(just_the_same_year) + ".csv", 'w')
# write header
s = "dependent_var,constant,age,age_squared,rank,left_censored,philanthropic,"
for year in years_since_1996_less_initial_and_last:
	s += "d" + year + ","
for duration in range(2, 27+1):
	if 13 <= duration <=15:
		pass
	else:
		s += "d" + str(duration) + ","
s += "d13_to_15" + ","
s += "d28_or_more" + "\n"
f.write(s)
# write data
for i in range(0, len(dependent_vars)):
	s = str(dependent_vars[i])
	for var in independent_vars[i]:
		s += ','
		s += str(var)
	s += '\n'
	f.write(s)
# close file
f.close()

## ESTIMATE PHILANTHROPIC MODEL ##

model = sm.MNLogit(dependent_vars, independent_vars)
fitted_for_philanthropic = model.fit(disp=False)
#print fitted_for_philanthropic.summary()

# define dict for storing results on probs of exits for a typical person
philanthropic_types = ['on philanthropy 50 list', 'not on philanthropy 50 list']
prob_of_exit_by_philanthropic_type = dict(zip(exits, [dict(zip(['lower', 'point', 'upper'], [dict(zip(philanthropic_types, [None for type in philanthropic_types])) for _ in ['lower', 'point', 'upper']])) for exit in exits]))

# set the following to True to re-estimate if already estimated
re_estimate = False
# check if already been estimated
filename = filepath + _ + "prob_of_exit_by_philanthropic_type_just_the_same_year_" + str(just_the_same_year) + ".pkl"
already_estimated = isfile(filename)
# estimate or re-estimate
if not already_estimated or re_estimate:
	# construct Krinsky-Robb confidence intervals by taking a large number of draws from a multivariate normal with means equal to the estimated parameters and var-covar matrix equal to the estimated var-covar matrix
	numpy.random.seed(250624) 
	redrawn_parameters = numpy.random.multivariate_normal(mean=fitted_for_philanthropic.params.flatten('F'), cov=fitted_for_philanthropic.cov_params(), size=10000)
	# for each type
	for type in philanthropic_types:
		X = [1, 64, 4096, 194, 0] # 64 years of age, rank of 194, no left censoring
		if type == 'on philanthropy 50 list':
			X.append(1)
		else:
			X.append(0)
		X.extend([0 for _ in years_since_1996_less_initial_and_last]) # 1996
		X.extend([0 for _ in range(1, 25+1)]) # duration of one year
		n = len(X)
		# point estimates for probs, given estimated parameters
		eXb1 = numpy.exp(numpy.dot(X, fitted_for_philanthropic.params.flatten('F')[0:n]))
		eXb2 = numpy.exp(numpy.dot(X, fitted_for_philanthropic.params.flatten('F')[n:]))
		denominator = 1. + eXb1 + eXb2
		prob_of_exit_by_philanthropic_type['remain wealthy']['point'][type] = 1. / denominator
		prob_of_exit_by_philanthropic_type['exit by decline']['point'][type] = eXb1 / denominator
		prob_of_exit_by_philanthropic_type['exit by death']['point'][type] = eXb2 / denominator
		# point estimates for probs, given redrawn parameters
		estimates_for = dict(zip(exits, [[] for exit in exits]))
		for b in redrawn_parameters:
			eXb1 = numpy.exp(numpy.dot(X, b[0:n]))
			eXb2 = numpy.exp(numpy.dot(X, b[n:]))
			denominator = 1. + eXb1 + eXb2
			estimates_for['remain wealthy'].append(1. / denominator)
			estimates_for['exit by decline'].append(eXb1 / denominator)
			estimates_for['exit by death'].append(eXb2 / denominator)
		# get 95% CI
		for exit in exits:
			prob_of_exit_by_philanthropic_type[exit]['lower'][type] = scipy.stats.mstats.mquantiles(estimates_for[exit], prob=[2.5/100.], alphap=1/3., betap=1/3.)[0]
			prob_of_exit_by_philanthropic_type[exit]['upper'][type] = scipy.stats.mstats.mquantiles(estimates_for[exit], prob=[97.5/100.], alphap=1/3., betap=1/3.)[0]
	# dump results
	filename = filepath + "prob_of_exit_by_philanthropic_type_just_the_same_year_" + str(just_the_same_year) + ".pkl"
	pickle.dump(prob_of_exit_by_philanthropic_type, open(filename, 'wb'))
else:
	# load results that were dumped using pickle
	prob_of_exit_by_philanthropic_type = pickle.load(open(filepath + "prob_of_exit_by_philanthropic_type_just_the_same_year_" + str(just_the_same_year) + ".pkl", 'rb'))
	
print "\non philanthropy 50 list vs. not on, typical person"
for exit in exits:
	for type in philanthropic_types:
		print exit, "\t", type, "\t", prob_of_exit_by_philanthropic_type[exit]['lower'][type], "\t", prob_of_exit_by_philanthropic_type[exit]['point'][type], "\t", prob_of_exit_by_philanthropic_type[exit]['upper'][type]

## PROFLIGATE HEIRS? ##

# The following code relates to the extension to the baseline duration model that includes a self-made dummy

## GET SURVIVAL DATA FOR SELF-MADE MODEL ##

# keep track of names associated with each spell
names_associated_w_spells = list()
# keep track of ages and ranks
ages_as_dependent_vars, ranks_as_dependent_vars = list(), list()
# list of data for spells
data_for_spells = list()
# for each name
for name in names:
	# get spells
	spells = []
	spell = []
	for year in years:
		wealth = data[name]['wealth'][year]
		if wealth != '':
			spell.append(year)
		else:
			spells.append(spell)
			spell = []
	spells.append(spell)
	# remove empty spells
	while [] in spells:
		spells.remove([])
	# for each spell, record data
	while len(spells) > 0:
		spell = spells.pop()
		# as long as the spell didn't start in the last year
		if spell == [years[-1]]:
			pass
		else:
			# list of data for spell
			data_for_spell = list()
			# for each year of the spell
			for i in range(0, len(spell)):
				# year
				year = spell[i]
				next_year = str(int(year)+1)
				# if year is not the last year
				if year != years[-1]:
					# data for given year of the spell
					data_for_year_of_spell = list()
					# remains wealthy?
					remains_wealthy = int(i != len(spell)-1)
					# exits by decline in wealth?
					exits_by_decline_in_wealth = int(data[name]['wealth'][next_year] == '')
					# exits by death?
					exits_by_death = int(name in dead_in[next_year])
					# exits by renunciation of citizenship?
					exits_by_renunciation_of_citizenship = int(name in renunciants_in[next_year])
					# record dependent variable
					if exits_by_renunciation_of_citizenship:
						dependent_var = 3 # where 3 = exits by renunciation of citizenship:
					elif exits_by_death:
						dependent_var = 2 # where 2 = exits by death
					elif exits_by_decline_in_wealth: 
						dependent_var = 1 # where 1 = exits by decline in wealth
					else: # remains wealthy
						dependent_var = 0 # where 0 = remains wealthy
					data_for_year_of_spell.append(dependent_var)
					# independent variables
					# age
					age = int(data[name]['age'][year])
					age_squared = age * age
					# rank
					rank = summary_stats['wealths'][year].index(float(data[name]['wealth'][year])) + 1
					# left-censored dummy
					left_censored = int('1982' in spell)
					data_for_year_of_spell.extend([1., age, age_squared, rank, left_censored]) # include a constant
					# self-made?
					if int(year) > 1995:
						data_for_year_of_spell.append(int(data[name]['selfmade or inherited'][year] == 'Self made'))
					else:
						data_for_year_of_spell.append('-999')
					# dummies for years, but drop 1996
					data_for_year_of_spell.extend([int(year == _) for _ in years_since_1996_less_initial_and_last])
					# dummies for durations of given number of years, except when there is a perfect prediction problem
					duration = (i + 1)
					for _ in range(2, 27+1):
						if 13 <= _ <= 15:
							pass
						else:
							data_for_year_of_spell.append(int(duration == _))
					data_for_year_of_spell.append(int(13 <= duration <= 15)) # 13, 14, or 15 years
					data_for_year_of_spell.append(int(duration > 27)) # 28 or more years
					# add to list
					# ignore spells that end with exit by renunciation of citizenship
					# also, ignore years of spells that are before 1996
					if (data_for_year_of_spell[0] != 3) and (int(year) > 1995):
						data_for_spell.append(data_for_year_of_spell)
						ages_as_dependent_vars.append(age)
						ranks_as_dependent_vars.append(rank)
			# add to list
			if (data_for_year_of_spell[0] != 3) and (int(year) > 1995):
				data_for_spells.append(data_for_spell)
				names_associated_w_spells.append(name)

# No one died after their 14th consecutive year on the list, so we'll make a dummy for 13, 14, or 15 years

## ORGANIZE SELF-MADE DATA AND EXPORT IT ##

# organize data for estimation
dependent_vars = list()
independent_vars = list()
for data_for_spell in data_for_spells:
	for data_for_year_of_spell in data_for_spell:
		dependent_vars.append(data_for_year_of_spell[0])
		independent_vars.append(numpy.array(data_for_year_of_spell[1:]))
dependent_vars = numpy.array(dependent_vars)
independent_vars = numpy.array(independent_vars)
# export to file
f = open(filepath + "exported_survival_data_w_self_made.csv", 'w')
# write header
s = "dependent_var,constant,age,age_squared,rank,left_censored,self_made,"
for year in years_since_1996_less_initial_and_last:
	s += "d" + year + ","
for duration in range(2, 27+1):
	if 13 <= duration <=15:
		pass
	else:
		s += "d" + str(duration) + ","
s += "d13_to_15" + ","
s += "d28_or_more" + "\n"
f.write(s)
# write data
for i in range(0, len(dependent_vars)):
	s = str(dependent_vars[i])
	for var in independent_vars[i]:
		s += ','
		s += str(var)
	s += '\n'
	f.write(s)
# close file
f.close()

## ESTIMATE SELF-MADE MODEL ##

model = sm.MNLogit(dependent_vars, independent_vars)
fitted_for_self_made = model.fit(disp=False)
#print fitted_for_self_made.summary()

# probs of exits by self made or not
self_made_types = ['self made', 'not self made']
prob_of_exit_by_self_made_type = dict(zip(exits, [dict(zip(['lower', 'point', 'upper'], [dict(zip(self_made_types, [None for type in self_made_types])) for _ in ['lower', 'point', 'upper']])) for exit in exits]))
prob_of_exit_diff_btw_self_made_and_not = dict(zip(exits, [dict(zip(['lower', 'point', 'upper'], [dict(zip(ranks, [None for rank in ranks])) for _ in ['lower', 'point', 'upper']])) for exit in exits]))

# set the following to True to re-estimate confidence intervals if they've already been estimated
re_estimate = False
# check if confidence intervals have already been estimated
filenames = [filepath + _ + ".pkl" for _ in ["prob_of_exit_by_self_made_type", "prob_of_exit_diff_btw_self_made_and_not"]]
already_estimated = False not in [isfile(filename) for filename in filenames]
# estimate or re-estimate
if not already_estimated or re_estimate:
	
	# construct Krinsky-Robb confidence intervals by taking a large number of draws from a multivariate normal with means equal to the estimated parameters and var-covar matrix equal to the estimated var-covar matrix
	numpy.random.seed(250624) 
	redrawn_parameters = numpy.random.multivariate_normal(mean=fitted_for_self_made.params.flatten('F'), cov=fitted_for_self_made.cov_params(), size=10000)
	
	# probs of exits for typical person who is or is not philanthropic
	for type in self_made_types:
		X = [1, 64, 4096, 194, 0] # 64 years of age, rank of 194, no left censoring
		if type == 'self made': # self made or not
			X.append(1)
		else:
			X.append(0)
		X.extend([0 for _ in years_since_1996_less_initial_and_last]) # 1996
		X.extend([0 for _ in range(1, 25+1)]) # duration of one year
		n = len(X)
		# point estimates for probs, given estimated parameters
		eXb1 = numpy.exp(numpy.dot(X, fitted_for_self_made.params.flatten('F')[0:n]))
		eXb2 = numpy.exp(numpy.dot(X, fitted_for_self_made.params.flatten('F')[n:]))
		denominator = 1. + eXb1 + eXb2
		prob_of_exit_by_self_made_type['remain wealthy']['point'][type] = 1. / denominator
		prob_of_exit_by_self_made_type['exit by decline']['point'][type] = eXb1 / denominator
		prob_of_exit_by_self_made_type['exit by death']['point'][type] = eXb2 / denominator
		# point estimates for probs, given redrawn parameters
		estimates_for = dict(zip(exits, [[] for exit in exits]))
		for b in redrawn_parameters:
			eXb1 = numpy.exp(numpy.dot(X, b[0:n]))
			eXb2 = numpy.exp(numpy.dot(X, b[n:]))
			denominator = 1. + eXb1 + eXb2
			estimates_for['remain wealthy'].append(1. / denominator)
			estimates_for['exit by decline'].append(eXb1 / denominator)
			estimates_for['exit by death'].append(eXb2 / denominator)
		# get 95% CI
		for exit in exits:
			prob_of_exit_by_self_made_type[exit]['lower'][type] = scipy.stats.mstats.mquantiles(estimates_for[exit], prob=[2.5/100.], alphap=1/3., betap=1/3.)[0]
			prob_of_exit_by_self_made_type[exit]['upper'][type] = scipy.stats.mstats.mquantiles(estimates_for[exit], prob=[97.5/100.], alphap=1/3., betap=1/3.)[0]

	# 95% CI for differences by rank
	for rank in ranks:
		X = dict(zip(self_made_types, [[1, 64, 4096, rank, 0] for type in self_made_types])) # age of 64, no left censoring
		# self made dummy
		for type in self_made_types:
			if type == 'self made':
				X[type].append(1)
			else:
				X[type].append(0)
		# year is 1996
		year_dummies = [0 for _ in years_since_1996_less_initial_and_last]
		for type in self_made_types:
			X[type].extend(year_dummies)
		# duration dummies
		for type in self_made_types:
			X[type].extend([0 for _ in range(1, 25+1)]) # duration of one year
		n = len(X[self_made_types[0]])
		# point estimates for probs, given estimated parameters
		eXb1 = dict(zip(self_made_types, [None for type in self_made_types]))
		eXb2 = dict(zip(self_made_types, [None for type in self_made_types]))
		denominator = dict(zip(self_made_types, [None for type in self_made_types]))
		for type in self_made_types:
			eXb1[type] = numpy.exp(numpy.dot(X[type], fitted_for_self_made.params.flatten('F')[0:n]))
			eXb2[type] = numpy.exp(numpy.dot(X[type], fitted_for_self_made.params.flatten('F')[n:]))
			denominator[type] = 1. + eXb1[type] + eXb2[type]
		prob_of_exit_diff_btw_self_made_and_not['remain wealthy']['point'][rank] = (1. / denominator['self made']) - (1. / denominator['not self made'])
		prob_of_exit_diff_btw_self_made_and_not['exit by decline']['point'][rank] = (eXb1['self made'] / denominator['self made']) - (eXb1['not self made'] / denominator['not self made'])
		prob_of_exit_diff_btw_self_made_and_not['exit by death']['point'][rank] = (eXb2['self made'] / denominator['self made']) - (eXb2['not self made'] / denominator['not self made'])
		# point estimates for probs, given redrawn parameters
		estimates_for = dict(zip(exits, [[] for exit in exits]))
		for b in redrawn_parameters:
			eXb1 = dict(zip(self_made_types, [None for type in self_made_types]))
			eXb2 = dict(zip(self_made_types, [None for type in self_made_types]))
			denominator = dict(zip(self_made_types, [None for type in self_made_types]))
			for type in self_made_types:
				eXb1[type] = numpy.exp(numpy.dot(X[type], b[0:n]))
				eXb2[type] = numpy.exp(numpy.dot(X[type], b[n:]))
				denominator[type] = 1. + eXb1[type] + eXb2[type]
			estimates_for['remain wealthy'].append((1. / denominator['self made']) - (1. / denominator['not self made']))
			estimates_for['exit by decline'].append((eXb1['self made'] / denominator['self made']) - (eXb1['not self made'] / denominator['not self made']))
			estimates_for['exit by death'].append((eXb2['self made'] / denominator['self made']) - (eXb2['not self made'] / denominator['not self made']))
		# get 95% CI
		for exit in exits:
			prob_of_exit_diff_btw_self_made_and_not[exit]['lower'][rank] = scipy.stats.mstats.mquantiles(estimates_for[exit], prob=[2.5/100.], alphap=1/3., betap=1/3.)[0]
			prob_of_exit_diff_btw_self_made_and_not[exit]['upper'][rank] = scipy.stats.mstats.mquantiles(estimates_for[exit], prob=[97.5/100.], alphap=1/3., betap=1/3.)[0]

	# dump results
	filename = filepath + "prob_of_exit_by_self_made_type.pkl"
	pickle.dump(prob_of_exit_by_self_made_type, open(filename, 'wb'))
	filename = filepath + "prob_of_exit_diff_btw_self_made_and_not.pkl"
	pickle.dump(prob_of_exit_diff_btw_self_made_and_not, open(filename, 'wb'))

else:

	# load results that were dumped using pickle
	prob_of_exit_by_self_made_type = pickle.load(open(filepath + "prob_of_exit_by_self_made_type.pkl", 'rb'))
	# load results that were dumped using pickle
	prob_of_exit_diff_btw_self_made_and_not = pickle.load(open(filepath + "prob_of_exit_diff_btw_self_made_and_not.pkl", 'rb'))

print "\nself made vs. not, typical person"
for exit in exits:
	for type in self_made_types:
		print exit, "\t", type, "\t", prob_of_exit_by_self_made_type[exit]['lower'][type], "\t", prob_of_exit_by_self_made_type[exit]['point'][type], "\t", prob_of_exit_by_self_made_type[exit]['upper'][type]

print "\ndifference by rank"
for exit in exits:
	for rank in ranks:
		print exit, "\t", rank, "\t", 
		print prob_of_exit_diff_btw_self_made_and_not[exit]['lower'][rank], "\t", prob_of_exit_diff_btw_self_made_and_not[exit]['point'][rank], "\t", prob_of_exit_diff_btw_self_made_and_not[exit]['upper'][rank]

## A GREAT RECESSION? ##

# The following code relates to the extension to the baseline duration model that includes a FIRE industry dummy and interaction terms between that dummy and the year dummies

## GET SURVIVAL DATA FOR FIRE MODEL ##

# no one in the FIRE industry died in either 1999, 2000, or 2012, so we'll just focus on the years between 2001 and 2011
years_from_2001_to_2011 = [str(y) for y in range(2001, 2011+1)]
# keep track of names associated with each spell
names_associated_w_spells = list()
# keep track of ages and ranks
ages_as_dependent_vars, ranks_as_dependent_vars = list(), list()
# list of data for spells
data_for_spells = list()
# for each name
for name in names:
	# get spells
	spells = []
	spell = []
	for year in years:
		wealth = data[name]['wealth'][year]
		if wealth != '':
			spell.append(year)
		else:
			spells.append(spell)
			spell = []
	spells.append(spell)
	# remove empty spells
	while [] in spells:
		spells.remove([])
	# for each spell, record data
	while len(spells) > 0:
		spell = spells.pop()
		# as long as the spell didn't start in the last year
		if spell == [years[-1]]:
			pass
		else:
			# list of data for spell
			data_for_spell = list()
			# for each year of the spell
			for i in range(0, len(spell)):
				# year
				year = spell[i]
				next_year = str(int(year)+1)
				# if year is not the last year
				if year != years[-1]:
					# data for given year of the spell
					data_for_year_of_spell = list()
					# remains wealthy?
					remains_wealthy = int(i != len(spell)-1)
					# exits by decline in wealth?
					exits_by_decline_in_wealth = int(data[name]['wealth'][next_year] == '')
					# exits by death?
					exits_by_death = int(name in dead_in[next_year])
					# exits by renunciation of citizenship?
					exits_by_renunciation_of_citizenship = int(name in renunciants_in[next_year])
					# record dependent variable
					if exits_by_renunciation_of_citizenship:
						dependent_var = 3 # where 3 = exits by renunciation of citizenship:
					elif exits_by_death:
						dependent_var = 2 # where 2 = exits by death
					elif exits_by_decline_in_wealth: 
						dependent_var = 1 # where 1 = exits by decline in wealth
						# would he or she been on the list with the same wealth as last year?
						#if not float(data[name]['wealth'][year]) >= summary_stats['min'][next_year]:
						#	print name, "\t", year, "\t", data[name]['wealth'][year], "\t", summary_stats['min'][next_year]
					else: # remains wealthy
						dependent_var = 0 # where 0 = remains wealthy
					data_for_year_of_spell.append(dependent_var)
					# independent variables
					# age
					age = int(data[name]['age'][year])
					age_squared = age * age
					# rank
					rank = summary_stats['wealths'][year].index(float(data[name]['wealth'][year])) + 1
					# left-censored dummy
					left_censored = int('1982' in spell)
					data_for_year_of_spell.extend([1., age, age_squared, rank, left_censored]) # include a constant
					# ` dummy and fire-year interaction dummies
					if int(year) > 1995:
						# fire dummy
						industry = data[name]['industry'][year]
						fire_industry = int(industry in ['Finance', 'Investments', 'Real estate'])
						data_for_year_of_spell.append(fire_industry)
						# fire-year interaction dummies
						for _ in years_from_2001_to_2011:
							data_for_year_of_spell.append(int(fire_industry * int(year == _)))
					else:
						data_for_year_of_spell.append('-999')
						for _ in years_from_2001_to_2011:
							data_for_year_of_spell.append('-999')					
					# dummies for years, but drop 1996
					data_for_year_of_spell.extend([int(year == _) for _ in years_since_1996_less_initial_and_last])
					# dummies for durations of given number of years, except when there is a perfect prediction problem
					duration = (i + 1)
					for _ in range(2, 27+1):
						if 13 <= _ <= 15:
							pass
						else:
							data_for_year_of_spell.append(int(duration == _))
					data_for_year_of_spell.append(int(13 <= duration <= 15)) # 13, 14, or 15 years
					data_for_year_of_spell.append(int(duration > 27)) # 28 or more years
					# add to list
					# ignore spells that end with exit by renunciation of citizenship
					# also, ignore years of spells that are before 1996
					if (data_for_year_of_spell[0] != 3) and (int(year) > 1995):
						data_for_spell.append(data_for_year_of_spell)
						ages_as_dependent_vars.append(age)
						ranks_as_dependent_vars.append(rank)
			# add to list
			if (data_for_year_of_spell[0] != 3) and (int(year) > 1995):
				data_for_spells.append(data_for_spell)
				names_associated_w_spells.append(name)

# No one died after their 14th consecutive year on the list, so we'll make a dummy for 13, 14, or 15 years

## ORGANIZE FIRE DATA AND EXPORT IT ##

# organize data for estimation
dependent_vars = list()
independent_vars = list()
for data_for_spell in data_for_spells:
	for data_for_year_of_spell in data_for_spell:
		dependent_vars.append(data_for_year_of_spell[0])
		independent_vars.append(numpy.array(data_for_year_of_spell[1:]))
dependent_vars = numpy.array(dependent_vars)
independent_vars = numpy.array(independent_vars)
# export to file
f = open(filepath + "exported_survival_data_w_fire.csv", 'w')
# write header
s = "dependent_var,constant,age,age_squared,rank,left_censored,fire,"
for year in years_from_2001_to_2011:
	s += "fire" + "d" + year + ","
for year in years_since_1996_less_initial_and_last:
	s += "d" + year + ","
for duration in range(2, 27+1):
	if 13 <= duration <=15:
		pass
	else:
		s += "d" + str(duration) + ","
s += "d13_to_15" + ","
s += "d28_or_more" + "\n"
f.write(s)
# write data
for i in range(0, len(dependent_vars)):
	s = str(dependent_vars[i])
	for var in independent_vars[i]:
		s += ','
		s += str(var)
	s += '\n'
	f.write(s)
# close file
f.close()

## ESTIMATE FIRE MODEL ##

model = sm.MNLogit(dependent_vars, independent_vars)
fitted_for_fire = model.fit(disp=False)
#print fitted_for_fire.summary()

# probs of exits by fire industry or not
industries = ['fire', 'other']
prob_of_exit_by_industry = dict(zip(exits, [dict(zip(['lower', 'point', 'upper'], [dict(zip(years_from_2001_to_2011, [dict(zip(industries, [None for industry in industries])) for year in years_from_2001_to_2011])) for _ in ['lower', 'point', 'upper']])) for exit in exits]))
prob_of_exit_diff_btw_fire_and_other = dict(zip(exits, [dict(zip(['lower', 'point', 'upper'], [dict(zip(years_from_2001_to_2011, [None for year in years_from_2001_to_2011])) for _ in ['lower', 'point', 'upper']])) for exit in exits]))
# set the following to True to re-estimate confidence intervals if they've already been estimated
re_estimate = False
# check if confidence intervals have already been estimated
filenames = [filepath + _ + ".pkl" for _ in ["prob_of_exit_by_industry", "prob_of_exit_diff_btw_fire_and_other"]]
already_estimated = False not in [isfile(filename) for filename in filenames]
# estimate or re-estimate
if not already_estimated or re_estimate:

	# construct Krinsky-Robb confidence intervals by taking a large number of draws from a multivariate normal with means equal to the estimated parameters and var-covar matrix equal to the estimated var-covar matrix
	numpy.random.seed(250624) 
	redrawn_parameters = numpy.random.multivariate_normal(mean=fitted_for_fire.params.flatten('F'), cov=fitted_for_fire.cov_params(), size=10000)
	import scipy.stats # for percentile confidence intervals

	# probs of exits for each industry and each year
	for industry in industries:
		for year in years_from_2001_to_2011:
			X = [1, 64, 4096, 194, 0] # 64 years of age, rank of 194, no left censoring
			# fire dummy and fire-year interaction terms
			if industry == 'fire':
				X.append(1)
				fire_year_interaction_terms = [0 for _ in years_from_2001_to_2011]
				if year == '1996':
					pass
				else:
					fire_year_interaction_terms[years_from_2001_to_2011.index(year)] = 1
			else:
				X.append(0)
				fire_year_interaction_terms = [0 for _ in years_from_2001_to_2011]
			X.extend(fire_year_interaction_terms)
			# year dummies	
			year_dummies = [0 for _ in years_since_1996_less_initial_and_last]
			if year == '1996':
				pass
			else:
				year_dummies[years_since_1996_less_initial_and_last.index(year)] = 1
			X.extend(year_dummies)
			# duration dummies
			X.extend([0 for _ in range(1, 25+1)]) # duration of one year
			n = len(X)
			# point estimates for probs, given estimated parameters
			eXb1 = numpy.exp(numpy.dot(X, fitted_for_fire.params.flatten('F')[0:n]))
			eXb2 = numpy.exp(numpy.dot(X, fitted_for_fire.params.flatten('F')[n:]))
			denominator = 1. + eXb1 + eXb2
			prob_of_exit_by_industry['remain wealthy']['point'][year][industry] = 1. / denominator
			prob_of_exit_by_industry['exit by decline']['point'][year][industry] = eXb1 / denominator
			prob_of_exit_by_industry['exit by death']['point'][year][industry] = eXb2 / denominator
			# point estimates for probs, given redrawn parameters
			estimates_for = dict(zip(exits, [[] for exit in exits]))
			for b in redrawn_parameters:
				eXb1 = numpy.exp(numpy.dot(X, b[0:n]))
				eXb2 = numpy.exp(numpy.dot(X, b[n:]))
				denominator = 1. + eXb1 + eXb2
				estimates_for['remain wealthy'].append(1. / denominator)
				estimates_for['exit by decline'].append(eXb1 / denominator)
				estimates_for['exit by death'].append(eXb2 / denominator)
			# get 95% CI
			for exit in exits:
				prob_of_exit_by_industry[exit]['lower'][year][industry] = scipy.stats.mstats.mquantiles(estimates_for[exit], prob=[2.5/100.], alphap=1/3., betap=1/3.)[0]
				prob_of_exit_by_industry[exit]['upper'][year][industry] = scipy.stats.mstats.mquantiles(estimates_for[exit], prob=[97.5/100.], alphap=1/3., betap=1/3.)[0]

	# 95% CI for differences by year
	for year in years_from_2001_to_2011:
		X = dict(zip(industries, [[1, 64, 4096, 194, 0] for industry in industries])) # 64 years of age, rank of 194, no left censoring
		# fire dummy and fire-year interaction terms
		for industry in industries:
			if industry == 'fire':
				X[industry].append(1)
				fire_year_interaction_terms = [0 for _ in years_from_2001_to_2011]
				if year == '1996':
					pass
				else:
					fire_year_interaction_terms[years_from_2001_to_2011.index(year)] = 1
			else:
				X[industry].append(0)
				fire_year_interaction_terms = [0 for _ in years_from_2001_to_2011]
			X[industry].extend(fire_year_interaction_terms)
		# year dummies
		year_dummies = [0 for _ in years_since_1996_less_initial_and_last]
		if year == '1996':
			pass
		else:
			year_dummies[years_since_1996_less_initial_and_last.index(year)] = 1
		for industry in industries:
			X[industry].extend(year_dummies)
		# duration dummies
		for industry in industries:
			X[industry].extend([0 for _ in range(1, 25+1)]) # duration of one year
		n = len(X[industries[0]])
		# point estimates for probs, given estimated parameters
		eXb1 = dict(zip(industries, [None for industry in industries]))
		eXb2 = dict(zip(industries, [None for industry in industries]))
		denominator = dict(zip(industries, [None for industry in industries]))
		for industry in industries:
			eXb1[industry] = numpy.exp(numpy.dot(X[industry], fitted_for_fire.params.flatten('F')[0:n]))
			eXb2[industry] = numpy.exp(numpy.dot(X[industry], fitted_for_fire.params.flatten('F')[n:]))
			denominator[industry] = 1. + eXb1[industry] + eXb2[industry]
		prob_of_exit_diff_btw_fire_and_other['remain wealthy']['point'][year] = (1. / denominator['fire']) - (1. / denominator['other'])
		prob_of_exit_diff_btw_fire_and_other['exit by decline']['point'][year] = (eXb1['fire'] / denominator['fire']) - (eXb1['other'] / denominator['other'])
		prob_of_exit_diff_btw_fire_and_other['exit by death']['point'][year] = (eXb2['fire'] / denominator['fire']) - (eXb2['other'] / denominator['other'])
		# point estimates for probs, given redrawn parameters
		estimates_for = dict(zip(exits, [[] for exit in exits]))
		for b in redrawn_parameters:
			eXb1 = dict(zip(industries, [None for industry in industries]))
			eXb2 = dict(zip(industries, [None for industry in industries]))
			denominator = dict(zip(industries, [None for industry in industries]))
			for industry in industries:
				eXb1[industry] = numpy.exp(numpy.dot(X[industry], b[0:n]))
				eXb2[industry] = numpy.exp(numpy.dot(X[industry], b[n:]))
				denominator[industry] = 1. + eXb1[industry] + eXb2[industry]
			estimates_for['remain wealthy'].append((1. / denominator['fire']) - (1. / denominator['other']))
			estimates_for['exit by decline'].append((eXb1['fire'] / denominator['fire']) - (eXb1['other'] / denominator['other']))
			estimates_for['exit by death'].append((eXb2['fire'] / denominator['fire']) - (eXb2['other'] / denominator['other']))
		# get 95% CI
		for exit in exits:
			prob_of_exit_diff_btw_fire_and_other[exit]['lower'][year] = scipy.stats.mstats.mquantiles(estimates_for[exit], prob=[2.5/100.], alphap=1/3., betap=1/3.)[0]
			prob_of_exit_diff_btw_fire_and_other[exit]['upper'][year] = scipy.stats.mstats.mquantiles(estimates_for[exit], prob=[97.5/100.], alphap=1/3., betap=1/3.)[0]

	# dump results
	filename = filepath + "prob_of_exit_by_industry.pkl"
	pickle.dump(prob_of_exit_by_industry, open(filename, 'wb'))
	filename = filepath + "prob_of_exit_diff_btw_fire_and_other.pkl"
	pickle.dump(prob_of_exit_diff_btw_fire_and_other, open(filename, 'wb'))

else:

	# load results that were dumped using pickle
	prob_of_exit_by_industry = pickle.load(open(filepath + "prob_of_exit_by_industry.pkl", 'rb'))
	# load results that were dumped using pickle
	prob_of_exit_diff_btw_fire_and_other = pickle.load(open(filepath + "prob_of_exit_diff_btw_fire_and_other.pkl", 'rb'))

# print results
print "\n\t\t", "fire", "\t\t\t", "other", "\t\t\t", "difference"
for exit in exits:
	for year in years_from_2001_to_2011:
		print exit, "\t", year, "\t", prob_of_exit_by_industry[exit]['lower'][year]['fire'], "\t", prob_of_exit_by_industry[exit]['point'][year]['fire'], "\t", prob_of_exit_by_industry[exit]['upper'][year]['fire'], "\t",
		print prob_of_exit_by_industry[exit]['lower'][year]['other'], "\t", prob_of_exit_by_industry[exit]['point'][year]['other'], "\t", prob_of_exit_by_industry[exit]['upper'][year]['other'], "\t", 
		print prob_of_exit_diff_btw_fire_and_other[exit]['lower'][year], "\t", prob_of_exit_diff_btw_fire_and_other[exit]['point'][year], "\t", prob_of_exit_diff_btw_fire_and_other[exit]['upper'][year]

## lastline=1673
if __name__ == '__main__':

	#### FIGURES ####

	import pylab
	from matplotlib import rc
	from matplotlib import lines
	from matplotlib.ticker import MultipleLocator

	# size of figures
	figwidth = 430.00627 / 72.27 # \textwidth in pts times pts per inch, following cookbook example 
	figheight = figwidth * (numpy.sqrt(5)-1.0) / 2.0 # golden ratio, following cookbook example
	figsize = (figwidth, figheight)

	rc('text', usetex=True)
	#rc('font',**{'family':'sans-serif','sans-serif':['Computer Modern Sans Serif']}) # for beamer
	#rc('font', family='sans-serif')
	rc('font',**{'family':'serif','serif':['Computer Modern Roman']}) # for document
	rc('font', family='serif')
	fontsize = 11
	pylab.rcParams['font.size'] = fontsize
	pylab.rcParams['axes.labelsize'] = fontsize
	pylab.rcParams['legend.fontsize'] = fontsize
	pylab.rcParams['xtick.labelsize'] = fontsize
	pylab.rcParams['ytick.labelsize'] = fontsize

	## FIG. 4.1: NUMBER OF EXITS BY TYPE BY YEAR ##

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=figsize)
	fig_ax = fig.gca()
	fig_ax.plot([int(dyear.split('--')[0]) for dyear in dyears], list(n_by_dyear_types_in[dyear]['remain wealthy'] for dyear in dyears), color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='Appeared again') # no markers if more than 10 data points
	fig_ax.plot([int(dyear.split('--')[0]) for dyear in dyears], list(n_by_dyear_types_in[dyear]['exit by decline'] for dyear in dyears), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label='Dropped off due to decline') 
	fig_ax.plot([int(dyear.split('--')[0]) for dyear in dyears], list(n_by_dyear_types_in[dyear]['exit by death'] for dyear in dyears), color='k', linestyle='dotted', lw=0.5, marker='o', markersize=0, clip_on=True, label='Died')
	# make a legend
	leg = fig_ax.legend(bbox_to_anchor=(0.0, 1.02, 1.0, 0.10), 
		loc='lower center', ncol=3, borderaxespad=0.0, 
		handlelength=2.2, handletextpad=0.25, columnspacing=0.5, 
		mode="expand", frameon=True)
	leg.get_frame().set_linewidth(0.5)
	# label the axes
	#fig_ax.set_xlabel('Year') # sentence-style capitalization
	fig_ax.set_ylabel('Number of people')
	# set limits
	pylab.xlim(1982, 2012)
	pylab.ylim(0.0, 400.0) #pylab.ylim(-0.15, 1.15)
	# major xticks
	pylab.xticks([1982, 1992, 2002, 2012], [r'\centering \begin{flushright}1982\\--1983\end{flushright}', r'\centering \begin{flushright}1992\\--1993\end{flushright}', r'\centering \begin{flushright}2002\\--2003\end{flushright}', r'\centering \begin{flushright}2012\\--2013\end{flushright}'])
	# minor xticks
	fig_ax.xaxis.set_minor_locator(MultipleLocator(1)) # tick every year
	# major yticks
	pylab.yticks([0, 200, 400], ['0', '200', '400'])
	# minor yticks
	#fig_ax.yaxis.set_minor_locator(MultipleLocator(100)) # tick every 100
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# adjust dimensions
	fig.subplots_adjust(left=0.12, bottom=0.14, right=0.96, top=0.90, wspace=0.0, hspace=0.0)
	# save figure
	pylab.savefig("./figures/figures_for_third_essay/number_of_exits_by_type_by_year.pdf")

	## FIG. 4.2: INVERSE HAZARD FUNCTION ##

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=figsize)
	fig_ax = fig.gca()
	# re-define durations
	durations = range(1, len(years), 1)
	# plot confidence intervals
	fig_ax.plot(durations[:-1], list(1. - upper_ci_for_hazard_function[duration] for duration in durations[:-1]), color='k', linestyle='dotted', lw=0.5, marker='o', markersize=0, clip_on=True, label='95\% CI')
	fig_ax.plot(durations[:-1], list(1. - lower_ci_for_hazard_function[duration] for duration in durations[:-1]), color='k', linestyle='dotted', lw=0.5, marker='o', markersize=0, clip_on=True)
	fig_ax.plot(durations[:-1], list(1. - point_estimate_for_hazard_function[duration] for duration in durations[:-1]), color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True) # no markers if more than 10 data points
	# make a legend
	leg = fig_ax.legend(loc='upper left', frameon=False, handlelength=3.3, handletextpad=0.5)
	# label the axes
	fig_ax.set_xlabel('Consecutive appearances') # sentence-style capitalization
	fig_ax.set_ylabel('Probability')
	# set limits
	pylab.xlim(1, 29)
	pylab.ylim(0.75, 1.0)
	# major xticks
	pylab.xticks([1, 5, 10, 15, 20, 25, 29], ['1', '5', '10', '15', '20', '25', '29'])
	# minor xticks
	fig_ax.xaxis.set_minor_locator(MultipleLocator(1)) # tick every year
	# major yticks
	pylab.yticks([0.75, 1.0], ['0.75', '1.0'])
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# adjust dimensions
	fig.subplots_adjust(left=0.11, bottom=0.14, right=0.97, top=0.90, wspace=0.0, hspace=0.0)
	# save figure
	pylab.savefig("./figures/figures_for_third_essay/inverse_hazard_rates.pdf", bbox_inches='tight')

	## FIG. 4.3: SURVIVAL FUNCTION ##

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=figsize)
	fig_ax = fig.gca()
	# re-define durations
	durations = range(1, len(years), 1)
	# plot survival curve as a step function or line?
	plot_as_step_function = False
	# plot as step function
	if plot_as_step_function:
		fig_ax.plot(durations, list(upper_ci_for_survivor_function[duration] for duration in durations), color='k', drawstyle='steps-post', linestyle='dotted', lw=0.5, marker='o', markersize=0, clip_on=True, label='95\% CI')
		fig_ax.plot(durations, list(lower_ci_for_survivor_function[duration] for duration in durations), color='k', drawstyle='steps-post', linestyle='dotted', lw=0.5, marker='o', markersize=0, clip_on=True)
		fig_ax.plot(durations, list(point_estimate_for_survivor_function[duration] for duration in durations), color='k', drawstyle='steps-post', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True) # no markers if more than 10 data points
	# plot as line
	else:
		fig_ax.plot(durations, list(upper_ci_for_survivor_function[duration] for duration in durations), color='k', linestyle='dotted', lw=0.5, marker='o', markersize=0, clip_on=True, label='95\% CI')
		fig_ax.plot(durations, list(lower_ci_for_survivor_function[duration] for duration in durations), color='k', linestyle='dotted', lw=0.5, marker='o', markersize=0, clip_on=True)
		fig_ax.plot(durations, list(point_estimate_for_survivor_function[duration] for duration in durations), color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True) # no markers if more than 10 data points
	# make a legend
	leg = fig_ax.legend(loc='upper right', frameon=False, handlelength=3.3, handletextpad=0.5)
	# label the axes
	fig_ax.set_xlabel('Consecutive appearances') # sentence-style capitalization
	fig_ax.set_ylabel('Probability')
	# set limits
	pylab.xlim(1, 31)
	pylab.ylim(0.0, 1.0)
	# major xticks
	pylab.xticks([1, 5, 10, 15, 20, 25, 30], ['1', '5', '10', '15', '20', '25', '30'])
	# minor xticks
	fig_ax.xaxis.set_minor_locator(MultipleLocator(1)) # tick every year
	# major yticks
	pylab.yticks([0, .5, 1], [r'0.0', r'0.5', r'1.0'])
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# adjust dimensions
	fig.subplots_adjust(left=0.11, bottom=0.14, right=0.97, top=0.90, wspace=0.0, hspace=0.0)
	# save figure
	pylab.savefig("./figures/figures_for_third_essay/survival_curve.pdf", bbox_inches='tight')

	## FIG. 4.4: PROBABILITY OF EXIT BY AGE ##

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=figsize)
	fig_ax = fig.gca()
	# plot confidence intervals
	fig_ax.plot([int(age) for age in ages], list(prob_of_exit_by_age['remain wealthy']['upper'][age] for age in ages), color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='Appear again') # no markers if more than 10 data points
	fig_ax.plot([int(age) for age in ages], list(prob_of_exit_by_age['exit by decline']['upper'][age] for age in ages), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label='Drop off due to decline') 
	fig_ax.plot([int(age) for age in ages], list(prob_of_exit_by_age['exit by death']['upper'][age] for age in ages), color='k', linestyle='dotted', lw=0.5, marker='o', markersize=0, clip_on=True, label='Die')
	fig_ax.plot([int(age) for age in ages], list(prob_of_exit_by_age['remain wealthy']['lower'][age] for age in ages), color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True)
	fig_ax.plot([int(age) for age in ages], list(prob_of_exit_by_age['exit by decline']['lower'][age] for age in ages), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True)
	fig_ax.plot([int(age) for age in ages], list(prob_of_exit_by_age['exit by death']['lower'][age] for age in ages), color='k', linestyle='dotted', lw=0.5, marker='o', markersize=0, clip_on=True)
	# make a legend
	leg = fig_ax.legend(bbox_to_anchor=(0.0, 1.02, 1.0, 0.10), \
		loc='lower center', ncol=3, borderaxespad=0.0, \
		handlelength=2.2, handletextpad=0.25, columnspacing=0.5, \
		mode="expand", frameon=True)
	leg.get_frame().set_linewidth(0.5)
	# label the axes
	fig_ax.set_xlabel('Age') # sentence-style capitalization
	fig_ax.set_ylabel('Probability')
	# set limits
	pylab.xlim(20, 100)
	pylab.ylim(0.0, 1.0)
	# add ticks
	pylab.xticks([20, 40, 60, 80, 100], ['20', '40', '60', '80', '100'])
	pylab.yticks([0, .5, 1], [r'0.0', r'0.5', r'1.0'])
	# minor xticks
	fig_ax.xaxis.set_minor_locator(MultipleLocator(10)) # tick every 10 ages
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# adjust dimensions
	fig.subplots_adjust(left=0.11, bottom=0.14, right=0.97, top=0.90, wspace=0.0, hspace=0.0)
	# save figure
	pylab.savefig("./figures/figures_for_third_essay/probs_of_exits_by_age.pdf")#, bbox_inches='tight')

	## FIG. 4.5: PROB OF EXIT BY RANK ##

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=figsize)
	fig_ax = fig.gca()
	# plot confidence intervals
	fig_ax.plot([int(rank) for rank in ranks], list(prob_of_exit_by_rank['remain wealthy']['upper'][rank] for rank in ranks), color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='Appear again') # no markers if more than 10 data points
	fig_ax.plot([int(rank) for rank in ranks], list(prob_of_exit_by_rank['exit by decline']['upper'][rank] for rank in ranks), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label='Drop off due to decline') 
	fig_ax.plot([int(rank) for rank in ranks], list(prob_of_exit_by_rank['exit by death']['upper'][rank] for rank in ranks), color='k', linestyle='dotted', lw=0.5, marker='o', markersize=0, clip_on=True, label='Die')
	fig_ax.plot([int(rank) for rank in ranks], list(prob_of_exit_by_rank['remain wealthy']['lower'][rank] for rank in ranks), color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True)
	fig_ax.plot([int(rank) for rank in ranks], list(prob_of_exit_by_rank['exit by decline']['lower'][rank] for rank in ranks), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True)
	fig_ax.plot([int(rank) for rank in ranks], list(prob_of_exit_by_rank['exit by death']['lower'][rank] for rank in ranks), color='k', linestyle='dotted', lw=0.5, marker='o', markersize=0, clip_on=True)
	# make a legend
	leg = fig_ax.legend(bbox_to_anchor=(0.0, 1.02, 1.0, 0.10), \
		loc='lower center', ncol=3, borderaxespad=0.0, \
		handlelength=2.2, handletextpad=0.25, columnspacing=0.5, \
		mode="expand", frameon=True)
	leg.get_frame().set_linewidth(0.5)
	# label the axes
	fig_ax.set_xlabel('Rank') # sentence-style capitalization
	fig_ax.set_ylabel('Probability')
	# set limits
	pylab.xlim(1, 400)
	pylab.ylim(0.0, 1.0)
	# add ticks
	pylab.xticks([1, 100, 200, 300, 400], ['1rst', '100th', '200th', '300th', '400th'])
	pylab.yticks([0, .5, 1], [r'0.0', r'0.5', r'1.0'])
	# minor xticks
	fig_ax.xaxis.set_minor_locator(MultipleLocator(50)) # tick every 50 ranks
	# invert x axis
	fig_ax.invert_xaxis()
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# adjust dimensions
	fig.subplots_adjust(left=0.11, bottom=0.14, right=0.97, top=0.90, wspace=0.0, hspace=0.0)
	# save figure
	pylab.savefig("./figures/figures_for_third_essay/probs_of_exits_by_rank.pdf")#, bbox_inches='tight')

	## FIG. 4.6: PROB OF EXIT BY YEAR, SPLITTING INTO TWO SUBPLOTS ##

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=figsize)
	# dimensions
	left = 0.12
	bottom = 0.14
	right = 0.975
	top = 0.90
	middle_space = 0.06
	bottom_height = ((top - bottom) / 2.) - (middle_space / 2.)
	width = right - left
	top_plot_dimensions = [left, bottom + bottom_height + middle_space, width, bottom_height] # [left, bottom, width, height]
	bottom_plot_dimensions = [left, bottom, width, bottom_height] # [left, bottom, width, height]
	# top plot
	fig.add_axes(top_plot_dimensions) # [left, bottom, width, height]
	fig_ax = fig.gca()
	fig_ax.plot([int(year) for year in years_less_last], list(prob_of_exit_by_year['remain wealthy']['upper'][year] for year in years_less_last), color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='Appear again') # no markers if more than 10 data points
	fig_ax.plot([int(year) for year in years_less_last], list(prob_of_exit_by_year['exit by decline']['upper'][year] for year in years_less_last), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label='Drop off due to decline') 
	fig_ax.plot([int(year) for year in years_less_last], list(prob_of_exit_by_year['exit by death']['upper'][year] for year in years_less_last), color='k', linestyle='dotted', lw=0.5, marker='o', markersize=0, clip_on=True, label='Die')
	fig_ax.plot([int(year) for year in years_less_last], list(prob_of_exit_by_year['remain wealthy']['lower'][year] for year in years_less_last), color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True)
	fig_ax.plot([int(year) for year in years_less_last], list(prob_of_exit_by_year['exit by decline']['lower'][year] for year in years_less_last), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True)
	fig_ax.plot([int(year) for year in years_less_last], list(prob_of_exit_by_year['exit by death']['lower'][year] for year in years_less_last), color='k', linestyle='dotted', lw=0.5, marker='o', markersize=0, clip_on=True)
	# make a legend
	leg = fig_ax.legend(bbox_to_anchor=(0.0, 1.05, 1.0, 0.10), \
		loc='lower center', ncol=3, borderaxespad=0.0, \
		handlelength=2.2, handletextpad=0.25, columnspacing=0.5, \
		mode="expand", frameon=True)
	leg.get_frame().set_linewidth(0.5)
	# label the axes
	#fig_ax.set_xlabel('Year') # sentence-style capitalization
	fig_ax.set_ylabel('Probability')
	# set limits
	pylab.xlim(1982, 2011)
	pylab.ylim(0.80, 1.0)
	# major xticks
	pylab.xticks([1982, 1992, 2002, 2011], ['', '', '', ''])
	# minor xticks
	fig_ax.xaxis.set_minor_locator(MultipleLocator(1)) # tick every year
	# major yticks
	pylab.yticks([0.75, 1], [r'0.75', r'1.0'])
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# bottom plot
	fig.add_axes(bottom_plot_dimensions)
	fig_ax = fig.gca()
	fig_ax.plot([int(year) for year in years_less_last], list(prob_of_exit_by_year['remain wealthy']['upper'][year] for year in years_less_last), color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='Appear again') # no markers if more than 10 data points
	fig_ax.plot([int(year) for year in years_less_last], list(prob_of_exit_by_year['exit by decline']['upper'][year] for year in years_less_last), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label='Drop off due to decline') 
	fig_ax.plot([int(year) for year in years_less_last], list(prob_of_exit_by_year['exit by death']['upper'][year] for year in years_less_last), color='k', linestyle='dotted', lw=0.5, marker='o', markersize=0, clip_on=True, label='Die')
	fig_ax.plot([int(year) for year in years_less_last], list(prob_of_exit_by_year['remain wealthy']['lower'][year] for year in years_less_last), color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True)
	fig_ax.plot([int(year) for year in years_less_last], list(prob_of_exit_by_year['exit by decline']['lower'][year] for year in years_less_last), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True)
	fig_ax.plot([int(year) for year in years_less_last], list(prob_of_exit_by_year['exit by death']['lower'][year] for year in years_less_last), color='k', linestyle='dotted', lw=0.5, marker='o', markersize=0, clip_on=True)
	# label the axes
	fig_ax.set_xlabel('Year') # sentence-style capitalization
	fig_ax.set_ylabel('Probability')
	# set limits
	pylab.xlim(1982, 2011)
	pylab.ylim(0.0, 0.25)
	# major xticks
	pylab.xticks([1982, 1992, 2002, 2011], ['1982', '1992', '2002', '2011'])
	# minor xticks
	fig_ax.xaxis.set_minor_locator(MultipleLocator(1)) # tick every year
	# major yticks
	pylab.yticks([0, .25], [r'0.0', r'0.25'])
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04-0.02) # extra -0.02
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# save figure
	pylab.savefig("./figures/figures_for_third_essay/probs_of_exits_by_year_split.pdf")

	## FIG. 4.7: PROB OF EXIT BY DURATION, SPLITTING INTO TWO SUBPLOTS ##

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=figsize)
	# re-define durations
	durations = range(1, 28+1)
	# dimensions
	left = 0.12
	bottom = 0.14
	right = 0.975
	top = 0.90
	middle_space = 0.06
	bottom_height = ((top - bottom) / 2.) - (middle_space / 2.)
	width = right - left
	top_plot_dimensions = [left, bottom + bottom_height + middle_space, width, bottom_height] # [left, bottom, width, height]
	bottom_plot_dimensions = [left, bottom, width, bottom_height] # [left, bottom, width, height]
	# top plot
	fig.add_axes(top_plot_dimensions) # [left, bottom, width, height]
	fig_ax = fig.gca()
	fig_ax.plot(range(1, len(durations)+1), list(prob_of_exit_by_duration['remain wealthy']['upper'][duration] for duration in durations), color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='Appear again') # no markers if more than 10 data points
	fig_ax.plot(range(1, len(durations)+1), list(prob_of_exit_by_duration['exit by decline']['upper'][duration] for duration in durations), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label='Drop off due to decline') 
	fig_ax.plot(range(1, len(durations)+1), list(prob_of_exit_by_duration['exit by death']['upper'][duration] for duration in durations), color='k', linestyle='dotted', lw=0.5, marker='o', markersize=0, clip_on=True, label='Die')
	fig_ax.plot(range(1, len(durations)+1), list(prob_of_exit_by_duration['remain wealthy']['lower'][duration] for duration in durations), color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True)
	fig_ax.plot(range(1, len(durations)+1), list(prob_of_exit_by_duration['exit by decline']['lower'][duration] for duration in durations), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True)
	fig_ax.plot(range(1, len(durations)+1), list(prob_of_exit_by_duration['exit by death']['lower'][duration] for duration in durations), color='k', linestyle='dotted', lw=0.5, marker='o', markersize=0, clip_on=True)
	# make a legend
	leg = fig_ax.legend(bbox_to_anchor=(0.0, 1.05, 1.0, 0.10), \
		loc='lower center', ncol=3, borderaxespad=0.0, \
		handlelength=2.2, handletextpad=0.25, columnspacing=0.5, \
		mode="expand", frameon=True)
	leg.get_frame().set_linewidth(0.5)
	# label the axes
	#fig_ax.set_xlabel('Consecutive appearances') # sentence-style capitalization
	fig_ax.set_ylabel('Probability')
	# set limits
	pylab.xlim(1, 28)
	pylab.ylim(0.75, 1.0)
	# major xticks
	pylab.xticks([1, 5, 10, 15, 20, 25, 28], ['', '', '', '', '', '', ''])
	# minor xticks
	fig_ax.xaxis.set_minor_locator(MultipleLocator(1)) # tick every year
	# major yticks
	pylab.yticks([0.75, 1], [r'0.75', r'1.0'])
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# bottom plot
	fig.add_axes(bottom_plot_dimensions)
	fig_ax = fig.gca()
	fig_ax.plot(range(1, len(durations)+1), list(prob_of_exit_by_duration['remain wealthy']['upper'][duration] for duration in durations), color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='Appear again') # no markers if more than 10 data points
	fig_ax.plot(range(1, len(durations)+1), list(prob_of_exit_by_duration['exit by decline']['upper'][duration] for duration in durations), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label='Drop off due to decline') 
	fig_ax.plot(range(1, len(durations)+1), list(prob_of_exit_by_duration['exit by death']['upper'][duration] for duration in durations), color='k', linestyle='dotted', lw=0.5, marker='o', markersize=0, clip_on=True, label='Die')
	fig_ax.plot(range(1, len(durations)+1), list(prob_of_exit_by_duration['remain wealthy']['lower'][duration] for duration in durations), color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True)
	fig_ax.plot(range(1, len(durations)+1), list(prob_of_exit_by_duration['exit by decline']['lower'][duration] for duration in durations), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True)
	fig_ax.plot(range(1, len(durations)+1), list(prob_of_exit_by_duration['exit by death']['lower'][duration] for duration in durations), color='k', linestyle='dotted', lw=0.5, marker='o', markersize=0, clip_on=True)
	# label the axes
	fig_ax.set_xlabel('Consecutive appearances') # sentence-style capitalization
	fig_ax.set_ylabel('Probability')
	# set limits
	pylab.xlim(1, 28)
	pylab.ylim(0.0, 0.25)
	# major xticks
	pylab.xticks([1, 5, 10, 15, 20, 25, 28], ['1', '5', '10', '15', '20', '25', r'$\geq$28'])
	# minor xticks
	fig_ax.xaxis.set_minor_locator(MultipleLocator(1)) # tick every year
	# major yticks
	pylab.yticks([0, .25], [r'0.0', r'0.25'])
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04-0.02) # extra -0.02
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# save figure
	pylab.savefig("./figures/figures_for_third_essay/probs_of_exits_by_duration_split.pdf")

	## FIG 4.8: EXIT DIFF BTW SELF-MADE AND NOT ##

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=figsize)
	fig_ax = fig.gca()
	# plot confidence intervals
	fig_ax.plot(ranks, list(prob_of_exit_diff_btw_self_made_and_not['remain wealthy']['upper'][rank] for rank in ranks), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label='95\% CI') # no markers if more than 10 data points
	fig_ax.plot(ranks, list(prob_of_exit_diff_btw_self_made_and_not['remain wealthy']['lower'][rank] for rank in ranks), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label='') 
	fig_ax.plot(ranks, list(prob_of_exit_diff_btw_self_made_and_not['remain wealthy']['point'][rank] for rank in ranks), color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='')
	fig_ax.plot([1, 400], [0.0, 0.0], color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='')
	# make a legend
	leg = fig_ax.legend(loc='lower right', frameon=False, handlelength=3.3, handletextpad=0.5)
	# label the axes
	fig_ax.set_xlabel('Rank') # sentence-style capitalization
	fig_ax.set_ylabel('Difference in probability')
	# set limits
	pylab.xlim(1, 400)
	pylab.ylim(-0.11, 0.01)
	# add ticks
	pylab.xticks([1, 100, 200, 300, 400], ['1rst', '100th', '200th', '300th', '400th'])
	pylab.yticks([-0.10, -0.05, 0.0], [r'$-$0.10', r'$-$0.05', r'0'])
	# minor xticks
	fig_ax.xaxis.set_minor_locator(MultipleLocator(50)) # tick every 50 ranks
	# invert x axis
	fig_ax.invert_xaxis()
	# minor yticks
	#fig_ax.yaxis.set_minor_locator(MultipleLocator(0.05)) # tick every 0.05
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# adjust dimensions
	fig.subplots_adjust(left=0.11, bottom=0.14, right=0.97, top=0.90, wspace=0.0, hspace=0.0)
	# save figure
	pylab.savefig("./figures/figures_for_third_essay/probs_of_exit_diff_btw_self_made.pdf", bbox_inches='tight')

	## FIG. 4.9: EXIT BY DECLINE DIFF BTW FIRE AND OTHER BY YEAR ##

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=figsize)
	fig_ax = fig.gca()
	# plot confidence intervals
	fig_ax.plot([int(year) for year in years_from_2001_to_2011], list(prob_of_exit_diff_btw_fire_and_other['exit by decline']['upper'][year] for year in years_from_2001_to_2011), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=2, clip_on=True, label='95\% CI') # no markers if more than 10 data points
	fig_ax.plot([int(year) for year in years_from_2001_to_2011], list(prob_of_exit_diff_btw_fire_and_other['exit by decline']['lower'][year] for year in years_from_2001_to_2011), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=2, clip_on=True, label='') 
	fig_ax.plot([int(year) for year in years_from_2001_to_2011], list(prob_of_exit_diff_btw_fire_and_other['exit by decline']['point'][year] for year in years_from_2001_to_2011), color='k', linestyle='solid', lw=0.5, marker='o', markersize=2, clip_on=True, label='')
	fig_ax.plot([2003, 2011], [0.0, 0.0], color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='')
	# make a legend
	leg = fig_ax.legend(loc='upper right', frameon=False, handlelength=3.3, handletextpad=0.5)
	# label the axes
	fig_ax.set_xlabel('Year') # sentence-style capitalization
	fig_ax.set_ylabel('Difference in probability')
	# set limits
	pylab.xlim(2003, 2011)
	pylab.ylim(-0.10, 0.10) #pylab.ylim(-0.15, 1.15)
	# major xticks
	pylab.xticks([2003, 2007, 2011], ['2003', '2007', '2011'])
	# minor xticks
	fig_ax.xaxis.set_minor_locator(MultipleLocator(1)) # tick every year
	# major yticks
	pylab.yticks([-0.10, 0.0, 0.10], ['$-$0.10', r'0', r'0.10'])
	# minor yticks
	fig_ax.yaxis.set_minor_locator(MultipleLocator(0.05)) # tick every 0.05
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# adjust dimensions
	fig.subplots_adjust(left=0.11, bottom=0.14, right=0.97, top=0.90, wspace=0.0, hspace=0.0)
	# save figure
	pylab.savefig("./figures/figures_for_third_essay/probs_of_exit_by_decline_diff_btw_industries.pdf", bbox_inches='tight')

	#### TABLES ####

	## TABLE C.1: NUMBER OF DROPOUTS ##

	# uses packages ctable and dcolumn
	f = open("./tables/tables_specific_to_third_essay/number_of_dropouts.txt", 'w') # write to this file
	f.write("{\ssp\n") # single-spacing
	f.write("\ctable[\n")
	f.write("\tcaption\t= {Number of Dropouts, 1982--2013},\n") # sentence-style without terminal period (Turabian 2007, sec. 26.2.2)
	f.write("\tlabel\t= number_of_dropouts,\n")
	f.write("\tpos\t= H,\n") # top or page by itself
	f.write("\tnotespar,\n") # footnotes must be flush left (Turabian 2007, sec. 26.2.7)
	f.write("\tnosuper,\n") # typeset sources and notes on the line, manually superscript everything else
	f.write("\tmincapwidth\t= 1.0\\textwidth,\n") # make the table caption and footnotes as wide as the text
	f.write("\tdoinside\t=\\small,\n") # table is too big for page, so shrink text size
	f.write("\t]{lD{.}{}{-2}ccc}{\n") # first column is flush left, other columns are centered
	f.write("\\tnote[Source:~\\textnormal{Data adapted from \\textit{Forbes Magazine} (1982--2013).}]{\\\} \n") # terminal period
	f.write("\\tnote[]{\\\} % for spacing\n")
	f.write("\\tnote[Note:~\\textnormal{This table shows, for the years 1982 to 2013 and as an average across those years, the number of people who dropped off \\textit{Forbes Magazine}'s list of the 400 wealthiest Americans between one year of that list and the next year because they died  (``decedents''), renounced their American citizenship (``renunciants''), or experienced a decline in their wealth (``other dropouts''). The total number of people who dropped off between one year and the next is also shown.}]{} \n")
	f.write("}{\FL[\lightrulewidth]\n")
	f.write("Year\t&\t\multicolumn{1}{c}{Decedents}\t&\t\multicolumn{1}{c}{Renunciants}\t&\t\multicolumn{1}{c}{Other dropouts}\t&\t\multicolumn{1}{c}{Total}\t\ML\n") # sentence style
	select_dyear_types = ['exit by death', 'exit by renunciation of citizenship', 'exit by decline', 'exit']
	for dyear in dyears:
		s = dyear + '\t&\t'
		for type in select_dyear_types:
			s += str(int(round(n_by_dyear_types_in[dyear][type], 0)))
			if type != select_dyear_types[-1]:
				s +=  '\t&\t'
			else:
				if dyear != '2012--2013':
					s += '\t\NN\n'
				else:
					s += '\t\ML\n' #'\t\NN\n'
					s += "Average" + '\t&\t'
					for type in select_dyear_types:
						avg = numpy.mean([n_by_dyear_types_in[_][type] for _ in dyears])
						s += str(int(round(avg, 0)))
						if type != select_dyear_types[-1]: 
							s +='\t&\t'
					s += '\t\LL[\lightrulewidth]\n'
		f.write(s)
	f.write("\NN % for spacing\n") # for spacing
	f.write("}\n") # end ctable
	f.write("}\n") # end single-spacing
	f.close()

	## TABLE 2.C: NUMBER OF ENTRANTS ##

	# uses packages ctable and dcolumn
	f = open("./tables/tables_specific_to_third_essay/number_of_entrants.txt", 'w') # write to this file
	f.write("{\ssp\n") # single-spacing
	f.write("\ctable[\n")
	f.write("\tcaption\t= {Number of Entrants, 1982--2013},\n") # sentence-style without terminal period (Turabian 2007, sec. 26.2.2)
	f.write("\tlabel\t= number_of_entrants,\n")
	f.write("\tpos\t= H,\n") # top or page by itself
	f.write("\tnotespar,\n") # footnotes must be flush left (Turabian 2007, sec. 26.2.7)
	f.write("\tnosuper,\n") # typeset sources and notes on the line, manually superscript everything else
	f.write("\tmincapwidth\t= 1.0\\textwidth,\n") # make the table caption and footnotes as wide as the text
	f.write("\tdoinside\t=\\small,\n") # table is too big for page, so shrink text size
	f.write("\t]{lcD{.}{}{-2}c}{\n") # first column is flush left, other columns are centered
	f.write("\\tnote[Source:~\\textnormal{Data adapted from \\textit{Forbes Magazine} (1982--2013).}]{\\\} \n") # terminal period
	f.write("\\tnote[]{\\\} % for spacing\n")
	f.write("\\tnote[Note:~\\textnormal{This table shows, for the years 1982 to 2013 and as an average across those years, the number of people who came onto \\textit{Forbes Magazine}'s list of the 400 wealthiest Americans between one year of that list and the next year for the first time since its inception in 1982 (``first-time'') or after an absence (``returnees''). The total number of people who came onto the list between one year and the next is also shown.}]{} \n")
	f.write("}{\FL[\lightrulewidth]\n")
	f.write("Year\t&\t\multicolumn{1}{c}{First-timers}\t&\t\multicolumn{1}{c}{Returnees}\t&\t\multicolumn{1}{c}{Total}\t\ML\n") # sentence style
	select_dyear_types = ['first-timers', 'returnees', 'entrants']
	for dyear in dyears:
		s = dyear + '\t&\t'
		for type in select_dyear_types:
			s += str(int(round(n_by_dyear_types_in[dyear][type], 0)))
			if type != select_dyear_types[-1]:
				s +=  '\t&\t'
			else:
				if dyear != '2012--2013':
					s += '\t\NN\n'
				else:
					s += '\t\ML\n' #'\t\NN\n'
					s += "Average" + '\t&\t'
					for type in select_dyear_types:
						avg = numpy.mean([n_by_dyear_types_in[_][type] for _ in dyears])
						s += str(int(round(avg, 0)))
						if type != select_dyear_types[-1]: 
							s +='\t&\t'
					s += '\t\LL[\lightrulewidth]\n'
		f.write(s)
	f.write("\NN % for spacing\n") # for spacing
	f.write("}\n") # end ctable
	f.write("}\n") # end single-spacing
	f.close()

	## TABLE C.3: NUMBER OF INCUMBENTS ##

	# uses packages ctable and dcolumn
	f = open("./tables/tables_specific_to_third_essay/number_of_incumbents.txt", 'w') # write to this file
	f.write("{\ssp\n") # single-spacing
	f.write("\ctable[\n")
	f.write("\tcaption\t= {Number of Incumbents, 1982--2013},\n") # sentence-style without terminal period (Turabian 2007, sec. 26.2.2)
	f.write("\tlabel\t= number_of_incumbents,\n")
	f.write("\tpos\t= H,\n") # top or page by itself
	f.write("\tnotespar,\n") # footnotes must be flush left (Turabian 2007, sec. 26.2.7)
	f.write("\tnosuper,\n") # typeset sources and notes on the line, manually superscript everything else
	f.write("\tmincapwidth\t= 1.0\\textwidth,\n") # make the table caption and footnotes as wide as the text
	f.write("\tdoinside\t=\\small,\n") # table is too big for page, so shrink text size
	f.write("\t]{lD{.}{}{3}cD{.}{}{-2}c}{\n") # first column is flush left, other columns are centered
	f.write("\\tnote[Source:~\\textnormal{Data adapted from \\textit{Forbes Magazine} (1982--2013).}]{\\\} \n") # terminal period
	f.write("\\tnote[]{\\\} % for spacing\n")
	f.write("\\tnote[Note:~\\textnormal{This table shows, for the years 1982 to 2013 and as an average across those years, the number of people who stayed on \\textit{Forbes Magazine}'s list of the 400 wealthiest Americans between one year of that list and the next year while seeing their rank in the distribution of wealth go up, down, or stay the same. The total number of people who stayed on the list between one year and the next is also shown.}]{} \n")
	f.write("}{\FL[\lightrulewidth]\n")
	f.write("Year\t&\t\multicolumn{1}{c}{Up in rank}\t&\t\multicolumn{1}{c}{Down in rank}\t&\t\multicolumn{1}{c}{Same rank}\t&\t\multicolumn{1}{c}{Total}\t\ML\n") # sentence style
	select_dyear_types = ['remain wealthy and move up', 'remain wealthy but move down', 'remain wealthy and stay the same', 'remain wealthy']
	for dyear in dyears:
		s = dyear + '\t&\t'
		for type in select_dyear_types:
			s += str(int(round(n_by_dyear_types_in[dyear][type], 0)))
			if type != select_dyear_types[-1]:
				s +=  '\t&\t'
			else:
				if dyear != '2012--2013':
					s += '\t\NN\n'
				else:
					s += '\t\ML\n' #'\t\NN\n'
					s += "Average" + '\t&\t'
					for type in select_dyear_types:
						avg = numpy.mean([n_by_dyear_types_in[_][type] for _ in dyears])
						s += str(int(round(avg, 0)))
						if type != select_dyear_types[-1]: 
							s +='\t&\t'
					s += '\t\LL[\lightrulewidth]\n'
		f.write(s)
	f.write("\NN % for spacing\n") # for spacing
	f.write("}\n") # end ctable
	f.write("}\n") # end single-spacing
	f.close()

	## TABLE 4.1: ESTIMATES FROM BASELINE MODEL ##

	def get_note(p_value):
		# returns LaTeX note for p-value
		note = "\\tmark[$^{"
		if p_value < 10. / 100.0:
			note += '*'
		else:
			pass
		if p_value < 5. / 100.0:
			note += '*'
		else:
			pass
		if p_value < 1. / 100.0:
			note += '*'
		else:
			pass
		note += "}$]"
		return note

	covariates = ['Constant', 'Age', 'Age squared', 'Rank', 'Left-censored dummy']
	covariates.extend(years_less_initial_and_last)
	covariates.extend([str(_) for _ in range(2, 28+1)])
	select_covariates = covariates[0:5]
	estimates = fitted_for_baseline.params
	std_errors = zip(numpy.sqrt(numpy.diag(fitted_for_baseline.cov_params())[0:len(fitted_for_baseline.params)]), numpy.sqrt(numpy.diag(fitted_for_baseline.cov_params())[len(fitted_for_baseline.params):]))

	# uses packages ctable and dcolumn
	f = open("./tables/tables_specific_to_third_essay/estimated_multinomial_logit_model.txt", 'w') # write to this file
	f.write("{\ssp\n") # single-spacing
	f.write("\ctable[\n")
	f.write("\tcaption\t= {Multinomial Logit Model of the Competing Risks to Appearing Again},\n") # sentence-style without terminal period
	f.write("\tlabel\t= estimated_multinomial_logit_model,\n")
	f.write("\tpos\t= H,\n") # top or page by itself
	f.write("\tnotespar,\n") # footnotes must be flush left (Turabian 2007, sec. 26.2.7)
	f.write("\tnosuper,\n") # typeset sources and notes on the line, manually superscript everything else
	f.write("\tmincapwidth\t= 1.0\\textwidth,\n") # make the table caption and footnotes as wide as the text
	f.write("\t]{lD{.}{.}{-2}D{.}{.}{-2}}{\n") # first column is flush left, other columns are centered
	f.write("\\tnote[Source:~\\textnormal{Data adapted from \\textit{Forbes Magazine} (1982--2013).}]{\\\} \n") # terminal period
	f.write("\\tnote[]{\\\} % for spacing\n")
	f.write("\\tnote[Note:~\\textnormal{This table shows the maximum-likelihood estimates for the multinomial logit model described in the text, in which the unit of observation is a person on \\textit{Forbes Magazine}'s list in a given year who could either appear on the list again in the next year, drop off due to an absolute or relative decline in his or her wealth, or drop off due to death. Appearing again was taken as the base case. The covariates, the number of observations, the percentage of those observations that were correctly predicted, the likelihood-ratio tests, and the independence-of-irrelevant-alternatives tests are all described in the text.}]{\\\} \n")
	f.write("\\tnote[]{\\\} % for spacing\n")
	f.write("\\tnote[$^{*}p<.10$ { } $^{**}p<.05$ { } $^{***}p<0.01$]{}") # notes for p-values (see Turabian 26.2.7)
	f.write("}{\FL[\lightrulewidth]\n")
	f.write("\t&\t\multicolumn{2}{c}{Competing risks}\t\NN\n")
	f.write("\cmidrule[\lightrulewidth](rl){2-3}\n") # rule 
	f.write("Covariates\t&\t\multicolumn{1}{c}{Decline}\t&\t\multicolumn{1}{c}{Death}\t\ML\n")
	for i in range(0, len(covariates)):
		covariate = covariates[i]
		if covariate not in select_covariates:
			pass
		else:
			s = covariate + '\t&\t'
			s += str(format(round(estimates[i][0], 2), '.2f')) 
			s += get_note(get_p_value(z_stat=estimates[i][0]/std_errors[i][0]))
			s += '\t&\t'
			s += str(format(round(estimates[i][1], 2), '.2f')) 
			s += get_note(get_p_value(z_stat=estimates[i][1]/std_errors[i][1]))
			s += '\t\NN\n'
			s += '\t&\t'
			s += '(' + str(format(round(std_errors[i][0], 2), '.2f')) + ')'
			s += '\t&\t'
			s += '(' + str(format(round(std_errors[i][1], 2), '.2f')) + ')'
			s += '\t\NN\n'
			f.write(s) 
	f.write('Year dummies' + '\t&\t' + '\multicolumn{1}{c}{\\textcolor{white}{$^{***}$}Yes}' + '\t&\t' + '\multicolumn{1}{c}{\\textcolor{white}{$^{***}$}Yes}' + '\t\NN\n')
	f.write('Duration dummies' + '\t&\t' + '\multicolumn{1}{c}{\\textcolor{white}{$^{***}$}Yes}' + '\t&\t' + '\multicolumn{1}{c}{\\textcolor{white}{$^{***}$}Yes}' + '\t\ML\n')
	# Some of the following stats are much easier to obtain from an econometric program like gretl, so we will not try to calculate it using the StatsModel module. Likewise for tables that follow. 
	stat = 'Number of observations'
	f.write(stat + '\t&\t' + str(format(int(fitted_for_baseline.nobs), ',.0f')) + '\t\NN\n')
	#
	stat = 'Percent correctly predicted'
	n_correctly_predicted = 10925.
	f.write(stat + '\t&\t' + str(format(round(100. * (n_correctly_predicted / fitted_for_baseline.nobs), 2), ',.2f')) + '\t\NN\n')
	#
	stat = 'Log-likelihood'
	llf = fitted_for_baseline.llf
	f.write(stat + '\t&\t' + str(format(round(llf, 2), ',.2f')) + '\t\NN\n')
	#
	test = 'Test for non-constant variables'	
	llr, dof = -5168.842, 122
	if scipy.stats.chisqprob(-2.*(llr-llf), dof) > 0.01: print test, "\t", "($X_{" + str(format(dof, '.0f')) + "}^{2} =$ " + str(format(-2.*(llr-llf), '.2f')) + " , $p =$ " + str(format(scipy.stats.chisqprob(-2.*(llr-llf), dof), '.2f'))
	f.write(test + '\t&\t' + str(format(round(-2.*(llr-llf), 2), ',.2f')) + get_note(scipy.stats.chisqprob(-2.*(llr-llf), dof)) + '\t\NN\n')
	#
	test = 'Test for age variables'
	llr, dof = -4125.762, 4
	if scipy.stats.chisqprob(-2.*(llr-llf), dof) > 0.01: print test, "\t", "($X_{" + str(format(dof, '.0f')) + "}^{2} =$ " + str(format(-2.*(llr-llf), '.2f')) + " , $p =$ " + str(format(scipy.stats.chisqprob(-2.*(llr-llf), dof), '.2f'))
	f.write(test + '\t&\t' + str(format(round(-2.*(llr-llf), 2), ',.2f')) + get_note(scipy.stats.chisqprob(-2.*(llr-llf), dof)) + '\t\NN\n')
	#
	test = 'Test for rank variables'
	llr, dof = -4673.364, 2
	if scipy.stats.chisqprob(-2.*(llr-llf), dof) > 0.01: print test, "\t", "($X_{" + str(format(dof, '.0f')) + "}^{2} =$ " + str(format(-2.*(llr-llf), '.2f')) + " , $p =$ " + str(format(scipy.stats.chisqprob(-2.*(llr-llf), dof), '.2f'))
	f.write(test + '\t&\t' + str(format(round(-2.*(llr-llf), 2), ',.2f')) + get_note(scipy.stats.chisqprob(-2.*(llr-llf), dof)) + '\t\NN\n')
	#
	test = 'Test for left-censored dummies'
	llr, dof = -3965.670, 2
	if scipy.stats.chisqprob(-2.*(llr-llf), dof) > 0.01: print test, "\t", "($X_{" + str(format(dof, '.0f')) + "}^{2} =$ " + str(format(-2.*(llr-llf), '.2f')) + " , $p =$ " + str(format(scipy.stats.chisqprob(-2.*(llr-llf), dof), '.2f'))
	f.write(test + '\t&\t' + str(format(round(-2.*(llr-llf), 2), ',.2f')) + get_note(scipy.stats.chisqprob(-2.*(llr-llf), dof)) + '\t\NN\n')
	#
	test = 'Test for year dummies'
	llr, dof = -4056.114, 60
	if scipy.stats.chisqprob(-2.*(llr-llf), dof) > 0.01: print test, "\t", "($X_{" + str(format(dof, '.0f')) + "}^{2} =$ " + str(format(-2.*(llr-llf), '.2f')) + " , $p =$ " + str(format(scipy.stats.chisqprob(-2.*(llr-llf), dof), '.2f'))	
	f.write(test + '\t&\t' + str(format(round(-2.*(llr-llf), 2), ',.2f')) + get_note(scipy.stats.chisqprob(-2.*(llr-llf), dof)) + '\t\NN\n')
	#
	test = 'Test for duration dummies'
	llr, dof = -3999.988, 54 
	if scipy.stats.chisqprob(-2.*(llr-llf), dof) > 0.01: print test, "\t", "($X_{" + str(format(dof, '.0f')) + "}^{2} =$ " + str(format(-2.*(llr-llf), '.2f')) + " , $p =$ " + str(format(scipy.stats.chisqprob(-2.*(llr-llf), dof), '.2f'))	
	f.write(test + '\t&\t' + str(format(round(-2.*(llr-llf), 2), ',.2f')) + get_note(scipy.stats.chisqprob(-2.*(llr-llf), dof)) + '\t') # last test
	#
	f.write('\t\LL[\lightrulewidth]\n')
	f.write("\NN % for spacing\n") # for spacing
	f.write("}\n") # end ctable
	f.write("}\n") # end single-spacing
	f.close()

	## TABLE FOR PHILANTHROPIC MODEL ##

	covariates = ['Constant', 'Age', 'Age squared', 'Rank', 'Left-censored dummy', 'Philanthropic dummy']
	covariates.extend([0 for _ in years_since_1996_less_initial_and_last])
	covariates.extend([0 for _ in range(1, 25+1)])
	select_covariates = covariates[0:6]
	estimates = fitted_for_philanthropic.params
	std_errors = zip(numpy.sqrt(numpy.diag(fitted_for_philanthropic.cov_params())[0:len(fitted_for_philanthropic.params)]), numpy.sqrt(numpy.diag(fitted_for_philanthropic.cov_params())[len(fitted_for_philanthropic.params):]))

	# uses packages ctable and dcolumn
	f = open("./tables/tables_specific_to_third_essay/estimated_multinomial_logit_model_w_philanthropic.txt", 'w') # write to this file
	f.write("{\ssp\n") # single-spacing
	f.write("\ctable[\n")
	f.write("\tcaption\t= {Multinomial Logit Model with Philanthropic Dummies},\n") # sentence-style without terminal period
	f.write("\tlabel\t= estimated_multinomial_logit_model_w_philanthropic,\n")
	f.write("\tpos\t= H,\n") # top or page by itself
	f.write("\tnotespar,\n") # footnotes must be flush left (Turabian 2007, sec. 26.2.7)
	f.write("\tnosuper,\n") # typeset sources and notes on the line, manually superscript everything else
	f.write("\tmincapwidth\t= 1.0\\textwidth,\n") # make the table caption and footnotes as wide as the text
	f.write("\t]{lD{.}{.}{-2}D{.}{.}{-2}}{\n") # first column is flush left, other columns are centered
	f.write("\\tnote[Source:~\\textnormal{Data adapted from \\textit{Chronicle of Philanthropy}; \\textit{Forbes Magazine} (1982--2013); and \\textit{Slate}.}]{\\\} \n") # terminal period
	f.write("\\tnote[]{\\\} % for spacing\n")
	f.write("\\tnote[Note:~\\textnormal{This table shows maximum-likelihood estimates for the model discussed in the text that was essentially the same as the model estimated in table \\ref{estimated_multinomial_logit_model}, except that it included a dummy variable that reflects people who were one of the wealthiest Americans in a given year according to \\textit{Forbes Magazine}'s list and also one of the most-generous Americans in that or any other earlier year according to either the Philanthropy 50 list (for the years since 2000) or the Slate 60 list (for the years between 1996 and 1999).}]{\\\} \n")
	f.write("\\tnote[]{\\\} % for spacing\n")
	f.write("\\tnote[$^{*}p<.10$ { } $^{**}p<.05$ { } $^{***}p<0.01$]{}") # notes for p-values (see Turabian 26.2.7)
	f.write("}{\FL[\lightrulewidth]\n")
	f.write("\t&\t\multicolumn{2}{c}{Competing risks}\t\NN\n")
	f.write("\cmidrule[\lightrulewidth](rl){2-3}\n") # rule 
	f.write("Covariates\t&\t\multicolumn{1}{c}{Decline}\t&\t\multicolumn{1}{c}{Death}\t\ML\n")
	for i in range(0, len(covariates)):
		covariate = covariates[i]
		if covariate not in select_covariates:
			pass
		else:
			s = covariate + '\t&\t'
			s += str(format(round(estimates[i][0], 2), '.2f')) 
			s += get_note(get_p_value(z_stat=estimates[i][0]/std_errors[i][0]))
			s += '\t&\t'
			s += str(format(round(estimates[i][1], 2), '.2f')) 
			s += get_note(get_p_value(z_stat=estimates[i][1]/std_errors[i][1]))
			s += '\t\NN\n'
			s += '\t&\t'
			s += '(' + str(format(round(std_errors[i][0], 2), '.2f')) + ')'
			s += '\t&\t'
			s += '(' + str(format(round(std_errors[i][1], 2), '.2f')) + ')'
			s += '\t\NN\n'
			f.write(s) 
	f.write('Year dummies' + '\t&\t' + '\multicolumn{1}{c}{\\textcolor{white}{$^{***}$}Yes}' + '\t&\t' + '\multicolumn{1}{c}{\\textcolor{white}{$^{***}$}Yes}' + '\t\NN\n')
	f.write('Duration dummies' + '\t&\t' + '\multicolumn{1}{c}{\\textcolor{white}{$^{***}$}Yes}' + '\t&\t' + '\multicolumn{1}{c}{\\textcolor{white}{$^{***}$}Yes}' + '\t\ML\n')
	#
	stat = 'Number of observations'
	_ = fitted_for_philanthropic
	f.write(stat + '\t&\t' + str(format(int(_.nobs), ',.0f')) + '\t\NN\n')
	#
	if not just_the_same_year:
		stat = 'Percent correctly predicted'
		n_correctly_predicted = 6063
		f.write(stat + '\t&\t' + str(format(round(100. * (n_correctly_predicted / _.nobs), 2), ',.2f')) + '\t\NN\n')
		#
		stat = 'Log-likelihood'
		llf = _.llf
		f.write(stat + '\t&\t' + str(format(round(llf, 2), ',.2f')) + '\t\NN\n')
		#
		test = 'Test for non-constant variables'	
		llr, dof = -2623.878, 92
		if scipy.stats.chisqprob(-2.*(llr-llf), dof) > 0.01: print test, "\t", "($X_{" + str(format(dof, '.0f')) + "}^{2} =$ " + str(format(-2.*(llr-llf), '.2f')) + " , $p =$ " + str(format(scipy.stats.chisqprob(-2.*(llr-llf), dof), '.2f'))
		f.write(test + '\t&\t' + str(format(round(-2.*(llr-llf), 2), ',.2f')) + get_note(scipy.stats.chisqprob(-2.*(llr-llf), dof)) + '\t\NN\n')
		#
		test = 'Test for philanthropic dummies'
		llr, dof = -1928.306, 2
		if scipy.stats.chisqprob(-2.*(llr-llf), dof) > 0.01: print test, "\t", "($X_{" + str(format(dof, '.0f')) + "}^{2} =$ " + str(format(-2.*(llr-llf), '.2f')) + " , $p =$ " + str(format(scipy.stats.chisqprob(-2.*(llr-llf), dof), '.2f'))	
		f.write(test + '\t&\t' + str(format(round(-2.*(llr-llf), 2), ',.2f')) + get_note(scipy.stats.chisqprob(-2.*(llr-llf), dof)) + '\t') # last test
	#
	f.write('\t\LL[\lightrulewidth]\n')	
	f.write("\NN % for spacing\n") # for spacing
	f.write("}\n") # end ctable
	f.write("}\n") # end single-spacing
	f.close()

	## TABLE FOR SELF-MADE MODEL ##

	covariates = ['Constant', 'Age', 'Age squared', 'Rank', 'Left-censored dummy', 'Self-made dummy']
	covariates.extend([0 for _ in years_since_1996_less_initial_and_last])
	covariates.extend([0 for _ in range(1, 25+1)])
	select_covariates = covariates[0:6]
	estimates = fitted_for_self_made.params
	std_errors = zip(numpy.sqrt(numpy.diag(fitted_for_self_made.cov_params())[0:len(fitted_for_self_made.params)]), numpy.sqrt(numpy.diag(fitted_for_self_made.cov_params())[len(fitted_for_self_made.params):]))

	# uses packages ctable and dcolumn
	f = open("./tables/tables_specific_to_third_essay/estimated_multinomial_logit_model_w_self_made.txt", 'w') # write to this file
	f.write("{\ssp\n") # single-spacing
	f.write("\ctable[\n")
	f.write("\tcaption\t= {Multinomial Logit Model with Self-made Dummies},\n") # sentence-style without terminal period
	f.write("\tlabel\t= estimated_multinomial_logit_model_w_self_made,\n")
	f.write("\tpos\t= H,\n") # top or page by itself
	f.write("\tnotespar,\n") # footnotes must be flush left (Turabian 2007, sec. 26.2.7)
	f.write("\tnosuper,\n") # typeset sources and notes on the line, manually superscript everything else
	f.write("\tmincapwidth\t= 1.0\\textwidth,\n") # make the table caption and footnotes as wide as the text
	f.write("\t]{lD{.}{.}{-2}D{.}{.}{-2}}{\n") # first column is flush left, other columns are centered
	f.write("\\tnote[Source:~\\textnormal{Data adapted from \\textit{Forbes Magazine} (1982--2013).}]{\\\} \n") # terminal period
	f.write("\\tnote[]{\\\} % for spacing\n")
	f.write("\\tnote[Note:~\\textnormal{This table shows maximum-likelihood estimates for the model discussed in the text that was essentially the same as the model estimated in table \\ref{estimated_multinomial_logit_model}, except that it included a dummy variable for whether someone's wealth was self-made or not. }]{\\\} \n")
	f.write("\\tnote[]{\\\} % for spacing\n")
	f.write("\\tnote[$^{*}p<.10$ { } $^{**}p<.05$ { } $^{***}p<0.01$]{}") # notes for p-values (see Turabian 26.2.7)
	f.write("}{\FL[\lightrulewidth]\n")
	f.write("\t&\t\multicolumn{2}{c}{Competing risks}\t\NN\n")
	f.write("\cmidrule[\lightrulewidth](rl){2-3}\n") # rule 
	f.write("Covariates\t&\t\multicolumn{1}{c}{Decline}\t&\t\multicolumn{1}{c}{Death}\t\ML\n")
	for i in range(0, len(covariates)):
		covariate = covariates[i]
		if covariate not in select_covariates:
			pass
		else:
			s = covariate + '\t&\t'
			s += str(format(round(estimates[i][0], 2), '.2f')) 
			s += get_note(get_p_value(z_stat=estimates[i][0]/std_errors[i][0]))
			s += '\t&\t'
			s += str(format(round(estimates[i][1], 2), '.2f')) 
			s += get_note(get_p_value(z_stat=estimates[i][1]/std_errors[i][1]))
			s += '\t\NN\n'
			s += '\t&\t'
			s += '(' + str(format(round(std_errors[i][0], 2), '.2f')) + ')'
			s += '\t&\t'
			s += '(' + str(format(round(std_errors[i][1], 2), '.2f')) + ')'
			s += '\t\NN\n'
			f.write(s) 
	f.write('Year dummies' + '\t&\t' + '\multicolumn{1}{c}{\\textcolor{white}{$^{***}$}Yes}' + '\t&\t' + '\multicolumn{1}{c}{\\textcolor{white}{$^{***}$}Yes}' + '\t\NN\n')
	f.write('Duration dummies' + '\t&\t' + '\multicolumn{1}{c}{\\textcolor{white}{$^{***}$}Yes}' + '\t&\t' + '\multicolumn{1}{c}{\\textcolor{white}{$^{***}$}Yes}' + '\t\ML\n')
	stat = 'Number of observations'
	_ = fitted_for_self_made
	f.write(stat + '\t&\t' + str(format(int(_.nobs), ',.0f')) + '\t\NN\n')
	#
	stat = 'Percent correctly predicted'
	n_correctly_predicted = 6063
	f.write(stat + '\t&\t' + str(format(round(100. * (n_correctly_predicted / _.nobs), 2), ',.2f')) + '\t\NN\n')
	#
	stat = 'Log-likelihood'
	llf = _.llf
	f.write(stat + '\t&\t' + str(format(round(llf, 2), ',.2f')) + '\t\NN\n')
	#
	test = 'Test for non-constant variables'	
	llr, dof = -2623.878, 92
	if scipy.stats.chisqprob(-2.*(llr-llf), dof) > 0.01: print test, "\t", "($X_{" + str(format(dof, '.0f')) + "}^{2} =$ " + str(format(-2.*(llr-llf), '.2f')) + " , $p =$ " + str(format(scipy.stats.chisqprob(-2.*(llr-llf), dof), '.2f'))
	f.write(test + '\t&\t' + str(format(round(-2.*(llr-llf), 2), ',.2f')) + get_note(scipy.stats.chisqprob(-2.*(llr-llf), dof)) + '\t\NN\n')
	#
	test = 'Test for self-made dummies'
	llr, dof = -1928.306, 2
	if scipy.stats.chisqprob(-2.*(llr-llf), dof) > 0.01: print test, "\t", "($X_{" + str(format(dof, '.0f')) + "}^{2} =$ " + str(format(-2.*(llr-llf), '.2f')) + " , $p =$ " + str(format(scipy.stats.chisqprob(-2.*(llr-llf), dof), '.2f'))	
	f.write(test + '\t&\t' + str(format(round(-2.*(llr-llf), 2), ',.2f')) + get_note(scipy.stats.chisqprob(-2.*(llr-llf), dof)) + '\t') # last test
	#
	f.write('\t\LL[\lightrulewidth]\n')
	f.write("\NN % for spacing\n") # for spacing
	f.write("}\n") # end ctable
	f.write("}\n") # end single-spacing
	f.close()

	## TABLE FOR FIRE MODEL ##

	covariates = ['Constant', 'Age', 'Age squared', 'Rank', 'Left-censored dummy', 'FIRE industry dummy']
	covariates.extend([0 for _ in years_since_1996_less_initial_and_last])
	covariates.extend([0 for _ in range(1, 25+1)])
	select_covariates = covariates[0:6]
	estimates = fitted_for_fire.params
	std_errors = zip(numpy.sqrt(numpy.diag(fitted_for_fire.cov_params())[0:len(fitted_for_fire.params)]), numpy.sqrt(numpy.diag(fitted_for_fire.cov_params())[len(fitted_for_fire.params):]))

	# uses packages ctable and dcolumn
	f = open("./tables/tables_specific_to_third_essay/estimated_multinomial_logit_model_w_fire_industry.txt", 'w') # write to this file
	f.write("{\ssp\n") # single-spacing
	f.write("\ctable[\n")
	f.write("\tcaption\t= {Multinomial Logit Model with FIRE Industry Dummies},\n") # sentence-style without terminal period
	f.write("\tlabel\t= estimated_multinomial_logit_model_w_fire_industry,\n")
	f.write("\tpos\t= H,\n") # top or page by itself
	f.write("\tnotespar,\n") # footnotes must be flush left (Turabian 2007, sec. 26.2.7)
	f.write("\tnosuper,\n") # typeset sources and notes on the line, manually superscript everything else
	f.write("\tmincapwidth\t= 1.0\\textwidth,\n") # make the table caption and footnotes as wide as the text
	f.write("\t]{lD{.}{.}{-2}D{.}{.}{-2}}{\n") # first column is flush left, other columns are centered
	f.write("\\tnote[Source:~\\textnormal{Data adapted from \\textit{Forbes Magazine} (1982--2013).}]{\\\} \n") # terminal period
	f.write("\\tnote[]{\\\} % for spacing\n")
	f.write("\\tnote[Note:~\\textnormal{This table shows maximum-likelihood estimates for the model discussed in the text that was essentially the same as the model estimated in table \\ref{estimated_multinomial_logit_model}, except that it included a dummy variable for whether someone's wealth was made in the FIRE industry or not and, also, interaction terms between that dummy and year dummies. }]{\\\} \n")
	f.write("\\tnote[]{\\\} % for spacing\n")
	f.write("\\tnote[$^{*}p<.10$ { } $^{**}p<.05$ { } $^{***}p<0.01$]{}") # notes for p-values (see Turabian 26.2.7)
	f.write("}{\FL[\lightrulewidth]\n")
	f.write("\t&\t\multicolumn{2}{c}{Competing risks}\t\NN\n")
	f.write("\cmidrule[\lightrulewidth](rl){2-3}\n") # rule 
	f.write("Covariates\t&\t\multicolumn{1}{c}{Decline}\t&\t\multicolumn{1}{c}{Death}\t\ML\n")
	for i in range(0, len(covariates)):
		covariate = covariates[i]
		if covariate not in select_covariates:
			pass
		else:
			s = covariate + '\t&\t'
			s += str(format(round(estimates[i][0], 2), '.2f')) 
			s += get_note(get_p_value(z_stat=estimates[i][0]/std_errors[i][0]))
			s += '\t&\t'
			s += str(format(round(estimates[i][1], 2), '.2f')) 
			s += get_note(get_p_value(z_stat=estimates[i][1]/std_errors[i][1]))
			s += '\t\NN\n'
			s += '\t&\t'
			s += '(' + str(format(round(std_errors[i][0], 2), '.2f')) + ')'
			s += '\t&\t'
			s += '(' + str(format(round(std_errors[i][1], 2), '.2f')) + ')'
			s += '\t\NN\n'
			f.write(s) 
	f.write('Year dummies' + '\t&\t' + '\multicolumn{1}{c}{\\textcolor{white}{$^{***}$}Yes}' + '\t&\t' + '\multicolumn{1}{c}{\\textcolor{white}{$^{***}$}Yes}' + '\t\NN\n')
	f.write('Duration dummies' + '\t&\t' + '\multicolumn{1}{c}{\\textcolor{white}{$^{***}$}Yes}' + '\t&\t' + '\multicolumn{1}{c}{\\textcolor{white}{$^{***}$}Yes}' + '\t\NN\n')
	f.write('Industry-year interaction terms' + '\t&\t' + '\multicolumn{1}{c}{\\textcolor{white}{$^{***}$}Yes}' + '\t&\t' + '\multicolumn{1}{c}{\\textcolor{white}{$^{***}$}Yes}' + '\t\ML\n')
	stat = 'Number of observations'
	_ = fitted_for_fire
	f.write(stat + '\t&\t' + str(format(int(_.nobs), ',.0f')) + '\t\NN\n')
	#
	stat = 'Percent correctly predicted'
	n_correctly_predicted = 6084
	f.write(stat + '\t&\t' + str(format(round(100. * (n_correctly_predicted / _.nobs), 2), ',.2f')) + '\t\NN\n')
	#
	stat = 'Log-likelihood'
	llf = _.llf
	f.write(stat + '\t&\t' + str(format(round(llf, 2), ',.2f')) + '\t\NN\n')
	#
	test = 'Test for non-constant variables'	
	llr, dof = -2623.878, 114
	if scipy.stats.chisqprob(-2.*(llr-llf), dof) > 0.01: print test, "\t", "($X_{" + str(format(dof, '.0f')) + "}^{2} =$ " + str(format(-2.*(llr-llf), '.2f')) + " , $p =$ " + str(format(scipy.stats.chisqprob(-2.*(llr-llf), dof), '.2f'))
	f.write(test + '\t&\t' + str(format(round(-2.*(llr-llf), 2), ',.2f')) + get_note(scipy.stats.chisqprob(-2.*(llr-llf), dof)) + '\t\NN\n')
	#
	test = 'Test for FIRE industry dummies and industry-year interaction terms'
	llr, dof = -1928.306, 24
	if scipy.stats.chisqprob(-2.*(llr-llf), dof) > 0.01: print test, "\t", "($X_{" + str(format(dof, '.0f')) + "}^{2} =$ " + str(format(-2.*(llr-llf), '.2f')) + " , $p =$ " + str(format(scipy.stats.chisqprob(-2.*(llr-llf), dof), '.2f'))	
	f.write('Test for FIRE industry dummies and ' + '\t\NN\n')
	f.write('\hspace{1em} industry-year interaction terms' + '\t&\t' + str(format(round(-2.*(llr-llf), 2), ',.2f')) + get_note(scipy.stats.chisqprob(-2.*(llr-llf), dof)) + '\t') # last test
	#
	f.write('\t\LL[\lightrulewidth]\n')
	f.write("\NN % for spacing\n") # for spacing
	f.write("}\n") # end ctable
	f.write("}\n") # end single-spacing
	f.close()