"""
Filename: essay_on_estimation_of_wealth_code.py
Python version: 2.7
Source: Capehart, Kevin W. Essays on the Wealthiest Americans. PhD dissertation, American University, Washington, DC, 2014. 
Description: This file generates results reported by the essay on the estimation of the wealth of the wealthiest Americans. 
"""

from __future__ import division
from load_forbes_400_dataset import * # load our Forbes 400 dataset
import decimal # import for trying to avoid any severe floating point errors
import scipy.stats # import for calculating quantiles
from os.path import isfile # import for checking whether a file exists
import pickle # import for dumping and loading computationally intensive results

## DEFINE A FUNCTION FOR CONVERTING FLOATS ##

# The estate-multiplier method often involves taking the reciprocal of small mortality rates and multiplying them by large wealths, so we will use Python's module for decimal floating point arithmetic in order to try to avoid any severe floating point errors

def dec(floating_point):
	""" Returns a decimal instead of a float """
	return decimal.Decimal(str(floating_point))

## LOAD SOME DATA ##

# load the names of the women who dropped off the Forbes 400 because they died
filepath = "./data/data_specific_to_first essay/"
filename = filepath + "women_who_died_data.txt"
raw_data = numpy.loadtxt(filename, delimiter='\t', skiprows=2, dtype={'names':['name'], 'formats':['<S50']})
names_of_dead_who_were_women = raw_data['name']

# load mortality rates used by Kopczuk and Saez (2004a,b)
filename = filepath + "mortality_rates_data.txt"
header = ['year', 'age', 'male', 'female', 'total']
raw_mortality_rates = numpy.loadtxt(filename, delimiter='\t', skiprows=9, dtype={'names':header, 'formats':['<S50']*len(header)})
filename = filepath + "relative_mortality_rates_data.txt"
header = ['age', 'male', 'female']
raw_relative_mortality_rates = numpy.loadtxt(filename, delimiter='\t', skiprows=7, dtype={'names':header, 'formats':['<S50']*len(header)})

# define dicts for mortality rates
genders = ['male', 'female']
genders_and_total = ['male', 'female', 'total']
ages = raw_relative_mortality_rates['age']
years_btw_1982_and_2012 = [str(y) for y in range(1982, 2012+1)]
years_btw_1982_to_2012_but_also_1918 = ['1918']
years_btw_1982_to_2012_but_also_1918.extend([year for year in years_btw_1982_and_2012])
mortality_rate_of = dict(zip(genders_and_total, [dict(zip(years_btw_1982_to_2012_but_also_1918, [dict(zip(raw_mortality_rates['age'][i:i+119+1], raw_mortality_rates[gender_or_total][i:i+119+1])) for i in range(0, (2012-1982+1+1)*119, 119+1)])) for gender_or_total in genders_and_total]))
relative_mortality_rate_of = dict(zip(genders, [dict(zip(raw_relative_mortality_rates['age'], raw_relative_mortality_rates[gender])) for gender in genders]))

# load Kopczuk and Saez (2004a,b) data on wealthiest 0.01%
filename = filepath + "ks_wealthiest_0_01_percent_data.txt"
header = ['year', 'population', 'top 0.01% average wealth', 'top 0.01% wealth threshold']
years_btw_1982_and_2000 = [str(y) for y in range(1982, 2000+1)] # only have estimates up to the year 2000
raw_ks_wealthiest_0_01_percent_data = numpy.loadtxt(filename, delimiter='\t', skiprows=4, dtype={'names':header, 'formats':['<S50']*len(header)})
ks_wealthiest_0_01_percent_data = dict(zip(['population', 'top 0.01% average wealth', 'top 0.01% wealth threshold'], [dict(zip(years_btw_1982_and_2000, [raw_ks_wealthiest_0_01_percent_data[_][i] for i in range(0, 2000-1982+1)])) for _ in ['population', 'top 0.01% average wealth', 'top 0.01% wealth threshold']]))

# load the price index used by Kopczuk and Saez (2004a,b), which is an earlier version of the CPI-U-RS with 2000 as the base year
filename = filepath + "ks_price_index_data.txt"
raw_data = numpy.loadtxt(filename, delimiter='\t', skiprows=3, dtype={'names':['year', 'index'], 'formats':['<S50','f']})
price_index_used_by_ks = dict(zip(raw_data['year'], raw_data['index']))

# load the latest version of the CPI-U-RS
filename = "./data/cpi_u_rs_data.txt"
raw_cpi_data = numpy.loadtxt(filename, delimiter='\t', skiprows=4, dtype={'names':['year', 'index'], 'formats':['<S50','f']})
cpi_u_rs_w_1977_as_base_year = dict(zip(raw_cpi_data['year'], raw_cpi_data['index']))
cpi_u_rs_w_2000_as_base_year = dict(zip(years, [dec(100.0) * (dec(cpi_u_rs_w_1977_as_base_year[year]) / dec(cpi_u_rs_w_1977_as_base_year['2000'])) for year in years]))

## DEFINE FUNCTIONS FOR CONVERTING DOLLAR AMOUNTS ##

# Kopczuk and Saez's (2004a,b) dollar figures are in CPI-U-RS dollars, but the version of the CPI-U-RS that they used is an older version, so we will re-inflate their constant dollar figures into current dollars in order to then deflate them using the latest version of the index

def not_in_2000_dollars(constant_dollar_amount, year):
	""" Returns a 2000 dollar amount in current dollars using the old version of the CPI-U-RS used by Kopczuk and Saez (2004a,b) """
	return constant_dollar_amount * (dec(price_index_used_by_ks[year]) / dec(100.0))

def in_2000_dollars(current_dollar_amount, year):
	""" Returns a current dollar amount in 2000 dollars using the latest version of the CPI-U-RS """
	return current_dollar_amount / (dec(cpi_u_rs_w_2000_as_base_year[year]) / dec(100.0))

## SEC. A.4: K&S VS. SCF ESTIMATES OF THE WEALTH OF WEALTHIEST ONE PERCENT ##

# The following code compares Kopczuk and Saez's (2004a,b) estate-multiplier estimates of the total wealth of the wealthiest one percent, on the one hand, to survey estimates of that group's wealth. 

# load SCF data
years_of_scf = ['1989', '1992', '1995', '1998', '2001', '2004', '2007', '2010']
scf_mean_wealth_in_thousands_of_2009_dollars_in = dict(zip(years_of_scf, [315.3, 279.5, 296.1, 373.2, 483.0, 509.6, 575.9, 493.1])) # Source: Table 2 of Kennickell's (2012) "The Other, Other Half: Changes in the Finances of the Least Wealthy 50%, 2007-2009."
number_of_households_in_millions = dict(zip(years_of_scf, [93.1, 95.9, 99.0, 102.6, 106.5, 112.1, 116.1, 117.6])) # Sources: Kennickell and Starr (1994, p. 880, n. 22); Kennickell et al. (2000, p. 27, n. 35); Bricker et al. (2012, p. 78, table A.3)
perct_of_wealth_held_by_wealthiest_one_perct_in = dict(zip(years_of_scf, [30.1, 30.1, 34.6, 33.9, 32.6, 33.4, 33.8, 34.5])) # Source: Ibid.

# define a function for converting Kennickell's 2009 constant dollars to current dollars
cpi_u_rs_w_2009_as_base_year = dict(zip(years, [dec(100.0) * (dec(cpi_u_rs_w_1977_as_base_year[year]) / dec(cpi_u_rs_w_1977_as_base_year['2009'])) for year in years]))
def not_in_2009_dollars(constant_dollar_amount, year):
	""" Returns a 2009 dollar amount in current dollars using the latest version of the CPI-U-RS """
	return constant_dollar_amount * (dec(cpi_u_rs_w_2009_as_base_year[year]) / dec(100.))

# calculate wealth of the wealthiest one percent based on the SCF data
scf_estimate_of_wealth_of_wealthiest_one_percent_in = dict(zip(years_less_last, ['TBD']*len(years_of_scf)))
for year in years_of_scf:
	mean_wealth = in_2000_dollars( not_in_2009_dollars( dec(scf_mean_wealth_in_thousands_of_2009_dollars_in[year]) * dec(1000.), year=year), year=year) # in 2000 dollars
	number_of_households = dec(number_of_households_in_millions[year]) * dec(1000000.) # in households
	total_wealth = number_of_households * mean_wealth # in 2000 dollars
	scf_estimate_of_wealth_of_wealthiest_one_percent_in[year] = ((dec(perct_of_wealth_held_by_wealthiest_one_perct_in[year]) / dec(100.)) * total_wealth) / dec(1000000000.) # in billions of 2000 dollars

# load Kopczuk and Saez's (2004a,b) estimates of the wealth of the wealthiest one percent
filename = filepath + "ks_wealthiest_one_percent_data.txt"
header = ['year', 'total wealth', 'top 1% wealth share']
raw_ks_wealthiest_one_percent_data = numpy.loadtxt(filename, delimiter='\t', skiprows=4, dtype={'names':header, 'formats':['<S50', '<S50']*len(header)})
ks_wealthiest_one_percent_data = dict(zip(['total wealth', 'top 1% wealth share'], [dict(zip(years_btw_1982_and_2000, [raw_ks_wealthiest_one_percent_data[_][i] for i in range(0, 2000-1982+1)])) for _ in ['total wealth', 'top 1% wealth share']]))
ks_estimate_of_wealth_of_wealthiest_one_percent_in = dict(zip(years_less_last, ['TBD']*len(years_btw_1982_and_2000)))
for year in years_btw_1982_and_2000:
	total_wealth = in_2000_dollars( not_in_2000_dollars( constant_dollar_amount=dec(ks_wealthiest_one_percent_data['total wealth'][year]), year=year), year=year)
	ks_estimate_of_wealth_of_wealthiest_one_percent_in[year] = total_wealth * (dec(ks_wealthiest_one_percent_data['top 1% wealth share'][year]) / dec(100.)) # billions of 2000 dollars

# print estimates of the wealth of the wealthiest one percent
for year in years:
	if year == years[0]: print "\nestimates of the wealth of the wealthiest one percent \nyear \tSCF estimate \tK&S estimate"
	print year, "\t", 
	if year in years_of_scf:
		print round(scf_estimate_of_wealth_of_wealthiest_one_percent_in[year]), "\t", 
	else:
		print "\t", 
	if year in years_btw_1982_and_2000:
		print round(ks_estimate_of_wealth_of_wealthiest_one_percent_in[year]), "\n", 
	else:
		print "\n",

## SEC. 2.2 TO 2.3: DIRECT VS. ESTATE-MULTIPLIER ESTIMATES OF THE WEALTH OF THE 400 WEALTHIEST ##

# The following code uses Kopczuk and Saez's (2004a,b) estate-multiplier estimates for the smallest group they consider (the wealthiest 0.01%) and also uses their Pareto extrapolation to method in order to get estate-multiplier estimates of the wealth of the 400 wealthiest. Those estate-multiplier estimates are then compared to Forbes Magazine's direct estimates. 

ks_estimated_wealth_of_400_wealthiest_in = dict(zip(years_btw_1982_and_2000, ['TBD' for year in years_btw_1982_and_2000]))
difference_in = dict(zip(years_less_last, ['TBD' for year in years_less_last]))
for year in years_btw_1982_and_2000:
	avg_wealth_at_top = dec(ks_wealthiest_0_01_percent_data['top 0.01% average wealth'][year]) # in their 2000 dollars
	number_in_top = (dec(0.01) / dec(100.)) * dec(ks_wealthiest_0_01_percent_data['population'][year])
	total_wealth_at_top = avg_wealth_at_top * number_in_top
	wealth_threshold_for_top = dec(ks_wealthiest_0_01_percent_data['top 0.01% wealth threshold'][year]) # in their 2000 dollars
	a = total_wealth_at_top / (total_wealth_at_top - wealth_threshold_for_top * number_in_top)
	ks_estimated_wealth_of_400_wealthiest_in_year_in_their_2000_dollars = total_wealth_at_top * ((dec(400.) / number_in_top) ** (dec(1) - (dec(1) / a)))
	ks_estimated_wealth_of_400_wealthiest_in[year] = in_2000_dollars( not_in_2000_dollars( constant_dollar_amount = ks_estimated_wealth_of_400_wealthiest_in_year_in_their_2000_dollars, year=year), year=year)
	direct_estimate = in_2000_dollars(dec(summary_stats['sum'][year]), year)
	difference_in[year] = ks_estimated_wealth_of_400_wealthiest_in[year] - direct_estimate
	# print results
	if year == years_btw_1982_and_2000[0]: print "\ndifference btw direct and estate-multiplier estimates of the wealth of the 400 wealthiest Americans \nyear \testate-multiplier estimate \tdirect estimate \t difference in dollars \t difference as a percentage of the direct estimate"
	print year, "\t",
	print round(ks_estimated_wealth_of_400_wealthiest_in[year]), "\t",
	print round(direct_estimate), "\t",
	print round(difference_in[year]), "\t",
	print round(dec(100.0) * difference_in[year] / direct_estimate, 2)

## SOME CALCULATIONS ##

print "\n400 wealthiest's wealth (millions of 2000 dollars) in 1990, according to K&S:\t", 
print round(ks_estimated_wealth_of_400_wealthiest_in['1990'])
print "400 wealthiest's wealth in 2000, according to K&S:\t", 
print round(ks_estimated_wealth_of_400_wealthiest_in['2000'])
print "400 wealthiest's share of wealth in 1990, according to K&S:\t", 
print round(dec(100.) * ks_estimated_wealth_of_400_wealthiest_in['1990'] / (in_2000_dollars( not_in_2000_dollars( constant_dollar_amount=dec(21588.2728070478), year='1990'), year='1990') * dec(1000.)), 2)
print "400 wealthiest's share of wealth in 2000, according to K&S:\t", 
aggregate_wealth_in_billions_according_to_ks = dec(32936.4508843353) # in billions of dollars
print round(dec(100.) * ks_estimated_wealth_of_400_wealthiest_in['2000'] / (aggregate_wealth_in_billions_according_to_ks * dec(1000)), 2)
print "difference in 2000 (millions of dollars)\t", 
print difference_in['2000']
print "difference in 2000 as a percent of 400 wealthiest's wealth, according to K&S:\t",
print round(dec(100.) * difference_in['2000'] / ks_estimated_wealth_of_400_wealthiest_in['2000'])
print "difference in 2000 as a percent of 400 wealthiest's wealth, according to Forbes:\t",
print round(dec(100.) * difference_in['2000'] / in_2000_dollars(dec(summary_stats['sum']['2000']), '2000'))
print "difference in 2000 as a percent of top 0.01%'s wealth, according to K&S:\t",
top_0_01_perct_wealth = (dec(3.89579841645598) / dec(100.)) * (in_2000_dollars( not_in_2000_dollars( constant_dollar_amount=aggregate_wealth_in_billions_according_to_ks, year='2000'), year='2000') * dec(1000)) # in millions
print round(dec(100.) * difference_in['2000'] / top_0_01_perct_wealth)
print "difference in 2000 as a percent of top 2%'s wealth, according to K&S:\t",
top_2_perct_wealth = (dec(26.4262930798121) / dec(100.)) * (in_2000_dollars( not_in_2000_dollars( constant_dollar_amount=aggregate_wealth_in_billions_according_to_ks, year='2000'), year='2000') * dec(1000)) # in millions
print round(dec(100.) * difference_in['2000'] / top_2_perct_wealth)

## SEC. 2.4.4: ESTIMATE MORTALITY RATES AMONG THE FORBES 400 ##

# The probability that someone on the Forbes 400 list in a given year will die by the next year can be estimated by assuming that members of the Forbes 400 live and die according to the Gompertz-Makeham law of mortality, as discussed in the dissertation. The parameters associated with that law must be estimated by using numerical methods. We estimated them using the econometric program gretl 1.9.14. The following Python code exports the data that we used, can print the gretl code that we used, and imports estimates made using gretl and our gretl code. 

# open file for all years
filename = filepath + "exported_gompertz_data_for_all_years.txt"
file_for_all_years = open(filename, 'w')
# write header
file_for_all_years.write("t,r,d,y\n")
for year in years_less_last:
	next_year = str(int(year) + 1)
	# open file for given year
	filename = filepath + "exported_gompertz_data_for_" + year + ".txt"
	file_for_one_year = open(filename, 'w')
	# write header
	file_for_one_year.write("t,r,d\n")
	# write data
	for name in names:
		if (data[name]['wealth'][year] != ''):
			age = data[name]['age'][year]
			s = age + ','
			wealth = float(data[name]['wealth'][year])
			rank = str(summary_stats['wealths'][year].index(wealth) + 1)
			s += rank + ','
			if (name in dead_in[next_year]):
				s += str(1)
			else:
				s += str(0)		
			file_for_all_years.write(s + ',' + str(years_less_last.index(year)) + '\n')
			file_for_one_year.write(s + '\n')
	# close file
	file_for_one_year.close()
file_for_all_years.close()

# de-comment to print gretl code
"""
for year in years_less_last:
	print "\nset seed 250624"
	print 'open "ABSOLUTE_PATH/data/data_specific_to_first essay/exported_gompertz_data_for_' + year + '.txt"' # note that the open command is not available in loop mode in gretl
	print "scalar a = 0.01"
	print "scalar b = 0.10"
	print "scalar c = 0"
	print "mle loglik = check ? (1 - d) * ((a/b) * (exp(b*t) - exp(b*(t+1))) - c) + d * log(1 - exp(((a/b) * (exp(b*t) - exp(b*(t+1))) - c))) : NA"
	print "\tscalar check = (a>=0) && (b>=0) && (c>=0)"
	print "\tparams a b"
	print "end mle --quiet"
	print "scalar a = a"
	print "scalar b = b"
	print "scalar c = a"
	print "mle loglik = check ? (1 - d) * ((a/b) * (exp(b*t) - exp(b*(t+1))) - c) + d * log(1 - exp(((a/b) * (exp(b*t) - exp(b*(t+1))) - c))) : NA"
	print "\tscalar check = (a>=0) && (b>=0) && (c>=0)"
	print "\tparams a b c" # note that you cannot export results to a file in gretl
	print "end mle --quiet --lbfgs"
	print "print a b c"
	print "restrict"
	print "\tb[3] = 0"
	print "end restrict"
"""

# import estimates that were made using gretl and the gretl code given above
filename = filepath + "gompertz_estimates.txt"
header = ['year', 'a', 'b', 'c']
raw_data = numpy.loadtxt(filename, delimiter='\t', skiprows=3, dtype={'names':header, 'formats':['f']*len(header)})
a_parameter_in = dict(zip(years_less_last, raw_data['a']))
b_parameter_in = dict(zip(years_less_last, raw_data['b']))
c_parameter_in = dict(zip(years_less_last, raw_data['c']))

# define a function that returns the mortality rates 
def estimated_mortality_rate_in(year, age):
	""" Returns the probability that someone on the Forbes 400 list in one year would die by the next. The estimates were made by assuming that members of the Forbes 400 live and die according to the Gompertz-Makeham law of mortality """
	return 1. - numpy.exp((a_parameter_in[year] / b_parameter_in[year]) * (numpy.exp(b_parameter_in[year] * float(age)) - numpy.exp(b_parameter_in[year] * (float(age) + 1.))) - c_parameter_in[year])

## SEC. 2.5: APPLY THE ESTATE-MULTIPLIER METHOD TO THE FORBES 400 LIST ##

class eme_of_wealth_of_400_or_other_wealthiest_in(object):
	""" 
	Class for applying the estate-multiplier method to the Forbes 400 list in the manner discussed in the initial exercise in the essay
	
	Parameters
	==========
	year : string of the year of one of the Forbes 400 list
	number_of_wealthiest : int of the number of people whose wealth will be estimated
	use_estimated_mortality_rates : bool for whether to use our estimated mortality rates or Kopczuk and Saez's (2004a,b)
	cut_wealths_in_half : bool for whether to undervalue a person's wealth by half
	"""
	
	def __init__(self, year, number_of_wealthiest=400, use_estimated_mortality_rates=False, cut_wealths_in_half=False):
		self.year = year
		self.number_of_wealthiest = dec(number_of_wealthiest)
		self.use_estimated_mortality_rates = use_estimated_mortality_rates
		self.cut_wealths_in_half = cut_wealths_in_half
		self.names_of_dead = dead_in[str(int(self.year) + 1)] # names of people on the Forbes 400 who died by next year
		self.estimated_wealth_of_number_of_wealthiest = 'TBD'
		self.bias = 'TBD'

	def calculate_raw_estimates(self):
		""" Apply the estate-multiplier method by assuming that we observe the wealths, ages, and genders of people who were on the Forbes 400, but died by the next year. Note that, in general, the estate-multiplier estimate of the number of people is not going to be equal to the number of people whose wealth we might wish to estimate, so we must interpolate or extrapolate the raw estate-multiplier estimates. Returns None """
		self.W = dec(0) # estimate of total wealth
		self.N = dec(0) # estimate of number of people
		self.wealths_and_implied_numbers = list() # store info
		self.var_of_raw_total = dec(0) # store variance
		for name in self.names_of_dead:
			age, wealth = data[name]['age'][self.year], in_2000_dollars(dec(data[name]['wealth'][self.year]), self.year) # wealth is in millions of 2000 dollars
			if self.use_estimated_mortality_rates:
				mortality_rate = dec(estimated_mortality_rate_in(year=self.year, age=age))
			else:				
				if name in names_of_dead_who_were_women:
					gender = 'female'
				else:
					gender = 'male'
				mortality_rate = dec(mortality_rate_of[gender][self.year][age]) * dec(relative_mortality_rate_of[gender][age]) # adjust mortality rates for social differential
			if self.cut_wealths_in_half:
				wealth *= dec(0.5) # cut wealth in half
			self.W += wealth / mortality_rate
			self.N += dec(1.) / mortality_rate
			self.wealths_and_implied_numbers.append((wealth, dec(1) / mortality_rate)) # note: list of tuples of wealths and number of people (which is typically not a whole number) with those wealths
			self.var_of_raw_total += ((dec(1) - mortality_rate) / (mortality_rate ** dec(2.))) * (wealth ** dec(2.))
	
	def get_interpolated_wealth_of_the_number_of_wealthiest(self):
		""" Returns the interpolated wealth of the 400 or other wealthiest. The interpolation method follows Kopczuk and Saez (2004b, appendix D.2) """
		# if the raw estate-multiplier estimate of the total number of people is already number_of_wealthiest, return the raw estate-multiplier estimate of their wealth
		if self.N == self.number_of_wealthiest:
			estimated_wealth_of_number_of_wealthiest = self.W
		else:
			# make sure that the 400 or other wealthiest can be interpolated
			assert self.N > self.number_of_wealthiest, "Wealth must be extrapolated"
			# if the raw estate-multipier estimate is different, then go through a list of sorted wealths and the number of people with those wealths until we get to number_of_wealthiest
			estimated_wealth_of_number_of_wealthiest = dec(0)
			people_so_far = dec(0)
			self.wealths_and_implied_numbers.sort(reverse=True) # sorted in descending order
			for wealth, number in self.wealths_and_implied_numbers:
				if self.number_of_wealthiest < (people_so_far + number):
					estimated_wealth_of_number_of_wealthiest += (self.number_of_wealthiest - people_so_far) * wealth
					people_so_far += (self.number_of_wealthiest - people_so_far)
				else:
					estimated_wealth_of_number_of_wealthiest += number * wealth
					people_so_far += number
		return estimated_wealth_of_number_of_wealthiest
	
	def get_extrapolated_wealth_of_the_number_of_wealthiest(self):
		""" Returns the extrapolated wealth of the 400 or other wealthiest. Assumes that wealth follows a Pareto distribution, following Kopczuk and Saez (2004b, p. 7, note 10) """
		# assert that the sample can be extrapolated
		assert ((self.w * self.N == self.W) != True), "Can't apply the Pareto extrapolation method"		
		# apply the Pareto extrapolation method
		w, W, N = self.w, self.W, self.N
		a = W / (W - N * w)
		estimated_min_wealth_of_number_of_wealthiest = W / (self.number_of_wealthiest * (a / (a - dec(1.))) * (N / self.number_of_wealthiest) ** (dec(1.) - (dec(1.) / a)))
		estimated_wealth_of_number_of_wealthiest = self.number_of_wealthiest * (a / (a - dec(1.))) * estimated_min_wealth_of_number_of_wealthiest
		return estimated_wealth_of_number_of_wealthiest
	
	def get_estimate(self):
		""" Returns the estate-multiplier estimate of the wealth of the 400 or other wealthiest """
		# calculate raw estimates
		self.calculate_raw_estimates()
		# assert that at least somebody died
		assert ((not self.wealths_and_implied_numbers) != True), "Nobody died"
		# if implied population is less than 400, then extrapolate, if extrapolation is possible
		if self.N < self.number_of_wealthiest:
			# get minimum wealth
			self.w = min(self.wealths_and_implied_numbers)[0]
			# extrapolate the sample
			self.estimated_wealth_of_number_of_wealthiest = self.get_extrapolated_wealth_of_the_number_of_wealthiest()
		# if implied population is greater than or equal to 400, then interpolate
		else:
			self.estimated_wealth_of_number_of_wealthiest = self.get_interpolated_wealth_of_the_number_of_wealthiest()
		return self.estimated_wealth_of_number_of_wealthiest # in millions of current dollars

	def get_bias(self):
		""" Returns the bias of the estate-multiplier estimator """
		if self.estimated_wealth_of_number_of_wealthiest == 'TBD':
			self.get_estimate()
		self.bias = self.estimated_wealth_of_number_of_wealthiest - in_2000_dollars( dec( sum( summary_stats['wealths'][self.year][0:int(self.number_of_wealthiest)])), self.year)
		return self.bias # in millions of current dollars
	
	def get_comparison_of_bias_w_combined_wealth_of_the_wealthiest(self):
		""" Returns an interperation of any bias in the estate-multiplier method's estimate """
		# get bias if not already calculated
		if self.bias == 'TBD':
			self.get_bias()
		# get whether bias is negative or positive
		if self.bias < 0:
			over_or_under_estimates = "under-estimates"
		elif self.bias > 0:
			over_or_under_estimates = "over-estimates"
		else:
			return "The estate-multiplier method as it was applied estimates the total wealth of the " + str(number_of_wealthiest) + " exactly"
		# compare the absolute value of the bias to the combined wealth of some of the wealthiest
		i = 1
		wealth_of_some_of_the_wealthiest = dec(0)
		while (wealth_of_some_of_the_wealthiest <= abs(self.bias)) and (i < int(self.number_of_wealthiest)):
			wealth_of_some_of_the_wealthiest += in_2000_dollars(dec(summary_stats['wealths'][self.year][int(i - 1)]), self.year)
			i += 1
		if i == int(self.number_of_wealthiest):
			more_than_or_about = "more than"
		else:
			more_than_or_about = "about"
			wealth_of_some_of_the_wealthiest -= in_2000_dollars(dec(summary_stats['wealths'][self.year][int(i - 1)]), self.year)
			i -= 1
		# return result of comparison
		return "The estate-multiplier method as it was applied " + over_or_under_estimates + " the total wealth of the " + str(number_of_wealthiest) + " by about " + str(format(round(abs(self.bias) / dec(1000000), 2), '.2f')) + " trillion constant (2000) dollars, which was " + more_than_or_about + " the total wealth of the " + str(i) + " wealthiest Americans on the magazine's list"

# use the mortality rates assumed by Kopczuk and Saez (2004a,b)
eme_of_wealth_of_400_wealthiest_in_w_ks_mortality_rates = dict(zip(years_less_last, ['TBD']*len(years_less_last)))
for year in years_less_last:
	estimator = eme_of_wealth_of_400_or_other_wealthiest_in(year)
	eme_of_wealth_of_400_wealthiest_in_w_ks_mortality_rates[year] = estimator.get_estimate()
	# print results
	if year == years_less_last[0]: print "\nwealth of the Forbes 400 using K&S mortality rates\nyear \testimated \tactual \tdifference"
	estimated = eme_of_wealth_of_400_wealthiest_in_w_ks_mortality_rates[year]
	actual = in_2000_dollars(dec(summary_stats['sum'][year]), year)
	difference = estimated - actual		
	print year, "\t", round(estimated), "\t", round(actual), "\t", round(difference)
	
# use the mortality rates assumed by Kopczuk and Saez (2004a,b) and also under-value wealths by half
eme_of_wealth_of_400_wealthiest_in_w_ks_mortality_rates_and_halved_wealths = dict(zip(years_less_last, ['TBD']*len(years_less_last)))
for year in years_less_last:
	estimator = eme_of_wealth_of_400_or_other_wealthiest_in(year, cut_wealths_in_half=True)
	eme_of_wealth_of_400_wealthiest_in_w_ks_mortality_rates_and_halved_wealths[year] = estimator.get_estimate()
	# print results
	if year == years_less_last[0]: print "\nwealth of the Forbes 400 using K&S mortality rates and under-valuing wealths by half\nyear \testimated \tactual\t difference"
	estimated = eme_of_wealth_of_400_wealthiest_in_w_ks_mortality_rates_and_halved_wealths[year]
	actual = in_2000_dollars(dec(summary_stats['sum'][year]), year)
	difference = estimated - actual		
	print year, "\t", round(estimated), "\t", round(actual), "\t", round(difference)

## SEC. 2.6.1: PROBABILITY OF UNDERESTIMATION ##

class sample_and_estimate_total_wealth_of_the_400_or_other_wealthiest_from(object):
	""" 
	Class for sampling from a given group of people and estimating the wealth of the group in the manner discussed in the extended exercise in the essay
	
	Parameters
	==========
	wealth_age_pairs : pairs of wealths and ages to sample from
	year_for_mortality_rates : string of the year to use for mortality rates
	number_of_wealthiest : int of the number of people whose wealth will be estimated. Must be less than or equal to 400 
	use_estimated_mortality_rates_to_sample : bool for whether to use our estimated mortality rates or Kopczuk and Saez's (2004a,b) to sample people
	use_estimated_mortality_rates_to_inflate : bool for whether to use our estimated mortality rates or Kopczuk and Saez's (2004a,b) to inflate wealths
	cut_wealths_in_half : bool for whether to undervalue a person's wealth by half
	"""
	
	def __init__(self, wealth_age_pairs, year_for_mortality_rates, number_of_wealthiest=400, cut_wealths_in_half=False, use_estimated_mortality_rates_to_sample=True, use_estimated_mortality_rates_to_inflate=True):
		self.wealth_age_pairs = wealth_age_pairs
		self.year_for_mortality_rates = year_for_mortality_rates
		self.number_of_wealthiest = dec(number_of_wealthiest) # number of people whose wealth will be estimated
		self.cut_wealths_in_half = cut_wealths_in_half
		self.use_estimated_mortality_rates_to_sample = use_estimated_mortality_rates_to_sample
		self.use_estimated_mortality_rates_to_inflate = use_estimated_mortality_rates_to_inflate
		self.min_of_wealth_for_wealth_age_pairs = min(wealth for wealth, age in wealth_age_pairs)
		self.empty_sample = False
		
	def draw_sample(self):
		""" Draws a sample from the wealth and age pairs. Returns None """
		self.W = dec(0)
		self.N = dec(0)
		self.wealths_and_implied_numbers = list()
		for pair in self.wealth_age_pairs:
			wealth, age = pair
			# get mortality rate for sampling
			if self.use_estimated_mortality_rates_to_sample:
				true_mortality_rate = dec(estimated_mortality_rate_in(year=self.year_for_mortality_rates, age=age))
			else:
				true_mortality_rate = dec(mortality_rate_of['male'][self.year_for_mortality_rates][age]) * dec(relative_mortality_rate_of['male'][age]) # use male mortality rates
			# sample wealth age pair?
			if numpy.random.uniform(0, 1) < float(true_mortality_rate):
				# misestimate mortality rates?
				if self.use_estimated_mortality_rates_to_inflate:
					assumed_mortality_rate = dec(estimated_mortality_rate_in(year=self.year_for_mortality_rates, age=age))
				else:
					assumed_mortality_rate = dec(mortality_rate_of['male'][self.year_for_mortality_rates][age]) * dec(relative_mortality_rate_of['male'][age])
				# cut wealth in half?
				if self.cut_wealths_in_half:
					wealth *= dec(0.50)
				self.W += dec(wealth) / assumed_mortality_rate
				self.N += dec(1) / assumed_mortality_rate
				self.wealths_and_implied_numbers.append((wealth, dec(1) / assumed_mortality_rate))
		# if sample is empty, then assume that number_of_wealthiest have the minimum wealth for the reasons discussed in the dissertation
		if (not self.wealths_and_implied_numbers):
			known_min_wealth = dec(self.min_of_wealth_for_wealth_age_pairs)
			if self.cut_wealths_in_half:
				known_min_wealth *= dec(0.50)
			self.W = known_min_wealth * self.number_of_wealthiest
			self.N = self.number_of_wealthiest
			self.wealths_and_implied_numbers = [(known_min_wealth, self.number_of_wealthiest)]
			self.empty_sample = True
	
	def get_estimate(self):
		""" Returns the estate-multiplier estimate of the wealth of the 400 or other wealthiest """
		self.draw_sample()
		W, N, wealths_and_implied_numbers = self.W, self.N, self.wealths_and_implied_numbers
		# extrapolate
		if N < self.number_of_wealthiest:
			# if each of the N < number_of_wealthiest have the same wealth, extrapolate by assuming that number_of_wealthiest have the same wealth, the shape parameter of the Pareto distribution can be thought of as infinite
			w = min(wealths_and_implied_numbers)[0]
			if (w * N == W):
				estimated_wealth_of_number_of_wealthiest = w * self.number_of_wealthiest
			# otherwise, extrapolate by assuming a pareto distribution
			else:
				a = W / (W - N * w)
				estimated_min_wealth_of_number_of_wealthiest = W / ( self.number_of_wealthiest * (a / (a - dec(1))) * (N / self.number_of_wealthiest) ** (dec(1) - (dec(1) / a)) )
				estimated_wealth_of_number_of_wealthiest = self.number_of_wealthiest * (a / (a - dec(1))) * estimated_min_wealth_of_number_of_wealthiest
		# interpolate
		else:
			estimated_wealth_of_number_of_wealthiest = dec(0)
			people_so_far = dec(0)
			wealths_and_implied_numbers.sort(reverse=True)
			for wealth, number in wealths_and_implied_numbers:
				if self.number_of_wealthiest < (people_so_far + number):
					estimated_wealth_of_number_of_wealthiest += (self.number_of_wealthiest - people_so_far) * wealth
					people_so_far += (self.number_of_wealthiest - people_so_far)
				else:
					estimated_wealth_of_number_of_wealthiest += number * wealth
					people_so_far += number
		# return estimate
		return estimated_wealth_of_number_of_wealthiest

# differences that were observed between the direct and estate-multiplier estimates
difference_in_2000_as_a_fraction_of_total_wealth_of_forbes_400 = (dec(ks_estimated_wealth_of_400_wealthiest_in['2000']) - dec(in_2000_dollars(dec(summary_stats['sum']['2000']), '2000'))) / dec(in_2000_dollars(dec(summary_stats['sum']['2000']), '2000'))
difference_in_year_as_a_fraction_of_total_wealth_of_forbes_400 = dict()
for year in years_btw_1982_and_2000:
	difference_in_year_as_a_fraction_of_total_wealth_of_forbes_400[year] = (dec(ks_estimated_wealth_of_400_wealthiest_in[year]) - dec(in_2000_dollars(dec(summary_stats['sum'][year]), year))) / dec(in_2000_dollars(dec(summary_stats['sum'][year]), year))

# define dicts for storing results
set_ups = ['right wealths and right rates', 'wrong wealths but right rates', 'right wealths but wrong rates', 'wrong wealths and wrong rates']
median_bias_for = dict(zip(set_ups, [dict(zip(years_less_last, ['TBD' for year in years_less_last])) for set_up in set_ups]))
prob_under_estimate_by_2000_amount_for = dict(zip(set_ups, [dict(zip(years_less_last, ['TBD' for year in years_less_last])) for set_up in set_ups]))
prob_under_estimate_by_same_amount_for = dict(zip(set_ups, [dict(zip(years_btw_1982_and_2000, ['TBD' for year in years_btw_1982_and_2000])) for set_up in set_ups]))
cumulative_prob_1996_to_2000_for = dict(zip(set_ups, [dec(1.) for set_up in set_ups]))
# set the following to True to re-estimate if already estimated
re_estimate = False
# check if already been estimated
filenames = [filepath + _ + ".pkl" for _ in ["median_bias_for", "prob_under_estimate_by_2000_amount_for", "prob_under_estimate_by_same_amount_for", "cumulative_prob_1996_to_2000_for"]]
already_estimated = False not in [isfile(filename) for filename in filenames]
# estimate or re-estimate
if not already_estimated or re_estimate:
	# set up simulations
	for set_up in set_ups:
		# number of resamples
		number_of_resamples = 10000
		# cut wealths in half?
		if 'wrong wealths' in set_up:
			cut_wealths_in_half = True
		else:
			cut_wealths_in_half = False
		# misestimate mortality rates?
		if 'wrong rates' in set_up:
			use_estimated_mortality_rates_to_sample = True
			use_estimated_mortality_rates_to_inflate = False
		else:
			use_estimated_mortality_rates_to_sample = True
			use_estimated_mortality_rates_to_inflate = True
		print set_up
		# run simulations for each year
		for year in years_less_last:
			numpy.random.seed(250624)
			biases_for_year = list()
			median_bias_for_year = list()
			prob_under_estimate_by_2000_amount_for_year = 0
			prob_under_estimate_by_same_amount_for_year = 0
			wealth_age_pairs = [(in_2000_dollars(dec(data[name]['wealth'][year]), year), data[name]['age'][year]) for name in names if (data[name]['wealth'][year] != '')]
			unknown_total_wealth_of_400_wealthiest = sum(zip(*wealth_age_pairs)[0])
			year_for_mortality_rates = year
			# for each resample
			for _ in range(1, number_of_resamples+1):
				# draw a sample
				estimate = sample_and_estimate_total_wealth_of_the_400_or_other_wealthiest_from( wealth_age_pairs = wealth_age_pairs, year_for_mortality_rates = year_for_mortality_rates, cut_wealths_in_half = cut_wealths_in_half, use_estimated_mortality_rates_to_sample = use_estimated_mortality_rates_to_sample, use_estimated_mortality_rates_to_inflate = use_estimated_mortality_rates_to_inflate).get_estimate()
				# get bias
				bias = (estimate - unknown_total_wealth_of_400_wealthiest) 
				biases_for_year.append(float(bias))
				# bias bigger than given amount?
				bias_as_a_fraction_of_total_wealth_of_forbes_400 = bias / (unknown_total_wealth_of_400_wealthiest) # bias as a percent
				if not bias_as_a_fraction_of_total_wealth_of_forbes_400 > difference_in_2000_as_a_fraction_of_total_wealth_of_forbes_400:
					prob_under_estimate_by_2000_amount_for_year += 1
				if year in years_btw_1982_and_2000:
					if not bias_as_a_fraction_of_total_wealth_of_forbes_400 > difference_in_year_as_a_fraction_of_total_wealth_of_forbes_400[year]:
						prob_under_estimate_by_same_amount_for_year += 1
			# record results
			median_bias_for[set_up][year] = str(round(scipy.stats.mstats.mquantiles( biases_for_year, prob=[50./100.], alphap=1/3., betap=1/3.)[0], 2))
			prob_under_estimate_by_2000_amount_for[set_up][year] = str(dec(100.) * dec(prob_under_estimate_by_2000_amount_for_year) / dec(number_of_resamples))
			if year in years_btw_1982_and_2000:
				prob_under_estimate_by_same_amount_for[set_up][year] = str(dec(100.) * dec(prob_under_estimate_by_same_amount_for_year) / dec(number_of_resamples))
				if int(1996) <= int(year) <= int(2000):
					cumulative_prob_1996_to_2000_for[set_up] *= dec(prob_under_estimate_by_same_amount_for_year) / dec(number_of_resamples)
			print year
	# dump results
	filename = filepath + "median_bias_for.pkl"
	pickle.dump(median_bias_for, open(filename, 'wb'))
	filename = filepath + "prob_under_estimate_by_2000_amount_for.pkl"
	pickle.dump(prob_under_estimate_by_2000_amount_for, open(filename, 'wb'))
	filename = filepath + "prob_under_estimate_by_same_amount_for.pkl"
	pickle.dump(prob_under_estimate_by_same_amount_for, open(filename, 'wb'))
	filename = filepath + "cumulative_prob_1996_to_2000_for.pkl"
	pickle.dump(cumulative_prob_1996_to_2000_for, open(filename, 'wb'))
else:	
	# load results that were dumped using pickle
	median_bias_for = pickle.load(open(filepath + "median_bias_for.pkl", 'rb'))
	prob_under_estimate_by_2000_amount_for = pickle.load(open(filepath + "prob_under_estimate_by_2000_amount_for.pkl", 'rb'))
	prob_under_estimate_by_same_amount_for = pickle.load(open(filepath + "prob_under_estimate_by_same_amount_for.pkl", 'rb'))
	cumulative_prob_1996_to_2000_for = pickle.load(open(filepath + "cumulative_prob_1996_to_2000_for.pkl", 'rb'))

# print results
for set_up in set_ups:
	for year in years_btw_1982_and_2000:
		if year == years_btw_1982_and_2000[0]: print "\nyear\t probability of underestimating by at least about 76 percent if " + set_up + " \tmedian misestmation"
		print year, "\t", 
		print round(float(prob_under_estimate_by_2000_amount_for[set_up][year]), 2), "\t", 
		print round(float(median_bias_for[set_up][year]))
set_up = 'wrong wealths and wrong rates'
for year in years_btw_1982_and_2000:
	if year == years_btw_1982_and_2000[0]: print "\nyear\t observed difference\t probability of observing that difference if " + set_up
	print year, "\t", 
	print round(float(dec(100.) * difference_in_year_as_a_fraction_of_total_wealth_of_forbes_400[year])), "\t", 
	print round(float(prob_under_estimate_by_same_amount_for[set_up][year]))
print "\nprobability of underestimating by observed differences in each year between 1996 and 2000:\t", cumulative_prob_1996_to_2000_for[set_up]

# Of note, medians were calculated by using one of the empirical quantile functions recommend by Hyndman and Yanan's (1996) "Sample Quantiles in Statistical Packackages" (specifically, the eighth function they consider). According to the authors, it gives approximately median-unbiased estimates of a quartile regardless of the underlying distribution. Numpy's median function yields similar results. 

## SEC. 2.6.2, ESP. FIG. 2.9: EFFECT OF INEQUALITY ##

# The following code looks at the probability of underestimating the total wealth of the Forbes 400 if the distribution of their wealth is changed and all other things are held equal. 

def get_pareto_index_if_wealthiest_perct_owns_one_minus_perct(perct):
	""" Returns the Pareto index that corresponds to a percentage of the population owning one minus their percentage of the population """
	perct_of_pop = perct
	pareto_index = numpy.log(perct_of_pop) / numpy.log(perct_of_pop / (1. - perct_of_pop))
	return pareto_index

def wealths_w_pareto_index_of(pareto_index, total_wealth=dec(summary_stats['sum']['2000']), number_of_people=400):
	""" Returns wealths that precisely follow a Pareto distribution """
	wealths = list()
	for i in range(1, number_of_people+1):
		p = dec(i) / dec(number_of_people) # perct of pop corresponding to person
		p_1 = dec(i - 1) / dec(number_of_people) # perct of pop corresponding to next person
		wealth = total_wealth * (p ** (dec(1.) - (dec(1.) / dec(pareto_index))) - p_1 ** (dec(1.) - (dec(1.) / dec(pareto_index))))
		wealths.append(wealth)
	return wealths

# define dicts for storing results
percts_of_wealth = [_ for _ in (100. * numpy.array(range(201, 399+1, 2)) / 400.)]
biases_for_w_different_inequality = dict(zip(percts_of_wealth, [list() for perct_of_wealth in percts_of_wealth]))
lower_bias_for_w_different_inequality = dict(zip(percts_of_wealth, ['TBD' for perct_of_wealth in percts_of_wealth]))
median_bias_for_w_different_inequality = dict(zip(percts_of_wealth, ['TBD' for perct_of_wealth in percts_of_wealth]))
upper_bias_for_w_different_inequality = dict(zip(percts_of_wealth, ['TBD' for perct_of_wealth in percts_of_wealth]))
prob_under_estimate_by_fixed_amount_for_w_different_inequality = dict(zip(percts_of_wealth, [0 for perct_of_wealth in percts_of_wealth]))
prob_over_estimate_by_fixed_amount_for_w_different_inequality = dict(zip(percts_of_wealth, [0 for perct_of_wealth in percts_of_wealth]))
# set the following to True to re-estimate if already estimated
re_estimate = False
# check if already been estimated
filenames = [filepath + _ + ".pkl" for _ in ["lower_bias_for_w_different_inequality", "median_bias_for_w_different_inequality", "upper_bias_for_w_different_inequality", "prob_under_estimate_by_fixed_amount_for_w_different_inequality", "prob_over_estimate_by_fixed_amount_for_w_different_inequality"]]
already_estimated = False not in [isfile(filename) for filename in filenames]
# estimate or re-estimate
if not already_estimated or re_estimate:
	# set seed
	numpy.random.seed(250624)
	# number of resamples
	number_of_resamples = 10000
	# cut wealths in half
	cut_wealths_in_half = True
	# misestimate mortality rate
	use_estimated_mortality_rates_to_sample = True
	use_estimated_mortality_rates_to_inflate = False
	# get ages, sorted by descending wealth with order of names breaking ties
	ranks = range(0, 400)
	ages_of = dict(zip(ranks, [list() for rank in ranks]))
	for name in names:
		if (data[name]['wealth']['2000'] != ''):
			rank = summary_stats['wealths']['2000'].index(float(data[name]['wealth']['2000']))
			age = str(data[name]['age']['2000'])
			ages_of[rank].append(age)
	ages_to_pair_w_wealths = list()
	for rank in ranks:
		ages_to_pair_w_wealths.extend(ages_of[rank])
	# get fixed amount
	fixed_amount = abs(difference_in['2000'])
	# for each perct of wealth
	for perct_of_wealth in percts_of_wealth:
		# get wealths to pair with ages
		wealths_to_pair_w_ages = wealths_w_pareto_index_of( pareto_index = get_pareto_index_if_wealthiest_perct_owns_one_minus_perct( perct = (100. - perct_of_wealth) / 100.)) # different distributions of wealth
		unknown_total_wealth_of_400_wealthiest = sum(wealths_to_pair_w_ages)
		# for each resample
		for _ in range(1, number_of_resamples+1):
			# pair wealths and ages
			wealth_age_pairs = zip(wealths_to_pair_w_ages, ages_to_pair_w_wealths)
			# draw a sample
			estimate = sample_and_estimate_total_wealth_of_the_400_or_other_wealthiest_from( wealth_age_pairs = wealth_age_pairs, year_for_mortality_rates='2000', cut_wealths_in_half = cut_wealths_in_half, use_estimated_mortality_rates_to_sample = use_estimated_mortality_rates_to_sample, use_estimated_mortality_rates_to_inflate = use_estimated_mortality_rates_to_inflate).get_estimate()
			# get bias
			biases_for_w_different_inequality[perct_of_wealth].append(float(estimate - unknown_total_wealth_of_400_wealthiest))
			if abs(estimate - unknown_total_wealth_of_400_wealthiest) > fixed_amount:
				if estimate - unknown_total_wealth_of_400_wealthiest < 0:
					prob_under_estimate_by_fixed_amount_for_w_different_inequality[perct_of_wealth] += 1
				if estimate - unknown_total_wealth_of_400_wealthiest > 0:
					prob_over_estimate_by_fixed_amount_for_w_different_inequality[perct_of_wealth] += 1
		# record results
		median_bias_for_w_different_inequality[perct_of_wealth] = str(round(scipy.stats.mstats.mquantiles( biases_for_w_different_inequality[perct_of_wealth], prob=[50./100.], alphap=1/3., betap=1/3.)[0]))	
		lower_bias_for_w_different_inequality[perct_of_wealth] = str(round(scipy.stats.mstats.mquantiles( biases_for_w_different_inequality[perct_of_wealth], prob=[2.5/100.], alphap=1/3., betap=1/3.)[0], 1))
		upper_bias_for_w_different_inequality[perct_of_wealth] = str(round(scipy.stats.mstats.mquantiles( biases_for_w_different_inequality[perct_of_wealth], prob=[97.5/100.], alphap=1/3., betap=1/3.)[0], 1))
		prob_under_estimate_by_fixed_amount_for_w_different_inequality[perct_of_wealth] = str(dec(100.) * dec(prob_under_estimate_by_fixed_amount_for_w_different_inequality[perct_of_wealth]) / dec(number_of_resamples))
		prob_over_estimate_by_fixed_amount_for_w_different_inequality[perct_of_wealth] = str(dec(100.) * dec(prob_over_estimate_by_fixed_amount_for_w_different_inequality[perct_of_wealth]) / dec(number_of_resamples))		
		print perct_of_wealth
	# dump results
	filename = filepath + "lower_bias_for_w_different_inequality.pkl"
	pickle.dump(lower_bias_for_w_different_inequality, open(filename, 'wb'))
	filename = filepath + "median_bias_for_w_different_inequality.pkl"
	pickle.dump(median_bias_for_w_different_inequality, open(filename, 'wb'))
	filename = filepath + "upper_bias_for_w_different_inequality.pkl"
	pickle.dump(upper_bias_for_w_different_inequality, open(filename, 'wb'))
	filename = filepath + "prob_under_estimate_by_fixed_amount_for_w_different_inequality.pkl"
	pickle.dump(prob_under_estimate_by_fixed_amount_for_w_different_inequality, open(filename, 'wb'))
	filename = filepath + "prob_over_estimate_by_fixed_amount_for_w_different_inequality.pkl"
	pickle.dump(prob_over_estimate_by_fixed_amount_for_w_different_inequality, open(filename, 'wb'))
else:
	# load results that were dumped using pickle
	lower_bias_for_w_different_inequality = pickle.load(open(filepath + "lower_bias_for_w_different_inequality.pkl", 'rb'))
	median_bias_for_w_different_inequality = pickle.load(open(filepath + "median_bias_for_w_different_inequality.pkl", 'rb'))
	upper_bias_for_w_different_inequality = pickle.load(open(filepath + "upper_bias_for_w_different_inequality.pkl", 'rb'))
	prob_under_estimate_by_fixed_amount_for_w_different_inequality = pickle.load(open(filepath + "prob_under_estimate_by_fixed_amount_for_w_different_inequality.pkl", 'rb'))
	prob_over_estimate_by_fixed_amount_for_w_different_inequality = pickle.load(open(filepath + "prob_over_estimate_by_fixed_amount_for_w_different_inequality.pkl", 'rb'))

print "\nmisestimation with different levels of inequality \nperct of wealth \tlower bias \tmedian bias \tupper bias \tprob underestimate \tprob overestimate"
for perct_of_wealth in percts_of_wealth:
	print perct_of_wealth, "\t", lower_bias_for_w_different_inequality[perct_of_wealth], "\t", median_bias_for_w_different_inequality[perct_of_wealth], "\t", upper_bias_for_w_different_inequality[perct_of_wealth], "\t", prob_under_estimate_by_fixed_amount_for_w_different_inequality[perct_of_wealth], "\t", prob_over_estimate_by_fixed_amount_for_w_different_inequality[perct_of_wealth]

## SEC. 2.6.2, ESP. FIG. 2.10: ESTIMATES WITH EXTREME INEQUALITY ##

# define a function for rounding wealth estimates
def get_rounded(dec_estimate_in_billions):
	""" Takes a dollar amount in billions and rounds it the nearest billion, if the nearest one billion dollars, if it was greater than five billion, and the nearest 100 million, otherwise. Dollar amounts are decimals, not floats """
	# less than five billion?
	if dec_estimate_in_billions < dec(5.0):
		round_to_nearest = dec(0.100) # hundred million
	else:
		round_to_nearest = dec(1.0) # billion
	return dec(round(float(dec_estimate_in_billions / round_to_nearest), 0)) * round_to_nearest

# define dict for storing results
prob_of_estimating_w_extreme_inequality = dict()
# set the following to True to re-estimate if already estimated
re_estimate = False
# check if already been estimated
filename = filepath + "prob_of_estimating_w_extreme_inequality.pkl"
already_estimated = isfile(filename)
# estimate or re-estimate
if not already_estimated or re_estimate:
	# set seed
	numpy.random.seed(250624)
	# number of resamples
	number_of_resamples = 10000
	# cut wealths in half
	cut_wealths_in_half = True
	# misestimate mortality rate
	use_estimated_mortality_rates_to_sample = True
	use_estimated_mortality_rates_to_inflate = False
	# get ages, sorted by descending wealth with order of names breaking ties
	ranks = range(0, 400)
	ages_of = dict(zip(ranks, [list() for rank in ranks]))
	for name in names:
		if (data[name]['wealth']['2000'] != ''):
			rank = summary_stats['wealths']['2000'].index(float(data[name]['wealth']['2000']))
			age = str(data[name]['age']['2000'])
			ages_of[rank].append(age)
	ages_to_pair_w_wealths = list()
	for rank in ranks:
		ages_to_pair_w_wealths.extend(ages_of[rank])
	# estimate probabilities
	biases = list()
	# for the most-extreme level of inequality
	for perct_of_wealth in [99.75]:
		# get wealths to pair with ages
		wealths_to_pair_w_ages = wealths_w_pareto_index_of( pareto_index = get_pareto_index_if_wealthiest_perct_owns_one_minus_perct( perct = (100. - perct_of_wealth) / 100.))
		unknown_total_wealth_of_400_wealthiest = sum(wealths_to_pair_w_ages)		
		# for each resample
		for resample in range(1, number_of_resamples+1):
			# pair wealths and ages
			wealth_age_pairs = zip(wealths_to_pair_w_ages, ages_to_pair_w_wealths)
			# draw a sample
			estimate = sample_and_estimate_total_wealth_of_the_400_or_other_wealthiest_from( wealth_age_pairs = wealth_age_pairs, year_for_mortality_rates='2000', cut_wealths_in_half = cut_wealths_in_half, use_estimated_mortality_rates_to_sample = use_estimated_mortality_rates_to_sample, use_estimated_mortality_rates_to_inflate = use_estimated_mortality_rates_to_inflate).get_estimate()
			estimate = str(get_rounded(estimate / dec(1000.))) # string of rounded estimate in billions
			if estimate in prob_of_estimating_w_extreme_inequality.keys():
				prob_of_estimating_w_extreme_inequality[estimate] += 1
			else:
				prob_of_estimating_w_extreme_inequality[estimate] = 1
			print resample
		for estimate in prob_of_estimating_w_extreme_inequality.keys():
			prob_of_estimating_w_extreme_inequality[estimate] = round(100 * (prob_of_estimating_w_extreme_inequality[estimate] / number_of_resamples), 2)
		# dump results
		filename = filepath + "prob_of_estimating_w_extreme_inequality.pkl"
		pickle.dump(prob_of_estimating_w_extreme_inequality, open(filename, 'wb'))
else:		
	# load results that were dumped using pickle
	prob_of_estimating_w_extreme_inequality = pickle.load(open(filepath + "prob_of_estimating_w_extreme_inequality.pkl", 'rb'))

## SEC. 2.6.2, ESP. TABLE 2.1: EFFECT OF CHANGES RELATIVE TO BASE YEAR ##

# The following code looks at the effect of changes between base_year and 2000 on the probability of underestimating the total wealth of the Forbes 400. 

base_year = '1990'
changes = ['No changes relative to base year', 'Ages', 'Sampling rates', 'Ages and sampling rates', 'Wealths', 'All changes']
# define dict for storing results
prob_under_estimate_by_2000_amount_for_w_changes = dict(zip(set_ups, [dict(zip(changes, [0 for change in changes])) for set_up in set_ups]))
# set the following to True to re-estimate if already estimated
re_estimate = False
# check if already been estimated
filename = filepath + "prob_under_estimate_by_2000_amount_for_w_changes.pkl"
already_estimated = isfile(filename)
# estimate or re-estimate
if not already_estimated or re_estimate:
	# set up simulations
	for set_up in set_ups:
		# number of resamples
		number_of_resamples = 10000
		# cut wealths in half?
		if 'wrong wealths' in set_up:
			cut_wealths_in_half = True
		else:
			cut_wealths_in_half = False
		# misestimate mortality rates?
		if 'wrong rates' in set_up:
			use_estimated_mortality_rates_to_sample = True
			use_estimated_mortality_rates_to_inflate = False
		else:
			use_estimated_mortality_rates_to_sample = True
			use_estimated_mortality_rates_to_inflate = True
		print set_up
		# for each change
		for change in changes:
			# set seed
			numpy.random.seed(250624)
			# get wealths in descending order by year
			# also get ages by year, sorted by descending wealth with order of names breaking ties
			ranks = range(0, 400)
			wealths_in_base_year = dict(zip(ranks, [list() for rank in ranks]))
			wealths_in_2000 = dict(zip(ranks, [list() for rank in ranks]))
			ages_in_base_year = dict(zip(ranks, [list() for rank in ranks]))
			ages_in_2000 = dict(zip(ranks, [list() for rank in ranks]))
			for name in names:
				if (data[name]['wealth'][base_year] != ''):
					rank = summary_stats['wealths'][base_year].index(float(data[name]['wealth'][base_year]))
					wealth = in_2000_dollars(dec(data[name]['wealth'][base_year]), base_year)
					age = str(data[name]['age'][base_year])
					wealths_in_base_year[rank].append(wealth)
					ages_in_base_year[rank].append(age)
				if (data[name]['wealth']['2000'] != ''):
					rank = summary_stats['wealths']['2000'].index(float(data[name]['wealth']['2000']))
					wealth = in_2000_dollars(dec(data[name]['wealth']['2000']), '2000')
					age = str(data[name]['age']['2000'])
					wealths_in_2000[rank].append(wealth)
					ages_in_2000[rank].append(age)
			wealths_in_base_year_to_pair_w_ages = list()
			wealths_in_2000_to_pair_w_ages = list()
			ages_in_base_year_to_pair_w_wealths = list()
			ages_in_2000_to_pair_w_wealths = list()
			for rank in ranks:
				wealths_in_base_year_to_pair_w_ages.extend(wealths_in_base_year[rank])
				wealths_in_2000_to_pair_w_ages.extend(wealths_in_2000[rank])
				ages_in_base_year_to_pair_w_wealths.extend(ages_in_base_year[rank])
				ages_in_2000_to_pair_w_wealths.extend(ages_in_2000[rank])
			# changes wealth, ages, or year of mortality rates
			if change == 'No changes relative to base year':
				# base year wealths, ages, and mortality rates
				wealth_age_pairs = [(in_2000_dollars(dec(data[name]['wealth'][base_year]), base_year), data[name]['age'][base_year]) for name in names if (data[name]['wealth'][base_year] != '')] # same order, so that we get the same result with the same seed
				unknown_total_wealth_of_400_wealthiest = sum(zip(*wealth_age_pairs)[0])
				year_for_mortality_rates = base_year
			elif change == 'All changes':
				# 2000 wealths, ages, and mortality rates
				wealth_age_pairs = [(in_2000_dollars(dec(data[name]['wealth']['2000']), '2000'), data[name]['age']['2000']) for name in names if (data[name]['wealth']['2000'] != '')] # same order, so that we get the same result with the same seed
				unknown_total_wealth_of_400_wealthiest = sum(zip(*wealth_age_pairs)[0])
				year_for_mortality_rates = '2000'
			else:
				# change wealth?
				if change == 'Wealths':
					# 2000 wealths, but 1990 ages
					wealth_age_pairs = zip(wealths_in_2000_to_pair_w_ages, ages_in_base_year_to_pair_w_wealths)
				# change ages?
				if 'Ages' in change:
					# 1990 wealths, but 2000 ages
					wealth_age_pairs = zip(wealths_in_base_year_to_pair_w_ages, ages_in_2000_to_pair_w_wealths)
				# total wealth
				unknown_total_wealth_of_400_wealthiest = sum(zip(*wealth_age_pairs)[0])
				# change mortality rates?
				if 'sampling rates' in change.lower():
					year_for_mortality_rates = '2000'
				else:
					year_for_mortality_rates = base_year
			# for each resample
			for _ in range(1, number_of_resamples+1):
				# draw a sample
				estimate = sample_and_estimate_total_wealth_of_the_400_or_other_wealthiest_from( wealth_age_pairs = wealth_age_pairs, year_for_mortality_rates = year_for_mortality_rates, cut_wealths_in_half = cut_wealths_in_half, use_estimated_mortality_rates_to_sample = use_estimated_mortality_rates_to_sample, use_estimated_mortality_rates_to_inflate = use_estimated_mortality_rates_to_inflate).get_estimate()
				# get bias
				bias = (estimate - unknown_total_wealth_of_400_wealthiest) / (unknown_total_wealth_of_400_wealthiest) # bias as a percent
				if not bias > difference_in_2000_as_a_fraction_of_total_wealth_of_forbes_400:
					prob_under_estimate_by_2000_amount_for_w_changes[set_up][change] += 1
			prob_under_estimate_by_2000_amount_for_w_changes[set_up][change] = str(dec(100.) * dec(prob_under_estimate_by_2000_amount_for_w_changes[set_up][change]) / dec(number_of_resamples))
			print change
	# dump results
	filename = filepath + "prob_under_estimate_by_2000_amount_for_w_changes.pkl"
	pickle.dump(prob_under_estimate_by_2000_amount_for_w_changes, open(filename, 'wb'))
else:		
	# load results that were dumped using pickle
	prob_under_estimate_by_2000_amount_for_w_changes = pickle.load(open(filepath + "prob_under_estimate_by_2000_amount_for_w_changes.pkl", 'rb'))

# print results
print "\neffect of changes between " + base_year + " and 2000"
for set_up in set_ups:
	print "\nset up: " + set_up + " \nchange \tprob of underestimating by 2000 amount given change"
	for change in changes:
		print change, "\t", prob_under_estimate_by_2000_amount_for_w_changes[set_up][change]

## SEC. A.5: EFFECT OF CHANGES BETWEEN 1918 AND 2000 ##

# get data on thirty wealthiest Americans in 1918
filename = filepath + "thirty_wealthiest_in_1918_data.txt"
header = ['Rank', 'Name', 'Wealth (in millions)', 'Born', 'Died', 'Gender', 'Age in 1918 (assuming birthday already occurred)', 'Prob of dying in 1918']
raw_thirty_wealthiest_in_1918_data = numpy.loadtxt(filename, delimiter='\t', skiprows=5, dtype={'names':header, 'formats':['<S50']*len(header)})
wealths_in_1918, ages_in_1918 = list(), list()
for i in range(0, len(raw_thirty_wealthiest_in_1918_data['Rank'])):
	wealth = dec(raw_thirty_wealthiest_in_1918_data['Wealth (in millions)'][i])
	age = raw_thirty_wealthiest_in_1918_data['Age in 1918 (assuming birthday already occurred)'][i]
	gender = raw_thirty_wealthiest_in_1918_data['Gender'][i].lower() # continue to ignore gender for simplicity
	wealths_in_1918.append(wealth)
	ages_in_1918.append(age)
wealth_age_pairs_for_1918 = zip(wealths_in_1918, ages_in_1918)

# get data on thirty wealthiest Americans in 2000
wealths_in_2000, ages_in_2000 = list(), list()
for year in ['2000']:
	for name in names:
		if data[name]['wealth'][year] != '':
			wealth = data[name]['wealth'][year]
			rank = summary_stats['wealths'][year].index(float(wealth)) # note: no ties for 30th wealthiest in 2000
			if rank < 30:
				wealths_in_2000.append(dec(wealth))
				age = data[name]['age'][year]
				if name in ['Walton, Alice L', 'Walton, Helen R', 'Anthony, Barbara Cox', 'Chambers, Anne Cox', 'Johnson, Abigail']:
					gender = 'female'
				else:
					gender = 'male' # continue to ignore gender for simplicity
				ages_in_2000.append(age)
wealth_age_pairs_for_2000 = zip(wealths_in_2000, ages_in_2000)

base_year = '1918'
# define dicts for storing results
biases_for = dict(zip(changes, [list() for change in changes]))
median_bias_as_a_perct_of_wealth_for_w_changes = dict(zip(changes, ['TBD' for change in changes]))
# set the following to True to re-estimate if already estimated
re_estimate = False
# check if already been estimated
filename = filepath + "median_bias_as_a_perct_of_wealth_for_w_changes.pkl"
already_estimated = isfile(filename)
# estimate or re-estimate
if not already_estimated or re_estimate:
	# number of resamples
	number_of_resamples = 10000
	# fully value wealths
	cut_wealths_in_half = False
	# use K&S rates to sample and inflate
	use_estimated_mortality_rates_to_sample = False
	use_estimated_mortality_rates_to_inflate = False
	# for each change
	for change in changes:
		# set seed
		numpy.random.seed(250624)
		# changes wealth, ages, or year of sampling rates
		if change == 'No changes relative to base year':
			# base year wealths, ages, and sampling rates
			wealth_age_pairs = wealth_age_pairs_for_1918
			unknown_total_wealth_of_30_wealthiest = sum(zip(*wealth_age_pairs)[0])
			year_for_mortality_rates = base_year
			wrong_rates_in_year = False
		elif change == 'All changes':
			# 2000 wealths, ages, and sampling rates
			wealth_age_pairs = wealth_age_pairs_for_2000
			unknown_total_wealth_of_30_wealthiest = sum(zip(*wealth_age_pairs)[0])
			year_for_mortality_rates = '2000'
			wrong_rates_in_year = False
		else:
			# change wealth?
			if change == 'Wealths':
				# 2000 wealths, but 1918 ages
				wealth_age_pairs = zip(wealths_in_2000, ages_in_1918)
			# change ages?
			if 'Ages' in change:
				# 1918 wealths, but 2000 ages
				wealth_age_pairs = zip(wealths_in_1918, ages_in_2000)
			# total wealth
			unknown_total_wealth_of_30_wealthiest = sum(zip(*wealth_age_pairs)[0])
			# change sampling rates?
			if 'sampling rates' in change.lower():
				year_for_mortality_rates = '2000'
				wrong_rates_in_year = False
			else:
				year_for_mortality_rates = base_year
				wrong_rates_in_year = False
		# for each resample
		for _ in range(1, number_of_resamples+1):
			# draw a sample and get estimate			
			estimate = sample_and_estimate_total_wealth_of_the_400_or_other_wealthiest_from( wealth_age_pairs = wealth_age_pairs, year_for_mortality_rates = year_for_mortality_rates, number_of_wealthiest=30, cut_wealths_in_half = cut_wealths_in_half, use_estimated_mortality_rates_to_sample = use_estimated_mortality_rates_to_sample, use_estimated_mortality_rates_to_inflate = use_estimated_mortality_rates_to_inflate).get_estimate()
			# get bias
			bias = (estimate - unknown_total_wealth_of_30_wealthiest) / (unknown_total_wealth_of_30_wealthiest) # bias as a percent
			biases_for[change].append(100. * float(bias))
		# record results
		median_bias_as_a_perct_of_wealth_for_w_changes[change] = str(round(scipy.stats.mstats.mquantiles( biases_for[change], prob=[50./100.], alphap=1/3., betap=1/3.)[0], 2))
		print change
	# dump results
	filename = filepath + "median_bias_as_a_perct_of_wealth_for_w_changes.pkl"
	pickle.dump(median_bias_as_a_perct_of_wealth_for_w_changes, open(filename, 'wb'))
else:		
	# load results that were dumped using pickle
	median_bias_as_a_perct_of_wealth_for_w_changes = pickle.load(open(filepath + "median_bias_as_a_perct_of_wealth_for_w_changes.pkl", 'rb'))

# print results
print "\neffect of changes between 1918 and 2000 \nset up: right wealths and right rates \nchange \tmedian misestimation as a perct of wealth given change"
for change in changes:
	print change, "\t", median_bias_as_a_perct_of_wealth_for_w_changes[change]

## lastline=986
if __name__ == '__main__':

	#### FIGURES ####

	# draw figure
	import pylab
	from matplotlib import rc
	from matplotlib import lines
	#import datetime
	#from matplotlib.dates import YearLocator, DateFormatter
	from matplotlib.ticker import MultipleLocator

	# size of figures
	figwidth = (430.00627) / 72.27 # \textwidth in pts times pts per inch, following cookbook example 
	figheight = figwidth * (numpy.sqrt(5)-1.0) / 2.0 # golden ratio, following cookbook example
	figsize = (figwidth, figheight)

	rc('text', usetex=True)
	#rc('font',**{'family':'sans-serif','sans-serif':['Computer Modern Sans Serif']}) # for beamer
	#rc('font', family='sans-serif')
	rc('font',**{'family':'serif','serif':['Computer Modern Roman']}) # for document
	rc('font', family='serif')
	fontsize = 11
	pylab.rcParams['font.size'] = fontsize
	pylab.rcParams['axes.labelsize'] = fontsize
	pylab.rcParams['legend.fontsize'] = fontsize
	pylab.rcParams['xtick.labelsize'] = fontsize
	pylab.rcParams['ytick.labelsize'] = fontsize

	## FIG A.1: K&S VS. SCF ESTIMATES OF THE WEALTH OF WEALTHIEST ONE PERCENT ##

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=figsize)
	fig_ax = fig.gca()
	# plot scf estimates converted to trillions of dollars
	fig_ax.plot([int(year) for year in ['1989', '1992', '1995', '1998', '2001']], list(scf_estimate_of_wealth_of_wealthiest_one_percent_in[year] / dec(1000.) for year in ['1989', '1992', '1995', '1998', '2001']), color='k', linestyle='solid', lw=0.5, marker='o', markersize=2, clip_on=False, label='Household survey estimates') # no markers if more than 10 data points
	# plot k & s estimates converted to trillions of dollars
	years_btw_1989_and_2000 = [str(y) for y in range(1989, 2000+1)]
	fig_ax.plot([int(year) for year in years_btw_1989_and_2000], list(ks_estimate_of_wealth_of_wealthiest_one_percent_in[year] / dec(1000.) for year in years_btw_1989_and_2000), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=2, clip_on=False, label='Estate-multiplier estimates') # no markers if more than 10 data points
	# make a legend
	leg = fig_ax.legend(loc='upper left', frameon=False, handlelength=3.3, handletextpad=0.5)
	# label the axes
	fig_ax.set_xlabel('Year')
	fig_ax.set_ylabel('Trillions of 2000 dollars') # sentence-style capitalization
	# set limits
	pylab.xlim(1989, 2001)
	pylab.ylim(0, 15)
	# major xticks
	pylab.xticks([1989, 1992, 1995, 1998, 2001], years_of_scf[0:4+1])
	# minor xticks
	fig_ax.xaxis.set_minor_locator(MultipleLocator(1)) # tick every year
	# add y ticks
	pylab.yticks([0, 5, 10, 15], ['0', '5', '10', '15'])
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# adjust dimensions
	fig.subplots_adjust(left=0.12, bottom=0.14, right=0.95, top=0.96, wspace=0.0, hspace=0.0)
	# save figure
	pylab.savefig("./figures/figures_for_first_essay/ks_vs_scf_estimates_for_wealthiest_one_percent.pdf", bbox_inches='tight')

	## FIG 2.1: DIRECT VS. ESTATE-MULTIPLIER ESTIMATES ##

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=figsize)
	fig_ax = fig.gca()
	# plot direct estimates
	years_btw_1982_and_2002 = [str(y) for y in range(1982, 2002+1)]
	fig_ax.plot([int(year) for year in years_btw_1982_and_2002], list(in_2000_dollars(dec(summary_stats['sum'][year]), year) / dec(1000.) for year in years_btw_1982_and_2002), color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='Direct estimates') # no markers if more than 10 data points
	# plot estate-multiplier estimates
	fig_ax.plot([int(year) for year in years_btw_1982_and_2000], list(ks_estimated_wealth_of_400_wealthiest_in[year] / dec(1000.) for year in years_btw_1982_and_2000), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label='Estate-multiplier estimates') # no markers if more than 10 data points
	# make a legend
	leg = fig_ax.legend(loc='upper left', frameon=False, handlelength=3.3, handletextpad=0.5)
	# label the axes
	fig_ax.set_xlabel('Year')
	fig_ax.set_ylabel('Trillions of 2000 dollars') # sentence-style capitalization
	# set limits
	pylab.xlim(1982, 2000)
	pylab.ylim(0, 1500)
	# major xticks
	pylab.xticks([1982, 1990, 2000], ['1982', '1990', '2000'])
	# minor xticks
	fig_ax.xaxis.set_minor_locator(MultipleLocator(1)) # tick every year
	# add y ticks
	pylab.yticks([0, 500, 1000, 1500], ['0', '0.5', '1.0', '1.5'])
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# adjust dimensions
	fig.subplots_adjust(left=0.12, bottom=0.14, right=0.95, top=0.96, wspace=0.0, hspace=0.0)
	# save figure
	pylab.savefig("./figures/figures_for_first_essay/direct_vs_emes.pdf", bbox_inches='tight')

	## FIG 2.2: ASSUMED VS. ESTIMATE MORTALITY RATES FOR MALES IN 2000 ##

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=figsize)
	fig_ax = fig.gca()
	ages = [str(age) for age in range(30, 90+1)] # ages range from 28 to 92 in 2000
	# plot assumed
	fig_ax.plot([int(age) for age in ages], [float(mortality_rate_of['male']['1990'][age]) * float(relative_mortality_rate_of['male'][age]) for age in ages], color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='Kopzuk-Saez for 1990') # no markers if more than 10 data points
	fig_ax.plot([int(age) for age in ages], [float(mortality_rate_of['male']['2000'][age]) * float(relative_mortality_rate_of['male'][age]) for age in ages], color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label='Kopzuk-Saez for 2000') # no markers if more than 10 data points
	# plot actual
	fig_ax.plot([int(age) for age in ages], [estimated_mortality_rate_in(year='1990', age=age) for age in ages], color='k', linestyle='dotted', lw=0.5, marker='o', markersize=0, clip_on=True, label="Estimated for 1990") # no markers if more than 10 data points
	fig_ax.plot([int(age) for age in ages], [estimated_mortality_rate_in(year='2000', age=age) for age in ages], color='k', linestyle='dashdot', lw=0.5, marker='o', markersize=0, clip_on=True, label="Estimated for 2000") # no markers if more than 10 data points
	# make a legend
	leg = fig_ax.legend(loc='upper left', frameon=False, handlelength=3.3, handletextpad=0.5)
	# label the axes
	fig_ax.set_xlabel('Age')
	fig_ax.set_ylabel('Mortality rate (\%)') # sentence-style capitalization
	# log the y axis
	fig_ax.set_yscale('log')
	# set limits
	pylab.xlim(30, 90)
	pylab.ylim(0.001, 0.18509)
	# major xticks
	pylab.xticks([30, 40, 50, 60, 70, 80, 90], ['30', '40', '50', '60', '70', '80', '90'])
	# add y ticks
	pylab.yticks([0.001, 0.01, 0.10, 0.18509], ['0.1', '1', '10', '19'])
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# adjust dimensions
	fig.subplots_adjust(left=0.12, bottom=0.14, right=0.95, top=0.96, wspace=0.0, hspace=0.0)
	# save figure
	pylab.savefig("./figures/figures_for_first_essay/assumed_and_estimated_mortality_rates_in_1990_and_2000.pdf", bbox_inches='tight')

	## FIGS 2.3 AND 2.11: ASSUMED VS. ESTIMATED ##

	# draw figure for both 1982 to 2000 and for 1982 to 2010
	for _ in ['1982_to_2000', '1982_to_2010']:
		# draw figure
		try:
			fignumber += 1
		except NameError:
			fignumber = 1
		fig = pylab.figure(fignumber, figsize=figsize)
		fig_ax = fig.gca()
		# plot actual
		fig_ax.plot([int(year) for year in years_less_last], list(in_2000_dollars(dec(summary_stats['sum'][year]), year) / dec(1000.) for year in years_less_last), color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='Assumed') # no markers if more than 10 data points
		# plot estate-multiplier estimates
		fig_ax.plot([int(year) for year in years_less_last], list(eme_of_wealth_of_400_wealthiest_in_w_ks_mortality_rates[year] / dec(1000.) for year in years_less_last), color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label='Estimated') # no markers if more than 10 data points
		# make a legend
		leg = fig_ax.legend(loc='upper left', frameon=False, handlelength=3.3, handletextpad=0.5)
		# label the axes
		fig_ax.set_xlabel('Year')
		fig_ax.set_ylabel('Trillions of 2000 dollars') # sentence-style capitalization
		# set x limits and draw major xticks
		if _ == '1982_to_2000':
			pylab.xlim(1982, 2000)
			pylab.xticks([1982, 1990, 2000], ['1982', '1990', '2000'])
		elif _ == '1982_to_2010':
			pylab.xlim(1982, 2010)
			pylab.xticks([1982, 1990, 2000, 2010], ['1982', '1990', '2000', '2010'])
		# minor xticks
		fig_ax.xaxis.set_minor_locator(MultipleLocator(1)) # tick every year
		# set y limits
		pylab.ylim(0, 1500)
		# add y ticks
		pylab.yticks([0, 500, 1000, 1500], ['0', '0.5', '1.0', '1.5'])
		# clean ticks
		for tickline in fig_ax.get_xticklines():
			tickline.set_markersize(8)
			tickline.set_markeredgewidth(0.5)
			tickline.set_marker(lines.TICKDOWN)
		for tickline in fig_ax.xaxis.get_minorticklines():
			tickline.set_markersize(4)
			tickline.set_markeredgewidth(0.5)
			tickline.set_marker(lines.TICKDOWN)
		for tickline in fig_ax.get_yticklines():
			tickline.set_markersize(8)
			tickline.set_markeredgewidth(0.5)
			tickline.set_marker(lines.TICKLEFT)
		for tickline in fig_ax.yaxis.get_minorticklines():
			tickline.set_markersize(4)
			tickline.set_markeredgewidth(0.5)
			tickline.set_marker(lines.TICKLEFT)
		# shift labels
		for ticklabel in fig_ax.get_xticklabels():
			ticklabel.set_y(0-0.04)
		for ticklabel in fig_ax.get_yticklabels():
			ticklabel.set_x(0-0.02)
		# set frame width
		for i in fig_ax.spines.keys():
			fig_ax.spines[i].set_linewidth(0.5)
		# get rid of ticks on top and right
		fig_ax.get_yaxis().tick_left()
		fig_ax.get_xaxis().tick_bottom()
		# adjust dimensions
		fig.subplots_adjust(left=0.12, bottom=0.14, right=0.95, top=0.96, wspace=0.0, hspace=0.0)
		# save figure
		pylab.savefig("./figures/figures_for_first_essay/assumed_vs_estimated_w_ks_mortality_rates_" + _ + ".pdf", bbox_inches='tight')

	## FIG 2.4: MISESTIMATION WITH RIGHT AND WRONG WEALTHS ##

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=figsize)
	fig_ax = fig.gca()
	# plot misestimation
	misestimation = list(((eme_of_wealth_of_400_wealthiest_in_w_ks_mortality_rates[year] - in_2000_dollars(dec(summary_stats['sum'][year]), year)) / dec(1000.)) for year in years_less_last)
	fig_ax.plot([int(year) for year in years_less_last], misestimation, color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='Right wealths') # no markers if more than 10 data points
	# plot misestimation
	misestimation = list(((eme_of_wealth_of_400_wealthiest_in_w_ks_mortality_rates_and_halved_wealths[year] - in_2000_dollars(dec(summary_stats['sum'][year]), year)) / dec(1000.)) for year in years_less_last)
	fig_ax.plot([int(year) for year in years_less_last], misestimation, color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label='Wrong wealths') # no markers if more than 10 data points
	# make a legend
	leg = fig_ax.legend(loc='lower left', frameon=False, handlelength=3.3, handletextpad=0.5)
	# label the axes
	fig_ax.set_xlabel('Year')
	fig_ax.set_ylabel('Misestimation ($\mathdollar$Ts)') # sentence-style capitalization
	# set limits and draw major xticks
	pylab.xlim(1982, 2000)
	pylab.ylim(-1000, 0)
	pylab.xticks([1982, 1990, 2000], ['1982', '1990', '2000'])
	# minor xticks
	fig_ax.xaxis.set_minor_locator(MultipleLocator(1)) # tick every year
	# add y ticks
	pylab.yticks([-1000, -500, 0], ['$-$1', '$-$0.5', '0'])
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# adjust dimensions
	fig.subplots_adjust(left=0.12, bottom=0.14, right=0.95, top=0.96, wspace=0.0, hspace=0.0)
	# save figure
	pylab.savefig("./figures/figures_for_first_essay/misestimation_w_actual_and_halved_wealths_1982_to_2000.pdf", bbox_inches='tight')

	## FIGS 2.5 TO 2.8: MISESTIMATION PROB AND AMOUNT UNDER DIFFERENT ASSUMPTIONS ##

	smaller_figwidth = (0.85 * 430.00627) / 72.27 # 85% of \textwidth in pts times pts per inch, following cookbook example 
	smaller_figheight = smaller_figwidth * (numpy.sqrt(5)-1.0) / 2.0 # golden ratio, following cookbook example
	smaller_figsize = (smaller_figwidth, smaller_figheight)

	# wrong wealths

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=smaller_figsize)
	fig_ax = fig.gca()
	# plot with halved wealths
	set_up = 'wrong wealths but right rates'
	fig_ax.plot([int(year) for year in years_less_last], [float(prob_under_estimate_by_2000_amount_for[set_up][year]) for year in years_less_last], color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='Wrong wealths') # no markers if more than 10 data points
	# plot without halved wealths
	set_up = 'right wealths and right rates'
	fig_ax.plot([int(year) for year in years_less_last], [float(prob_under_estimate_by_2000_amount_for[set_up][year]) for year in years_less_last], color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label='Right wealths') # no markers if more than 10 data points
	# make a legend
	leg = fig_ax.legend(loc='upper left', frameon=False, handlelength=3.3, handletextpad=0.5, ncol=1, columnspacing=0.5)
	# label the axes
	fig_ax.set_xlabel('Year')
	fig_ax.set_ylabel(r'Prob$\{\hbox{Misestimation} \leq -76\%\}$') # sentence-style capitalization
	# set limits
	pylab.xlim(1982, 2000)
	pylab.ylim(0, 100)
	# major xticks
	pylab.xticks([1982, 1990, 2000], ['1982', '1990', '2000'])
	# minor xticks
	fig_ax.xaxis.set_minor_locator(MultipleLocator(1)) # tick every year
	# add y ticks
	pylab.yticks([0, 50, 100], ['0', '.5', '1.0'])
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# adjust dimensions
	fig.subplots_adjust(left=0.12, bottom=0.14, right=0.95, top=0.96, wspace=0.0, hspace=0.0)
	# save figure
	pylab.savefig("./figures/figures_for_first_essay/misestimation_prob_by_year.pdf", bbox_inches='tight')

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=smaller_figsize)
	fig_ax = fig.gca()
	# plot with halved wealths
	set_up = 'wrong wealths but right rates'
	fig_ax.plot([int(year) for year in years_less_last], [float(median_bias_for[set_up][year]) / 1000. for year in years_less_last], color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='Wrong wealths') # no markers if more than 10 data points
	# plot without halved wealths
	set_up = 'right wealths and right rates'
	fig_ax.plot([int(year) for year in years_less_last], [float(median_bias_for[set_up][year]) / 1000. for year in years_less_last], color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label='Right wealths') # no markers if more than 10 data points
	# make a legend
	leg = fig_ax.legend(loc='lower left', frameon=False, handlelength=3.3, handletextpad=0.5)
	# label the axes
	fig_ax.set_xlabel('Year')
	fig_ax.set_ylabel('Median misestimation ($\mathdollar$Ts)') # sentence-style capitalization
	# set limits
	pylab.xlim(1982, 2000)
	pylab.ylim(-1000, 0)
	# major xticks
	pylab.xticks([1982, 1990, 2000], ['1982', '1990', '2000'])
	# minor xticks
	fig_ax.xaxis.set_minor_locator(MultipleLocator(1)) # tick every year
	# add y ticks
	pylab.yticks([-1000, -500, 0], ['$-$1.0', '$-$0.5', '0'])
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# adjust dimensions
	fig.subplots_adjust(left=0.12, bottom=0.14, right=0.95, top=0.96, wspace=0.0, hspace=0.0)
	# save figure
	pylab.savefig("./figures/figures_for_first_essay/misestimation_amount_by_year.pdf", bbox_inches='tight')

	# wrong rates

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=smaller_figsize)
	fig_ax = fig.gca()
	# plot with halved wealths and misestimated mortality rates
	set_up = 'wrong wealths and wrong rates'
	fig_ax.plot([int(year) for year in years_less_last], [float(prob_under_estimate_by_2000_amount_for[set_up][year]) for year in years_less_last], color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='Wrong wealths and rates') # no markers if more than 10 data points
	# plot with misestimated mortality rates, but without halved wealths
	set_up = 'right wealths but wrong rates'
	fig_ax.plot([int(year) for year in years_less_last], [float(prob_under_estimate_by_2000_amount_for[set_up][year]) for year in years_less_last], color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label='Right wealths, but wrong rates') # no markers if more than 10 data points
	# plot without halved wealths or misestimated mortality rates
	set_up = 'right wealths and right rates'
	fig_ax.plot([int(year) for year in years_less_last], [float(prob_under_estimate_by_2000_amount_for[set_up][year]) for year in years_less_last], color='k', linestyle='dotted', lw=0.5, marker='o', markersize=0, clip_on=True, label='Right wealths and rates') # no markers if more than 10 data points
	# make a legend
	leg = fig_ax.legend(loc='upper left', frameon=False, handlelength=3.3, handletextpad=0.5, ncol=1, columnspacing=0.5)
	# label the axes
	fig_ax.set_xlabel('Year')
	fig_ax.set_ylabel(r'Prob$\{\hbox{Misestimation} \leq -76\%\}$') # sentence-style capitalization
	# set limits
	pylab.xlim(1982, 2000)
	pylab.ylim(0, 100)
	# major xticks
	pylab.xticks([1982, 1990, 2000], ['1982', '1990', '2000'])
	# minor xticks
	fig_ax.xaxis.set_minor_locator(MultipleLocator(1)) # tick every year
	# add y ticks
	pylab.yticks([0, 50, 100], ['0', '.5', '1.0'])
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# adjust dimensions
	fig.subplots_adjust(left=0.12, bottom=0.14, right=0.95, top=0.96, wspace=0.0, hspace=0.0)
	# save figure
	pylab.savefig("./figures/figures_for_first_essay/misestimation_prob_by_year_w_diff_mortality_rates.pdf", bbox_inches='tight')

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=smaller_figsize)
	fig_ax = fig.gca()
	# plot with halved wealths and misestimated mortality rates
	set_up = 'wrong wealths and wrong rates'
	fig_ax.plot([int(year) for year in years_less_last], [float(median_bias_for[set_up][year]) / 1000. for year in years_less_last], color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='Wrong wealths and rates') # no markers if more than 10 data points
	# plot with misestimated mortality rates, but without halved wealths
	set_up = 'right wealths but wrong rates'
	fig_ax.plot([int(year) for year in years_less_last], [float(median_bias_for[set_up][year]) / 1000. for year in years_less_last], color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label='Right wealths, but wrong rates') # no markers if more than 10 data points
	# plot without halved wealths or misestimated mortality rates
	set_up = 'right wealths and right rates'
	fig_ax.plot([int(year) for year in years_less_last], [float(median_bias_for[set_up][year]) / 1000. for year in years_less_last], color='k', linestyle='dotted', lw=0.5, marker='o', markersize=0, clip_on=True, label='Right wealths and rates') # no markers if more than 10 data points
	# make a legend
	leg = fig_ax.legend(loc='lower left', frameon=False, handlelength=3.3, handletextpad=0.5)
	# label the axes
	fig_ax.set_xlabel('Year')
	fig_ax.set_ylabel('Median misestimation ($\mathdollar$Ts)') # sentence-style capitalization
	# set limits
	pylab.xlim(1982, 2000)
	pylab.ylim(-1000, 0)
	# major xticks
	pylab.xticks([1982, 1990, 2000], ['1982', '1990', '2000'])
	# minor xticks
	fig_ax.xaxis.set_minor_locator(MultipleLocator(1)) # tick every year
	# add y ticks
	pylab.yticks([-1000, -500, 0], ['$-$1.0', '$-$0.5', '0'])
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# adjust dimensions
	fig.subplots_adjust(left=0.12, bottom=0.14, right=0.95, top=0.96, wspace=0.0, hspace=0.0)
	# save figure
	pylab.savefig("./figures/figures_for_first_essay/misestimation_amount_by_year_w_diff_mortality_rates.pdf", bbox_inches='tight')

	## FIG 2.9: EFFECT OF INEQUALITY ##

	# prob of misestimation

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=smaller_figsize)
	fig_ax = fig.gca()
	# plot under estimate
	fig_ax.plot(percts_of_wealth, [float(prob_under_estimate_by_fixed_amount_for_w_different_inequality[perct_of_wealth]) for perct_of_wealth in percts_of_wealth], color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='Misestimation $\leq$ $-$76\%') # no markers if more than 10 data points
	# plot over estimate
	fig_ax.plot(percts_of_wealth, [float(prob_over_estimate_by_fixed_amount_for_w_different_inequality[perct_of_wealth]) for perct_of_wealth in percts_of_wealth], color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label=r"Misestimation $\geq$ $+$76\%") # no markers if more than 10 data points
	# make a legend
	leg = fig_ax.legend(loc='upper left', frameon=False, handlelength=3.3, handletextpad=0.5, ncol=1, columnspacing=0.5)
	# label the axes
	fig_ax.set_xlabel('Percentage of wealth held by inverse percentage of population')
	fig_ax.set_ylabel('Probability') # sentence-style capitalization
	# set limits
	pylab.xlim(50.25, 99.75)
	pylab.ylim(0.0, 100.0)
	# major xticks
	pylab.xticks([50.25, 60.0, 70.0, 80.0, 90.0, 99.75], ['50.25', '60', '70', '80', '90', '99.75'])
	# add y ticks
	pylab.yticks([0.0, 50.0, 100.0], ['0', '.5', '1.0'])
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# adjust dimensions
	fig.subplots_adjust(left=0.12, bottom=0.14, right=0.95, top=0.96, wspace=0.0, hspace=0.0)
	# save figure
	pylab.savefig("./figures/figures_for_first_essay/misestimation_prob_w_different_wealths.pdf", bbox_inches='tight')

	# median misestimation

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=smaller_figsize)
	fig_ax = fig.gca()
	# plot median bias
	fig_ax.plot(percts_of_wealth, [float(median_bias_for_w_different_inequality[perct_of_wealth]) / 1000. for perct_of_wealth in percts_of_wealth], color='k', linestyle='solid', lw=0.5, marker='o', markersize=0, clip_on=True, label='Median') # no markers if more than 10 data points
	# plot 95% CI
	fig_ax.plot(percts_of_wealth, [float(upper_bias_for_w_different_inequality[perct_of_wealth]) / 1000. for perct_of_wealth in percts_of_wealth], color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True, label='95\% CI') # no markers if more than 10 data points
	fig_ax.plot(percts_of_wealth, [float(lower_bias_for_w_different_inequality[perct_of_wealth]) / 1000. for perct_of_wealth in percts_of_wealth], color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True) # no markers if more than 10 data points
	# make a legend
	leg = fig_ax.legend(loc='lower left', frameon=False, handlelength=3.3, handletextpad=0.5)
	# label the axes
	fig_ax.set_xlabel('Percentage of wealth held by inverse percentage of population')
	fig_ax.set_ylabel('Misestimation ($\mathdollar$Ts)') # sentence-style capitalization
	# set limits
	pylab.xlim(50.25, 99.75)
	pylab.ylim(-1500, 0.0)
	# major xticks
	pylab.xticks([50.25, 60, 70, 80, 90, 99.75], ['50.25', '60', '70', '80', '90', '99.75'])
	# add y ticks
	pylab.yticks([-1500, -1000, -500, 0.0], ['$-$1.5', '$-$1.0', '$-$0.5', '0'])
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# adjust dimensions
	fig.subplots_adjust(left=0.12, bottom=0.14, right=0.95, top=0.96, wspace=0.0, hspace=0.0)
	# save figure
	pylab.savefig("./figures/figures_for_first_essay/misestimation_amount_w_different_wealths.pdf", bbox_inches='tight')

	## FIG 2.10: DISTRIBUTION OF ESTIMATES UNDER EXTREME INEQUALITY ##

	try:
		fignumber += 1
	except NameError:
		fignumber = 1
	fig = pylab.figure(fignumber, figsize=figsize)
	fig_ax = fig.gca()
	# plot assumed
	fig_ax.plot([1198, 1198], [0.001, 110], color='k', linestyle='dashed', lw=0.5, marker='o', markersize=0, clip_on=True) # no markers if more than 10 data points
	estimates = [dec(estimate) for estimate in prob_of_estimating_w_extreme_inequality.keys()]
	# plot estimates
	estimates = prob_of_estimating_w_extreme_inequality.keys()
	for estimate in estimates:
		if estimate == '0.10' or estimate == '238933.00':
			lw = 7.0 # so the largest and smallest stand out
		else:
			lw = 4.0
		fig_ax.plot([estimate, estimate], [0.001, prob_of_estimating_w_extreme_inequality[estimate]], color='k', linestyle='solid', drawstyle='step-mid', lw=lw, marker='o', markersize=0, clip_on=True) # no markers if more than 10 data points
	estimate = estimated_mortality_rate_in(year='2000', age=data['Gates, William Henry III']['age']['2000'])
	print "\nNote that the estimated mortality rate for a person like Bill Gates in 2000 was " + str(estimate) + ". So, a person like him should haved died about 1-in-" + str(format(round(1./estimate), '.0f')) + " times."
	# label the axes
	fig_ax.set_xlabel('Estimate')
	fig_ax.set_ylabel('Frequency') # sentence-style capitalization
	# log the axes
	fig_ax.set_xscale('log')
	# major xticks
	pylab.xticks([0.10, 1, 10, 100, 1198, 10000, 238933], ['$\mathdollar$100M', '$\mathdollar$1B', '$\mathdollar$10B', '$\mathdollar$100B', '$\mathdollar$1.198T', '$\mathdollar$10T', '$\mathdollar$24T'])
	# add y ticks
	pylab.yticks([0.0, 15, 30], ['0', '0.15', '0.30'])
	# set limits
	pylab.xlim(0.10, 238933)
	pylab.ylim(0.0, 30)
	# clean ticks
	for tickline in fig_ax.get_xticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.xaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKDOWN)
	for tickline in fig_ax.get_yticklines():
		tickline.set_markersize(8)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	for tickline in fig_ax.yaxis.get_minorticklines():
		tickline.set_markersize(4)
		tickline.set_markeredgewidth(0.5)
		tickline.set_marker(lines.TICKLEFT)
	# shift labels
	for ticklabel in fig_ax.get_xticklabels():
		ticklabel.set_y(0-0.04)
	for ticklabel in fig_ax.get_yticklabels():
		ticklabel.set_x(0-0.02)
	# set frame width
	for i in fig_ax.spines.keys():
		fig_ax.spines[i].set_linewidth(0.5)
	# get rid of ticks on top and right
	fig_ax.get_yaxis().tick_left()
	fig_ax.get_xaxis().tick_bottom()
	# adjust dimensions
	fig.subplots_adjust(left=0.12, bottom=0.14, right=0.95, top=0.96, wspace=0.0, hspace=0.0)
	# save figure
	pylab.savefig("./figures/figures_for_first_essay/estimates_w_extreme_inequality.pdf", bbox_inches='tight')

	#### TABLES ####

	## TABLE 2.1: EFFECT OF CHANGES BETWEEN 1990 AND 2000 ##

	# uses packages ctable and dcolumn
	f = open("./tables/tables_specific_to_first_essay/effect_of_changes_btw_1990_and_2000_on_misestimation.txt", 'w') # write to this file
	f.write("{\ssp\n") # single-spacing
	f.write("\ctable[\n")
	f.write("\tcaption\t= {Effect of Changes between 1990 and 2000 on Misestimation},\n") # sentence-style without terminal period
	f.write("\tlabel\t= effect_of_changes_btw_1990_and_2000_on_misestimation,\n")
	f.write("\tpos\t= H,\n") # top or page by itself
	f.write("\tnotespar,\n") # footnotes must be flush left (Turabian 2007, sec. 26.2.7)
	f.write("\tnosuper,\n") # typeset sources and notes on the line, manually superscript everything else
	f.write("\tmincapwidth\t= 1.0\\textwidth,\n") # make the table caption and footnotes as wide as the text
	f.write("\t]{lc}{\n") # first column is flush left, other columns are centered
	f.write("\\tnote[Sources:~\\textnormal{Data adapted from \citet{brown.et.al.-2002}; \\textit{Forbes Magazine} (1990, 2000); \citet{mortality.database}.}]{\\\} \n") # terminal period
	f.write("\\tnote[]{\\\} % for spacing\n")
	f.write("\\tnote[Note:~\\textnormal{This table shows how certain changes that occurred between 1990 and 2000 would have affected the probability that the estate-multiplier method would underestimate what the 400 wealthiest Americans were assumed to be worth by at least 76 percent of their wealth, given different assumptions about the valuation of wealth and the rates at which people are sampled.}]{\\\} \n")
	f.write("}{\FL[\lightrulewidth]\n")
	f.write("\t&\t\multicolumn{1}{c}{Probability of}\t\NN\n")
	f.write("\t&\t\multicolumn{1}{c}{underestimating by}\t\NN\n")
	f.write("Changes\t&\t\multicolumn{1}{c}{at least 76\%}\t\ML\n")
	#...

	# right wealths and rates
	set_up = 'right wealths and right rates'
	f.write('\NN\n')
	f.write('\t&\t' + '\multicolumn{1}{c}{\\textit{Right wealths and rates}}' + '\t\NN\n')
	f.write('\NN\n')
	for i in range(0, len(changes)):
		change = changes[i]
		if change == 'No changes relative to base year':
			s = 'No changes'
		else:
			s = change
		s += '\t&\t'
		if change == 'No changes relative to base year':
			s += "{\color{white}\%}"
		s += '$' + str(format(dec(prob_under_estimate_by_2000_amount_for_w_changes[set_up][change]), '.2f')) + '$'
		if change == 'No changes relative to base year':
			s += "\%"	
		s += '\t\NN\n'
		if change in ['Ages', 'Sampling rates', 'Ages and sampling rates']:
			pass # the figure would be too long if these were printed
		else:
			f.write(s)

	# right wealths, but wrong rates
	set_up = 'right wealths but wrong rates'
	f.write('\NN\n')
	f.write('\t&\t' + '\multicolumn{1}{c}{\\textit{Right wealths, but wrong rates}}' + '\t\NN\n')
	f.write('\NN\n')
	for i in range(0, len(changes)):
		change = changes[i]
		if change == 'No changes relative to base year':
			s = 'No changes'
		else:
			s = change
		s += '\t&\t'
		if change == 'No changes relative to base year':
			s += "{\color{white}\%}"
		s += '$' + str(format(dec(prob_under_estimate_by_2000_amount_for_w_changes[set_up][change]), '.0f')) + '$'
		if change == 'No changes relative to base year':
			s += "\%"	
		s += '\t\NN\n'
		if change in ['Ages', 'Sampling rates', 'Ages and sampling rates']:
			pass # the figure would be too long if these were printed
		else:
			f.write(s)

	# wrong wealths, but right rates
	set_up = 'wrong wealths but right rates'
	f.write('\NN\n')
	f.write('\t&\t' + '\multicolumn{1}{c}{\\textit{Wrong wealths, but right rates}}' + '\t\NN\n')
	f.write('\NN\n')
	for i in range(0, len(changes)):
		change = changes[i]
		if change == 'No changes relative to base year':
			s = 'No changes'
		else:
			s = change
		s += '\t&\t'
		if change == 'No changes relative to base year':
			s += "{\color{white}\%}"
		s += '$' + str(format(dec(prob_under_estimate_by_2000_amount_for_w_changes[set_up][change]), '.0f')) + '$'
		if change == 'No changes relative to base year':
			s += "\%"	
		s += '\t\NN\n'
		if change in ['Ages', 'Sampling rates', 'Ages and sampling rates']:
			pass # the figure would be too long if these were printed
		else:
			f.write(s)

	# wrong wealths and rates
	set_up = 'wrong wealths and wrong rates'
	f.write('\NN\n')
	f.write('\t&\t' + '\multicolumn{1}{c}{\\textit{Wrong wealths and rates}}' + '\t\NN\n')
	f.write('\NN\n')
	for i in range(0, len(changes)):
		change = changes[i]
		if change == 'No changes relative to base year':
			s = 'No changes'
		else:
			s = change
		s += '\t&\t'
		if change == 'No changes relative to base year':
			s += "{\color{white}\%}"
		s += '$' + str(format(dec(prob_under_estimate_by_2000_amount_for_w_changes[set_up][change]), '.0f')) + '$'
		if change == 'No changes relative to base year':
			s += "\%"	
		s += '\t\NN\n'
		if change in []:#['Ages', 'Sampling rates', 'Ages and sampling rates']:
			pass # the figure would be too long if these were printed
		else:
			f.write(s)

	#...
	f.write('\t\LL[\lightrulewidth]\n')
	f.write("\NN % for spacing\n") # for spacing
	f.write("}\n") # end ctable
	f.write("}\n") # end single-spacing
	f.close()

	## TABLE A.1: EFFECT OF CHANGES BETWEEN 1918 AND 2000 ##

	# uses packages ctable and dcolumn
	f = open("./tables/tables_specific_to_first_essay/effect_of_changes_btw_1918_and_2000_on_misestimation.txt", 'w') # write to this file
	f.write("{\ssp\n") # single-spacing
	f.write("\ctable[\n")
	f.write("\tcaption\t= {Effect of Changes between 1918 and 2000 on Misestimation},\n") # sentence-style without terminal period
	f.write("\tlabel\t= effect_of_changes_btw_1918_and_2000_on_misestimation,\n")
	f.write("\tpos\t= H,\n") # top or page by itself
	f.write("\tnotespar,\n") # footnotes must be flush left (Turabian 2007, sec. 26.2.7)
	f.write("\tnosuper,\n") # typeset sources and notes on the line, manually superscript everything else
	f.write("\tmincapwidth\t= 1.0\\textwidth,\n") # make the table caption and footnotes as wide as the text
	f.write("\t]{lc}{\n") # first column is flush left, other columns are centered
	f.write("\\tnote[Sources:~\\textnormal{Data adapted from \citet{brown.et.al.-2002}; \\textit{Forbes Magazine} (1918, 1983); \citet{mortality.database}.}]{\\\} \n") # terminal period
	f.write("\\tnote[]{\\\} % for spacing\n")
	f.write("\\tnote[Note:~\\textnormal{This table shows how certain changes that occurred between 1918 and 2000 would have affected the median amount by which the estate-multiplier method would underestimate what the 30 wealthiest Americans were assumed to be worth. The median amount is reported as a percentage of their wealth. For both years, people were sampled at the Kopzuk-Saez mortality rates, the people who were sampled were inflated by the rates at which they were sampled, and their fully valued wealths were also inflated by those rates.}]{\\\} \n")
	f.write("}{\FL[\lightrulewidth]\n")
	f.write("\t&\t\multicolumn{1}{c}{Median}\t\NN\n")
	f.write("Changes\t&\t\multicolumn{1}{c}{misestimation}\t\ML\n")
	#...
	f.write('\NN\n')
	for i in range(0, len(changes)):
		change = changes[i]
		if change == 'No changes relative to base year':
			s = 'No changes'
		else:
			s = change
		s += '\t&\t'
		if change == 'No changes relative to base year':
			s += "{\color{white}\%}"
		s += "$" + str(format(dec(median_bias_as_a_perct_of_wealth_for_w_changes[change]), '.0f')) + "$"
		if change == 'No changes relative to base year':
			s += '\%'
		s += '\t\NN\n'
		f.write(s)
	#...
	f.write('\t\LL[\lightrulewidth]\n')
	f.write("\NN % for spacing\n") # for spacing
	f.write("}\n") # end ctable
	f.write("}\n") # end single-spacing
	f.close()