"""
Filename: load_forbes_400_dataset.py
Python version: 2.7
Source: Capehart, Kevin W. Essays on the Wealthiest Americans. PhD dissertation, American University, Washington, DC, 2014. 
Description: This file loads a dataset based on Forbes Magazine's annual list of the 400 wealthiest Americans. See the dissertation cited above for more details on the dataset. Some other relevant sets of data are also loaded, including data on the dates for which the magazine tried to take a snapshot of wealth, the people who dropped off the magazine's list because they died, and the people who dropped off the list because they renounced their American citizenship. 
"""

from __future__ import division
import numpy 
import datetime # import for dealing with dates

## LOAD FORBES 400 DATA ##

# define the most-recent year of the list
last_year = 2013
# define the years between the earliest and most-recent lists
years = [str(y) for y in range(1982, last_year+1)]
# define some other sets of years that will prove useful
years_less_initial = [str(y) for y in range(1983, last_year+1)]
years_less_last = [str(y) for y in range(1982, (last_year-1)+1)]
years_less_initial_and_last = [str(y) for y in range(1983, (last_year-1)+1)]
descending_years = [str(y) for y in range(last_year, 1982-1, -1)]

# load the Forbes 400 data from the data subdirectory
filename = "./data/forbes_400_dataset_1982_to_" + str(last_year) + ".txt"
header = ['name']
for info in ['wealth','age','source','industry','selfmade or inherited']:
	header.extend(info+year for year in years)
rawdata = numpy.loadtxt(filename, delimiter='\t', skiprows=7, dtype={'names':(header), 'formats':(['<S99']*len(header))})
# create a dictionary of the data
data = dict(zip(rawdata['name'], [dict(zip(['wealth', 'age', 'source', 'industry', 'selfmade or inherited'], [dict(zip(years, [rawdata['wealth'+y][i] for y in years])), dict(zip(years, [rawdata['age'+y][i] for y in years])), dict(zip(years, [rawdata['source'+y][i] for y in years])), dict(zip(years, [rawdata['industry'+y][i] for y in years])), dict(zip(years, [rawdata['selfmade or inherited'+y][i] for y in years]))])) for i in range(0, len(rawdata['name']))]))
# get list of names sorted alphabetically
names = sorted(data.keys())

# load data on Forbes 400 members who dropped out because they died
filename = "./data/exit_by_death_data.txt"
header = ['name', 'year']
rawdata = numpy.loadtxt(filename, delimiter='\t', skiprows=2, dtype={'names':(header), 'formats':(['<S50' for _ in range(0, len(header))])})
# get dict of names of the dead by year
year_dead_in_for = dict(zip(rawdata['name'], rawdata['year']))
dead_in = dict(zip(years_less_initial, [[] for year in years_less_initial]))
for name in year_dead_in_for.keys():
	year = year_dead_in_for[name]
	if year in years:
		dead_in[year].append(name)

# load data on Forbes 400 members who dropped out because they renounced their citizenship
filename = "./data/exit_by_renunciation_of_citizenship_data.txt"
header = ['name', 'year']
rawdata = numpy.loadtxt(filename, delimiter='\t', skiprows=8, dtype={'names':(header), 'formats':(['<S50' for _ in range(0, len(header))])})
# get dict of names of those who renounced their citizenship by year
year_renounced_citizenship_in_for = dict(zip(rawdata['name'], rawdata['year']))
renunciants_in = dict(zip(years_less_initial, [[] for year in years_less_initial]))
for name in year_renounced_citizenship_in_for.keys():
	year = year_renounced_citizenship_in_for[name]
	if year in years:
		renunciants_in[year].append(name)

## RECORD SOME SUMMARY STATISTICS ##

# Note that M.S. Forbes and his imputed wealth are included in the following. 

# define dicts for storing infos
infos = ['wealths', 'len', 'sum', 'min', 'median', 'mean', 'max']
summary_stats = dict(zip(infos, [dict(zip(years, [[] for year in years])) for _ in infos]))

# get sorted wealths by year
for year in years:
	for name in names:
		wealth = data[name]['wealth'][year]
		if (wealth != ''):
			summary_stats['wealths'][year].append(float(wealth))
	summary_stats['wealths'][year].sort(reverse=True) # sort in descending order

# get stats on wealths
for year in years:
	summary_stats['len'][year] = len(summary_stats['wealths'][year])
	summary_stats['sum'][year] = sum(summary_stats['wealths'][year])
	summary_stats['min'][year] = min(summary_stats['wealths'][year])
	summary_stats['median'][year] = numpy.median(summary_stats['wealths'][year])
	summary_stats['mean'][year] = numpy.mean(summary_stats['wealths'][year])
	summary_stats['max'][year] = max(summary_stats['wealths'][year])

## DATES OF VALUATION ##

# load the dates (formatted as year, month, day) when Forbes priced publicly traded stocks
valued_on_in = dict(zip(years, [datetime.date(1982, 8, 25), # imputed (see comments below)
	datetime.date(1983, 8, 24),
	datetime.date(1984, 8, 17),
	datetime.date(1985, 9, 6),
	datetime.date(1986, 9, 12),
	datetime.date(1987, 9, 11),
	datetime.date(1988, 8, 30),
	datetime.date(1989, 9, 8),
	datetime.date(1990, 9, 5),
	datetime.date(1991, 9, 4),
	datetime.date(1992, 8, 27), # last Thursday of August
	datetime.date(1993, 8, 27), # last Friday of August * 
	datetime.date(1994, 8, 31), # last day of August, a Wednesday
	datetime.date(1995, 8, 23), # second to last Wednesday of August
	datetime.date(1996, 8, 23), # second to last Friday of August * 
	datetime.date(1997, 8, 22), # second to last Friday of August * 
	datetime.date(1998, 9, 1), # first day of September, a Tuesday
	datetime.date(1999, 8, 27), # imputed
	datetime.date(2000, 8, 25), # imputed
	datetime.date(2001, 8, 27), # last Monday of August
	datetime.date(2002, 8, 16), # mid-August, a Friday * 
	datetime.date(2003, 8, 29), # imputed
	datetime.date(2004, 8, 27), # imputed
	datetime.date(2005, 8, 26), # imputed
	datetime.date(2006, 8, 31), # last day of August, a Thursday
	datetime.date(2007, 8, 31), # last day of August, a Friday * 
	datetime.date(2008, 8, 29), # last Friday of August * 
	datetime.date(2009, 9, 10), # second Thursday of September
	datetime.date(2010, 8, 25), # last Wednesday of August
	datetime.date(2011, 8, 26), # last Friday of August * 
	datetime.date(2012, 8, 24), # second to last Friday of August * 
	datetime.date(2013, 8, 23)])) # second to last Friday of August *

# In the 1982 Forbes 400, the valuation date was reported as "mid-August," 1982. We'll impute the valuation date as Wednesday, August 25, 1982, based on the fact that the valuation date for the 1983 Forbes 400 was Wednesday, August 24, 1983. To our knowledge, neither Forbes Magazine nor Forbes.com reported the valuation dates in 1999, 2000, or 2003 to 2005. We'll impute the valuation dates as the last Friday of August of the respective years, given that, in recent years (i.e., since 1992), most of the valuation dates (9 out of 15) have been on a Friday towards the end of August (the last or second to last Friday of the month). 
years_w_imputed_valuation_dates = ['1982', '1999', '2000', '2003', '2004', '2005']