"""
Filename: unit_tests_code.py
Python version: 2.7
Source: Capehart, Kevin W. Essays on the Wealthiest Americans. PhD dissertation, American University, Washington, DC, 2014. 
Description: This files includes unit tests for the other files.
"""

## IMPORT THE UNIT TEST MODULE ##

import unittest

# In computer science, unit testing is method by which programmers try to ensure that their programs work as intended. If economics is to become a branch of applied computer science in the same way that it has sometimes been nothing more than a branch of applied mathematics, then such methods can be adopted without much if any adaption. That said, exactly how such methods should be adopted and adapted by economists who engage in computationally intensive research is an open question. Here, we simply try to ensure that our code works as intended from the perspective of an economist. An economist would expect that converting current dollars to constant dollars and back should yield the same current dollar amount, for example. Much of the code was admittedly written through trial and error, but these tests at least formalize some of the trials that did not result in errors. Any remaining errors in the code are the author's. 

## UNIT TESTS FOR FIRST ESSAY ##

import essay_on_estimation_of_wealth_code 
# functions are: dec, in_2000_dollars, not_in_2000_dollars, not_in_2009_dollars, estimated_mortality_rate_in, get_pareto_index_if_wealthiest_perct_owns_one_minus_perct, wealths_w_pareto_index_of, and get_rounded

# Note that 'dec' simply takes a float and returns a decimal. 
# Note that, because Kopczuk and Saez (2004a,b) used an earlier version of the same price index that we use, the in_2000_dollars function and the not_in_2000_dollars function are not the inverse of each other. We use the two functions to covert their constant dollars into current dollars and then into our constant dollars. See the unit tests for the data appendix code for an example of a unit test where similar functions for deflating and re-inflating dollar amounts are supposed to be the inverse of each other. 
# See the unit tests for the data appendix code for a test of a function like 'not_in_2009_dollars'
# The functional form of 'estimated_mortality_rate_in' is what one must be sure is correct. See sec. A.3 for more on the functional form. 
# 'eme_of_wealth_of_400_or_other_wealthiest_in' is a class with its own functions and assertions
# 'sample_and_estimate_total_wealth_of_the_400_or_other_wealthiest_from' is a class with its own functions and assertions
# See below for a test of 'get_pareto_index_if_wealthiest_perct_owns_one_minus_perct' and 'wealths_w_pareto_index_of', but also see sec. A.2 and B.2 for more on the relevant properties of Pareto distributions

class FirstEssayCodeTestCases(unittest.TestCase):
	""" Tests for code for the first essay """

	def test_get_pareto_index_if_wealthiest_perct_owns_one_minus_perct(self):
		""" Test wealthiest perct really does own one minus perct. Jointly tests 'get_pareto_index_if_wealthiest_perct_owns_one_minus_perct' and 'wealths_w_pareto_index_of' """
		# get Pareto index for, say, wealthiest 20 perct owning 80 perct of wealth
		pareto_index = essay_on_estimation_of_wealth_code.get_pareto_index_if_wealthiest_perct_owns_one_minus_perct(perct=0.20)
		# get wealths distributed with that Pareto index by using 'wealths_w_pareto_index_of'
		wealths = essay_on_estimation_of_wealth_code.wealths_w_pareto_index_of(pareto_index=pareto_index, total_wealth=essay_on_estimation_of_wealth_code.dec(1000), number_of_people=1000)
		# test that wealthiest 20 perct really does own 80 perct
		twenty_perct_of_pop = int(len(wealths) * 0.20)
		wealth = sum(wealths[1-1:int(twenty_perct_of_pop)-1])
		wealth_share = wealth / sum(wealths)
		self.assertAlmostEqual(essay_on_estimation_of_wealth_code.dec(0.80), wealth_share, places=2)

	def test_get_rounded(self):
		""" Test that dollar amounts in billions are really rounded in the desired manner """
		# five billion or more rounded to nearest billion?
		amount_in_billions = 11.11 # greater than 5 billion but otherwise arbitrary
		should_round_to = 11.00 # 11.11 billion should round to 11 billion
		dec_amount_in_billions = essay_on_estimation_of_wealth_code.dec(amount_in_billions)
		rounded_to = essay_on_estimation_of_wealth_code.get_rounded(dec_amount_in_billions)
		self.assertAlmostEqual(essay_on_estimation_of_wealth_code.dec(should_round_to), rounded_to, places=2)
		# less than five billion rounded to nearest 100 million?
		amount_in_billions = 0.110 # less than 5 billion but otherwise arbitrary
		should_round_to = 0.100 # 110 million should round to 100 million
		dec_amount_in_billions = essay_on_estimation_of_wealth_code.dec(amount_in_billions)
		rounded_to = essay_on_estimation_of_wealth_code.get_rounded(dec_amount_in_billions)
		self.assertAlmostEqual(essay_on_estimation_of_wealth_code.dec(should_round_to), rounded_to, places=2)

## UNIT TESTS FOR SECOND ESSAY ##

import essay_on_distribution_of_wealth_code 
# functions are: get_pareto_index_using_klass_et_al_method, random_pareto, random_bounded_pareto, get_p_value

# Note that 'get_pareto_index_using_klass_et_al_method' is essentially just the OLS estimate of a slope parameter, so its results can be checked against the results of econometric programs like gretl. 

class SecondEssayCodeTestCases(unittest.TestCase):
	""" Tests for code for the second essay """

	def test_p_value(self):
		""" Test that the p-value for a z-test with a z-stat of zero is unity """
		self.assertAlmostEqual(1., essay_on_distribution_of_wealth_code.get_p_value(0.))
	
	def test_random_pareto_good_fit_to_pareto(self):
		""" Test that the random_pareto function generates random draws that really are a good fit to a Pareto distribution. Jointly tests 'resample_d_stats_for_unknown_pareto_distribution', 'k_s_like_test_against_unknown_pareto_distribution', 'fit_pareto_to', and 'random_pareto' """
		# import some modules
		import numpy # import numpy for seeding random number generator, although test should pass for almost any seed
		from os.path import isfile # import for checking whether a file exists
		import pickle # import for dumping and loading computationally intensive results
		# set the following to True to re-estimate if already estimated
		re_estimate = False
		# check if already been estimated
		filename = "./data/data_specific_to_second_essay/p_value_for_random_pareto_test.pkl"
		already_estimated = isfile(filename)
		# estimate or re-estimate
		if not already_estimated or re_estimate:
			# get random draws from a Pareto distribution
			numpy.random.seed(250624)
			large_number = 1000 # large but otherwise arbitrary number of random samples to draw
			lower_bound = 1.0 # arbitrary lower bound parameter
			shape_parameter = 1.1609640474436811 # arbitrary shape parameter, corresponds to 80-20 rule
			wealths = essay_on_distribution_of_wealth_code.random_pareto(lower_bound=lower_bound, shape_parameter=shape_parameter, size=large_number)
			# get discrepancy stat against fitted Pareto distribution
			k_s_like_test = essay_on_distribution_of_wealth_code.k_s_like_test_against_unknown_pareto_distribution(wealths=wealths)
			discrepancy_statistic = k_s_like_test.get_discrepancy_statistic()
			# get p-value for D-stat
			numpy.random.seed(250624)
			resample_d_stats = essay_on_distribution_of_wealth_code.resample_d_stats_for_unknown_pareto_distribution(wealths=wealths, round_to_nearest_for={}, number_of_resamples=large_number, fitting=True, rounding=False)
			p_value_for_random_pareto_test = resample_d_stats.get_p_value(discrepancy_statistic)
			# dump result
			pickle.dump(p_value_for_random_pareto_test, open(filename, 'wb'))
		else:
			# load result
			p_value_for_random_pareto_test = pickle.load(open(filename, 'rb'))
		# fail to reject Pareto distribution at conventional levels of statistical significance?
		self.assertTrue(p_value_for_random_pareto_test > 0.10)
		
	def test_random_bounded_pareto_good_fit_to_bounded_pareto(self):
		""" Tests that the random_bounded_pareto function generates random draws that really are a good fit to a bounded Pareto distribution. Like the unit test for the random_pareto function, jointly tests associated functions and classes """
		# import some modules
		import numpy # import numpy for seeding random number generator, although test should pass for almost any seed
		from os.path import isfile # import for checking whether a file exists
		import pickle # import for dumping and loading computationally intensive results
		# set the following to True to re-estimate if already estimated
		re_estimate = False
		# check if already been estimated
		filename = "./data/data_specific_to_second_essay/p_value_for_random_bounded_pareto_test.pkl"
		already_estimated = isfile(filename)
		# estimate or re-estimate
		if not already_estimated or re_estimate:
			# get random draws from a Pareto distribution
			numpy.random.seed(250624)
			large_number = 1000 # large but otherwise arbitrary number of random samples to draw
			lower_bound = 1.0 # arbitrary lower bound parameter
			shape_parameter = 1.1609640474436811 # arbitrary shape parameter
			upper_bound = 100.0 # arbitrary upper bound parameter
			wealths = essay_on_distribution_of_wealth_code.random_bounded_pareto(lower_bound=lower_bound, shape_parameter=shape_parameter, upper_bound=upper_bound, size=large_number)
			# get discrepancy stat against fitted bounded Pareto distribution
			k_s_like_test = essay_on_distribution_of_wealth_code.k_s_like_test_against_unknown_bounded_pareto_distribution(wealths=wealths)
			discrepancy_statistic = k_s_like_test.get_discrepancy_statistic()
			# get p-value for D-stat
			numpy.random.seed(250624)
			round_to_nearest_for = dict(zip(['billionaires', 'sub-billionaires'], [0.001, 0.001])) # round to the nearest one thousandth, a small but otherwise arbitrary amount
			resample_d_stats = essay_on_distribution_of_wealth_code.resample_d_stats_for_unknown_pareto_distribution(wealths=wealths, round_to_nearest_for=round_to_nearest_for, number_of_resamples=large_number) # always fits and rounds
			p_value_for_random_bounded_pareto_test = resample_d_stats.get_p_value(discrepancy_statistic)
			# dump result
			pickle.dump(p_value_for_random_bounded_pareto_test, open(filename, 'wb'))
		else:
			# load result
			p_value_for_random_bounded_pareto_test = pickle.load(open(filename, 'rb'))
		# fail to reject Pareto distribution at conventional levels of statistical significance?
		self.assertTrue(p_value_for_random_bounded_pareto_test > 0.10)

## UNIT TESTS FOR THIRD ESSAY ##

import essay_on_duration_of_wealth_code 
# functions are: get_normal_ci, get_cp_ci, and get_p_value

# It can be noted that, for the multinomial logit models in the third essay, estimates made in the code were checked against estimates made by the open source econometric program gretl. See the code for a check of the baseline model. 
# See the second essay's unit tests for a test of the 'get_p_value' function, which is used in a different context, but which has the same functional form

class ThirdEssayCodeTestCases(unittest.TestCase):
	""" Tests for code for the third essay """
	
	def test_get_normal_ci(self):
		""" Test that the z-stat for a 95% CI based on the Normal distribution is about equal to 1.96 """
		lower, upper = essay_on_duration_of_wealth_code.get_normal_ci(point=0.0, se=1.0, confidence_level=0.95)
		self.assertAlmostEqual(1.96, upper, places=2)
			
	def test_get_cp_ci(self):
		""" Test that we are using the btdtri function properly by having it return the zero and 100 percent quantiles """
		# use a number of trials and failures that are otherwise arbitrary except that trials is greater than failures
		# so that we can get the 100 percent quantile from the btdri function, set confidence_level to -1
		should_be_unity, _ = essay_on_duration_of_wealth_code.get_cp_ci(trials=100, failures=50, confidence_level=-1.)
		self.assertAlmostEqual(1., should_be_unity)
		# so that we can get the zero percent quantile from the btdri function, set confidence_level to 1
		should_be_zero, _ = essay_on_duration_of_wealth_code.get_cp_ci(trials=100, failures=50, confidence_level=1.)
		self.assertAlmostEqual(0., should_be_zero)

## UNIT TESTS FOR DATA APPENDIX CODE ##

import data_appendix_code 
# functions are: deflate_by_CPI, not_in_2009_dollars, arcsinh, and sinh

class DataAppendixCodeTestCases(unittest.TestCase):
	""" Tests for code for the data appendix """
		
	def test_not_in_2009_dollars_returns_current_dollars(self):
		""" Test that the not_in_2009_dollars function really does return current dollars. Jointly tests 'deflate_by_CPI' and 'not_in_2009_dollars' """
		years = [str(y) for y in range(1977, 2013+1)]
		for year in years:
			one_hundred_year_dollars_in_2000_dollars = data_appendix_code.deflate_by_CPI(current_dollar_amount=100.00, year=year, base_year='2009')
			dollars_in_current_dollars = data_appendix_code.not_in_2009_dollars(constant_dollar_amount=one_hundred_year_dollars_in_2000_dollars, year=year)
			self.assertAlmostEqual(100.00, dollars_in_current_dollars, places=2)
	
	def test_arcsinh_is_sinh_inverse_and_vice_versa(self):
		""" Test that the arcsinh function is the inverse of the sinh function and vice versa. Jointly tests 'sinh' and 'arcsinh' """
		self.assertAlmostEqual(data_appendix_code.sinh(data_appendix_code.arcsinh(100.0)), 100.0)
		self.assertAlmostEqual(data_appendix_code.arcsinh(data_appendix_code.sinh(100.0)), 100.0)

## RUN UNIT TESTS ##

if __name__ == '__main__':
	unittest.main()